#!/bin/sh
#
# edit-paths -p perms -e sededit in out
#
# Substitute a standard set of tokenes in a script or config file.  This
# puts in build-time paths and file names.  Tokens in the form @name@ are
# replaced.
#  o -p perms - Sets file permissions to perms, which is in the forms
#    accepted by chmod
#  o -e sededit - An command to pass to sed.
#

# The only paths edited in:
ABS_ROOT="%ABS_ROOT%"
MAKE="%MAKE%"
SCRIPTS_DIR=${ABS_ROOT}/build/scripts

set -e

perms=""
edits=""

while true ; do
    case "$1" in
        -p) shift
            perms="$1"
            shift;;
        -e) shift
            edits="$edits -e $1"
            shift;;
        -*) echo "Unknown option: $1" >&2
            exit 1;;
         *) break;;
    esac
done

if [ $# != 2 ] ; then
    echo "wrong # args: edit-paths -p perms -e sededit in out" >&2
    exit 1
fi

in="$1"
out="$2"

# Get the variables to substitute directly from make.
eval `cd ${SCRIPTS_DIR} && ${MAKE} --no-print-directory list-paths`
mkdir -p "`dirname $out`"
rm -f "$out"

sed -e "s|@JAVA@|${JAVA}|" \
    -e "s|@JAVAC@|${JAVAC}|" \
    -e "s|@JAVADOC@|${JAVADOC}|" \
    -e "s|@JDKDIR@|${JDKDIR}|" \
    -e "s|@JDKVERSION@|${JDKVERSION}|" \
    -e "s|@ENHYDRA_DIR@|${ENHYDRA_DIR}|" \
    -e "s|@ENHYDRA_CLASSES@|${ENHYDRA_CLASSES}|" \
    -e "s|@SWING_DIR@|${SWING_DIR}|" \
    -e "s|@SWING_CLASSES@|${SWING_CLASSES}|" \
    -e "s|@ENHYDRA_CLASSPATH@|${ENHYDRA_CLASSPATH}|" \
    -e "s|@OUTPUT@|${OUTPUT}|" \
    -e "s|@ROOT@|${ROOT}|" \
    -e "s|@ENHYDRA_VERSION@|${ENHYDRA_VERSION}|" \
    -e "s|@MAKE@|${MAKE}|" \
    $edits \
    < "$in"  > "$out"

if [ "X$perms" != "X" ] ; then
    chmod $perms "$out"
fi
