#
# Enhydra Java Application Server Project
# 
# The contents of this file are subject to the Enhydra Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License on
# the Enhydra web site ( http://www.enhydra.org/ ).
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
# the License for the specific terms governing rights and limitations
# under the License.
# 
# The Initial Developer of the Enhydra Application Server is Lutris
# Technologies, Inc. The Enhydra Application Server and portions created
# by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
# All Rights Reserved.
# 
# Contributor(s):
# 
# $Id: config.mk,v 1.128.2.12.2.7 2000/12/12 05:10:22 builder Exp $
#

#
# **** BEFORE EDITING, READ THIS! THIS MEANS YOU ****
#  
# If you want to set variables local to your tree, create a file
#    config.local.mk
# and set the make variables there.  This way, you will not accidently
# check in the changes.
#
# The most common variables that you will need to override are:
# JDKDIR - the location of the JDK.
# JAVACCDIR - the location of the Sun JavaCC (Java Compiler Compiler).
# SWINGDIR - the location of the Sun Swing Set (JFC).  This is only necessary
#	if you are using a 1.1 JDK or you wish to use the Enhydra Debugger
#	under other servlet runners like Apache JServ.
#

#
# WARNING: Modify this version string and the constant in
#     modules/EnhydraAppManager/src/com/lutris/Enhydra.java
# when changing Enhydra versions.
#
# *WARNING*: read above notice before changing the version number.
#
ENHYDRA_VERSION = 3.1

#
# CVS information for making a source distribution.  This is only used by the
# make dist target.  One only need modify this if one is building a full
# enhydra distribution outside of Lutris.
#
#   ENHYDRA_CVS_REPOSITORY - The -d string
#   ENHYDRA_CVS_MODULE - The CVS module containing Enhydra.
#
ENHYDRA_CVS_REPOSITORY = www.enhydra.org:/u/cvs
ENHYDRA_CVS_MODULE = Projects/EnhydraOrg/Enhydra

#
# Override defined for since we are compiling the Enhydra tree rather
# than a app.
# NOTE:  Absolute paths to the class output directory and module class
# path components are used because long relative paths (lots of ../..)
# broke the NT compile of certain modules. Yea, don't know why, but
# using absolute paths fixed it.
#
RELEASE_NAME = enhydra
ENHYDRA_DIR = $(ROOT)/output/${RELEASE_NAME}${ENHYDRA_VERSION}
OUTPUT = $(ENHYDRA_DIR)

JOLTC = $(ENHYDRA_DIR)/bin/joltc
ifeq ($(JARNAME),)
    JARNAME = enhydra
endif
ifeq ($(CLASS_OUTPUT),)
    CLASS_OUTPUT = $(ABS_ROOT)/classes/$(JARNAME)
endif
ENHYDRA_CLASSES = $(ABS_ROOT)/classes/enhydra

# Set to YES for apps, NO for Enhydra build.
ENHYDRA_CHECK = NO

# Only relevant for Enhydra build.
BUILD_CHECK = YES

# This is so that the Entering/Leaving directory messages will
# show up on Windows. 
MAKEFLAGS:="-w"

#
# Scripts used in the build
#
EDIT_PATHS = ${ROOT}/build/scripts/edit-paths
COPY_FILE = ${ROOT}/build/scripts/copy-file

#
# Directories, etc.
#
DIST_OUTPUT = ${OUTPUT}/dist
DIST_BIN_OUTPUT = ${OUTPUT}/dist/bin

#
# Local override
#
ifeq ($(shell test -r $(ROOT)/config.local.mk && echo YES),YES)
    include $(ROOT)/config.local.mk
endif

#
# Internally to Lutris, we keep most Enhydra dependencies in a CVS
# repository.  We pick up the checked-out repository components here
# if they are available to ensure a correctly build binary distribution.
# This overrides the default settings in stdconf.mk.  Wish we could
# redistribute this stuff.
#
DEPEND_DIST_DIR = ${ABS_ROOT}/../Dist

ifeq (${SWINGDIR},)
    SWINGDIR=$(wildcard ${DEPEND_DIST_DIR}/Sun/Swing)
endif
ifeq (${JAVACCDIR},)
    JAVACCDIR=$(wildcard ${DEPEND_DIST_DIR}/Sun/JavaCC)
endif
ifeq (${JSSEDIR},)
    JSSEDIR=$(wildcard ${DEPEND_DIST_DIR}/Sun/JSSE)
endif
ifeq (${JBUILDER_DIR},)
    JBUILDER_DIR=$(wildcard ${DEPEND_DIST_DIR}/Borland/JBuilder)
endif
ifeq (${JBFOUNDATION_DIR},)
    JBFOUNDATION_DIR=$(wildcard ${DEPEND_DIST_DIR}/Borland/JBFoundation)
endif

#
# We point at the pre-install stdconf.mk to get around chicken & egg.
#
STDCONF=$(ROOT)/lib/make/stdconf.mk
include $(ROOT)/lib/make/stdrules.mk

#
# Must add all modules to the CLASSPATH so that javac can find them.
# Must be done after stdrules.mk to get ${PS}.
#
MODULES=${ABS_ROOT}/modules
ENHYDRA_CLASSPATH:=${MODULES}/HTTPClient/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/HttpServerTest/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/TestBed/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/EnhydraAppManager/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/Core/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/DODS/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/DODS${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/DatabaseManager/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/JDDI/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/JTidy/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/MultiServer/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/MultiServerAdmin/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/Xerces/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/XMLC/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/gnu.regexp/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/NewApp/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/Tomcat/src/share${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/I18N/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${MODULES}/ServletContainer/src${PS}${ENHYDRA_CLASSPATH}
ENHYDRA_CLASSPATH:=${JDKDIR}/lib/tools.jar${PS}${ENHYDRA_CLASSPATH}

