/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HtmlTable.java,v 1.8.12.1 2000/10/19 17:58:53 jasona Exp $
 */





package com.lutris.html;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This utility class is used to compose an HTML table.
 *
 * @see HtmlTableRow
 * @see HtmlTableDataCell
 * @see HtmlTableHeaderCell
 * @author Kyle Clark
 * @author Paul Morgan
 * @version 1.0 (File $Revision: 1.8.12.1 $)
 */
public class HtmlTable {

    private String caption = null;
    private int percentWidth = 0;
    private int borderWidth = -1;
    private int cellPadding = -1;
    private int cellSpacing = -1;
    private String bgColor = null;
    private String rules = null;
    private String frame = null;
    private Vector rows = new Vector();
    private Object thead = null;

    public HtmlTable()
    {
        this.caption = null;
    }

    public HtmlTable(String caption)
    {
        this.caption = caption;
    }

    public void setBorderWidth(int pixels)
    {
        this.borderWidth = pixels;
    }

    public void addRow(HtmlTableRow row)
    {
        rows.addElement(row);
    }

    public void addRow(String row)
    {
        rows.addElement(row);
    }

    public void setHeader(Object row)
    {
        thead = row;
    }

    public void setPercentWidth(int percent)
    {
        if (percent > 0 && percent <= 100)
            this.percentWidth = percent;
    }

    public void setCaption(String caption)
    {
        this.caption = caption;
    }

    public void setCellSpacing(int size)
    {
        if (size >= 0)
            cellSpacing = size;
    }

    public void setCellPadding(int size)
    {
        if (size >= 0)
            cellPadding = size;
    }

    public void setBackgroundColor(String bgColor)
    {
        this.bgColor = bgColor;
    }

    public void setFrame(String frame)
    {
        this.frame = frame;
    }

    public void setRules(String rules)
    {
        this.rules = rules;
    }

    public String toString()
    {
        StringBuffer html = new StringBuffer();

        html.append("<TABLE");
        if (borderWidth >= 0) 
            html.append(" BORDER=" + borderWidth);
        if (percentWidth > 0) 
            html.append(" WIDTH=" + percentWidth + "%");
        if (cellPadding >= 0) 
            html.append(" CELLPADDING=" + cellPadding);
        if (cellSpacing >= 0) 
            html.append(" CELLSPACING=" + cellSpacing);
        if (bgColor != null)
            html.append(" BGCOLOR=" + bgColor);
        if (rules != null)
            html.append(" RULES=" + rules);
        if (frame != null)
            html.append(" FRAME=" + frame);
        html.append(">\n");

        if (caption != null) {
            html.append("<CAPTION>");
            html.append(caption);
            html.append("</CAPTION>\n");
        }

        if (thead != null) {
            html.append("<THEAD>");
            html.append(thead.toString());
            html.append("</THEAD>\n");
        }
        
        Enumeration e = rows.elements();
        while (e.hasMoreElements()) {
            html.append(e.nextElement().toString());
        }
        html.append("</TABLE>\n");

        return html.toString();
    }
}
