/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HtmlTableRow.java,v 1.8.12.1 2000/10/19 17:58:53 jasona Exp $
 */





package com.lutris.html;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This utility class is used to compose a row within a
 * HTML table.
 *
 * @see HtmlTable
 * @see HtmlTableDataCell
 * @see HtmlTableHeaderCell
 * @author Kyle Clark
 * @author Paul Morgan
 * @version 1.0 (File $Revision: 1.8.12.1 $)
 */
public class HtmlTableRow {

    public static final int ALIGN_LEFT = 1;
    public static final int ALIGN_CENTER = 2;
    public static final int ALIGN_RIGHT = 3;
    public static final int VALIGN_TOP = 1; 
    public static final int VALIGN_MIDDLE = 2;
    public static final int VALIGN_BOTTOM = 3;

    private String caption = null;
    private String align = null;
    private String valign = null;
    private Vector row = new Vector();
    private int colSpan = 0;
    private int rowSpan = 0;
    private String bgColor = null;

    public HtmlTableRow()
    {
        // noop
    }

    public HtmlTableRow(HtmlTableCell td)
    {
        addCell(td);
    }

    public HtmlTableRow(String td)
    {
        addCell(td);
    }

    public void addCell(HtmlTableCell td)
    {
        row.addElement(td);
    }

    public void addCell(String td)
    {
        row.addElement(td);
    }

    public void setHorizontalAlignment(int alignment)
    {
        switch (alignment) {
        case ALIGN_CENTER:
            this.align = "CENTER";
            break;
        case ALIGN_RIGHT:
            this.align = "RIGHT";
            break;
        default:
            this.align = "LEFT";
            break;
        }
    }

    public void setVerticalAlignment(int alignment)
    {
        switch (alignment) {
        case VALIGN_TOP:
            this.valign = "TOP";
            break;
        case VALIGN_MIDDLE:
            this.valign = "MIDDLE";
            break;
        default:
            this.valign = "BOTTOM";
            break;
        }
    }

    public void setRowSpan(int rowSpan)
    {
        if (rowSpan > 0)
            this.rowSpan = rowSpan;
    }

    public void setColSpan(int colSpan)
    {
        if (colSpan > 0)
            this.colSpan = colSpan;
    }

    public void setBackgroundColor(String bgColor)
    {
        this.bgColor = bgColor;
    }

    public String toString()
    {
        StringBuffer html = new StringBuffer();

        html.append("<TR");
        if (align != null)
            html.append(" ALIGN=" + align);
        if (valign != null)
            html.append(" VALIGN=" + valign);
        if (colSpan > 0)
            html.append(" COLSPAN=" + colSpan);
        if (rowSpan > 0)
            html.append(" ROWSPAN=" + rowSpan);
        if (bgColor != null)
            html.append(" BGCOLOR=" + bgColor);
        html.append(">\n");

        Enumeration e = row.elements();
        while (e.hasMoreElements()) {
            html.append(e.nextElement().toString());
        }
        html.append("</TR>\n");

        return html.toString();
    }
}
