/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ContentHeader.java,v 1.7.12.1 2000/10/19 17:58:53 jasona Exp $
 */




package com.lutris.mime;

/**
 * Parses and represents Mime header fields that have the form of the
 * <code>Content-Type</code> header as defined in RFC2045 Section 5.1
 * Page 11.  The format of these headers is informally:
 * <PRE>
 * 	<Header-Type>: <value> [ ; <parameter> ... ]
 * </PRE>
 * Examples:
 * <PRE>
 * 	Content-Type: application/octet-stream
 * 	Content-Type: multipart/form-data; boundary="---123456"
 * 	Content-Disposition: form-data; name=upload; filename="/tmp/myfile"
 * </PRE>
 * In addition to the actual header syntax, RFC822 header comments are
 * allowed.  Such comments consist of text enclosed in parentheses.
 */
public class ContentHeader extends MimeHeader 
{
    /**
     * Constructs a new ContentTypeHeader object from an unparsed Mime 
     * <code>Content-Type</code> header line.  The header line should
     * include the <i>entire</i> line as read from the input source.
     * 
     * @param	headerLine	The entire line containing the Mime header.
     */
    public
    ContentHeader(String headerLine)
    {
	super(headerLine);
	int pos = headerLine.indexOf(';');
	if (pos >= 0) parseParameters(headerLine.substring(pos+1));
    }

    /**
     * Constructs a new ContentTypeHeader object from an existing
     * <code>MimeHeader</code> object.
     * 
     * @param	hdr		An existing <code>MimeHeader</code> object.
     */
    public
    ContentHeader(MimeHeader hdr)
    {
	super(hdr.getRawHeaderLine());
	String headerLine = hdr.getRawHeaderLine();
	int pos = headerLine.indexOf(';');
	if (pos >= 0) parseParameters(headerLine.substring(pos+1));
    }
}
