/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MultipartMimeInputTests.java,v 1.8.8.1 2000/10/19 17:58:53 jasona Exp $
 */





package com.lutris.mime.tests;
import	java.io.*;
import	java.util.*;
import	java.security.*;
import	com.lutris.mime.*;
import	com.lutris.testFramework.*;
import	com.lutris.util.*;

/**
 * Tests for MultipartMimeInput object. 
 *
 * @version	$Revision: 1.8.8.1 $
 * @author	John Marco
 * @since	Jolt1.0
 */
public class MultipartMimeInputTests {
    
    public static
    void runStdinTest(InputStream in)
    throws Exception
    {
	int b;
	StringBuffer sb = new StringBuffer();
	outer: while ((b = in.read()) >= 0) {
	    switch ((char) b) {
		case '\n':
		    break outer;
		default:
		    sb.append((char)b);
		    break;
	    }
	}
	String ct = sb.toString();
	if (ct.endsWith("\r")) ct = ct.substring(0, ct.length() - 1);
	System.out.println("ct: " + ct);
	
	ContentHeader cheader = new ContentHeader(ct);
	
	MultipartMimeInput mime = new MultipartMimeInput(in, cheader);
	
	MultipartMimeInputStream mis;
	
	String[] sHeaders = null;

	
	while ((mis = mime.nextPart()) != null) {
	    System.out.print("==============================");
	    System.out.print(" NEXT STREAM ");
	    System.out.println("==============================");
	    sHeaders = mis.getRawHeaders();
	    if (sHeaders != null) {
		for (int i=0; i<sHeaders.length; i++) {
		    System.out.println(((sHeaders[i] != null) ? sHeaders[i] : "(null)"));
		}
	    }
	    sHeaders = mis.getGarbageHeaders();
	    if (sHeaders != null) {
		for (int i=0; i<sHeaders.length; i++) {
		    System.out.println("(Garbage) " +
				    ((sHeaders[i]!= null)  ? sHeaders[i] : "(null)"));
		}
	    }
	    System.out.print("------------------------------");
	    System.out.print(" END HEADERS ");
	    System.out.println("------------------------------");
	    while ((b = mis.read()) >= 0) {
		System.out.print((char) b);
	    }
	    System.out.println("++ENDMARKER++");
	}
	System.out.print("==============================");
	System.out.print("    DONE     ");
	System.out.println("==============================");
    }
		    
    public static
    void runSingleTest(long[] lvec)
    throws Exception
    {
	String boundary = "0123456789ABCDEF";
	TestInputStream tis = new TestInputStream(lvec, boundary);
	String ctype = "Content-Type: multipart/form-data; " +
		"boundary=\"" + boundary + "\"";
	ContentHeader cheader = new ContentHeader(ctype);
	MultipartMimeInput mime = new MultipartMimeInput(tis, cheader);
	MultipartMimeInputStream mis;
	int snum = 0;
	byte[] sink = new byte[1024];
	while ((mis = mime.nextPart()) != null) {
	    MessageDigest md = MessageDigest.getInstance("SHA");
	    DigestInputStream dis = new DigestInputStream(mis, md);
	    while (dis.read(sink) > 0) ;
	    dis.close();
	    mis.close();
	    byte[] digb = dis.getMessageDigest().digest();
	    String digest = Convert.toBase64String(digb);
	    String s = "{";
	    String c = " ";
	    for (int i=0; i<lvec.length; i++) {
		if (i == snum) s += c + "(" + lvec[i] + ")";
		else s += c + lvec[i];
		c = ", ";
	    }
	    s += " }";
	    if (!digest.equals(tis.digests[snum])) {
		System.out.println("    FAIL: " + s);
		System.out.println("        Got '" + digest + "'" +
			    " Expected '" + tis.digests[snum] + "'");
		throw new Exception("Bad checksum");
	    }
	    System.out.println("    OK: " + s);
	    ++snum;
	}
    }
    
    public static
    void runTests()
    throws Exception
    {
	int i, j;
	long lvec[] = new long[2];
	for (i=1; i<=100; i++) {
	    for (j=1; j<=100; j++) {
		lvec[0] = i;
		lvec[1] = j;
		runSingleTest(lvec);
	    }
	}
    }
}

class TestInputStream extends InputStream {
    
    public String[]	digests;

    DigestInputStream	dis;

    char[]		buf;
    int			bufPos;
    
    long[]		lengthVec;
    int			lengthPos;
    
    final int		ST_HEADER=1;
    final int		ST_DATA=2;
    final int		ST_END=3;
    int			state = ST_HEADER;
    
    String		boundary;
    String		CRLF	 = "\r\n";
    
    TestInputStream(long[] lengths, String sep)
    {
	lengthVec = lengths;
	lengthPos = 0;
	boundary = sep;
	String sbuf;
	digests = null;
	
	if ((lengthVec == null) || (lengthPos >= lengthVec.length)) {
	    sbuf = "--" + boundary + "--" + CRLF;
	    state = ST_END;
	} else  {
	    digests = new String[lengthVec.length];
	    sbuf = "--" + boundary + CRLF +
		"Test-Name: testinput" + (lengthPos + 1) + CRLF +
		"Test-Length: " + lengths[0] + CRLF +
		"Content-Disposition: test-data; name=\"testinput" +
		    (lengthPos + 1) + "\"" + CRLF +
		CRLF;
	    state = ST_HEADER;
	}
	buf = sbuf.toCharArray();
	bufPos = 0;
    }
    
    public
    int read()
    throws IOException
    {
	int ch;
	String sbuf;
	switch (state) {
	    case ST_HEADER:
		ch = buf[bufPos++];
		if (bufPos >= buf.length) {
		    InputStream is =
			new RandomInputStream(lengthVec[lengthPos], true);
		    try {
			MessageDigest md = MessageDigest.getInstance("SHA");
			dis = new DigestInputStream(is, md);
		    } catch (Throwable t) {}
		    state = ST_DATA;
		}
		return ch;
	    case ST_DATA:
		ch = dis.read();
		if (ch < 0) {
		    dis.close();
		    byte[] digb = dis.getMessageDigest().digest();
		    digests[lengthPos++] = Convert.toBase64String(digb);
		    if (lengthPos >= lengthVec.length) {
			sbuf = CRLF + "--" + boundary + "--" + CRLF;
			buf = sbuf.toCharArray();
			bufPos = 1;
			ch = buf[0];
			state = ST_END;
		    } else {
			sbuf =
			  CRLF + "--" + boundary + CRLF +
			  "Test-Name: testinput" + (lengthPos + 1) + CRLF +
			  "Test-Length: " + lengthVec[lengthPos] + CRLF +
			  "Content-Disposition: test-data; name=\"testinput" +
				(lengthPos + 1) + "\"" + CRLF +
			  CRLF;
			buf = sbuf.toCharArray();
			bufPos = 1;
			ch = buf[0];
			state = ST_HEADER;
		    }
		}
		return ch;
	    case ST_END:
		if (bufPos >= buf.length) return -1;
		return buf[bufPos++];
	}
	return -1;
    }
}

class RandomInputStream extends InputStream {
    long	bytesLeft;
    Random	rand;
    boolean	isPrintable;
    
    public
    RandomInputStream(long length, boolean printable)
    {
	rand = new Random();
	bytesLeft = length;
	isPrintable = printable;
    }
    
    public
    int read()
    {
	if (bytesLeft <= 0) return -1;
	--bytesLeft;
	if (isPrintable) {
	    if (bytesLeft == 0) return '\n';
	    int ch = (int) Math.abs((long)rand.nextInt()) % 96 + 32;
	    if (ch == 127) ch = '\n';
	    return ch;
	} else 
	    return ((int)Math.abs((long)rand.nextInt()) % 256);
    }
}

