/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Sorter.java,v 1.11.12.1 2000/10/19 17:58:53 jasona Exp $
 */

package com.lutris.testutil;

/*
 * Interface for ObjSort comparsions.
 */
interface Comparer {
    int compare(Object obj1, Object obj2);
}

/*
 * Generic sorting class.
 */
class ObjSort {
    /*
     * Sort object array.
     */
    public Object[] sort(Object[] objs,
                         Comparer comparer) {
        Object[] result = (Object[])objs.clone ();
        boolean sorted;

        do {
            sorted = true;
            for (int idx = 0; idx < objs.length - 1; idx++) {
                if (comparer.compare(result[idx],
                                     result[idx + 1]) > 0) {
                    Object tmp = result[idx];
                    result[idx] = result[idx + 1];
                    result[idx + 1] = tmp;
                    sorted = false;
                }
            }
        } while (!sorted);

        return result;
    }
}

/**
 * Result sorter.
 *
 * @version	$Revision: 1.11.12.1 $
 * @author	Mark Diekhans
 * @since	Jolt1.0
 */
public class Sorter {
    /**
     * Sort a string array in assending order.
     */
    public static String[] sort(String[] strings) {
        return (String[])new ObjSort().sort(strings,
            new Comparer() {
                public int compare(Object obj1, Object obj2) {
                    return ((String) obj1).compareTo((String)obj2);
                }
            });
    }

    /**
     * Sort a Float array in assending order.
     */
    public static Float[] sort(Float[] nums) {
        return (Float[]) new ObjSort().sort(nums,
            new Comparer() {
                public int compare(Object obj1, Object obj2) {
                    float num1 = ((Float)obj1).floatValue();
                    float num2 = ((Float)obj2).floatValue();
                    if (num1 < num2) {
                        return -1;
                    } else if (num1 > num2) {
                        return 1;
                    } else {
                        return 0;
                    }
                }
            });
    }
}
