/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: FilePersistentStoreTests.java,v 1.9.8.1 2000/10/19 17:58:53 jasona Exp $
 */





package com.lutris.util.tests;
import com.lutris.util.*;
import java.io.*;

/**
 * Class that tests the FilePersistentStore object.
 * TODO: more robust tests are needed.
 *
 * @author Kyle Clark
 */
public class FilePersistentStoreTests {

    private static final String INTEGER_KEY = "Integer Key !@#$%^&*()_++_";
    private static final String KEYWORD_VALUE_KEY = "KeywordValue";
    private static FilePersistentStore file;

    static void testInteger()
        throws Exception
    {
        Integer i = new Integer(4);
        file.delete(INTEGER_KEY);
        file.store(INTEGER_KEY, i);
        if (file.exists(INTEGER_KEY))
            System.out.println("Integer object was successfully archived.");
        System.out.println(i);
        i = (Integer)file.retrieve(INTEGER_KEY);
        System.out.println(i);
    }

    static void testKeywordValueTable()
        throws Exception
    {
        KeywordValueTable kv = new KeywordValueTable();
        kv.set("Integer4", new Integer(4));
        kv.set("Integer8", new Integer(8));
        // kv.set("KeywordValue", kv);  this will fail miserably and it should
        file.delete(KEYWORD_VALUE_KEY);
        file.store(KEYWORD_VALUE_KEY, kv);
        if (file.exists(KEYWORD_VALUE_KEY))
            System.out.println("KeywordValue object was successfully archived.");
        System.out.println(kv);
        kv = (KeywordValueTable)file.retrieve(KEYWORD_VALUE_KEY);
        System.out.println(kv);
    }

    public static void runTests()
        throws Exception
    {
        file = new FilePersistentStore();
        testInteger();
        testKeywordValueTable();
        java.util.Enumeration e = file.keys();
        System.out.println("Keys:");
        while(e.hasMoreElements()) {
            String k = (String)e.nextElement();
            System.out.println("\t" + k);
            file.delete(k);
        }

    }

    public static void main (String [] args)
        throws Exception
    {
        runTests();
    }
}
