/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SourceGenerator.java,v 1.3.6.1.2.1 2000/12/04 21:32:04 jay Exp $
 */




package com.lutris.dods.builder.generator.base;

import com.lutris.dods.store.doa.*;
import com.lutris.dods.exceptions.*;
import com.lutris.dods.builder.engine.*;

import com.lutris.util.Config;

import java.io.*;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import java.text.*;

abstract public class SourceGenerator {

    /*
    String		templateDir;
    TemplateReader	templateFileHeader;
    TemplateReader	templateFileFooter;
    TemplateReader	templateMemberStuff;
    */
    protected GeneratorInfo	info;
    TemplateFilter		filterFileHeader;
    TemplateFilter		filterFileFooter;
    TemplateFilter		filterMemberStuff;

    protected TagSet		tagSet;
    static public String TEMPLATE_BEGIN_REMARK  = "TEMPLATE_BEGIN_REMARK";
    static public String TEMPLATE_END_REMARK    = "TEMPLATE_END_REMARK";
    static public String AUTHOR			= "AUTHOR";
    static public String PROJECT_NAME		= "PROJECT_NAME";
    static public String CATEGORY		= "CATEGORY";
    static public String SUFFIX			= "SUFFIX";
    static public String PACKAGE		= "PACKAGE";
    static public String CLASS_NAME		= "CLASS_NAME";
    static public String EXTENDS		= "EXTENDS";
    static public String TABLE_NAME		= "TABLE_NAME";
    static public String DB_VENDOR		= "DB_VENDOR";
    static public String FILE_NAME		= "FILE_NAME";
    static public String MEMBER_NAME		= "MEMBER_NAME";
    static public String UPPER_MEMBER_NAME	= "UPPER_MEMBER_NAME";
    static public String MEMBER_JAVA_TYPE	= "MEMBER_JAVA_TYPE";
    static public String MEMBER_JAVA_TYPE_SAFE_NAME= "MEMBER_JAVA_TYPE_SAFE_NAME";
    static public String MEMBER_JAVA_DEFAULT_VALUE= "MEMBER_JAVA_DEFAULT_VALUE";
    static public String MEMBER_JDBC_TYPE	= "MEMBER_JDBC_TYPE";
    static public String MEMBER_DB_TYPE		= "MEMBER_DB_TYPE";
    static public String MEMBER_DB_DEFAULT_VALUE= "MEMBER_DB_DEFAULT_VALUE";
    static public String MEMBER_JAVADOC		= "MEMBER_JAVADOC";
    static public String DO_CLASS_NAME        	= "DO_CLASS_NAME";
    static public String DO_INTERFACE_NAME      = "DO_INTERFACE_NAME";
    static public String QUERY_CLASS_NAME     	= "QUERY_CLASS_NAME";
    static public String BASE_NAME     		= "BASE_NAME";

    static public String START_CONCRETE       	= "START_CONCRETE";
    static public String STOP_CONCRETE        	= "STOP_CONCRETE";

    static public String START_CACHING		= "START_CACHING";
    static public String STOP_CACHING		= "STOP_CACHING";
    static public String START_FULL_CACHING	= "START_FULL_CACHING";
    static public String STOP_FULL_CACHING	= "STOP_FULL_CACHING";
    static public String TRUE_IF_PARTIAL_CACHING= "TRUE_IF_PARTIAL_CACHING";
    static public String TRUE_IF_FULL_CACHING	= "TRUE_IF_FULL_CACHING";
    static public String TRUE_IF_NO_CACHING	= "TRUE_IF_NO_CACHING";

    static public String DATABASE_SECTION       = "Database";

    static public String STRING_QUOTE_CHARACTER	= "STRING_QUOTE_CHARACTER";
    static public String STRING_MATCH		= "STRING_MATCH";
    static public String STRING_WILDCARD	= "STRING_WILDCARD";

    static public String BEGIN_COMMENT		= "BEGIN_COMMENT";
    static public String END_COMMENT		= "END_COMMENT";


    static public String VERSION_DB_COL_NAME	= "VERSION_DB_COL_NAME";
    static public String OID_DB_COL_NAME	= "OID_DB_COL_NAME";
    static public String OID_DB_TYPE		= "OID_DB_TYPE";
    static public String OID_JDBC_TYPE		= "OID_JDBC_TYPE";
    static public String INT_DB_TYPE		= "INT_DB_TYPE";

    static public String NAMED_CONSTRAINTS	= "NAMED_CONSTRAINTS";
    static public String DELETE_CASCADES	= "DELETE_CASCADES";

    // Tuples used to copy config section values to tag values.
    static final String[][] configSectionName_tagName_pairs = {
	{    "StringQuoteCharacter",	STRING_QUOTE_CHARACTER },
	{    "StringMatch",		STRING_MATCH },
	{    "StringWildcard",		STRING_WILDCARD },
	{    "CommentStart",		BEGIN_COMMENT },
	{    "CommentEnd",		END_COMMENT },
	{    "OidDbColumnName",		OID_DB_COL_NAME },
	{    "VersionDbColumnName",	VERSION_DB_COL_NAME },
	{    "OidDbType",		OID_DB_TYPE },
	{    "OidJdbcType",		OID_JDBC_TYPE },
	{    "NamedConstraint",		NAMED_CONSTRAINTS },
	{    "DeleteCascade",		DELETE_CASCADES },
    };


    private void setTagsFromConfig( Config config, String vendor ) 
    throws Exception {
        for ( int i = 0; i < configSectionName_tagName_pairs.length; i++ ) {
            String[] pair =  configSectionName_tagName_pairs[i];
	    setTagFromConfig( config, pair[0], vendor, pair[1] );
        }
    }

    private String setTagFromConfig(    Config config, 
					String sectionName, String vendor,
					String tag )
    throws Exception {
        String tagValue = null;
        try {
//System.err.println("sectionName ="+sectionName );
	    Config section = (Config) config.getSection( sectionName );
//System.err.println("vendor ="+vendor );
	    tagValue = section.getString( vendor );
//System.err.println("tagValue ="+tagValue );
	    if ( null != tag )
		tagSet.setTagValue( tag, tagValue );
        } catch ( Exception e ) { 
            Builder.print("No dods.conf value for " +
			    sectionName + "." + vendor );
	    throw e;
        }
	return tagValue;
    }
    
    protected String val( String tagName ) 
    throws Exception {
	return tagSet.getTagValue( tagName );
    }

    static final String BEGIN_IS_REF		= "BEGIN_IS_REF";
    static final String END_IS_REF		= "END_IS_REF";

    protected String		db_vendor;
    protected Config 		db 		= null;
    protected Config 		jdbcConfig	= null;
    protected String 		vendor_key	= null;

    protected DataObject	currentObj;

    protected SourceWriter	output;

    protected boolean 	    	cascadeDelete =	false;
    protected boolean 	    	namedConstraint = false;

    static final String FILE_HEADER_TEMPLATE_NAME  = "FileHeader";
    static final String FILE_FOOTER_TEMPLATE_NAME  = "FileFooter";
    static final String MEMBER_STUFF_TEMPLATE_NAME = "MemberStuff";

    /* Builder.generateSQLcommands() uses oidDbType */
    public    static String oidDbType = null;

    protected static boolean columnIsText( Attribute a ) {
	return columnIsText( a.getDBType() );
    }
    protected static boolean columnIsText( String columnType ) {
	return "LONGVARCHAR".equals( columnType );
    }

    protected TemplateFilter createFilter( String templateKey )
    throws Exception {
	TemplateReader reader = info.getTemplateReader( templateKey );
	try {
	    /*
	    reader.mark( 100000 );
	    reader.reset();
	    */
	} catch ( Exception e ) {
          Builder.print("CANNOT RESET READER: " + e.getMessage() );
	}
	return new TemplateFilter( reader, output );
    }

    static private Hashtable fixedDefaults = null;

    static public void reset() {
	fixedDefaults = new Hashtable();
    }

   /**
     * Adds tag values to the <CODE>TagSet</CODE>.
     * A subclass of <CODE>SourceGenerator</CODE> can override this method
     * to set additional tag values.
     * An overrided <CODE>objectPrep()</CODE>
     * should invoke <CODE>super.objectPrep()</CODE>
     * to get the basic tag settings performed by this method.
     * A subclass of <CODE>SourceGenerator</CODE>
     * will call <CODE>objectPrep()</CODE>
     * and then call its implementation of one of the following methods:
     *    <CODE>
     *    writeFileHeader
     *    writeFileFooter
     *    writeMemberStuff
     *    </CODE>
     * These methods should invoke
     *    <CODE>TemplateFilter.process( tagSet )</CODE>
     * which will create the output source file.
     *
     * @param o
     *   DataObject for which source code will be generated by
     *    <CODE>writeFileHeader</CODE> and
     *    <CODE>writeFileFooter</CODE>
     * @exception Exception
     *   If an unrecoverable error occurs.
     *
     * @author    Jay Gunter
     * @since     LBS1.8
     * @version   $Revision: 1.3.6.1.2.1 $
     *
     */

    public void objectPrep(	GeneratorInfo	sgi,
				DODSProject	project,
				DataObject	o )
    throws Exception {
	info = sgi;
	currentObj = o;
	String flavor = info.outputFileFlavor;
	if ( null == flavor )
		flavor = info.outputFileFlavor;
	String suffix = info.outputFileSuffix;
	tagSet = new TagSet();
	tagSet.setTagValue( TEMPLATE_BEGIN_REMARK, TemplateFilter.skipStart );
	tagSet.setTagValue( TEMPLATE_END_REMARK,   TemplateFilter.skipStop );
	tagSet.setTagValue( CATEGORY,  flavor );
	tagSet.setTagValue( SUFFIX,    suffix );
	tagSet.setTagValue( BASE_NAME, o.getName() );
	tagSet.setTagValue( CLASS_NAME, getOutputClassName( o, flavor ) );
	DataObject parentClass = o.getExtensionOf();

	tagSet.setTagValue( TRUE_IF_PARTIAL_CACHING, "false" );
	tagSet.setTagValue( TRUE_IF_FULL_CACHING,    "false" );
	tagSet.setTagValue( TRUE_IF_NO_CACHING,      "true" );

	if ( o.isCaching() && o.isConcrete() ) {
	    tagSet.setTagValue( TRUE_IF_NO_CACHING,      "false" );
	    tagSet.setTagValue( TRUE_IF_PARTIAL_CACHING, "true" );
	    tagSet.setTagValue( START_CACHING,     "" );
	    tagSet.setTagValue( STOP_CACHING,      "" );
	} else {
	    tagSet.setTagValue( START_CACHING,     TemplateFilter.skipStart );
	    tagSet.setTagValue( STOP_CACHING,      TemplateFilter.skipStop );
	}

	if ( o.isFullCaching() && o.isConcrete() ) {
	    tagSet.setTagValue( TRUE_IF_NO_CACHING,      "false" );
	    tagSet.setTagValue( TRUE_IF_PARTIAL_CACHING, "false" );
	    tagSet.setTagValue( TRUE_IF_FULL_CACHING,    "true" );
	    tagSet.setTagValue( START_FULL_CACHING,     "" );
	    tagSet.setTagValue( STOP_FULL_CACHING,      "" );
	} else {
	    tagSet.setTagValue( START_FULL_CACHING,     TemplateFilter.skipStart );
	    tagSet.setTagValue( STOP_FULL_CACHING,      TemplateFilter.skipStop );
	}

	if ( o.isConcrete() ) {
	    tagSet.setTagValue( START_CONCRETE,     "" );
	    tagSet.setTagValue( STOP_CONCRETE,      "" );
	} else {
	    tagSet.setTagValue( START_CONCRETE,     TemplateFilter.skipStart );
	    tagSet.setTagValue( STOP_CONCRETE,      TemplateFilter.skipStop );
	}

	// If the Data Object does not derive from another Data Object,
	// and the OutputClassParent setting is not set for this SourceGenerator
	// then the output class does not need an "extends" clause.
	tagSet.setTagValue( EXTENDS, "" );
	if ( null != info.outputClassParent ) {
	    // If a Data Object does not derive from another Data Object,
	    // but an OutputClassParent is specified in the config file,
	    // the extends clause names that parent class.
	    tagSet.setTagValue( EXTENDS,
		"extends " + info.outputClassParent );
	}
	if ( null != parentClass ) {
	    // If a Data Object Y derives from another Data Object X,
	    // the extends clause must reflect this.
	    tagSet.setTagValue( EXTENDS,
		    "extends " + parentClass.getAdjustedPackageName() + "." +
		    getOutputClassName( parentClass, flavor ) );
	}

	db_vendor = project.getDatabase();
	tagSet.setTagValue( DB_VENDOR,    db_vendor );
	tagSet.setTagValue( TABLE_NAME,   getTableName( o ) );
	tagSet.setTagValue( PACKAGE,      o.getAdjustedPackageName() );
	tagSet.setTagValue( AUTHOR,       System.getProperty( "user.name" ) );
	tagSet.setTagValue( PROJECT_NAME, project.getName() );

	// FIX the strings "DO" and "Query" should really come from
	// the dods.conf file.  These suffixes should not be hard-coded here.
	tagSet.setTagValue( DO_CLASS_NAME,	o.getName() + "DO" );
	tagSet.setTagValue( DO_INTERFACE_NAME,	o.getName() + "DOI" );
	tagSet.setTagValue( QUERY_CLASS_NAME,	o.getName() + "Query" );

	// set tag values and variables from config file values
	db = (Config) info.config.getSection( DATABASE_SECTION );
	jdbcConfig = (Config)  db.getSection( "JDBCtype" );
	vendor_key = db_vendor.replace( ' ', '_' );
	setTagsFromConfig( db, vendor_key );
	tagSet.setTagValue( INT_DB_TYPE,
		    adjustDbType( jdbcConfig, "INTEGER", vendor_key  ) );
	oidDbType = 		val( OID_DB_TYPE );
	cascadeDelete =		val( DELETE_CASCADES   ).equals( "true" );
	namedConstraint =	val( NAMED_CONSTRAINTS ).equals( "true" );

	// prepare the output file (the source generator product)
	outputFilename =	info.outputDir + File.separator +
				o.getName() + flavor + "." + suffix;
Builder.print("Generating "+outputFilename );
	output = new SourceWriter( outputFilename );
	tagSet.setTagValue( FILE_NAME, fixFileName( outputFilename ) );
	filterFileHeader  = createFilter( GeneratorInfo.WRITE_FILE_HEADER );
	filterFileFooter  = createFilter( GeneratorInfo.WRITE_FILE_FOOTER );
	filterMemberStuff = createFilter( GeneratorInfo.WRITE_MEMBER_STUFF );

	setOutputFilename( outputFilename );
    }

    private String outputFilename = null;
    private void setOutputFilename( String ofn ) {
	outputFilename = ofn;
    }

    public String getOutputFilename() {
	return outputFilename;
    }

    private String fixFileName( String s ) {
	/*
	    On Windows platforms, it is common to get filenames like
		C:/usr/...
	    except the forward slashes are backslashes.
	    When javacc sees the pattern \ followed by u
	    it tries to interpret it as a Unicode character
	    (even if it occurs in a comment)
	    and because it was not valid Unicode, you got an error.
	 */
	return s.replace( '\\', '/' );
    }

    private String getOutputClassName( DataObject o, String flavor ) {
	return o.getName() + flavor;
    }

    static Hashtable tableNames = new Hashtable();
    private String getTableName( DataObject o )
    throws Exception {
	//String name  = o.getName().toLowerCase().trim();
	String name  = o.getDBTableName();
	return name;
    }

    public void memberPrep(	Attribute a ) throws Exception {
	String name = a.getName();
	String type = a.getJavaType();
	tagSet.setTagValue( MEMBER_NAME,       name );
	String firstChar = name.substring( 0, 1 ).toUpperCase();
	name = firstChar + name.substring( 1 );
	tagSet.setTagValue( UPPER_MEMBER_NAME,		name );
	tagSet.setTagValue( MEMBER_JAVA_TYPE,		type );

	// FIX if only SourceGenerator_DO needs MEMBER_JAVA_TYPE_SAFE_NAME
	// then move this line to SourceGenerator_DO.memberPrep().
/* 	tagSet.setTagValue( MEMBER_JAVA_TYPE_SAFE_NAME,	JavaType( type ) ); */
	if ( a.isObjectRef() )
	    tagSet.setTagValue( MEMBER_JAVA_TYPE_SAFE_NAME, "DO" );
	else
	    tagSet.setTagValue( MEMBER_JAVA_TYPE_SAFE_NAME, JavaType( type ) );

	tagSet.setTagValue( MEMBER_JAVA_DEFAULT_VALUE,  fixDefault( a ) );
	tagSet.setTagValue( MEMBER_JDBC_TYPE,		JdbcType( type ) );

	tagSet.setTagValue( BEGIN_IS_REF, beginNoCodeIf( ! a.isObjectRef() ));
	tagSet.setTagValue( END_IS_REF,   endNoCodeIf(   ! a.isObjectRef() ));

	if ( a.isObjectRef() )
	    tagSet.setTagValue( MEMBER_DB_TYPE,		oidDbType );
	else
	    tagSet.setTagValue( MEMBER_DB_TYPE, 
				    adjustDbType(   db, a.getDBType(), vendor_key  ) );
	tagSet.setTagValue( MEMBER_DB_DEFAULT_VALUE,	a.getInitialValue() );
	tagSet.setTagValue( MEMBER_JAVADOC,		a.getJavadoc() );
    }
    
    private static Hashtable dbTypes = new Hashtable();
		    
    private String adjustDbType( 
			Config config, String jdbcType, String vendor )
    throws Exception {
//System.err.println("jdbcType="+jdbcType);
if ( jdbcType.equals( "VARCHAR2" ) )
    jdbcType = "VARCHAR";
else if ( jdbcType.startsWith( "NUMERIC" ) )
    jdbcType = "INTEGER";
else if ( jdbcType.startsWith( "DECIMAL" ) )
    jdbcType = "INTEGER";
	String vendorType = jdbcType;
	String x = (String) dbTypes.get( jdbcType );
	if ( null != x ) {
	    vendorType = x;
	} else if ( null != jdbcConfig ) {
	    Config typeConfig = (Config) jdbcConfig.getSection( jdbcType );
	    if ( null != typeConfig ) {
		vendorType = typeConfig.getString( vendor );
		dbTypes.put( jdbcType, vendorType );
	    }
	}
	return vendorType;
    }
	    
		    

    private String fixDefault( Attribute a ) {
	String type  = a.getJavaType();
	String value = a.getInitialValue();
	// FIX handle arrays
	if ( null == value ) value = "";
	String fixedValue = value;
	String defaultValue = null;
	if ( type.endsWith( "DO" ) )
	    fixedValue = "null";	// no default value allowed for DO refs
	else if ( type.equals( "String" ) ) {
	    if ( fixedValue.length() == 0 )
		fixedValue = "null";
	    else
		fixedValue = "\"" + value + "\"";
	} else if ( type.equals( "java.math.BigDecimal" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "null";
	    else {
		try {
		    java.math.BigDecimal test = new java.math.BigDecimal( fixedValue );
		    fixedValue = "new " + type + "(" + fixedValue + ")";
		} catch ( Exception e ) {
		    defaultValue = "null";
		}
	    }
	} else if ( type.equals( "java.sql.Date" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "null";
	    else {
		try {
/* 		    java.util.Date d = new java.util.Date( fixedValue ); */
		    java.util.Date d = str2date( fixedValue );
		    java.sql.Date test = new java.sql.Date( d.getTime() );
		    fixedValue = "new " + type + "(" + fixedValue + ")";
		} catch ( Exception e ) {
		    defaultValue = "null";
		}
	    }
	} else if ( type.equals( "java.sql.Time" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "null";
	    else {
		try {
		    java.util.Date d = str2date( fixedValue );
		    java.sql.Time test = new java.sql.Time( d.getTime() );
		    fixedValue = "new " + type + "(" + fixedValue + ")";
		} catch ( Exception e ) {
		    defaultValue = "null";
		}
	    }
	} else if ( type.equals( "java.sql.Timestamp" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "null";
	    else {
		try {
		    java.util.Date d = str2date( fixedValue );
		    java.sql.Timestamp test = new java.sql.Timestamp( d.getTime() );
		    fixedValue = "new " + type + "(" + fixedValue + ")";
		} catch ( Exception e ) {
		    defaultValue = "null";
		}
	    }
	} else if ( type.equals( "int" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "0";
	    else {
		try {
		    int i = Integer.parseInt( value );
		} catch ( Exception e ) {
		    defaultValue = "0";
		}
	    }
	} else if ( type.equals( "boolean" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "false";
	    else {
		try {
		    if ( Boolean.valueOf( value ).booleanValue() )
			fixedValue = "true";
		    else
			fixedValue = "false";
		} catch ( Exception e ) {
		    defaultValue = "false";
		}
	    }
	} else if ( type.equals( "float" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "0";
	    else {
		try {
		    Float.valueOf( value );
		} catch ( Exception e ) {
		    defaultValue = "0";
		}
	    }
	} else if ( type.equals( "byte" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "0";
	    else {
		try {
		    Byte.parseByte( value );
		} catch ( Exception e ) {
		    defaultValue = "0";
		}
	    }
	} else if ( type.equals( "short" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "0";
	    else {
		try {
		    Short.parseShort( value );
		} catch ( Exception e ) {
		    defaultValue = "0";
		}
	    }
	} else if ( type.equals( "long" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "0";
	    else {
		try {
		    Long.parseLong( value );
		} catch ( Exception e ) {
		    defaultValue = "0";
		}
	    }
	} else if ( type.equals( "double" ) ) {
	    if ( value.length() == 0 )
		fixedValue = "0";
	    else {
		try {
		    Double.valueOf( value );
		} catch ( Exception e ) {
		    defaultValue = "0";
		}
	    }
	} else if ( type.equals( "byte[]" ) ) {
	    try {
		// FIX
		defaultValue = "{0}";
	    } catch ( Exception e ) {
	    }
	} else {
		defaultValue = "null";
	}
	if ( null != defaultValue ) {
	    if ( null == fixedDefaults.get( a.getName() ) ) {
		fixedDefaults.put( a.getName(), "" );
		Builder.print("WARNING: Attribute '" + a.getName() +
				"' has a bad default value: " + value +
				", changed to " + defaultValue );
	    }
	    fixedValue = defaultValue;
	}
	return fixedValue;
    }

    private java.util.Date str2date( String s ) {
	String[] formats = {
	    "yyyy-MM-dd H:mm:ss a",		"yyyy MM dd H:mm:ss a",
	    "yyyy-MM-dd H:mm:ss",		"yyyy MM dd H:mm:ss",
	    "yyyy-MM-dd H:mm a",		"yyyy MM dd H:mm a",
	    "yyyy-MM-dd H:mm",		"yyyy MM dd H:mm",
	    "yyyy-MM-dd",			"yyyy MM dd",
	    "MMM dd yyyy h:mm:ss a",	"MMM dd, yyyy h:mm:ss a",
	    "MMM dd yyyy h:mm:ss",		"MMM dd, yyyy h:mm:ss",
	    "MMM dd yyyy h:mm a",		"MMM dd, yyyy h:mm a",
	    "MMM dd yyyy h:mm",		"MMM dd, yyyy h:mm" };
	java.util.Date ret = null;
	for ( int i = 0; i < formats.length; i++ ) {
	    SimpleDateFormat df = new SimpleDateFormat( formats[ i ] );
	    try {
		ret = df.parse( s );
		break;
	    } catch ( ParseException e ) {
	    }
	}
	return ret;
    }

    private String convertArrayType( String type ) {
	String at = type;
	int i = type.indexOf( "[]" );
	if ( -1 != i )
		at = type.substring( 0, i ) + "s";
/* 		at = type.substring( 0, i ) + "Array"; */
	return at;
    }

    private String JavaType( String type ) {
/* 	return type.replace( '.', '_' ); */
	String jt = type.replace( '.', '_' );
	return convertArrayType( jt );
    }

    private String JdbcType( String javaType ) {
	String type = javaType;
	int dot = type.lastIndexOf( '.' );
	if ( -1 != dot )
		type = type.substring( dot + 1 );
	String firstChar = type.substring( 0, 1 ).toUpperCase();
/* 	return firstChar + type.substring( 1 ); */
	return convertArrayType( firstChar + type.substring( 1 ) );
    }

    public void adjustOutputFile() throws Exception {
	if ( null == output ) {
	    // The SourceGenerator subclass decided not to generate
	    // an output file.  For example, there is no
	    // SQL output file needed for an abstract class.
	    return;
	}
	output.close();

	String[] argv;
        int status;

	/*
	argv = new String[ 3 ];

        argv[0] = "chown";
        argv[1] = "???";
        argv[2] = output.getPath();

        Process chown = Runtime.getRuntime().exec( argv );
        status = chown.waitFor();
        if (status != 0 ) {
                     throw new Exception("Change ownership failed");
        }

        argv = new String[ 3 ];

        argv[0] = "chgrp";
        argv[1] = "???";
        argv[2] = output.getPath();

        Process chgrp  = Runtime.getRuntime().exec( argv );

        status = chgrp.waitFor();
        if (status != 0) {
                    throw new Exception("Change group failed");
        }
	*/

    }

    public void defaultWriteFileHeader(  DataObject o ) throws Exception {
/* System.err.println("*** defaultWriteFileHeader"); */
	filterFileHeader.reset();
	filterFileHeader.process( tagSet );
	filterFileHeader.close();
    }
    public void defaultWriteFileFooter(  DataObject o ) throws Exception {
/* System.err.println("*** defaultWriteFileFooter"); */
	filterFileFooter.reset();
	filterFileFooter.process( tagSet );
	filterFileFooter.close();
	filterMemberStuff.close();
    }
    public void defaultWriteMemberStuff( Attribute  a ) throws Exception {
/* System.err.println("*** defaultWriteMemberStuff"); */
	filterMemberStuff.reset();
	filterMemberStuff.process( tagSet );
/* 	filterMemberStuff.close(); */
    }

    abstract public void writeFileHeader(  DataObject o ) throws Exception;
    abstract public void writeFileFooter(  DataObject o ) throws Exception;
    abstract public void writeMemberStuff( Attribute  a ) throws Exception;

    /*
    public void openTemplates( String templateDirFromConfig ) {
	templateDir = templateDirFromConfig;
	File templateFile;
	templateFile = File( templateDir + "/" + FILE_HEADER_TEMPLATE_NAME );
	templateFileHeader = new TemplateReader( templateFile.getPath() );
	templateFile = File( templateDir + "/" + FILE_FOOTER_TEMPLATE_NAME );
	templateFileFooter = new TemplateReader( templateFile.getPath() );
	templateFile = File( templateDir + "/" + MEMBER_STUFF_TEMPLATE_NAME );
	templateMemberStuff = new TemplateReader( templateFile.getPath() );
    }
    */

    /** Array impl #2 (see $qdb/todo)
     *  If DataObject XXX has an array Attribute AAA,
     *  AAA is converted to a new DataObject that refers to XXX.
     *  That new DataObject is processed normally by the Builder,
     *  generating all necessary .java and schema files.
     */
    public static DataObject handleArray( DataObject o, Attribute a ) {
	String tableName = o.getName() + "_" + a.getName();
	DataObject ao   = new DataObject( tableName );
	ao.setAbstract( false );
	ao.setCaching( false );
	try {
	    ao.setDBTableName( tableName );
	} catch ( DataObjectNameException e ) {
	    Builder.print(
		"ERROR!! DataObjectNameException for tableName="+tableName);
	}
	try {
	    ao.setDBTableType( DataObject.CONCRETE );
	} catch ( DataObjectTypeException e ) {
	    Builder.print(
		"ERROR!! DataObjectTypeException for tableName="+tableName);
	}
	ao.setFinal( true );
	ao.setFullCaching( false );
	ao.setIndex( false );
	ao.setLazyLoading( false );
	try {
	    ao.setPackageName( o.getAdjustedPackageName() );
	} catch ( DataObjectNameException e ) {
	    Builder.print(
		"ERROR!! DataObjectNameException for tableName="+tableName);
	}

	/*
	Attribute  ref = new Attribute(  o.getName() + "_ref" );  // ref to o
	ref.setArray( false );
	ref.setCanBeNull( true );
	ref.setConstant( false );
	if ( -1 == o.getName().indexOf( "DO" ) )
	    ref.setDBType( "BigDecimal" );
	else
	    ref.setDBType( a.getJavaType() );
	ref.setIndex( false );
	ref.setJavaType( o.getName() + "DO" );
	ref.setJavadoc( "/" + "* ref to " + o.getName() + " *" + "/" );
	ref.setObjectRef( true );
	ref.setRefConstraint( false );
	ref.setUsedForQuery( false );
	ao.addAttribute( ref );
	*/

	Attribute index = new Attribute( "index" );
	index.setArray( false );
	index.setCanBeNull( false );
	index.setConstant( false );
	index.setDBType( "INTEGER" );
	index.setIndex( false );
	index.setJavaType( "int" );
	index.setJavadoc( "/* index for array table " + tableName + " */" );
	index.setObjectRef( false );
	index.setRefConstraint( false );
	index.setUsedForQuery( false );
	ao.addAttribute( index );

	Attribute  value = new Attribute( "value" );
	value.setArray( false );
	value.setCanBeNull( true );
	value.setConstant( false );
	if ( -1 == o.getName().indexOf( "DO" ) )
	    value.setDBType( "BigDecimal" );
	else
	    value.setDBType( a.getJavaType() );
	value.setIndex( false );
	value.setJavaType( a.getJavaType() );
	value.setJavadoc( a.getJavadoc() );
	value.setObjectRef( a.isObjectRef() );
	value.setRefConstraint( false );
	value.setUsedForQuery( false );
	ao.addAttribute( value );

	return ao;
    }

    /**
     * If condition is true, return tag to temporarily suspend code generation.
     * This allows conditionally processed sections within a .template file.
     */
    public String beginNoCodeIf( boolean condition ) {
	return condition ? TemplateFilter.skipStart : "";
    }

    /**
     * If condition is true, return tag to resume code generation.
     */
    public String endNoCodeIf( boolean condition ) {
	return condition ? TemplateFilter.skipStop : "";
    }

}
