/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SourceGenerator_Query.java,v 1.3.6.1 2000/10/05 00:56:04 jay Exp $
 */




package com.lutris.dods.builder.generator.query;

import com.lutris.dods.store.doa.DataObject;
import com.lutris.dods.store.doa.Attribute;
import com.lutris.dods.store.doa.DODSProject;

import com.lutris.dods.builder.engine.*;
import com.lutris.dods.builder.generator.base.*;

import java.io.File;
import java.io.StringWriter;
import java.util.Vector;

public class SourceGenerator_Query extends SourceGenerator {

//    final String NULL_FLAG		= "NULL_FLAG";
    final String CAN_QUERY		= "CAN_QUERY";
    final String QUERY_COLUMN_NAMES	= "QUERY_COLUMN_NAMES";
    final String QUERY_OID_AND_VERSION	= "QUERY_OID_AND_VERSION";

    public SourceGenerator_Query() throws Exception {
    }

   /**
     * Adds tag values to the <CODE>TagSet</CODE>.
     * This method invokes
     *    <CODE>TemplateFilter.process( tagSet )</CODE>
     * which will create the output source file.
     *
     * @param o
     *   DataObject for which source code will be generated by
     *    <CODE>writeFileHeader</CODE> and
     *    <CODE>writeFileFooter</CODE>
     * @exception Exception
     *   If an unrecoverable error occurs.
     *
     */
    public void objectPrep(	GeneratorInfo sgi,
				DODSProject project,
				DataObject o )
    throws Exception {
	super.objectPrep( sgi, project, o );
    }

    public void memberPrep(	Attribute a ) throws Exception {
	if ( ! a.isUsedForQuery() ) return;
	super.memberPrep( a );
	/*
	tagSet.setTagValue( NULL_FLAG,
		a.canBeNull() ? "NULL_OK" : "NOT_NULL" );
		*/
	tagSet.setTagValue( CAN_QUERY,
		a.isUsedForQuery() ? "public" : "private" );
    }

    private String sql_query_column_names = "";
    private String table_prefix = "";

    public void writeFileHeader(  DataObject o ) throws Exception {
	defaultWriteFileHeader( o );

	table_prefix = tagSet.getTagValue( TABLE_NAME ) + ".";

        Vector ancestors = o.getAncestors();
	if ( null != ancestors ) {
	    for ( int i = 0; i < ancestors.size(); i++ ) {
		DataObject p = (DataObject) ancestors.elementAt( i );
		if ( null != p ) {
		    Vector members = p.getAttributes();
		    for ( int j = 0; j < members.size(); j++ ) {
			Attribute attr = (Attribute) members.elementAt( j );
			memberPrep( attr );
			writeMemberStuff(   attr );
		    }
		}
	    }
	}
    }

    private String query_comma = "";

    public void writeMemberStuff( Attribute  a ) throws Exception {
	// Spell out the names of all columns, so that the generated code
	//  will not break if a new table definition changes the column order.
	sql_query_column_names += query_comma + table_prefix + a.getName();
	query_comma = ", ";

	if ( ! a.isUsedForQuery() ) return;
	defaultWriteMemberStuff( a );
    }

    public void writeFileFooter(  DataObject o ) throws Exception {
	tagSet.setTagValue( QUERY_COLUMN_NAMES,	sql_query_column_names );
	// When spelling out the names of the columns,
	// we do not automatically include "oid" and "version".
	// This is because some DOs are used as VIEWs
	// whose definitions are created by hand.
	tagSet.setTagValue( QUERY_OID_AND_VERSION,
	    ", " + table_prefix + val( OID_DB_COL_NAME ) +
	    ", " + table_prefix + val( VERSION_DB_COL_NAME ) );

	defaultWriteFileFooter( o );
    }
}
