/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 */
package com.lutris.dods.gui;

import com.lutris.dods.util.Util;

import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;

import java.beans.Beans;
import java.awt.*;
import java.util.ResourceBundle;

public class AboutBox extends JDialog {
  static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
  JPanel panelMain;
  JPanel panelButton;
  JPanel panelLabel;
  JLabel labelPrimary;
  JLabel labelSecondary;
  JLabel labelVersion;
  JButton buttonOK;
  GridBagLayout layoutMain;
  GridBagLayout layoutButton;
  GridBagLayout layoutLabel;

  private boolean standalone = false;
  private LocalActionListener actionListener = null;

  public AboutBox(Frame owner) {
    super(owner, null, true);
    construct();
  }

  private void construct() {
    try  {
      jbInit();
      pmInit();
      pack();
      setResizable(true);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
  }

  ///
  protected void clearAll() {
    buttonOK.removeAll();
    buttonOK.removeActionListener(actionListener);
    labelPrimary.removeAll();
    labelSecondary.removeAll();
    labelVersion.removeAll();
    panelButton.removeAll();
    panelLabel.removeAll();
    panelMain.removeAll();
    removeAll();
    getContentPane().removeAll();
    actionListener = null;
    buttonOK       = null;
    labelVersion   = null;
    layoutButton   = null;
    layoutLabel    = null;
    layoutMain     = null;
    panelButton    = null;
    panelLabel     = null;
    panelMain      = null;
    setTitle(null);
  }

  ///
  public void show() {
    Point centeringPoint;
    pack();
    centeringPoint = Util.getCenteringPoint(getSize());
    setLocation(centeringPoint);
    buttonOK.requestFocus();
    super.show();
  }

  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == buttonOK) {
      closeDialog();
    }
  }

  public static void main(String[] args) {
    Util.setLookForWindowsOnly();
    AboutBox dlg;
    dlg = new AboutBox(new Frame());
    dlg.standalone = true;
    dlg.show();
  }

  public String getPrimary() {
    return labelPrimary.getText();
  }

  public String getSecondary() {
    return labelSecondary.getText();
  }


  public void setPrimary(String p) {
    labelPrimary.setText(p);
  }

  public void setSecondary(String s) {
    labelSecondary.setText(s);
    setTitle(res.getString("About") + s);
  }


  public String getVersion() {
    return labelVersion.getText();
  }

  public void setVersion(String v) {
    labelVersion.setText(v);
  }
  ///
  ///

  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      closeDialog();
    }
    super.processWindowEvent(e);
  }

  protected void disposeDialog() {
    setVisible(false);
    removeAll();
    dispose();
  }

  ///
  ///

  private void closeDialog() {
    disposeDialog();
    if (standalone) {
      System.exit(0);
    }
  }

  private void pmInit() {
    actionListener = new LocalActionListener();
    buttonOK.addActionListener(actionListener);
  }

  private void jbInit() throws Exception  {
    panelMain   = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    panelLabel  = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    panelButton = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    labelPrimary    = (JLabel) Beans.instantiate(getClass().getClassLoader(), JLabel.class.getName());
    labelSecondary  = (JLabel) Beans.instantiate(getClass().getClassLoader(), JLabel.class.getName());
    labelVersion   = (JLabel) Beans.instantiate(getClass().getClassLoader(), JLabel.class.getName());
    buttonOK       = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    layoutMain   = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
    layoutButton = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
    layoutLabel  = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());

    buttonOK.setText(res.getString("OK"));
    buttonOK.setMaximumSize(new Dimension(100, 30));
    buttonOK.setMinimumSize(new Dimension(60, 15));
    buttonOK.setPreferredSize(new Dimension(75, 20));
    buttonOK.setMargin(new Insets(2, 12, 2, 12));
    buttonOK.setMnemonic('0');

    panelLabel.setLayout(layoutLabel);
    labelPrimary.setFont(new java.awt.Font("Dialog", 3, 18));
    labelPrimary.setToolTipText("");
    labelPrimary.setText(res.getString("Primary"));
    labelSecondary.setFont(new java.awt.Font("Dialog", 0, 18));
    labelSecondary.setText(res.getString("Secondary"));
    labelVersion.setMaximumSize(new Dimension(200, 17));
    labelVersion.setText(res.getString("Version"));
    panelLabel.add(labelPrimary, new GridBagConstraintsX(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(10, 20, 3, 20), 40, 0));
    panelLabel.add(labelSecondary, new GridBagConstraintsX(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, 20, 5, 20), 40, 0));
    panelLabel.add(labelVersion, new GridBagConstraintsX(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 20, 10, 20), 40, 0));

    panelButton.setLayout(layoutButton);
    panelButton.add(buttonOK, new GridBagConstraintsX(0, 0, 1, 1, 0.5, 0.5
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 10, 0), 0, 11));

    panelMain.setLayout(layoutMain);
    panelMain.setMaximumSize(new Dimension(500, 400));
    panelMain.setMinimumSize(new Dimension(300, 150));
    panelMain.setPreferredSize(new Dimension(350, 150));
    panelMain.add(panelLabel, new GridBagConstraintsX(0, 1, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    panelMain.add(panelButton, new GridBagConstraintsX(0, 2, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));

    this.getContentPane().add(panelMain);
  }


  class LocalActionListener implements ActionListener {
    public void actionPerformed(ActionEvent event) {
      closeDialog();
    }
  }
}


