/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DODSFileChooser.java,v 1.12.14.2 2000/12/04 19:17:01 chip Exp $
 *-----------------------------------------------------------------------------
 * Constructor to set working directory added by David Wood (dwood@plugged.net.au)
 */



package com.lutris.dods.gui;

import java.awt.Component;
import com.lutris.dods.store.doa.*;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.filechooser.*;
import java.io.File;
import java.util.ResourceBundle;
/**
 * This class is used to save and open files for DODS
 */
public class DODSFileChooser extends JFileChooser {
  static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");

    private InternalFilter filterDOML = null;
  private InternalFilter filterDOA  = null;

  public DODSFileChooser() {
    setFileSelectionMode(FILES_ONLY);
    filterDOML = new InternalFilter(res.getString("doml"));
    filterDOA  = new InternalFilter(res.getString("doa"));
    resetChoosableFileFilters();
    addChoosableFileFilter(filterDOML);
    setFileFilter(filterDOML);
    removeChoosableFileFilter(getAcceptAllFileFilter());
  }

  public int showOpenDialog(Component parent) {
    FileFilter[] filters = getChoosableFileFilters();
    for (int i = 0 ; i < filters.length ; i++ ) {
      removeChoosableFileFilter(filters[i]);
    }
    addChoosableFileFilter(filterDOML);
    addChoosableFileFilter(filterDOA);
    setFileFilter(filterDOML);
    return super.showOpenDialog(parent);
  }

  public int showSaveDialog(Component parent) {
    resetChoosableFileFilters();
    addChoosableFileFilter(filterDOML);
    removeChoosableFileFilter(getAcceptAllFileFilter());
    return super.showSaveDialog(parent);
  }

  class InternalFilter extends FileFilter {
        private String extension = null;

    InternalFilter(String ex) {
      extension  = ex;
    }

    public boolean accept(File f) {
      boolean acceptFile = false;
      if (f != null) {
        if (f.isDirectory()) {
          return true;
        }
        String fileExt = com.lutris.dods.util.Util.getExtension(f);
        if (fileExt != null && fileExt.equalsIgnoreCase(extension)) {
          acceptFile = true;
        }
      }
      return acceptFile;
    }

    public String getDescription() {
      return res.getString("DODS_Files_") + " (*." + extension + ")";
    }
  }

  class InternalFileView extends FileView {
    InternalFileView() {}
    public Icon getIcon(File f) {

    if (f.isDirectory()) {
      return Icons.getIcon("folder.gif");
    }
      return Icons.getIcon("object_ref.gif");
    }

    public String getName(File f) {
      String fileName = null;
      if(f != null) {
        fileName = f.getName();
        if (fileName.equals("")) {
          fileName = f.getPath();
        }
      }
      return fileName;
    }


    public String getDescription(File f) {
      return f.getName();
    }

    public String getTypeDescription(File f) {
      if (f.isDirectory()) {
        return "";
      } else {
        return "";
      }
    }

    public Boolean isTraversable(File f) {
      if (f.isDirectory()) {
        return Boolean.TRUE;
      } else {
        return Boolean.FALSE;
      }
    }
  }
}



