/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DirectoryChooser.java,v 1.7.14.1 2000/12/01 22:27:39 chip Exp $
 *-----------------------------------------------------------------------------
 */



package com.lutris.dods.gui;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public final class DirectoryChooser extends Dialog
implements ActionListener, TextListener {
    static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    public  File	dir       = null;

    private TextField	tfDir     = new TextField( 50 );
    private JButton	setBtn    = new JButton( res.getString("OK") );
    private JButton	cancelBtn = new JButton( res.getString("Cancel1") );

    public DirectoryChooser( Frame f, String defaultDir ) {
	super( f, res.getString("Choose_Target"), true );
	setLayout( new GridLayout( 3, 1 ) );
	setSize( 200, 500 );
	setLocation( 50, 100 );
	if ( null != defaultDir )
	    tfDir.setText( defaultDir );
	setBtn.addActionListener( this );
	cancelBtn.addActionListener( this );
	Panel p1 = new Panel();
	p1.add( new Label( res.getString("Target_Directory_"), Label.LEFT ) );
	p1.add( tfDir );
	Panel p3 = new Panel();
	p3.add( setBtn );
	p3.add( cancelBtn );
	add( p1 );
	add( p3 );
	pack();
	show();
    }

    public void actionPerformed( ActionEvent event ) {
	dir = null;
	String what = event.getActionCommand();
	if ( "Cancel".equals( what ) ) {
	    dispose();
	} else {
	    String path = tfDir.getText();
	    if ( null == path || 0 == path.length() ) {
/* 		path = ""; */
		int result = JOptionPane.showConfirmDialog( this,
				res.getString("Do_you_wish_to_Build"),
				res.getString("No_directory"),
				JOptionPane.YES_NO_OPTION,
				JOptionPane.QUESTION_MESSAGE );
		if ( JOptionPane.NO_OPTION == result )
		    dispose();
		return;
	    } else {
		dir = new File( path );
		if ( dir.isFile() ) {
		    int result = JOptionPane.showConfirmDialog( this,
				res.getString("That_is_an_existing") +
				res.getString("Do_you_wish_to_enter"),
				res.getString("Cannot_create"),
				JOptionPane.YES_NO_OPTION,
				JOptionPane.ERROR_MESSAGE );
		    if ( JOptionPane.NO_OPTION == result ) {
			dir = null;
			dispose();
		    }
		    return;
		} else if ( ! dir.isDirectory() ) {
		    int result = JOptionPane.showConfirmDialog( this,
				res.getString("Create_that_directory"),
				res.getString("Directory_does_not"),
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.QUESTION_MESSAGE );
		    if ( JOptionPane.CANCEL_OPTION == result ) {
			dir = null;
			dispose();
			return;
		    }
		    if ( JOptionPane.NO_OPTION == result ) {
			dir = null;
			return;
		    }
		    dir.mkdir();
		    dispose();
		} else {
		    int result = JOptionPane.showConfirmDialog( this,
				res.getString("Destroy_current"),
				res.getString("Directory_contents1"),
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.QUESTION_MESSAGE );
		    if ( JOptionPane.CANCEL_OPTION == result ) {
			dir = null;
			dispose();
			return;
		    }
		    if ( JOptionPane.NO_OPTION == result ) {
			dir = null;
			return;
		    }
		    dispose();
		}
	    }
	}
    }

    public void textValueChanged( TextEvent e ) {
	int cols = tfDir.getColumns();
	if ( tfDir.getText().length() == cols ) {
	    tfDir.setColumns( cols + cols );
	    pack();
	}
    }
}
