/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: GraphicalPanel.java,v 1.13.14.1 2000/12/01 22:27:39 chip Exp $
 *-----------------------------------------------------------------------------
 */


   package com.lutris.dods.gui;

   import com.lutris.dods.store.doa.*;
   import com.lutris.dods.gui.tree.*;
   import javax.swing.JPanel;
   import javax.swing.JViewport;
   import javax.swing.JTree;
   import java.awt.Dimension;
   import java.awt.Graphics;
   import java.awt.Color;
   import java.awt.Point;
   import java.awt.Image;
   import java.awt.Graphics;
   import java.awt.event.MouseEvent;
   import java.awt.event.MouseListener;
   import java.awt.event.MouseMotionListener;
   import java.util.Vector;
import java.util.ResourceBundle;
/**
 * This class will develop into the graphical view of the hierarchy of
 * DataObjects in a given project.
 *
 * @author  Philip Martin
 * @version $Revision: 1.13.14.1 $
 */
   public class GraphicalPanel extends JViewport
   implements MouseListener, MouseMotionListener {
    /*
     * This is the DODSProject which contains all of the information we need
     */
      static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    private DODSProject project;
    /*
     * The set of DataObject Hierarchies for this Project.
     */
      private Vector branches;
    /*
     * The preferred width is large enough to hold all dataObjects
     */
      private int rows = 0;
      private int width = 500;
    /*
     * The preferred height is large enough to hold all dataObjects
     */
      private int height = 200;
      private Vector allBranches;
      private JPanel jpanel;
      private Graphics graphics;
      private Image image;
      private DOTreeModel doTreeModel;
      private JTree tree;
      private DataObject selectedDO;
      private Vector allDOS;
    /**
     * Just calls the super constructor, and initiallizes all variables.
     */
      public GraphicalPanel(DODSProject dodsProject, JTree tre) {
         super();
         project = dodsProject;
         tree = tre;
         doTreeModel = (DOTreeModel)tree.getModel();
         allBranches = new Vector();
         allDOS = new Vector();
         jpanel = new JPanel();
         jpanel.setPreferredSize(new Dimension(width, height));
         setView(jpanel);
         this.addMouseListener(this);
         this.addMouseMotionListener(this);
         updateBranches();
      }
    /**
     * Used to set up the initial layout size Currently, the preferred size is (100,100).
     */
      public Dimension getPreferredSize() {
         setPreferredSize(new Dimension(width, height));
         image = createImage(width, height);
	 if ( null != image )
	     graphics = image.getGraphics();
         jpanel.setPreferredSize(new Dimension(width, height));
         return new Dimension(width, height);
      }
    /**
     * This method gathers all of the DataObjects into Branches which can
     * be displayed.
     */
      public void updateBranches() {
         branches = new Vector();
         allDOS = new Vector();
         project.getAllDataObjects(allDOS);
         allDOS = DataObject.alphabetizeVector(allDOS);
         int newX = 20;
         if(allDOS.size() > 0) {
            rows = 1;
         }
         for(int i = 0; i < allDOS.size(); i++) {
            DataObject DO = (DataObject)allDOS.elementAt(i);
            if(DO.getExtensionOf() == null) {
               GraphicalBranch branch = new GraphicalBranch(DO, 1, this);
               branches.addElement(branch);
               branch.setPosition(newX, 30);
               branch.updateBranches();
               if(branch.getBottomRow() > rows) {
                  rows = branch.getBottomRow();
               }
               newX += branch.getWidth();
               branch.updateBranchPositions();
            }
         }
         width = newX;
         height = 50*rows + 30;
         if(height < 200) {
            height = 200;
         }
         allBranches = new Vector();
         for(int i = 0; i < branches.size(); i++) {
            allBranches.addElement(branches.elementAt(i));
            ((GraphicalBranch)branches.elementAt(i)).addBranches(allBranches);
         }
      }
      public void setDODSProject(DODSProject pro) {
         project = pro;
      }
      public void update(Graphics g) {
         paint(g);
      }
      public void paint(Graphics g) {
	 if ( null == g || null == graphics )
		return;
         graphics.setColor(Color.white);
         graphics.fillRect(0, 0, width, height);
         graphics.setColor(Color.black);
         drawObjectRefs(graphics, getSelectedBranch());
         for(int i = 0; i < branches.size(); i++) {
            GraphicalBranch gb = (GraphicalBranch)branches.elementAt(i);
            gb.draw(graphics);
         }
         int x = this.getViewPosition().x;
         int y = this.getViewPosition().y;
         g.drawImage(image, -x, -y, width, height, null);
      }
      public void drawObjectRefs(Graphics g, GraphicalBranch b) {
         if(b == null) {
            return;
         }
         DataObject DO = b.getDataObject();
         for(int i = 0 ; i < DO.getAttributes().size() ; i++) {
            Attribute att = (Attribute)DO.getAttributes().elementAt(i);
            if(att.isObjectRef()) {
               GraphicalBranch refBranch = getBranchWithDO(att.getObjectRefDO());
               g.setColor(Color.red.darker());
               int x1 = b.getRectangle().x + b.getRectangle().width/2;
               int y1 = b.getRectangle().y + b.getRectangle().height/2;
               int x2 = refBranch.getRectangle().x + refBranch.getRectangle().width/2;
               int y2 = refBranch.getRectangle().y + refBranch.getRectangle().height/2;
               int choice = 0;
               double slope = 0;
               double slopeInverse = 0;
               if(x1 == x2) {
                  if(y1 <= y2) {
                     choice = 3;
                  }
                  else {
                     choice = 1;
                  }
               }
               else {
                  slope = (-y1+y2 + 0.0)/(x1-x2);
                  if(slope != 0) {
                     slopeInverse = -1/slope;
                  }
                  else {
                     slopeInverse = Math.PI;
                  }
                  if(Math.abs(slope) < 1) {
                     if(x1 <= x2) {
                        choice = 4;
                     }
                     else {
                        choice = 2;
                     }
                  }
                  else {
                     if(y1 <= y2) {
                        choice = 3;
                     }
                     else {
                        choice = 1;
                     }
                  }
               }
               switch(choice) {
                  case 0:
                     break;
                  case 1:
                     y1 += -b.getRectangle().height/2-3;
                     y2 += refBranch.getRectangle().height/2-1;
                     break;
                  case 2:
                     x1 += -b.getRectangle().width/2-3;
                     x2 += refBranch.getRectangle().width/2;
                     break;
                  case 3:
                     y1 += b.getRectangle().height/2-2;
                     y2 += -refBranch.getRectangle().height/2-3;
                     break;
                  case 4:
                     x1 += b.getRectangle().width/2;
                     x2 += -refBranch.getRectangle().width/2-3;
                     break;
                  default:
                     break;
               }
               if(x1 != x2) {
                  slope = (- y1 + y2+ 0.0)/(x1 - x2);
               }
               else {
                  slope = Math.PI;
               }
               if(slope == 0) {
                  slopeInverse = Math.PI;
               }
               else {
                  slopeInverse = -1/slope;
               }
               int x3 = 0;
               int y3 = 0;
               int ax1 = 0;
               int ay1 = 0;
               int ax2 = 0;
               int ay2 = 0;
               int adown = 15;
               int aout = 5;
               double fracs = 1/Math.sqrt(1 + slope*slope);
               double fraci = 1/Math.sqrt(1 + slopeInverse*slopeInverse);
               boolean slopepos = true;
               if(slope < 0) {
                  slopepos = false;
                  slope = -slope;
               }
               else {
                  slopepos = true;
                  slopeInverse = -slopeInverse;
               }
               switch(choice) {
                  case 0:
                     break;
                  case 1:
                     if(slope == Math.PI) {
                        x3 = x2;
                        y3 = y2 + adown;
                        ax1 = x3 - aout;
                        ay1 = y3;
                        ax2 = x3 + aout;
                        ay2 = y3;
                     }
                     else {
                        if(slopepos) {
                           x3 = (int)(x2 - adown*fracs);
                           y3 = (int)(y2 + adown*slope*fracs);
                           ax1 = (int)(x3 - aout*fraci);
                           ay1 = (int)(y3 - aout*slopeInverse*fraci);
                           ax2 = (int)(x3 + aout*fraci);
                           ay2 = (int)(y3 + aout*slopeInverse*fraci);
                        }
                        else {
                           x3 = (int)(x2 + adown*fracs);
                           y3 = (int)(y2 + adown*slope*fracs);
                           ax1 = (int)(x3 - aout*fraci);
                           ay1 = (int)(y3 + aout*slopeInverse*fraci);
                           ax2 = (int)(x3 + aout*fraci);
                           ay2 = (int)(y3 - aout*slopeInverse*fraci);
                        }
                     }
                     break;
                  case 2:
                     if(slopeInverse == Math.PI) {
                        x3 = x2 + adown;
                        y3 = y2;
                        ax1 = x3;
                        ay1 = y3 + aout;
                        ax2 = x3;
                        ay2 = y3 - aout;
                     }
                     else {
                        if(slopepos) {
                           x3 = (int)(x2 + adown*fracs);
                           y3 = (int)(y2 - adown*slope*fracs);
                           ax1 = (int)(x3 + aout*fraci);
                           ay1 = (int)(y3 + aout*slopeInverse*fraci);
                           ax2 = (int)(x3 - aout*fraci);
                           ay2 = (int)(y3 - aout*slopeInverse*fraci);
                        }
                        else {
                           x3 = (int)(x2 + adown*fracs);
                           y3 = (int)(y2 + adown*slope*fracs);
                           ax1 = (int)(x3 - aout*fraci);
                           ay1 = (int)(y3 + aout*slopeInverse*fraci);
                           ax2 = (int)(x3 + aout*fraci);
                           ay2 = (int)(y3 - aout*slopeInverse*fraci);
                        }
                     }
                     break;
                  case 3:
                     if(slope == Math.PI) {
                        x3 = x2;
                        y3 = y2 - adown;
                        ax1 = x3 + aout;
                        ay1 = y3;
                        ax2 = x3 - aout;
                        ay2 = y3;
                     }
                     else {
                        if(slopepos) {
                           x3 = (int)(x2 + adown*fracs);
                           y3 = (int)(y2 - adown*slope*fracs);
                           ax1 = (int)(x3 + aout*fraci);
                           ay1 = (int)(y3 + aout*slopeInverse*fraci);
                           ax2 = (int)(x3 - aout*fraci);
                           ay2 = (int)(y3 - aout*slopeInverse*fraci);
                        }
                        else {
                           x3 = (int)(x2 - adown*fracs);
                           y3 = (int)(y2 - adown*slope*fracs);
                           ax1 = (int)(x3 + aout*fraci);
                           ay1 = (int)(y3 - aout*slopeInverse*fraci);
                           ax2 = (int)(x3 - aout*fraci);
                           ay2 = (int)(y3 + aout*slopeInverse*fraci);
                        }
                     }
                     break;
                  case 4:
                     if(slopeInverse == Math.PI) {
                        x3 = x2 - adown;
                        y3 = y2;
                        ax1 = x3;
                        ay1 = y3 - aout;
                        ax2 = x3;
                        ay2 = y3 + aout;
                     }
                     else {
                        if(slopepos) {
                           x3 = (int)(x2 - adown*fracs);
                           y3 = (int)(y2 + adown*slope*fracs);
                           ax1 = (int)(x3 - aout*fraci);
                           ay1 = (int)(y3 - aout*slopeInverse*fraci);
                           ax2 = (int)(x3 + aout*fraci);
                           ay2 = (int)(y3 + aout*slopeInverse*fraci);
                        }
                        else {
                           x3 = (int)(x2 - adown*fracs);
                           y3 = (int)(y2 - adown*slope*fracs);
                           ax1 = (int)(x3 + aout*fraci);
                           ay1 = (int)(y3 - aout*slopeInverse*fraci);
                           ax2 = (int)(x3 - aout*fraci);
                           ay2 = (int)(y3 + aout*slopeInverse*fraci);
                        }
                     }
                     break;
                  default:
                     break;
               }
               g.drawLine(x2, y2, ax1, ay1);
               g.drawLine(x2, y2, ax2, ay2);
               g.drawLine(x1, y1, x2, y2);
               g.setColor(Color.black);
            }
         }
      }
      public void mouseMoved(MouseEvent e) {
         GraphicalBranch branch = null;
         int x = e.getX()+this.getViewPosition().x;
         int y = e.getY()+this.getViewPosition().y;
         for(int i = 0; i < allBranches.size(); i++) {
            branch = (GraphicalBranch)allBranches.elementAt(i);
            if(branch.getRectangle().contains(new Point(x, y))) {
               if(!branch.getMouseOver()){
                  branch.setMouseOver(true);
                  update(this.getGraphics());
               }
            }
            else {
               if(branch.getMouseOver()){
                  branch.setMouseOver(false);
                  update(this.getGraphics());
               }
            }
         }
      }
      public void mouseDragged(MouseEvent e) {
      }
      public void mousePressed(MouseEvent e) {
         GraphicalBranch branch = null;
         int x = e.getX()+this.getViewPosition().x;
         int y = e.getY()+this.getViewPosition().y;
         selectedDO = null;
         for(int i = 0; i < allBranches.size(); i++) {
            branch = (GraphicalBranch)allBranches.elementAt(i);
            if(branch.getRectangle().contains(new Point(x, y))) {
               selectedDO = branch.getDataObject();
               if(!branch.getSelected()){
                  branch.setSelected(true);
                  update(this.getGraphics());
               }
            }
            else {
               if(branch.getSelected()){
                  branch.setSelected(false);
                  update(this.getGraphics());
               }
            }
         }
      }
      public void mouseClicked(MouseEvent e) {
      }
      public void mouseReleased(MouseEvent e) {
         doTreeModel = (DOTreeModel)tree.getModel();
         if(selectedDO == null) {
            tree.setSelectionPath(null);
         }
         else {
            tree.setSelectionPath(
               doTreeModel.findPathWithDO((DOTreeNode)doTreeModel.getRoot(),
               selectedDO,
               null)
               );
         }
      }
      public void mouseEntered(MouseEvent e) {
      }
      public void mouseExited(MouseEvent e) {
         GraphicalBranch branch = null;
         int x = e.getX()+this.getViewPosition().x;
         int y = e.getY()+this.getViewPosition().y;
         for(int i = 0; i < allBranches.size(); i++) {
            branch = (GraphicalBranch)allBranches.elementAt(i);
            if(branch.getMouseOver()) {
               branch.setMouseOver(false);
               update(this.getGraphics());
               return;
            }
         }
      }
      public void setSelectedDO(DataObject Do) {
         if(selectedDO != null && Do != null) {
            if(selectedDO.equals(Do)) {
               return;
            }
         }
         getBranchWithDO(selectedDO).setSelected(false);
         selectedDO = Do;
         getBranchWithDO(selectedDO).setSelected(true);
         update(this.getGraphics());
      }
      public DataObject getSelectedDO() {
         return selectedDO;
      }
      public GraphicalBranch getBranchWithDO(DataObject data) {
         if(data == null) {
            return new GraphicalBranch(new DataObject(res.getString("asdffdsa")), -1, this);
         }
         for(int i = 0; i < allBranches.size() ; i++) {
            GraphicalBranch branch = (GraphicalBranch)allBranches.elementAt(i);
            if(branch.getDataObject().equals(data)) {
               return branch;
            }
         }
         return new GraphicalBranch(new DataObject(res.getString("asdffsda")), -1, this);
      }
      public Vector getAllBranches(){
         return allBranches;
      }
      public Vector getAllDOS() {
         return allDOS;
      }
      public GraphicalBranch getSelectedBranch() {
         for(int i = 0; i < allBranches.size() ; i++) {
            GraphicalBranch ret = (GraphicalBranch)allBranches.elementAt(i);
            if(ret.getSelected()) {
               return ret;
            }
         }
         return null;
      }
   }








