/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: AttributeEditor.java,v 1.12.14.1 2000/12/01 22:27:39 chip Exp $
 *-----------------------------------------------------------------------------
 */



   package com.lutris.dods.gui.editors;

   import com.lutris.dods.gui.*;
   import com.lutris.dods.gui.tree.*;
   import com.lutris.dods.store.doa.*;
   import javax.swing.JDialog;
   import javax.swing.JFrame;
   import javax.swing.JOptionPane;
   import javax.swing.JTree;
   import javax.swing.JButton;
   import javax.swing.JPanel;
   import javax.swing.JTabbedPane;
   import javax.swing.JTextField;
   import javax.swing.JLabel;
   import javax.swing.JScrollPane;
   import javax.swing.JComboBox;
   import javax.swing.JCheckBox;
   import javax.swing.JTextArea;
   import javax.swing.border.TitledBorder;
   import javax.swing.event.TreeSelectionEvent;
   import javax.swing.event.ChangeListener;
   import javax.swing.event.ChangeEvent;
   import javax.swing.tree.TreePath;
   import java.awt.Dimension;
   import java.awt.GridBagLayout;
   import java.awt.Color;
   import java.awt.GridBagConstraints;
   import java.awt.Component;
   import java.awt.FlowLayout;
   import java.awt.Insets;
   import java.awt.event.ActionListener;
   import java.awt.event.ActionEvent;
   import java.util.Vector;
import java.util.ResourceBundle;
/**
 * This class allows you to edit an Attribute and all of its attributes.
 *
 * @author  Philip Martin
 * @version $Revision: 1.12.14.1 $
 */
   public class AttributeEditor extends JDialog {
    /**
     * This Constructor lays out all of the panels neatly.
     * @param oldtre The original package tree from the parent frame.
     * @param da The parent DataObjectApplication.
     */
      public AttributeEditor(JTree oldtre, DataObjectApplication da) {
         super(da);
        //Initializing the tree.
         oldTree = oldtre;
         doa = da;
         tree = new JTree();
         tree.setMinimumSize(new Dimension(0,0));
        //Initializing the Frame.
         setTitle(res.getString("AttributeEditor"));
         setModal(true);
         setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
         getContentPane().setLayout(new GridBagLayout());
         GridBagConstraints gbc;
         setBackground(Color.lightGray);
        //Initializing the panel container.
         jp = new JPanel();
         jp.setLayout(new GridBagLayout());
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,0,0,0);
         ((GridBagLayout)getContentPane().getLayout()).setConstraints(jp, gbc);
         getContentPane().add(jp);
        //Initializing Button Panel.
         buttonPanel = new JPanel();
         buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 0.3;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,0,0,5);
         ((GridBagLayout)jp.getLayout()).setConstraints(buttonPanel, gbc);
         jp.add(buttonPanel);
         okButton = new JButton();
         okButton.setText(res.getString("OK1"));
         buttonPanel.add(okButton);
         cancelButton = new JButton();
         cancelButton.setText(res.getString("Cancel"));
         buttonPanel.add(cancelButton);
         helpButton = new JButton();
         helpButton.setText(res.getString("Help1"));
         buttonPanel.add(helpButton);
         helpButton.setEnabled(false);
        //Initializing the TabbedPane.
         jtp = new JTabbedPane();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(10,10,0,10);
         ((GridBagLayout)jp.getLayout()).setConstraints(jtp, gbc);
         jp.add(jtp);
        //Initializing the generalPanel.
         generalPanel = new JPanel();
         generalPanel.setLayout(new GridBagLayout());
         nameLabel = new JLabel();
         nameLabel.setText(res.getString("Name_"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(20,20,2,0);
         ((GridBagLayout)generalPanel.getLayout()).setConstraints(nameLabel, gbc);
         generalPanel.add(nameLabel);
         name = new JTextField();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,20,0,20);
         ((GridBagLayout)generalPanel.getLayout()).setConstraints(name, gbc);
         generalPanel.add(name);
         viewLabel = new JLabel();
         viewLabel.setText(res.getString("Place_In_"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,20,2,0);
         ((GridBagLayout)generalPanel.getLayout()).setConstraints(viewLabel, gbc);
         generalPanel.add(viewLabel);
         treeView = new JScrollPane();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 3;
         gbc.weightx = 1.0;
         gbc.weighty = 50.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,20,0,20);
         ((GridBagLayout)generalPanel.getLayout()).setConstraints(treeView, gbc);
         generalPanel.add(treeView);
         className = new JTextField();
         className.setEditable(false);
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 4;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(5,20,20,20);
         ((GridBagLayout)generalPanel.getLayout()).setConstraints(className, gbc);
         generalPanel.add(className);
         className.setEnabled(false);
        //Initializing javaPanel.
         javaPanel = new JPanel();
         javaPanel.setLayout(new GridBagLayout());
         jtypeLabel = new JLabel();
         jtypeLabel.setText(res.getString("Java_Type_"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(20,20,2,0);
         ((GridBagLayout)javaPanel.getLayout()).setConstraints(jtypeLabel, gbc);
         javaPanel.add(jtypeLabel);
         javaType = new JComboBox();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,20,0,20);
         ((GridBagLayout)javaPanel.getLayout()).setConstraints(javaType, gbc);
         javaPanel.add(javaType);
         initLabel = new JLabel();
         initLabel.setText(res.getString("Initial_Value_"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,20,2,0);
         ((GridBagLayout)javaPanel.getLayout()).setConstraints(initLabel, gbc);
         javaPanel.add(initLabel);
         initValue = new JTextField();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 3;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,20,0,20);
         ((GridBagLayout)javaPanel.getLayout()).setConstraints(initValue, gbc);
         javaPanel.add(initValue);
         javadocLabel = new JLabel();
         javadocLabel.setText(res.getString("Javadoc_Text_"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 4;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,20,2,0);
         ((GridBagLayout)javaPanel.getLayout()).setConstraints(javadocLabel, gbc);
         javaPanel.add(javadocLabel);
         javadocView = new JScrollPane();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 5;
         gbc.weightx = 1.0;
         gbc.weighty = 50.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,20,20,20);
         ((GridBagLayout)javaPanel.getLayout()).setConstraints(javadocView, gbc);
         javaPanel.add(javadocView);
        //Initializing dbPanel.
         dbPanel = new JPanel();
         dbPanel.setLayout(new GridBagLayout());
         dbTypeLabel = new JLabel();
         dbTypeLabel.setText(res.getString("db_Type_"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(20,20,2,0);
         ((GridBagLayout)dbPanel.getLayout()).setConstraints(dbTypeLabel, gbc);
         dbPanel.add(dbTypeLabel);
         dbType = new JComboBox();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,20,0,20);
         ((GridBagLayout)dbPanel.getLayout()).setConstraints(dbType, gbc);
         dbPanel.add(dbType);
         optionPanel = new JPanel();
      /*         optionPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5)); */
         optionPanel.setLayout(new GridBagLayout());
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 4;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.BOTH;
      /*         gbc.insets = new Insets(0,20,20,20); */
         gbc.insets = new Insets(0,0,0,0);
         ((GridBagLayout)dbPanel.getLayout()).setConstraints(optionPanel, gbc);
         dbPanel.add(optionPanel);

         gbc.gridy = 0;
         gbc.insets = new Insets(0,10,0,10);

         isIndex = new JCheckBox();
         isIndex.setText(res.getString("Is_an_Index1"));
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(isIndex, gbc);
         optionPanel.add(isIndex);

         gbc.gridy++;

         canBeNull = new JCheckBox();
         canBeNull.setText(res.getString("Can_be_Null"));
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(canBeNull, gbc);
         optionPanel.add(canBeNull);

         gbc.gridy++;

         usedForQuery = new JCheckBox();
         usedForQuery.setText(res.getString("Can_be_queried"));
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(usedForQuery, gbc);
         optionPanel.add(usedForQuery);

         gbc.gridy++;

         refConstraint = new JCheckBox();
         refConstraint.setText(res.getString("Referenced_DO_must"));
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(refConstraint, gbc);
         optionPanel.add(refConstraint);

         gbc.gridy++;

         constant = new JCheckBox();
         constant.setText(res.getString("Value_is_constant"));
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(constant, gbc);
         optionPanel.add(constant);

         sizePanel = new JPanel();
         sizePanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
         ((FlowLayout)sizePanel.getLayout()).setAlignment(FlowLayout.LEFT);
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,15,0,20);
         ((GridBagLayout)dbPanel.getLayout()).setConstraints(sizePanel, gbc);
         dbPanel.add(sizePanel);
         sizeLabel = new JLabel();
         sizeLabel.setText(res.getString("Size_"));
         sizePanel.add(sizeLabel);
         size = new JTextField();
         size.setColumns(5);
         sizePanel.add(size);

	/* FIX
		disabled until code generation is more robust.

        //Initializing ArrayPanel.
         arrayPanel = new JPanel();
         arrayPanel.setLayout(new GridBagLayout());
         isArray = new JCheckBox();
         isArray.setText("Is an Array");
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.WEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(20,20,10,0);
         ((GridBagLayout)arrayPanel.getLayout()).setConstraints(isArray, gbc);
         arrayPanel.add(isArray);
         arrayAttributesPanel = new JPanel();
         arrayAttributesPanel.setLayout(new GridBagLayout());
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,20,20,20);
         ((GridBagLayout)arrayPanel.getLayout()).setConstraints(arrayAttributesPanel, gbc);
         arrayPanel.add(arrayAttributesPanel);
         arraySizePanel = new JPanel();
         arraySizePanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.WEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,0,0,0);
         ((GridBagLayout)arrayAttributesPanel.getLayout()).setConstraints(arraySizePanel, gbc);
         arrayAttributesPanel.add(arraySizePanel);
         arraySizeLabel = new JLabel();
         arraySizeLabel.setText("Size:");
         arraySizePanel.add(arraySizeLabel);
         arraySize = new JTextField();
         arraySize.setColumns(5);
         arraySizePanel.add(arraySize);
	*/

        //Initializing Tree view.
         listener = new AttributeTreeSelectionListener(tree, className);
         tree.addTreeSelectionListener(listener);
         tree.setMinimumSize(new Dimension(0,0));
         treeView.setViewportView(tree);
         treeView.setMinimumSize(new Dimension(0,0));
        //Initializing borders.
         optionPanel.setBorder(new TitledBorder(res.getString("Options")));
	 /*
         arrayAttributesBorder = new TitledBorder("Array Attributes");
         arrayAttributesPanel.setBorder(arrayAttributesBorder);
	 */
        //Initializing javadoc TextArea.
         jta = new JTextArea();
         jta.setMinimumSize(new Dimension(0,0));
         javadocView.setViewportView(jta);
         javadocView.setPreferredSize(new Dimension(0,0));
        //Other Frame Initializations.
         setSize(350,500);
         setResizable(true);
         jtp.addTab(res.getString("General"), generalPanel);
         jtp.addTab(res.getString("Java"), javaPanel);
         jtp.addTab(res.getString("Database1"), dbPanel);
	 /*
         jtp.addTab("Array", arrayPanel);
	 */
        //FIX:
         dbType.setEnabled(true);
        //Initializing Listeners.
         SymAction lSymAction = new SymAction();
         okButton.addActionListener(lSymAction);
         cancelButton.addActionListener(lSymAction);
         name.addActionListener(lSymAction);
         javaType.addActionListener(lSymAction);
         SymChange lSymChange = new SymChange();
	 /*
         isArray.addChangeListener(lSymChange);
	 */
      }
      static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    private JPanel jp;
      private JPanel buttonPanel;
      private JButton okButton;
      private JButton cancelButton;
      private JButton helpButton;
      private JTabbedPane jtp;
      private JPanel generalPanel;
      private JLabel nameLabel;
      private JTextField name;
      private JLabel viewLabel;
      private JScrollPane treeView;
      private JTextField className;
      private JPanel javaPanel;
      private JLabel jtypeLabel;
      private JComboBox javaType;
      private JLabel initLabel;
      private JTextField initValue;
      private JLabel javadocLabel;
      private JScrollPane javadocView;
      private JPanel dbPanel;
      private JLabel dbTypeLabel;
      private JComboBox dbType;
      private JPanel optionPanel;
      private JCheckBox isIndex;
      private JCheckBox canBeNull;
      private JCheckBox usedForQuery;
      private JCheckBox refConstraint;
      private JCheckBox constant;
      private JPanel sizePanel;
      private JLabel sizeLabel;
      private JTextField size;
      /*
      private JPanel arrayPanel;
      private JCheckBox isArray;
      private JPanel arrayAttributesPanel;
      private JPanel arraySizePanel;
      private JLabel arraySizeLabel;
      private JTextField arraySize;
      private TitledBorder arrayAttributesBorder;
      */
      private JTextArea jta;
      private AttributeTreeSelectionListener listener;
      private JTree tree;
      private JTree oldTree;
      private DataObjectApplication doa;
      private DataObject DO;
      private Attribute att;
    /*
     * When this method is called, All of the Attribute's attributes are set
     * to the appropriate values and then the dialog is hidden.
     */
      private void okButton_actionPerformed(ActionEvent event) {
      //System.err.println("AttributeEditor.okButton_actionPerformed");
         if(className.getText().equals("")) {
            jtp.setSelectedIndex(0);
            JOptionPane.showMessageDialog(this,
               res.getString("Please_select_a_class"),
               res.getString("Error_creating"),
               JOptionPane.WARNING_MESSAGE);
            return;
         }
         DataObject DOA = ((DOTreeNode)tree.getLastSelectedPathComponent()).getDataObject();
         if(DO.getAttributes().contains(att)) {
            DO.getAttributes().removeElement(att);
         }
         if(!DOA.getAttributes().contains(att)) {
            DOA.addAttribute(att);
         }
         att.setName(name.getText());
         att.setJavaType(javaType.getSelectedItem().toString());
         att.setInitialValue(initValue.getText());
         att.setJavadoc(jta.getText());
         att.setDBType(dbType.getSelectedItem().toString());
         att.setIndex(isIndex.isSelected());
         att.setCanBeNull(canBeNull.isSelected());
         att.setUsedForQuery(usedForQuery.isSelected());
         att.setRefConstraint(refConstraint.isSelected());
         att.setConstant(constant.isSelected());
         att.setSize(size.getText());
	 /*
         att.setArray(isArray.isSelected());
	 */
         if(javaType.getSelectedItem() instanceof DataObject) {
            att.setObjectRefDO((DataObject)javaType.getSelectedItem());
         }
         else {
            att.setObjectRefDO(null);
         }
	 /*
         try{
            att.setArraySize((new Integer(arraySize.getText())).intValue());
         }
         catch(NumberFormatException e) {
               att.setArraySize(0);
         }
	 */
         if(dbType.getSelectedItem().toString().equals("none")){
            att.setObjectRef(true);
         }
         else{
            att.setObjectRef(false);
         }

      	//this ensures that the new attribute will show in the table of attributes
         doa.refreshDODS();
      	//hide this window
         setVisible(false);
      }
    /*
     * This Method just hides the dialog when cancel is pressed.
     */
      private void cancelButton_actionPerformed(ActionEvent event) {
         setVisible(false);
      }
    /*
     * When enter is pressed in the name field, it is as if the ok button was pressed.
     */
      private void name_actionPerformed(ActionEvent event) {
         okButton.doClick();
      }
    /*
     * This method initializes the choices for JavaTypes.
     */
      private void initializeJavaChoices() {
         try {
            javaType.removeAllItems();
         }
            catch(IndexOutOfBoundsException e){
            }
         Vector DOS = new Vector();
         ((DOPackage)((DOTreeNode)tree.getModel().getRoot()).getDataObject()).getAllDataObjects(DOS);
         javaType.addItem("String");
         javaType.addItem("java.math.BigDecimal");
         javaType.addItem("boolean");
         javaType.addItem("byte");
         javaType.addItem("short");
         javaType.addItem("int");
         javaType.addItem("long");
         javaType.addItem("float");
         javaType.addItem("double");
         javaType.addItem("byte[]");
         javaType.addItem("java.sql.Date");
         javaType.addItem("java.sql.Time");
         javaType.addItem("java.sql.Timestamp");
         for(int i = 0; i < DOS.size(); i++) {
            if(!((DataObject)DOS.elementAt(i)).isAbstract()) {
               javaType.addItem(((DataObject)DOS.elementAt(i)));
            }
         }
      }
    /*
     * When a javatype is chosen, only certain DBTypes are allowed. This method
     * Sets up the choices for DBType. If there is only one chioce, the comboBox
     * is disabled.
     */
      private void setDBChoices() {
         try {
            dbType.removeAllItems();
         }
            catch(IndexOutOfBoundsException e){
            }
         if(javaType.getSelectedItem()==null) {
            return;
         }
         String javaChoice = javaType.getSelectedItem().toString();
         if(javaChoice==null) {
            return;
         }
         if(javaChoice.equals("String")) {
            dbType.addItem("VARCHAR");
            dbType.addItem("CHAR");
            dbType.addItem("LONGVARCHAR");
            dbType.setEnabled(true);
         }
         else if(javaChoice.equals("java.math.BigDecimal")) {
            dbType.addItem("NUMERIC");
            dbType.addItem("DECIMAL");
            dbType.setEnabled(true);
         }
         else if(javaChoice.equals("boolean")) {
            dbType.addItem("BIT");
            dbType.addItem("CHAR");
            dbType.setEnabled(true);
         }
         else if(javaChoice.equals("byte[]")) {
            dbType.addItem("BINARY");
            dbType.addItem("VARBINARY");
            dbType.addItem("LONGVARBINARY");
            dbType.setEnabled(true);
         }
         else if(javaChoice.equals("byte")) {
            dbType.addItem("TINYINT");
            dbType.setEnabled(false);
         }
         else if(javaChoice.equals("short")) {
            dbType.addItem("SMALLINT");
            dbType.setEnabled(false);
         }
         else if(javaChoice.equals("int")) {
            dbType.addItem("INTEGER");
            dbType.setEnabled(false);
         }
         else if(javaChoice.equals("long")) {
            dbType.addItem("BIGINT");
            dbType.setEnabled(false);
         }
         else if(javaChoice.equals("float")) {
            dbType.addItem("REAL");
            dbType.setEnabled(false);
         }
         else if(javaChoice.equals("double")) {
            dbType.addItem("DOUBLE");
            dbType.setEnabled(false);
         }
         else if(javaChoice.equals("java.sql.Date")) {
            dbType.addItem("DATE");
            dbType.setEnabled(false);
         }
         else if(javaChoice.equals("java.sql.Time")) {
            dbType.addItem("TIME");
            dbType.setEnabled(false);
         }
         else if(javaChoice.equals("java.sql.Timestamp")) {
            dbType.addItem("TIMESTAMP");
            dbType.setEnabled(false);
         }
         else {
            dbType.addItem("none");
            dbType.setEnabled(false);
         }
      }
    /*
     * when the javaType is chosen, the DBChoices need to be updated.
     */
      private void javaType_actionPerformed(ActionEvent event) {
         setDBChoices();
      }
    /*
     * When the isArray choice changes, the array options are either
     * enabled or disabled.
     */
      /*
      private void isArray_stateChanged(ChangeEvent event) {
         if(isArray.isSelected()) {
            arraySize.setEnabled(true);
            arraySize.setBackground(Color.white);
            arraySizeLabel.setForeground(Color.black);
            arrayAttributesBorder.setTitleColor(Color.black);
         }
         else {
            arraySize.setEnabled(false);
            arraySize.setBackground(Color.lightGray);
            arraySizeLabel.setForeground(Color.gray);
            arrayAttributesBorder.setTitleColor(Color.gray);
         }
         repaint();
      }
      */
    /**
     * This method initializes all of the fields to their appropriate values given
     * the attribute being edited and the DataObject it is in.
     * @param at The attribute to be edited.
     * @param Do The DataObject it is in.
     */
      public void setAttribute(Attribute at, DataObject Do) {
         jtp.setSelectedIndex(0);
         att = at;
         DO = Do;
         jta.setText(att.getJavadoc());
         name.setText(att.getName());
         initValue.setText(att.getInitialValue());
         isIndex.setSelected(att.isIndex());
         canBeNull.setSelected(att.canBeNull());
         usedForQuery.setSelected(att.isUsedForQuery());
         refConstraint.setSelected(att.isRefConstraint());
         constant.setSelected(att.isConstant());
         size.setText(att.getSize());
	 /*
         isArray.setSelected(att.isArray());
         arraySize.setText(att.getArraySize()+"");
         if(!att.isArray()) {
            arraySize.setEnabled(false);
            arraySize.setBackground(Color.lightGray);
            arraySizeLabel.setForeground(Color.gray);
            arrayAttributesBorder.setTitleColor(Color.gray);
         }
	 */
         DOTreeModel treeModel = new DOTreeModel((DOTreeNode)oldTree.getModel().getRoot(), doa);
         tree.setModel(treeModel);
         tree.setCellRenderer(new DOTreeRenderer());
         tree.setSelectionPath(oldTree.getSelectionPath());
         tree.expandPath(oldTree.getSelectionPath());
         initializeJavaChoices();
         if(att.getObjectRefDO()!=null) {
            javaType.setSelectedItem(att.getObjectRefDO());
         }
         else {
            javaType.setSelectedItem(att.getJavaType());
         }
         setDBChoices();
         if(tree.getSelectionPath()==null){
            Object[] rootpath = {tree.getModel().getRoot()};
            tree.setSelectionPath(new TreePath(rootpath));
         }
         listener.valueChanged(new TreeSelectionEvent(this, null, null, null, null));
      }
    /*
     *
     * the inner classes are just listeners for events.
     *
     */
      class SymAction implements ActionListener {
         public void actionPerformed(ActionEvent event) {
            Object object = event.getSource();
            if (object == okButton)
               okButton_actionPerformed(event);
            else if (object == cancelButton)
               cancelButton_actionPerformed(event);
            else if (object == name)
               name_actionPerformed(event);
            else if (object == javaType)
               javaType_actionPerformed(event);
         }
      }
      class SymChange implements ChangeListener {
         public void stateChanged(ChangeEvent event) {
            Object object = event.getSource();
	    /*
            if (object == isArray)
               isArray_stateChanged(event);
	    */
         }
      }
   }
