/*
 * -----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement:
 * This product includes Enhydra software developed by Lutris
 * Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * -----------------------------------------------------------------------------
 * $Id: PackageEditor.java,v 1.10.14.1 2000/12/01 22:27:39 chip Exp $
 * -----------------------------------------------------------------------------
 */
package com.lutris.dods.gui.editors;

import com.lutris.dods.store.doa.DOPackage;
import com.lutris.dods.store.doa.DataObject;
import com.lutris.dods.gui.Icons;
import com.lutris.dods.gui.DataObjectApplication;
import com.lutris.dods.gui.tree.DOTreeNode;
import com.lutris.dods.gui.tree.DOTreeModel;
import com.lutris.dods.gui.tree.PackageTree;
import com.lutris.dods.gui.tree.PackageTreeModel;
import com.lutris.dods.gui.tree.PackageTreeNode;
import com.lutris.dods.gui.tree.PackageTreeSelectionListener;
import com.lutris.dods.exceptions.DataObjectNameException;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JTree;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import java.awt.Component;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.Font;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Vector;
import java.util.ResourceBundle;

/**
 * This class creates a pop-up window which allows you to edit a package by
 * changing its name and location.
 *
 * @author  Paul Mahar
 * @version $Revision: 1.10.14.1 $
 */
public final class PackageEditor extends JDialog {
    static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    private DOPackage			 editObject;
    private PackageTree			 tree;
    private JTree			 structTree;
    private DataObjectApplication	 doa;
    private PackageTreeSelectionListener listener;
    private JTextField			 textName;
    private JTextField			 packageName;
    private JScrollPane			 treeView;
    private JLabel			 packageLabel;
    private JLabel			 nameLabel;
    private JPanel			 buttonPanel;
    private JButton			 okButton;
    private JButton			 cancelButton;
    private JButton			 helpButton;

    /**
     * This Constructor initializes all of the fields and lays everything out.
     * @param tre The JTree from the parent frame.
     * @param da The parent DataObjectApplication.
     */
    public PackageEditor(JTree tre, DataObjectApplication da) {
	super(da);

	// initializing the Tree.
	structTree = tre;
	doa = da;
	tree = new PackageTree();

	DOTreeNode       oldRoot = null;
	DOPackage	 rootPack = null;
	PackageTreeNode  packRoot = null;
	PackageTreeModel treeModel = null;

	oldRoot = (DOTreeNode) structTree.getModel().getRoot();
	rootPack = (DOPackage) oldRoot.getDataObject();
	packRoot = new PackageTreeNode(rootPack, editObject);
	treeModel = new PackageTreeModel(packRoot);

	tree.setModel(treeModel);
	tree.setMinimumSize(new Dimension(0, 0));

	// Initializing the frame.
	setTitle(res.getString("Package_Editor"));
	setResizable(true);
	setModal(true);
	setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	setSize(300, 250);
	getContentPane().setLayout(new GridBagLayout());

	GridBagConstraints gbc;

	// Initializing Fields;
	textName = new JTextField();
	gbc = new GridBagConstraints();
	gbc.gridx = 0;
	gbc.gridy = 1;
	gbc.gridwidth = 2;
	gbc.weightx = 1.0;
	gbc.weighty = 1.0;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(0, 20, 0, 20);

	((GridBagLayout) getContentPane().getLayout()).setConstraints(textName,
		gbc);
	getContentPane().add(textName);

	packageName = new JTextField();
	gbc = new GridBagConstraints();
	gbc.gridx = 0;
	gbc.gridy = 4;
	gbc.gridwidth = 2;
	gbc.weightx = 1.0;
	gbc.weighty = 1.0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(0, 20, 0, 20);

	((GridBagLayout) getContentPane().getLayout()).setConstraints(packageName,
		gbc);
	getContentPane().add(packageName);
	packageName.setEnabled(false);

	treeView = new JScrollPane();
	gbc = new GridBagConstraints();
	gbc.gridx = 0;
	gbc.gridy = 3;
	gbc.gridwidth = 3;
	gbc.weightx = 1.0;
	gbc.weighty = 10.0;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.insets = new Insets(0, 20, 0, 20);

	((GridBagLayout) getContentPane().getLayout()).setConstraints(treeView,
		gbc);
	getContentPane().add(treeView);

	packageLabel = new JLabel();

	packageLabel.setText(res.getString("Place_in_"));

	gbc = new GridBagConstraints();
	gbc.gridx = 0;
	gbc.gridy = 2;
	gbc.weightx = 1.0;
	gbc.weighty = 1.0;
	gbc.anchor = GridBagConstraints.SOUTHWEST;
	gbc.fill = GridBagConstraints.NONE;
	gbc.insets = new Insets(0, 20, 2, 0);

	((GridBagLayout) getContentPane().getLayout()).setConstraints(packageLabel,
		gbc);
	getContentPane().add(packageLabel);

	nameLabel = new JLabel();

	nameLabel.setText(res.getString("Name_1"));

	gbc = new GridBagConstraints();
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.weightx = 1.0;
	gbc.weighty = 1.0;
	gbc.anchor = GridBagConstraints.SOUTHWEST;
	gbc.fill = GridBagConstraints.NONE;
	gbc.insets = new Insets(0, 20, 2, 0);

	((GridBagLayout) getContentPane().getLayout()).setConstraints(nameLabel,
		gbc);
	getContentPane().add(nameLabel);

	buttonPanel = new JPanel();

	buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));

	gbc = new GridBagConstraints();
	gbc.gridx = 0;
	gbc.gridy = 5;
	gbc.gridwidth = 3;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.insets = new Insets(5, 15, 0, 15);

	((GridBagLayout) getContentPane().getLayout()).setConstraints(buttonPanel,
		gbc);
	getContentPane().add(buttonPanel);

	okButton = new JButton();

	okButton.setText(res.getString("OK"));
	buttonPanel.add(okButton);

	cancelButton = new JButton();

	cancelButton.setText(res.getString("Cancel1"));
	buttonPanel.add(cancelButton);

	helpButton = new JButton();

	helpButton.setText(res.getString("Help"));
	buttonPanel.add(helpButton);
	helpButton.setEnabled(false);

	// Initializing TreeView.
	listener = new PackageTreeSelectionListener(tree, packageName);

	tree.addTreeSelectionListener(listener);
	tree.setMinimumSize(new Dimension(0, 0));
	treeView.setViewportView(tree);
	treeView.setMinimumSize(new Dimension(0, 0));

	// Initializing Listeners.
	LocalActionListener lLocalActionListener = new LocalActionListener();

	okButton.addActionListener(lLocalActionListener);
	cancelButton.addActionListener(lLocalActionListener);
	textName.addActionListener(lLocalActionListener);
    }

    /**
     * This method sets the packages name and location and then hides the dialog.
     */
    private void commitAndExit() {
	if (!tree.isNameUniqueForSelection(textName.getText().trim(),
					   editObject)) {
	    JOptionPane.showMessageDialog(this,
					  res.getString("An_object_with_that3"),
					  res.getString("Error_creating1"),
					  JOptionPane.WARNING_MESSAGE);
	} else {
	    try {
		updatePackage();
                tree.updateStructure(structTree, editObject);
	    } catch (DataObjectNameException e) {
		doa.namingExceptionOccurred(e, this);

		return;
	    }

	    doa.updateTree();
	    closeDialog();
	}
    }

    /**
     * Update the package object.
     *
     * @throws DataObjectNameException
     * Thrown if new name is not valid.
     */
    private void updatePackage() throws DataObjectNameException {
        editObject.setName(textName.getText());
    }

    /**
     * This method initializes all of the Fields for the editor.
     * @param DoP The DOPackage being edited.
     */
    public void setDOPackage(DOPackage newPack) {
	editObject = newPack;

	textName.setText(editObject.getName());

	TreePath	 selectPath = null;
	DOTreeNode       structNode = null;
	DOPackage	 rootPack = null;
	DOPackage	 parentPack = null;
	PackageTreeNode  rootNode = null;
	PackageTreeNode  parentNode = null;
	PackageTreeModel treeModel = null;

	structNode = (DOTreeNode) structTree.getModel().getRoot();
	rootPack = (DOPackage) structNode.getDataObject();
	rootNode = new PackageTreeNode(rootPack, editObject);
	treeModel = new PackageTreeModel(rootNode);

	tree.setModel(treeModel);
	tree.setMinimumSize(new Dimension(0, 0));
	tree.synchSelectionWithStuctureTree(structTree);
	listener.valueChanged(new TreeSelectionEvent(this, null, null, null,
		null));
    }

    /**
     * Close the dialog and release associated resources.
     */
    public void closeDialog() {
	setVisible(false);
	removeAll();
	dispose();
    }


    /**
     * This class is just an action listener for this dialog.
     */
    class LocalActionListener implements ActionListener {

	/**
	 * Figure out what happened and fire the associated method.
	 */
	public void actionPerformed(ActionEvent event) {
	    Object object = event.getSource();

	    if (object == okButton) {
		commitAndExit();
	    } else if (object == cancelButton) {
	    	closeDialog();
	    } else if (object == textName) {
		okButton.doClick();
	    }
	}

    }

}

