/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DOTreeMouseListener.java,v 1.9.14.1 2000/12/01 22:27:39 chip Exp $
 *-----------------------------------------------------------------------------
 */



package com.lutris.dods.gui.tree;

import com.lutris.dods.store.doa.*;
import com.lutris.dods.gui.*;
import com.lutris.dods.gui.editors.*;
import javax.swing.JTree;
import javax.swing.JTable;
import javax.swing.JOptionPane;
import javax.swing.tree.TreePath;
import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionListener;
import java.util.Vector;
import java.util.ResourceBundle;
/**
 * This class handles when you want to drag a Node in the tree to a new parent node.
 */
public final class DOTreeMouseListener extends MouseAdapter implements MouseMotionListener {
    static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    private DataObjectApplication doa;
    private JTree tree;
    private DOTreeNode selectedNode;
    private Cursor normal = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
    private TreePath currentPath;
    private TreePath originalPath;
    private JTable table;
    /**
     * Initializes the appropriate variables.
     */
    public DOTreeMouseListener(DataObjectApplication da, JTable tab) {
        doa = da;
        table = tab;
    }
    /**
     * When the mouse is pressed, the node it is over is selected and can
     * be dragged.
     */
    public void mousePressed(MouseEvent e) {
        tree = (JTree)e.getSource();
        TreePath path = tree.getPathForLocation(e.getX(), e.getY());
        tree.setSelectionPath(path);
        if(path == null){
            originalPath = null;
            currentPath = null;
            selectedNode = null;
            return;
        }
        currentPath = path;
        originalPath = path;
        selectedNode = (DOTreeNode)path.getLastPathComponent();
        tree.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    }
    /**
     * When the Mouse is released if the selected node can be placed in the
     * node where the mouse is released then it will be.
     */
    public void mouseReleased(MouseEvent e) {
        tree = (JTree)e.getSource();
        tree.setCursor(normal);
        TreePath path = currentPath;
        if(path == null || selectedNode == null || path.equals(originalPath)){
            if(originalPath != null) {
                tree.setSelectionPath(originalPath);
            }
            originalPath = null;
            currentPath = null;
            selectedNode = null;
            return;
        }
        else {
            DOTreeNode newParent = (DOTreeNode)path.getLastPathComponent();
            for(int i = 0; i < newParent.getChildren().size(); i++) {
                DOTreeNode donode = (DOTreeNode)newParent.getChildren().elementAt(i);
                if(donode.getDataObject().getName().equals(selectedNode.getDataObject().getName()) && !donode.equals(selectedNode)) {
                    JOptionPane.showMessageDialog(tree,
                                      res.getString("An_object_with_that2"),
                                      res.getString("Error_moving_Data"),
                                      JOptionPane.WARNING_MESSAGE);
                    tree.setSelectionPath(originalPath);
                    originalPath = null;
                    currentPath = null;
                    selectedNode = null;
                    return;
                }
            }
            DataObject DO = selectedNode.getDataObject();

            if(!newParent.isLeaf() && !selectedNode.contains(newParent)) {
                DOTreeNode findnode = ((DOTreeNode)((DOTreeModel)tree.getModel()).getRoot()).getParentWithChildContaining(DO);
                DOTreeNode dotn;
                Vector children;
                if(findnode != null) {
                    children = findnode.getChildren();
                }
                else {
                    children = new Vector();
                }
                for(int i = 0; i < children.size(); i++) {
                    dotn = (DOTreeNode)children.elementAt(i);
                    if(DO.equals(dotn.getDataObject())){
                        children.removeElement(dotn);
                        ((DOPackage)findnode.getDataObject()).getDataObjects().removeElement(dotn.getDataObject());
                        break;
                    }
                }
                DOTreeNode newNode;
                if(DO instanceof DOPackage) {
                    DOPackage DOP = (DOPackage)DO;
                    ((DOPackage)newParent.getDataObject()).addDOPackage(DOP);
                    newNode = new DOTreeNode(DOP, true);
                    newParent.getChildren().addElement(newNode);
                }
                else {
                    ((DOPackage)newParent.getDataObject()).addDataObject(DO);
                    newNode = new DOTreeNode(DO, false);
                    newParent.getChildren().addElement(newNode);
                }

                doa.updateTree();

                Object[] newPathComponents = new Object[path.getPath().length +1];
                for(int i = 0; i < path.getPath().length; i++) {
                    newPathComponents[i] = path.getPath()[i];
                }
                newPathComponents[path.getPath().length] = newNode;

                TreePath newPath = new TreePath(newPathComponents);
                tree.setSelectionPath(newPath);
                selectedNode = null;
                originalPath = null;
                currentPath = null;
            }
        }
    }
    /**
     * If you double click on a Leaf in the tree, an editor will be launched.
     */
    public void mouseClicked(MouseEvent e) {
        if(e.getClickCount() == 2 ) {
            tree = (JTree)e.getSource();
            DOTreeNode node;
            TreePath path;

            path = tree.getPathForLocation(e.getX(), e.getY());
            if(path == null){
                return;
            }
            node = (DOTreeNode)path.getLastPathComponent();

            if(!node.equals(tree.getModel().getRoot())) {
                Object[] newpath = new Object[path.getPath().length-1];
                for(int i = 0; i < path.getPath().length-1; i++) {
                    newpath[i] = path.getPath()[i];
                }
                tree.setSelectionPath(new TreePath(newpath));
                if(node.getDataObject() instanceof DOPackage) {
                }
                else if(node.getDataObject() instanceof DataObject) {
                    DataObject DO = (DataObject)node.getDataObject();
                    DataObjectEditor doe = new DataObjectEditor(tree, doa);
                    doe.setDataObject(DO);
                    doe.setVisible(true);
                }
                doa.updateMenus();
            }
        }
    }
    /**
     * Does nothing.
     */
    public void mouseMoved(MouseEvent e){
    }
    /**
     * If the mouse is dragged over a node where the selected node can be put
     * then that node is highlighted, else the selected node is highlighted.
     */
    public void mouseDragged(MouseEvent e) {
        if(selectedNode != null && originalPath != null) {
            tree = (JTree)e.getSource();
            TreePath path = tree.getPathForLocation(e.getX(), e.getY());
            if(path == null || path.equals(originalPath)){
                tree.setSelectionPath(originalPath);
                currentPath = originalPath;
                return;
            }
            else{
                DOTreeNode Node = (DOTreeNode)path.getLastPathComponent();
                if(!Node.isLeaf() && !Node.equals(selectedNode) && !selectedNode.contains(Node)) {
                    tree.setSelectionPath(path);
                    currentPath = path;
                }
                else {
                    tree.setSelectionPath(originalPath);
                    currentPath = originalPath;
                }
            }
        }
    }
}
