/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DOTreeRenderer.java,v 1.10 2000/01/29 07:01:42 markd Exp $
 *-----------------------------------------------------------------------------
 */



package com.lutris.dods.gui.tree;

import com.lutris.dods.gui.*;
import com.lutris.dods.store.doa.*;

import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.ImageIcon;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import java.awt.Component;
import java.awt.Color;
/**
 * The Renderer for the Tree in DODS.  It controls the text, icon, and tooltip.
 *
 * @author  Philip Martin
 * @version $Revision: 1.10 $
 */
public class DOTreeRenderer implements TreeCellRenderer {
    private JLabel label;
    private ImageIcon Node;
    private ImageIcon OpenNode;
    private ImageIcon DOLeaf;
    private ImageIcon DOALeaf;
    private ImageIcon DOFLeaf;
    private ImageIcon DOAFLeaf;
    /**
     * The constructor initializes the JLabel which is the renderer for the Tree node.
     * It also initializes all of the icons which can be used in the tree.
     */
    public DOTreeRenderer() {
        label = new JLabel();
        label.setOpaque(true);
        label.setBorder(new EmptyBorder(1, 1, 1, 1));
        
        DOLeaf = Icons.getIcon("dataobject.gif");
        DOALeaf = Icons.getIcon("abstract_dataobject.gif");
        DOFLeaf = Icons.getIcon("final_dataobject.gif");
        DOAFLeaf = Icons.getIcon("abstract_final_dataobject.gif");
        Node = Icons.getIcon("folder.gif");
        OpenNode = Icons.getIcon("open_folder.gif");
    }
    /**
     * Sets up the JLabel which it returns depending on the various parameters.
     * @param tree The Tree that needs the Renderer.
     * @param value The DOTreeNode which is being displayed.
     * @param selected When the node is selected, it is outlined.
     * @param expanded If a folder is open the icon changes.
     * @param leaf Folers and Leaves have different icons.
     * @param row The row this node is on.
     * @param hasFocus Isn't used.
     */
    public Component getTreeCellRendererComponent(JTree tree,
                                                  Object value,
                                                  boolean selected,
                                                  boolean expanded,
                                                  boolean leaf,
                                                  int row,
                                                  boolean hasFocus) {
        DOTreeNode node = (DOTreeNode)value;
        DataObject DO = node.getDataObject();
        
        label.setFont(tree.getFont());
        
        if(selected) {
            label.setBackground(tree.getBackground());
            label.setForeground(tree.getForeground());
            label.setBorder(new LineBorder(Color.blue));
        }
        else {
            label.setBackground(tree.getBackground());
            label.setForeground(tree.getForeground());
            label.setBorder(new LineBorder(tree.getBackground()));
        }
        if(leaf) {
            if(DO.isAbstract() && DO.isFinal()) {
                label.setIcon(DOAFLeaf);
                label.setToolTipText("abstract final class " + DO.getName() + "DO");
            }
            else if(DO.isAbstract() && !DO.isFinal()) {
                label.setIcon(DOALeaf);
                label.setToolTipText("abstract class " + DO.getName() + "DO");
            }
            else if(!DO.isAbstract() && DO.isFinal()) {
                label.setIcon(DOFLeaf);
                label.setToolTipText("final class " + DO.getName() + "DO");
            }
            else {
                label.setIcon(DOLeaf);
                label.setToolTipText("class " + DO.getName() + "DO");
            }
        }
        else if(!leaf && !expanded) {
            label.setIcon(Node);
            if(DO.getPackageName().equals("")) {
                label.setToolTipText("package " + DO.getName());
            }
            else {
                label.setToolTipText("package " + DO.getPackageName() + "." + DO.getName());
            }
        }
        else if(!leaf && expanded) {
            label.setIcon(OpenNode);
            if(DO.getPackageName().equals("")) {
                label.setToolTipText("package " + DO.getName());
            }
            else {
                label.setToolTipText("package " + DO.getPackageName() + "." + DO.getName());
            }
        }
        
        label.setText(node.getDataObject().getName() + " ");
        
        return label;
    }
}
