/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package com.lutris.dods.gui.tree;

import com.lutris.dods.gui.Icons;
import com.lutris.dods.store.doa.DOPackage;
import com.lutris.dods.store.doa.DataObject;
import java.awt.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class PackageTree extends JTree {

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public PackageTree() {
	pmInit();
    }

    /**
     * Method declaration
     *
     *
     * @param structTree
     * @param editObject
     *
     * @see
     */
    public void updateStructure(JTree structTree, DataObject editObject) {
	TreePath	path;
	DOTreeModel     structModel;
	DOTreeNode      foundNode;
	DOTreeNode      childNode;
	DOTreeNode      newNode;
	DOPackage       parentPack;
	PackageTreeNode parentNode;
	Vector		children;

	structModel = (DOTreeModel) structTree.getModel();
	foundNode =
	    ((DOTreeNode) ((DOTreeModel) structTree.getModel()).getRoot()).getParentWithChildContaining(editObject);

	if (foundNode != null) {
	    children = foundNode.getChildren();
	} else {
	    children = new Vector();
	}

	for (int i = 0; i < children.size(); i++) {
	    childNode = (DOTreeNode) children.elementAt(i);

	    if (editObject.equals(childNode.getDataObject())) {
		children.removeElement(childNode);
		((DOPackage) foundNode.getDataObject()).getDataObjects().removeElement(childNode.getDataObject());

		break;
	    }
	}

	parentNode = (PackageTreeNode) getLastSelectedPathComponent();
	parentPack = (DOPackage) parentNode.getUserObject();

	if (editObject instanceof DOPackage) {
	    parentPack.addDOPackage((DOPackage) editObject);

	    newNode = new DOTreeNode(editObject, true);
	} else {
	    parentPack.addDataObject(editObject);

	    newNode = new DOTreeNode(editObject, false);
	}

	foundNode = structModel.getNodeContaining(parentPack);

	foundNode.getChildren().addElement(newNode);

	// The following makes sure that the DataObject is selected after
	// it is created.
	path = structModel.getPathContaining(editObject);

	structTree.setSelectionPath(path);
	structTree.expandPath(path);
    }

    /**
     * Method declaration
     *
     *
     * @param structTree
     *
     * @see
     */
    public void synchSelectionWithStuctureTree(JTree structTree) {
	DOPackage       selectedPack = null;
	DOTreeNode      structNode = null;
	PackageTreeNode selectNode = null;
	TreePath	path = null;
	int		count = 0;

	path = structTree.getSelectionPath();

	if (path != null) {
	    count = path.getPathCount();
	}

	for (int i = (count - 1); i > -1; i--) {
	    structNode = (DOTreeNode) path.getPathComponent(i);

	    if (structNode.getDataObject() instanceof DOPackage) {
		selectedPack = (DOPackage) structNode.getDataObject();

		break;
	    }
	}

	path = null;

	if (selectedPack != null) {
	    selectNode =
		((PackageTreeModel) getModel()).getNodeContaining(selectedPack);
	}

	if (selectNode != null) {
	    path = selectNode.getTreePath();
	}

	if (path == null) {
	    setSelectionRow(0);
	} else {
	    setSelectionPath(path);
	    expandPath(path);
	}
    }

    /**
     * Method declaration
     *
     *
     * @param newName
     * @param editObject
     *
     * @return
     *
     * @see
     */
    public boolean isNameUniqueForSelection(String newName,
					    DataObject editObject) {
	boolean		unique = true;
	int		count = 0;
	PackageTreeNode parentNode = null;
	DOPackage       parentPack = null;
	Vector		children = null;
	String		beforeName = null;
	String		afterName = null;

	newName = newName.trim();
	parentNode = (PackageTreeNode) getLastSelectedPathComponent();
	parentPack = (DOPackage) parentNode.getUserObject();
	beforeName = editObject.getPackageName() + "." + editObject.getName();

	if (parentPack.getPackageName().trim().length() == 0) {
	    afterName = parentPack.getName() + "." + newName;
	} else {
	    afterName = parentPack.getPackageName() + "."
			+ parentPack.getName() + "." + newName;
	}

	if (!beforeName.trim().equals(afterName.trim())) {
	    children = parentPack.getDataObjects();
	    count = children.size();

	    for (int i = 0; i < count; i++) {
		DataObject data = (DataObject) children.elementAt(i);
		String     currentName = data.getName().trim();

		if (currentName.equals(newName)) {
		    unique = false;
		}
	    }
	}

	return unique;
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
	setCellRenderer(new LocalCellRenderer());
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    private class LocalCellRenderer implements TreeCellRenderer {
	private JLabel    label;
	private ImageIcon Node;
	private ImageIcon OpenNode;

	/**
	 * The constructor initializes the JLabel which is the renderer for the Tree node.
	 * It also initializes all of the icons which can be used in the tree.
	 */
	public LocalCellRenderer() {
	    label = new JLabel();

	    label.setOpaque(true);
	    label.setBorder(new EmptyBorder(1, 1, 1, 1));

	    Node = Icons.getIcon("folder.gif");
	    OpenNode = Icons.getIcon("open_folder.gif");
	}

	/**
	 * Sets up the JLabel which it returns depending on the various parameters.
	 * @param tree The Tree that needs the Renderer.
	 * @param value The DOTreeNode which is being displayed.
	 * @param selected When the node is selected, it is outlined.
	 * @param expanded If a folder is open the icon changes.
	 * @param leaf Folers and Leaves have different icons.
	 * @param row The row this node is on.
	 * @param hasFocus Isn't used.
	 */
	public Component getTreeCellRendererComponent(JTree tree,
		Object value, boolean selected, boolean expanded,
		boolean leaf, int row, boolean hasFocus) {

	    label.setFont(tree.getFont());

	    if (selected) {
		label.setBackground(tree.getBackground());
		label.setForeground(tree.getForeground());
		label.setBorder(new LineBorder(Color.blue));
	    } else {
		label.setBackground(tree.getBackground());
		label.setForeground(tree.getForeground());
		label.setBorder(new LineBorder(tree.getBackground()));
	    }

	    if (expanded) {
		label.setIcon(OpenNode);
	    } else {
		label.setIcon(Node);
	    }

	    if ((value != null) && (value instanceof PackageTreeNode)) {
	      PackageTreeNode node = (PackageTreeNode) value;
	      label.setText(node.toString());
	    }
	    return label;
	}

    }

}



/*--- formatting done in "Sun Java Convention" style on 01-19-2000 ---*/

