/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DODSProject.java,v 1.3 2000/01/29 07:01:44 markd Exp $
 *-----------------------------------------------------------------------------
 */



package com.lutris.dods.store.doa;

import java.io.File;
/**
 * A DODSProject is a DOPackage which is the root of a project.  It contains
 * the information about what file it is stored in and what kind of database
 * these DataObjects will be contained in. (ie. Oracle, Informix, etc.)
 *
 * @author  Philip Martin
 * @version $Revision: 1.3 $
 */
public class DODSProject extends DOPackage {
    /**
     * This is the file that the DODSProject is being stored in.  It is null
     * if the project hasn't been saved yet.
     */
    protected File file;
    /**
     * This is the type of DataBase that the project is being mapped to.
     */
    protected String database = "Standard"; // default db
    /**
     * This constructor sets up the name of the Root Package.
     */
    public DODSProject(String name) {
        super(name);
    }
    /**
     * This constructor uses "ProjectRoot" as the Root package name.
     */
    public DODSProject() {
        super("ProjectRoot");
    }
    /**
     * @return The type of Database this Project is for.
     */
    public String getDatabase() {
        return database;
    }
    /**
     * Sets the database this is for.
     * FIX: Should throw an exception if an invalid DataBase is chosen.
     */
    public void setDatabase(String db) {
       if (db.trim().toLowerCase().startsWith("standard")) {
         database = "Standard";
       } else {
         database = db;
       }
    }
    /**
     * Sets the file that this project is stored in.
     */
    public void setFile(File f) {
        file = f;
    }
    /**
     * @return The file to store this project in.
     */
    public File getFile() {
        return file;
    }
}
