/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *-----------------------------------------------------------------------------
 *
 * This class Copyright 1999 Plugged In Communications Pty Ltd
 * All rights reserved. Released under Enhydra Licence 10 August 1999
 *
 *-----------------------------------------------------------------------------
 *
 * Author      : Chris Ryan (cryan@pisoftware.com)
 *
 * Description : This class provides various static convenience methods
 *               for often used processes.
 *
 */

package com.lutris.dods.util;

import java.io.File;
import java.text.*;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import javax.swing.UIManager;


public final class Util {


  /**
   * Constructor.  Don't want anyone to invoke one of these.
   */
  protected Util() {
  }


  /**
   * This method replaces all instances of one string to
   * another string within the main string.  For example,
   * if the main string is "Foo bar baz", and the string
   * to replace is "ba" with "cu", then the returned
   * result will be "Foo cur cuz".
   *
   * @param main_value  The value to do the replacing on
   * @param search_value  The value to search for
   * @param sub_value  The value to replace any found values
   * @return see comment.
   */
  public static String replace(String main_value, String search_value, String sub_value) {

    int pos = 0;
    int old_pos = 0;
    StringBuffer result = new StringBuffer();

    while ((old_pos < main_value.length()) &&
           ((pos = main_value.indexOf(search_value, old_pos)) != -1)) {

      // add everything from the old position to here
      if (pos != old_pos)
        result.append(main_value.substring(old_pos, pos));

      // add the replacement value
      result.append(sub_value);

      // move the old position along
      old_pos = pos + search_value.length();

    }

    // did we get to the end of the string?
    if (old_pos <= main_value.length()) {

      // yes, add the remainder to the result
      result.append(main_value.substring(old_pos));

    }

    return result.toString();
  }


  /**
   * This method attempts to convert the string passed into
   * a date.
   *
   * @param datestr  The string to convert
   * @return The converted string, or null if any errors occur
   */
  public static java.util.Date stringToDate(String datestr) {

    // basic checks...
    if (datestr == null || datestr.length() == 0) {    
      return null;
    }

    // These are the date formats we can accept
    String[] formats = {
      "yyyy/MM/dd H:mm:ss a",    "yyyy/MM/dd K:mm:ss",
      "d/M/yyyy H:mm:ss a",    "d/M/yyyy K:mm:ss",
      "MM/dd/yyyy H:mm:ss a",    "MM/dd/yyyy K:mm:ss",
      "yyyy-MM-dd H:mm:ss a",    "yyyy MM dd H:mm:ss a",
      "yyyy-MM-dd H:mm:ss",    "yyyy MM dd H:mm:ss",
      "yyyy-MM-dd H:mm a",    "yyyy MM dd H:mm a",
      "yyyy-MM-dd H:mm",    "yyyy MM dd H:mm",
      "yyyy/MM/dd",      "dd/MM/yyyy",
      "yyyy-MM-dd",      "yyyy MM dd",
      "MMM dd yyyy h:mm:ss a",  "MMM dd, yyyy h:mm:ss a",
      "MMM dd yyyy h:mm:ss",    "MMM dd, yyyy h:mm:ss",
      "MMM dd yyyy h:mm a",    "MMM dd, yyyy h:mm a",
      "MMM dd yyyy h:mm",    "MMM dd, yyyy h:mm" };

    java.util.Date result = null;

    // try each format...
    for ( int i = 0; i < formats.length; i++ ) {

      // create the date formatter with this format string
      SimpleDateFormat df = new SimpleDateFormat( formats[ i ] );

      try {

        // parse the string with this date formatter
        result = df.parse( datestr );

        // jump out if we get here, since we've got a valid result
        break;
      } catch ( ParseException e ) {

        // ignore this - if the string can't be parsed we'll get this exception
      }

    }

    // return the parsed date... hopefully
    return result;

  }

  public static String getExtension(File file) {
    String ext  = null;
    String name = null;
    int dotAt = 0;
    if (file != null) {
      name = file.getName();
      dotAt = name.lastIndexOf('.');
      if (dotAt > 0) {
        ext = name.substring(dotAt + 1).toLowerCase().trim();
      }
    }
    return ext;
  }

  public static String getBase(File file) {
    String base  = null;
    String name = null;
    int dotAt = 0;
    if (file != null) {
      name = file.getName();
      dotAt = name.lastIndexOf('.');
      if (dotAt > 0) {
        base = name.substring(0, dotAt).trim();
      }
    }
    return base;
  }

  public static Point getCenteringPoint(Dimension size) {
    Point point = new Point();
    Dimension screen;
    screen = Toolkit.getDefaultToolkit().getScreenSize();
    if (size.height > screen.height) {
      size.height = screen.height;
    }
    if (size.width > screen.width) {
      size.width = screen.width;
    }
    point.x = (screen.width  - size.width)  / 2;
    point.y = (screen.height - size.height) / 2;
    return point;
  }

  public static void setLookForWindowsOnly() {
    try {
      String systemLook = UIManager.getSystemLookAndFeelClassName();
      if (systemLook.trim().endsWith("WindowsLookAndFeel")) {
        UIManager.setLookAndFeel(systemLook);
      }
    }
    catch (javax.swing.UnsupportedLookAndFeelException e) {}
    catch (IllegalAccessException e) {}
    catch (InstantiationException e) {}
    catch (ClassNotFoundException e) {};
  }

}
