/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Test1.java,v 1.1.2.2 2000/11/30 22:50:20 jay Exp $
 */


package tests.org.enhydra.dods.test1;

import tests.org.enhydra.dods.TestHarness;

import java.io.*;
import java.util.*;
import java.sql.*;
import java.lang.reflect.*;

import com.lutris.util.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;

import com.lutris.dods.builder.generator.dataobject.*;
import com.lutris.dods.builder.generator.query.*;

import tests.org.enhydra.dods.test1.data.*;

public class Test1 extends TestHarness {

    /*
       TestHarness can find these method names via reflection,
       but they must be run in a certain order.
       This list also makes it easier to turn tests on/off.
    */
    public static String[] tests = { 
	"dropAndRecreateTables",
	"deleteAllData",
	"createData",
	"checkData0",
	"checkData1",
	"checkData2",
	"checkData3",
	"checkData4",
	"limitRows",
	"printAllHandles",
	"checkNullParent",
	"checkAddWhereIn",
	"selfRefAndQueryReset",
	"checkOidArray",
	//"checkEmpsForProject0",
	//"checkEmpsForProject1",
	//"checkEmpsForProject2",
	//"checkEmpsForProject3",
	null
    };

    public static void main( String args[] ) {
	run( new Test1(), tests, args );
    }

    public Test1() {
//	super();
//System.err.println("Test1 ctor");
    }

    public static boolean dropAndRecreateTables() throws Exception {
	print( TITLE, 
	    "dropAndRecreateTables: exercise drop and create table commands.");
	runSqlFromFile( "data/drop_tables.sql",   true, true, false );
	runSqlFromFile( "data/create_tables.sql", false, true, true );
	return true;
    }

    public static boolean printAllHandles() throws Exception {
	print( TITLE, 
	    "printAllHandles: Verify all classes and tables are present.");
	if ( ! opDOArray( ManagerQuery.class,		PRINT_OID ) )
	    return false;
	if ( ! opDOArray( EmployeeQuery.class,		PRINT_OID ) )
	    return false;
	if ( ! opDOArray( ProjectQuery.class,		PRINT_OID ) )
	    return false;
	if ( ! opDOArray( Project_EmployeeQuery.class,	PRINT_OID ) )
	    return false;
	
	return true;
    }

    public static boolean checkOidArray() throws Exception {
	print( TITLE, 
	    "checkOidArray: get an array of oids and retrieve DOs from them.");
	Vector v = new Vector();
	v.addElement( EmployeeDO.PrimaryKey );
	QueryBuilder qb = new QueryBuilder( v );
	RDBRow[] oids = qb.getRows();
	EmployeeQuery eq = new  EmployeeQuery();
	int x = 0, y = 0;
	String s = oids[0].get( EmployeeDO.PrimaryKey ).getBigDecimal().toString();
System.err.println("s ="+s );
	eq.setQueryHandle( s );
	EmployeeDO e = eq.getNextDO();
	if ( null == e ) 
	    return false;
System.err.println("e.getHandle()="+e.getHandle());
	return true;
    }

    public static boolean deleteAllData() throws Exception {
	print( TITLE, 
	    "deleteAllData: delete all rows of test data.");
	if ( ! opDOArray( EmployeeQuery.class,		DELETE_DO ) )
	    return false;
	if ( ! opDOArray( ManagerQuery.class,		DELETE_DO ) )
	    return false;
	if ( ! opDOArray( ProjectQuery.class,		DELETE_DO ) )
	    return false;
	if ( ! opDOArray( Project_EmployeeQuery.class,	DELETE_DO ) )
	    return false;
	return true;
    }

    public static int numEmps = 10;
    public static EmployeeDO[] emps = new EmployeeDO[ numEmps ];
    public static Hashtable empHandlesByName = new Hashtable();

    public static int numMgrs = 4;
    public static ManagerDO[] mgrs = new ManagerDO[ numMgrs ];
    public static Hashtable mgrHandlesByTitle = new Hashtable();
    public static Hashtable mgrHandleByEmpHandle = new Hashtable();

    public static int numProjs = 4;
    public static ProjectDO[] projs = new ProjectDO[ numProjs ];
    public static int[] artMgrIndexForProject = { 0, 0, 1, 1 };
    public static int[] engMgrIndexForProject = { 2, 2, 3, 3 };
    public static int[] parentIndexForProject = { -1, 0, 0, 2 };
    public static Hashtable projHandlesByName = new Hashtable();

    public static int numProjEmps = 6;
    public static Project_EmployeeDO[] proj_emps = new Project_EmployeeDO[ numProjEmps ];
    public static int[] empIndexForProjectEmp  = { 4, 5, 6, 7, 8, 9 };
    public static int[] projIndexForProjectEmp = { 0, 0, 1, 1, 2, 2 };
    public static Hashtable proj_empHandlesByEmp = new Hashtable();

    /*
       totalRows is a Hashtable in TestHarness.
       TestHarness.opDOArray uses totalRows to perform query validation.
    */
    static {
	setTotalRows( EmployeeQuery.class,             numEmps );
	setTotalRows( ManagerQuery.class,              numMgrs );
	setTotalRows( ProjectQuery.class,              numProjs );
	setTotalRows( Project_EmployeeQuery.class,     numProjEmps );
    }

    public static boolean createData() throws Exception {
	print( TITLE, "createData: create initial data set");
        for ( int i = 0; i < numEmps; i++ ) {
	    EmployeeDO e = emps[ i ] = EmployeeDO.createVirgin();
	    String name = "e" + i;
	    e.setName( name );
	    e.commit();
	    empHandlesByName.put( name, e.getHandle() );
	    if ( i < numMgrs ) {
		ManagerDO m = mgrs[ i ] = ManagerDO.createVirgin();
		m.setMyEmpInfo( e );
		String title = "m" + i;
		m.setTitle( title );
		m.commit();
		print( DEBUG3, "Committed mgr with handle="+m.getHandle());
		mgrHandlesByTitle.put( title, m.getHandle() );
		mgrHandleByEmpHandle.put( e.getHandle(), m.getHandle() );
	    }
	}
        for ( int i = 0; i < numProjs; i++ ) {
	    ProjectDO p = projs[ i ] = ProjectDO.createVirgin();
	    String name = "p" + i;
	    p.setName( name );
	    p.setArtManager( mgrs[ artMgrIndexForProject[ i ] ] );
	    p.setEngManager( mgrs[ engMgrIndexForProject[ i ] ] );
	    int n = parentIndexForProject[ i ];
	    if ( -1 == n )
		p.setParentProject( null );
	    else
		p.setParentProject( projs[ n ] );
	    p.commit();
	    projHandlesByName.put( name, p.getHandle() );
        }
        for ( int i = 0; i < numProjEmps; i++ ) {
            Project_EmployeeDO x = proj_emps[i] = Project_EmployeeDO.createVirgin();
	    EmployeeDO e = emps[ empIndexForProjectEmp[ i ] ];
	    x.setEmployee( e );
	    x.setProject( projs[ projIndexForProjectEmp[ i ] ] );
	    x.commit();
        }
	return true;
    }

    public static boolean checkData0() throws Exception {
	print( TITLE, "checkData: verify initial data set was written to db");
        for ( int i = 0; i < mgrs.length; i++ ) {
            String h = mgrs[i].getHandle();
	    print( DEBUG3, "Looking for mgr with handle="+mgrs[i].getHandle());
	    ManagerQuery q = new ManagerQuery();
	    q.setQueryHandle( h );
	    QueryBuilder qb = q.getQueryBuilder();
//	    qb.debug();
	    ManagerDO m = q.getNextDO();
	    if ( null == m ) {
		print( PASS_FAIL, 
		    "Manager with handle " + h + " not found." );
		return false;
	    }
        }
	return true;
    }


    private static ProjectDO[] chk_projs = null;

    public static boolean checkData1() throws Exception {
	print( TITLE, "checkData: verify initial data set was written to db");
	ProjectQuery pq = new ProjectQuery();
	chk_projs = pq.getDOArray();
        for ( int i = 0; i < chk_projs.length; i++ ) {
	    String name = chk_projs[ i ].getName();
	    String handle = chk_projs[ i ].getHandle();
	    String phbn = (String) projHandlesByName.get( name );
            if ( ! phbn.equals( handle ) ) {
		print( PASS_FAIL, 
		    "Proj named " + name + " did not have handle " + handle );
		return false;
	    }
        }
	return true;
    }

    private static EmployeeDO[] chk_emps = null;

    public static boolean checkData2() throws Exception {
	EmployeeQuery eq = new EmployeeQuery();
	chk_emps = eq.getDOArray();
	if ( chk_emps.length != numEmps ) {
	    print( PASS_FAIL, 
		"Expected " + numEmps + "employees, but got " + chk_emps.length );
	    return false;
	}
        for ( int i = 0; i < chk_emps.length; i++ ) {
	    EmployeeDO e = chk_emps[ i ];
	    String name = e.getName();
	    String handle = e.getHandle();
            if ( ! empHandlesByName.get( name ).equals( handle ) ) {
		print( PASS_FAIL, 
		    "Emp named " + name + " did not have handle " + handle );
		return false;
	    }
        }
	return true;
    }

    public static boolean checkData3() throws Exception {
        for ( int i = 0; i < chk_emps.length; i++ ) {
	    EmployeeDO e = chk_emps[ i ];

	    ProjectDO[] pv = e.getProjectDOArray_via_Project_Employee();

	    Project_EmployeeQuery peq = new Project_EmployeeQuery();
	    peq.setQueryEmployee( e );
	    Project_EmployeeDO[] pea = peq.getDOArray();
            for ( int j = 0; j < pea.length; j++ ) {
		ProjectDO p = pea[j].getProject();
		String pHandle = p.getHandle();
                for ( int k = 0; k < proj_emps.length; k++ ) {
                    Project_EmployeeDO pe = proj_emps[k];
		    EmployeeDO ee = pe.getEmployee();
		    if ( ee.getHandle().equals( e.getHandle() ) ) {
			String ppHandle = pe.getProject().getHandle();
			if ( ! p.getHandle().equals( ppHandle ) ) {
			    print( PASS_FAIL, 
				"ProjEmp for Emp handle/name " + 
				e.getHandle() + "/" + e.getName() +  
				" got Proj handle " + pHandle +
				" instead of " + ppHandle );
			    return false;
			}
		    }
                }
            }
        }
	return true;
    }

    private static ManagerDO[] chk_mgrs = null;

    public static boolean checkData4() throws Exception {
	ManagerQuery mq = new ManagerQuery();
	chk_mgrs = mq.getDOArray();
	if ( chk_mgrs.length != numMgrs ) {
	    print( PASS_FAIL, 
		"Expected " + numMgrs + "managers, but got " + chk_mgrs.length );
	    return false;
	}
        for ( int i = 0; i < chk_mgrs.length; i++ ) {
	    String title = chk_mgrs[ i ].getTitle();
	    String handle = chk_mgrs[ i ].getHandle();
            if ( ! mgrHandlesByTitle.get( title ).equals( handle ) ) {
		print( PASS_FAIL, 
		    "Mgr titled " + title + " did not have handle " + handle );
		return false;
	    }
	    String empHandle = chk_mgrs[ i ].getMyEmpInfo().getHandle();
	    if ( ! mgrHandleByEmpHandle.get( empHandle ).equals( handle ) ) {
		print( PASS_FAIL, 
		    "Mgr with emp handle " + empHandle + 
		    " did not have handle " + handle );
		return false;
	    }
        }

	return true;
    }

    public static boolean limitRows() throws Exception {
	EmployeeQuery eq = new EmployeeQuery();
	int max = 3;
	eq.setMaxRows( max );
	EmployeeDO[] arr = eq.getDOArray();
	if ( max != arr.length ) {
	    print( PASS_FAIL, 
		"Expected " + max + " employees, but got " + arr.length );
	    return false;
	}
	eq.reset();
	arr = eq.getDOArray();
	if ( numEmps != arr.length ) {
	    print( PASS_FAIL, 
		"Expected " + numEmps + " employees, but got " + arr.length );
	    return false;
	}
	return true;
    }

    public static boolean checkAddWhereIn() throws Exception {
	print( TITLE, "checkAddWhereIn: check behavior of addWhereIn");
	EmployeeQuery eq = new EmployeeQuery();
	QueryBuilder qb = eq.getQueryBuilder();
	String[] names = { "e1", "e2" };
	qb.addWhereIn( EmployeeDO.Name, names );
	EmployeeDO[] a = eq.getDOArray();
	if ( 2 != a.length ) {
	    print( PASS_FAIL, "addWhereIn #1 returned " + a.length +
			    "rows, expecting " + names.length );
	    return false;
	}
	eq.reset();
	qb = eq.getQueryBuilder();
	qb.addWhereIn( EmployeeDO.PrimaryKey, a );
	EmployeeDO[] b = eq.getDOArray();
	if ( 2 != b.length ) {
	    print( PASS_FAIL, "addWhereIn #2 returned " + b.length +
			    "rows, expecting " + a.length );
	    return false;
	}
	return true;
    }

    public static boolean checkNullParent() throws Exception {
	print( TITLE, "checkNullParent: check behavior of null reference");
	ProjectQuery q = new ProjectQuery();
	q.setQueryHandle( projs[ 0 ].getHandle() );
	ProjectDO child = q.getNextDO();
	if ( null == child ) {
	    print( PASS_FAIL, "Child project not found." );
	    return false;
	}
	ProjectDO parent = child.getParentProject();
	if ( null != parent ) {
	    print( PASS_FAIL, "Parent project should be null." );
	    print( DEBUG2, "Parent=" + parent );
	    return false;
	}
	return true;
    }

    public static boolean selfRefAndQueryReset() throws Exception {
	print( TITLE, "selfRefAndQueryReset: check self reference");
	ProjectQuery q = new ProjectQuery();
	q.setQueryHandle( projs[ 3 ].getHandle() );
	ProjectDO child = q.getNextDO();
	String childHandle = child.getHandle();
	print( DEBUG3, "Child project handle " + childHandle );
	q.reset();
	print( DEBUG3, "Child project parent handle " + 
			child.getParentProject().getHandle() );
	q.setQueryHandle( child.getParentProject().getHandle() );
	ProjectDO parent = q.getNextDO();
	if ( null == parent ) {
	    print( PASS_FAIL,
		"Child project " + childHandle + " has null parent." );
	    print( DEBUG2, "Child project " + child );
	    return false;
	}
	String parentHandle = parent.getHandle();
	print( DEBUG2, "parent = " + parent );
	ProjectDO expectedParent = projs[ 2 ];
	String expectedHandle = expectedParent.getHandle();
	print( DEBUG3, "Expected parent project handle " + expectedHandle );
	if ( ! parentHandle.equals( expectedHandle ) ) {
	    print( PASS_FAIL,
		"Child project " + childHandle +
		" has incorrect parent " + parent.getHandle() +
		", expected parent to be " + expectedHandle );
	    print( DEBUG2, "Child project " + child );
	    return false;
	}
	return true;
    }




//-----------------------------------------------------------------------------

    public static int[][] empsPerProj = {
	{ },		// Project 0
	{ 5, 6 },	// Project 1
	{ 7, 8 },	// Project 2
	{ 9, 10 }	// Project 3
    };

    public static boolean checkEmpsForProject0() throws Exception {
	return checkEmpsForProject( 0 );
    }

    public static boolean checkEmpsForProject1() throws Exception {
	return checkEmpsForProject( 1 );
    }

    public static boolean checkEmpsForProject2() throws Exception {
	return checkEmpsForProject( 2 );
    }

    public static boolean checkEmpsForProject3() throws Exception {
	return checkEmpsForProject( 3 );
    }

    public static boolean checkEmpsForProject( int oid ) throws Exception {
	//print( ERROR,  "checkProjectsForEmp" + oid);
	Vector fields = new Vector();
	fields.addElement( Project_EmployeeDO.Employee );
	QueryBuilder qb = new QueryBuilder( fields );
	qb.addWhere( Project_EmployeeDO.Project, oid );
	/*
	Project_EmployeeQuery q = new Project_EmployeeQuery();
	QueryBuilder qb = q.getQueryBuilder();
	qb.addWhere( Project_EmployeeDO.Project, oid );
	Method g = q.getClass().getDeclaredMethod( "getEmployee", params );
	return checkDOArray( q, empsPerProj[oid], g );
	*/
	return checkResults( qb, Project_EmployeeDO.Employee, empsPerProj[oid] );
    }
}
