/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Test2.java,v 1.1.2.4 2000/11/29 23:59:05 jay Exp $
 */


package tests.org.enhydra.dods.test2;

import tests.org.enhydra.dods.TestHarness;

import java.io.*;
import java.util.*;
import java.sql.*;
import java.lang.reflect.*;

import com.lutris.util.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;

import com.lutris.dods.builder.generator.dataobject.*;
import com.lutris.dods.builder.generator.query.*;

import tests.org.enhydra.dods.test2.data.*;

public class Test2 extends TestHarness {

    /*
       TestHarness can find these method names via reflection,
       but they must be run in a certain order.
       This list also makes it easier to turn tests on/off.
    */
    public static String[] tests = { 
	"dropAndRecreateTables",
	"deleteAllData",
	"createData",
	"checkDataStandard",
	"checkQueryByLong",
	"checkDataTime",
	"checkDates",
	"checkQueryWild_CaseSensitive",
	"checkQueryWild_CaseInsensitive",
	"checkQueryNull",
	"checkQueryBoolean",
	"printAllHandles",
	"checkDualRefCascadeDelete",
	"checkRedundantDelete",
	null
    };

    public static void main( String args[] ) {
	run( new Test2(), tests, args );
    }

    private Test2() {
    }

    public static boolean dropAndRecreateTables() throws Exception {
	print( TITLE, 
	    "dropAndRecreateTables: exercise drop and create table commands.");
	runSqlFromFile( "data/drop_tables.sql",   true, true, false );
	runSqlFromFile( "data/create_tables.sql", false, true, true );
	return true;
    }

    public static boolean printAllHandles() throws Exception {
	print( TITLE, 
	    "printAllHandles: Verify all classes and tables are present.");
	if ( ! opDOArray( XQuery.class,			PRINT_OID ) )
	    return false;
	/*
	if ( ! opDOArray( MultipleRefs2Query.class,	PRINT_OID ) )
	    return false;
	if ( ! opDOArray( MultipleRefs3Query.class,	PRINT_OID ) )
	    return false;
	    */
	return true;
    }

    public static boolean deleteAllData() throws Exception {
	print( TITLE, 
	    "deleteAllData: delete all rows of test data.");
	if ( ! opDOArray( XQuery.class,			DELETE_DO ) )
	    return false;
	if ( ! opDOArray( MultipleRefs2Query.class,	DELETE_DO ) )
	    return false;
	if ( ! opDOArray( MultipleRefs3Query.class,	DELETE_DO ) )
	    return false;
	return true;
    }

    public static int numX = 2;
    public static XDO[] xs = new XDO[ numX ];

    public static int numM2 = 1;
    public static MultipleRefs2DO[] m2s = new MultipleRefs2DO[ numM2 ];

    public static int numM3 = 1;
    public static MultipleRefs3DO[] m3s = new MultipleRefs3DO[ numM3 ];

    /*
       totalRows is a Hashtable in TestHarness.
       TestHarness.opDOArray uses totalRows to perform query validation.
    */
    static {
	setTotalRows( XQuery.class,             numX );
	setTotalRows( MultipleRefs3Query.class,	numM2 );
	setTotalRows( MultipleRefs2Query.class, numM3 );
    }


private static java.sql.Date      da0 = java.sql.Date.valueOf( "2000-12-24" );
private static long		  ms0 = da0.getTime();
private static java.sql.Timestamp ts0 = new java.sql.Timestamp( ms0 );
private static long		  ss0 = 86400000;
private static java.sql.Time      ti0 = new java.sql.Time(      ss0 );

private static java.sql.Date      da1 = java.sql.Date.valueOf( "2000-12-25" );
private static long		  ms1 = da1.getTime();
private static java.sql.Timestamp ts1 = new java.sql.Timestamp( ms1 );
private static long		  ss1 = 86400001;
private static java.sql.Time      ti1 = new java.sql.Time(      ss1 );

private static java.sql.Date      da2 = java.sql.Date.valueOf( "2000-12-26" );
private static long		  ms2 = da2.getTime();
private static java.sql.Timestamp ts2 = new java.sql.Timestamp( ms2 );
private static long		  ss2 = 86400002;
private static java.sql.Time      ti2 = new java.sql.Time(      ss2 );

    private static String makeString( int i ) {
	return "a" + i + "b";
    }

    public static boolean createData() throws Exception {
	print( TITLE, "createData: create initial data set");
        for ( int i = 0; i < numX; i++ ) {
	    XDO x = xs[i] = XDO.createVirgin();
	    String s = makeString( i );
	    x.setBIT_boolean ( 			i == 0 );

	    x.setCHAR_String ( 			s );
	    x.setVARCHAR_String ( 		s );

	    x.setINTEGER_int ( 			i );
	    x.setTINYINT_byte ( 	(byte)  i );
	    x.setSMALLINT_short ( 	(short)	i );
	    x.setBIGINT_long ( 			i );
	    x.setREAL_float ( 			i );
	    x.setFLOAT_double ( 		i );
	    x.setDOUBLE_double ( 		i );

	    x.setDECIMAL_BigDecimal ( 		new java.math.BigDecimal( i ) );
	    x.setNUMERIC_BigDecimal ( 		new java.math.BigDecimal( i ) );

	    x.setDATE_Date ( 			da1 );
	    x.setTIMESTAMP_Timestamp ( 		ts1 );
	    x.setTIME_Date ( 			ti1 );
	    print( DEBUG3, "ti1.getTime()="+ti1.getTime());
	    print( DEBUG3, "ss1="+ss1);
	    x.commit();

        }
	return true;
    }

    private static XDO[] chk_x = null;

    public static boolean checkDataStandard() throws Exception {
	print( TITLE, "checkDataStandard: verify initial data set was written to db");
	XQuery xq = new XQuery();
	chk_x = xq.getDOArray();
        for ( int i = 0; i < chk_x.length; i++ ) {
//System.err.println("TTTTTTTTTTTTTTTTTT");
//System.err.println("chk_x["+i+"].getHandle()="+chk_x[i].getHandle());
//System.err.println("i ="+i );
	    XDO x = chk_x[i];
	    String s = makeString( i );
	    String getter = "???";
	    String failedGetters = "";
            try {
		getter = "getCONSTANT_int";
	        if ( 123 != x.getCONSTANT_int() )
		    failedGetters += getter + " ";

		getter = "getBIT_boolean";
	        if ( ( 0 == i ) != ( x.getBIT_boolean() ) )
		    failedGetters += getter + " ";

		getter = "getCHAR_String";
	        if ( 0 != x.getCHAR_String().indexOf( s ) ) 
		    failedGetters += getter + " ";
		getter = "getVARCHAR_String";
	        if ( ! s.equals( x.getVARCHAR_String() ) ) 
		    failedGetters += getter + " ";

		getter = "getINTEGER_int";
	        if ( i != x.getINTEGER_int() ) 
		    failedGetters += getter + " ";
		getter = "getTINYINT_byte";
	        if ( i != x.getTINYINT_byte() ) 
		    failedGetters += getter + " ";
		getter = "getSMALLINT_short";
	        if ( i != x.getSMALLINT_short() ) 
		    failedGetters += getter + " ";
		getter = "getBIGINT_long";
	        if ( i != x.getBIGINT_long() ) 
		    failedGetters += getter + " ";
		getter = "getREAL_float";
	        if ( i != x.getREAL_float() ) 
		    failedGetters += getter + " ";
		getter = "getFLOAT_double";
	        if ( i != x.getFLOAT_double() ) 
		    failedGetters += getter + " ";
		getter = "getDOUBLE_double";
	        if ( i != x.getDOUBLE_double() ) 
		    failedGetters += getter + " ";

		getter = "getDECIMAL_BigDecimal";
	        if ( i != x.getDECIMAL_BigDecimal().intValue() ) 
		    failedGetters += getter + " ";
		getter = "getNUMERIC_BigDecimal";
	        if ( i != x.getNUMERIC_BigDecimal().intValue() ) 
		    failedGetters += getter + " ";

		getter = "getDATE_Date";
		long l = x.getDATE_Date().getTime();
	        if ( ms1 != l ) {
		    print( REPORT2, test + 
			": getDATE_Date returned " + l +
			" instead of " + ms1 + "." );
		    failedGetters += getter + " ";
		}

		getter = "getTIMESTAMP_Timestamp";
		l = x.getTIMESTAMP_Timestamp().getTime();
	        if ( ms1 != l ) {
		    print( REPORT2, test + 
			": getTIMESTAMP_Timestamp returned " + l +
			" instead of " + ms1 + "." );
		    failedGetters += getter + " ";
		}

		getter = "DONE";
            } catch ( Exception e ) { 
		print( ERROR, test + 
			": got exception after calling get method: " + getter +
			": Exception=" + e.getMessage() );
		return false;
            }
	    if ( 0 != failedGetters.length() ) {
		print( REPORT1, test + 
			": get methods failed: " + failedGetters + "." );
		return false;
	    }
        }
	return true;
    }

    public static boolean checkQueryByLong() throws Exception {
	print( TITLE, "checkQueryByLong: verify querying by long uses long");
	XQuery xq = new XQuery();
	xq.setQueryBIGINT_long( 961104760933L );
//	xq.getQueryBuilder().debug();
	XDO[] a = xq.getDOArray();
	return true;
    }

    public static boolean checkDataTime() throws Exception {
	print( TITLE, "checkDataTime: verify initial time data was written to db");
	XQuery xq = new XQuery();
	chk_x = xq.getDOArray();
        for ( int i = 0; i < chk_x.length; i++ ) {
//System.err.println("TTTTTTTTTTTTTTTTTT");
//System.err.println("chk_x["+i+"].getHandle()="+chk_x[i].getHandle());
//System.err.println("i ="+i );
	    XDO x = chk_x[i];
	    String s = makeString( i );
	    String getter = "???";
	    String failedGetters = "";
            try {
		getter = "getTIME_Date";
		long l = x.getTIME_Date().getTime();
	        if ( ss1 != l ) {
		    print( REPORT2, test + 
			": getTIME_Date returned " + l +
			" instead of " + ss1 + "." );
		    failedGetters += getter + " ";
		}

		getter = "DONE";
            } catch ( Exception e ) { 
		print( ERROR, test + 
			": got exception after calling get method: " + getter +
			": Exception=" + e.getMessage() );
		return false;
            }
	    if ( 0 != failedGetters.length() ) {
		print( REPORT1, test + 
			": get methods failed: " + failedGetters + "." );
		return false;
	    }
        }
	return true;
    }

    public static boolean checkDates() throws Exception {
	print( TITLE, "checkDates: verify dates work properly");
	String task = null;
        try {
	    task = "query by date 1";
	    XQuery q = new XQuery();
	    q.setQueryDATE_Date( da1 );
	    XDO[] a = q.getDOArray();
	    if ( numX != a.length ) {
		print( PASS_FAIL, test + ": date EQUAL query failed" );
		return false;
	    }

	    task = "query by date 2";
	    q.reset();
	    QueryBuilder qb = q.getQueryBuilder();
	    qb.addWhere( XDO.DATE_Date, da0, QueryBuilder.GREATER_THAN );
	    a = q.getDOArray();
	    if ( numX != a.length ) {
		print( PASS_FAIL, test + ": date GREATER_THAN query failed" );
		return false;
	    }
        } catch ( Exception e ) { 
	    print( ERROR, test + ": failed trying to " + task + 
			": Exception=" + e.getMessage() );
	    return false;
        }
	return true;
    }

    public static boolean checkQueryWild_CaseSensitive() throws Exception {
	print( TITLE, "checkQueryWild_CaseSensitive: test column LIKE '%x%'");
	String task = null;
        try {
	    task = "test CASE_SENSITIVE_CONTAINS";
	    XQuery q = new XQuery();
//	    QueryBuilder qb = q.getQueryBuilder();
//	    qb.debug();
//	    qb.addWhere( XDO.VARCHAR_String, "a", 
	    q.setQueryVARCHAR_String( "a", 
				QueryBuilder.CASE_SENSITIVE_CONTAINS );
	    XDO[] a = q.getDOArray();
//System.err.println("a.length="+a.length);
	    if ( chk_x.length != a.length ) {
		print( PASS_FAIL, test + ": CASE_SENSITIVE_CONTAINS failed" );
		return false;
	    }
	    task = "test CASE_SENSITIVE_ENDS_WITH";
	    q = new XQuery();
//	    QueryBuilder qb = q.getQueryBuilder();
//	    qb.debug();
	    q.setQueryVARCHAR_String( "b", 
				QueryBuilder.CASE_SENSITIVE_ENDS_WITH );
	    a = q.getDOArray();
//System.err.println("a.length="+a.length);
	    if ( chk_x.length != a.length ) {
		print( PASS_FAIL, test + 
				": CASE_SENSITIVE_ENDS_WITH failed" );
		return false;
	    }
        } catch ( Exception e ) { 
	    print( ERROR, test + ": failed trying to " + task + 
			": Exception=" + e.getMessage() );
	    return false;
        }
	return true;
    }

    public static boolean checkQueryWild_CaseInsensitive() throws Exception {
	print( TITLE, "checkQueryWild_CaseInsensitive: test LOWER(column) LIKE '%x%'");
	String task = null;
        try {
	    task = "test CASE_INSENSITIVE_ENDS_WITH";
	    XQuery q = new XQuery();
	    QueryBuilder qb = q.getQueryBuilder();
//	    qb.debug();
	    qb.addWhere( XDO.VARCHAR_String, "B", 
				QueryBuilder.CASE_INSENSITIVE_ENDS_WITH );
	    XDO[] a = q.getDOArray();
//System.err.println("a.length="+a.length);
	    if ( chk_x.length != a.length ) {
		print( PASS_FAIL, test + 
				": CASE_INSENSITIVE_ENDS_WITH failed" );
		return false;
	    }

	    task = "test deprecated 'exact' method";
	    q.reset();
	    q.setQueryVARCHAR_String( "B", false );
//	    q.getQueryBuilder().debug();
	    a = q.getDOArray();
//System.err.println("a.length="+a.length);
	    if ( chk_x.length != a.length ) {
		print( PASS_FAIL, test + 
				": inexact failed" );
		return false;
	    }
        } catch ( Exception e ) { 
	    print( ERROR, test + ": failed trying to " + task + 
			": Exception=" + e.getMessage() );
	    return false;
        }
	return true;
    }

    public static boolean checkQueryNull() throws Exception {
	print( TITLE, "checkQueryNull: verify can query for nulls");
	String task = null;
        try {
	    task = "query by date 1";
	    XQuery q = new XQuery();
	    q.setQueryDATE_Date( da1 );
	    XDO[] a = q.getDOArray();
	    if ( numX != a.length ) {
		print( PASS_FAIL, test + ": date EQUAL query failed" );
		return false;
	    }

	    task = "set date to null";
	    a[0].setDATE_Date( null );
	    a[0].commit();

	    task = "query by date 2";
	    q.reset();
	    QueryBuilder qb = q.getQueryBuilder();
//	    qb.debug();
	    q.setQueryDATE_Date( null );
	    a = q.getDOArray();
	    if ( ( numX - 1 ) != a.length ) {
		print( PASS_FAIL, test + ": date NULL query failed" );
		return false;
	    }
        } catch ( Exception e ) { 
	    print( ERROR, test + ": failed trying to " + task + 
			": Exception=" + e.getMessage() );
	    return false;
        }
	return true;
    }

    public static boolean checkQueryBoolean() throws Exception {
	print( TITLE, "checkQueryBoolean: verify can query for booleans");
	String task = null;
        try {
	    task = "query by boolean false";
	    XQuery q = new XQuery();
	    q.setQueryBIT_boolean( false );
	    XDO[] a = q.getDOArray();
//System.err.println("a.length ="+a.length );
//System.err.println("numX ="+numX );
	    if ( 1 != a.length ) {
		print( PASS_FAIL, test + ": boolean EQUAL false query failed" );
		return false;
	    }

	    task = "query by boolean false";
	    q.reset();
	    q.setQueryBIT_boolean( true );
	    a = q.getDOArray();
//System.err.println("a.length ="+a.length );
//System.err.println("numX ="+numX );
	    if ( 1 != a.length ) {
		print( PASS_FAIL, test + ": boolean EQUAL true query failed" );
		return false;
	    }

        } catch ( Exception e ) { 
	    print( ERROR, test + ": failed trying to " + task + 
			": Exception=" + e.getMessage() );
	    return false;
        }
	return true;
    }

    public static boolean checkDualRefCascadeDelete() throws Exception {
	print( TITLE, 
	"checkDualRefCascadeDelete: check cascading delete in dual ref case");
	m2s[0] = MultipleRefs2DO.createVirgin();
	m2s[0].setRef1( xs[0] );
	m2s[0].setRef2( xs[0] );
	xs[0].delete();
	return true;
    }

    public static boolean checkRedundantDelete() throws Exception {
	print( TITLE, 
	"checkRedundantDelete: check 2nd delete of same row in trans is ignored.");
	DBTransaction dbt = Enhydra.getDatabaseManager().createTransaction(); 
	xs[1].delete( dbt );
	// StandardDBTransaction.delete should detect this 2nd delete of xs[1]
	// and simply not include the redundant delete in the transaction.
	xs[1].delete( dbt );	
	dbt.commit();
	// make sure the delete actually happened.
	XQuery xq = new XQuery();
	xq.setQueryOId( xs[1].getOId() );
	if ( null != xq.getNextDO() ) {
	    print( PASS_FAIL, test + ": twice deleted object still in database" );
	    return false;
	}
	return true;
    }
}
