/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Test3.java,v 1.1.2.2 2000/11/30 22:50:20 jay Exp $
 */


package tests.org.enhydra.dods.test3;

import tests.org.enhydra.dods.TestHarness;

import java.io.*;
import java.util.*;
import java.sql.*;
import java.lang.reflect.*;

import com.lutris.util.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;

import com.lutris.dods.builder.generator.dataobject.*;
import com.lutris.dods.builder.generator.query.*;

import tests.org.enhydra.dods.test3.data.*;

public class Test3 extends TestHarness {

    /*
       TestHarness can find these method names via reflection,
       but they must be run in a certain order.
       This list also makes it easier to turn tests on/off.
    */
    public static String[] tests = { 
	"dropAndRecreateTables",
	"deleteAllData",
	"createData",
	"checkDataBLOB",
	"printAllHandles",
	null
    };

    public static void main( String args[] ) {
	run( new Test3(), tests, args );
    }

    private Test3() {
    }

    public static boolean dropAndRecreateTables() throws Exception {
	print( TITLE, 
	    "dropAndRecreateTables: exercise drop and create table commands.");
	runSqlFromFile( "data/drop_tables.sql",   true, true, false );
	runSqlFromFile( "data/create_tables.sql", false, true, true );
	return true;
    }

    public static boolean printAllHandles() throws Exception {
	print( TITLE, 
	    "printAllHandles: Verify all classes and tables are present.");
	if ( ! opDOArray( BQuery.class,			PRINT_OID ) )
	    return false;
	return true;
    }

    public static boolean deleteAllData() throws Exception {
	print( TITLE, 
	    "deleteAllData: delete all rows of test data.");
	if ( ! opDOArray( BQuery.class,			DELETE_DO ) )
	    return false;
	return true;
    }

    public static int numB = 2;
    public static BDO[] xs = new BDO[ numB ];

    /*
       totalRows is a Hashtable in TestHarness.
       TestHarness.opDOArray uses totalRows to perform query validation.
    */
    static {
	setTotalRows( BQuery.class,             numB );
    }

    public static boolean createData() throws Exception {
	print( TITLE, "createData: create initial data set");
        for ( int i = 0; i < numB; i++ ) {
	    String task = "init byte array data";
            try {
		byte[] ba = new byte[3];
		for ( int k = 0; k < ba.length; k++ ) {
		    ba[k] = (byte)( 100 * ( i + 1 ) + k );
		}

for ( int j = 0; j < ba.length; j++ ) {
print(DEBUG3, "ba["+j+"]="+ba[j]);
}
		task = "create virgin DO";
		BDO x = xs[i] = BDO.createVirgin();
		String s = "" + i;

		task = "set BINARY byte array";
		x.setBINARY_bytearray ( 		ba );

byte[] BA = x.getBINARY_bytearray();
print(DEBUG3, "BA.length;="+BA.length);
for ( int k = 0; k < 3; k++ ) {
print(DEBUG3, "BA["+k+"]="+BA[k]);
}
print(DEBUG3, "x.getBINARY_bytearray().length="+x.getBINARY_bytearray().length);

/*
		task = "set VARBINARY byte array";
		x.setVARBINARY_bytearray ( 		ba );
		task = "set LONGVARBINARY byte array";
		x.setLONGVARBINARY_bytearray ( 	ba );
		*/

		task = "commit DO";
		x.commit();
            } catch ( Exception e ) { 
		print( PASS_FAIL,
		    "createData: cannot " + task +
		    ": exception=" + e.getMessage() );
		return false;
            }
        }
	return true;
    }

    private static BDO[] chk_x = null;

    public static boolean checkDataBLOB() throws Exception {
	print( TITLE, "checkDataBLOB: verify byte array data set was written to db");
print(DEBUG3, "BBBBBBBBBBBBBBBBBBBBBBB");
	BQuery bq = new BQuery();
	chk_x = bq.getDOArray();
	for ( int i = 0; i < chk_x.length; i++ ) {
print(DEBUG3, "chk_x["+i+"].getHandle()="+chk_x[i].getHandle());
print(DEBUG3, "i ="+i );
	    BDO x = chk_x[i];
	    String getter = "???";
	    String failedGetters = "";
            try {
		getter = "getBINARY_bytearray";
/*
byte[] ba = x.getBINARY_bytearray();
print(DEBUG3, "ba.length;="+ba.length);
for ( int k = 0; k < 3; k++ ) {
print(DEBUG3, "ba["+k+"]="+ba[k]);
}
print(DEBUG3, "x.getBINARY_bytearray().length="+x.getBINARY_bytearray().length);
*/
	        if ( i != x.getBINARY_bytearray()[0] ) 
		    failedGetters += getter + " ";
/*
		getter = "getVARBINARY_bytearray";
print(DEBUG3, "x.getVARBINARY_bytearray().length="+x.getVARBINARY_bytearray().length);
byte[] ba = x.getVARBINARY_bytearray();
print(DEBUG3, "ba.length;="+ba.length);
for ( int k = 0; k < 3; k++ ) {
print(DEBUG3, "ba["+k+"]="+ba[k]);
}
print(DEBUG3, "x.getVARBINARY_bytearray().length="+x.getVARBINARY_bytearray().length);
	        if ( i != x.getVARBINARY_bytearray()[0] ) 
		    failedGetters += getter + " ";
		getter = "getLONGVARBINARY_bytearray";
print(DEBUG3, "x.getLONGVARBINARY_bytearray().length="+x.getLONGVARBINARY_bytearray().length);
	        if ( i != x.getLONGVARBINARY_bytearray()[0] ) 
		    failedGetters += getter + " ";
*/

		getter = "DONE";
            } catch ( Exception e ) { 
		print( ERROR, test + 
			": got exception after calling get method: " + getter +
			": Exception=" + e.getMessage() );
		return false;
            }
	    if ( 0 != failedGetters.length() ) {
		print( DEBUG1, test + 
			": get methods failed: " + failedGetters + "." );
		return false;
	    }
        }
	return true;
    }

}
