/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Test6.java,v 1.1.2.1 2000/11/30 22:50:20 jay Exp $
 */


package tests.org.enhydra.dods.test6;

import tests.org.enhydra.dods.TestHarness;

import java.io.*;
import java.util.*;
import java.sql.*;
import java.lang.reflect.*;

import com.lutris.util.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;

import com.lutris.dods.builder.generator.dataobject.*;
import com.lutris.dods.builder.generator.query.*;

import tests.org.enhydra.dods.test6.data.*;

public class Test6 extends TestHarness {

    /*
       TestHarness can find these method names via reflection,
       but they must be run in a certain order.
       This list also makes it easier to turn tests on/off.
    */
    public static String[] tests = { 
	"dropAndRecreateTables",
	"deleteAllData",
	"createData",
	"checkLazyLoading",
	null
    };

    public static void main( String args[] ) {
	run( new Test6(), tests, args );
    }

    private Test6() {
    }

    public static boolean dropAndRecreateTables() throws Exception {
	print( TITLE, 
	    "dropAndRecreateTables: exercise drop and create table commands.");
	runSqlFromFile( "data/drop_tables.sql",   true, true, false );
	runSqlFromFile( "data/create_tables.sql", false, true, true );
	return true;
    }

    public static boolean deleteAllData() throws Exception {
	print( TITLE, 
	    "deleteAllData: delete all rows of test data.");
	if ( ! opDOArray( X6Query.class,			DELETE_DO ) )
	    return false;
	return true;
    }

    public static int numX = 2;
    public static X6DO[] xs = new X6DO[ numX ];

    /*
       totalRows is a Hashtable in TestHarness.
       TestHarness.opDOArray uses totalRows to perform query validation.
    */
    static {
	setTotalRows( X6Query.class,             numX );
    }

    private static String makeString( int i ) {
	return "a" + i + "b";
    }

    public static boolean createData() throws Exception {
	print( TITLE, "createData: create initial data set");
        for ( int i = 0; i < numX; i++ ) {
	    X6DO x = xs[i] = X6DO.createVirgin();
	    x.setINTEGER_int ( 100 + i );
	    x.commit();
        }
	return true;
    }

    private static X6DO[] chk_x = null;

    public static boolean checkLazyLoading() throws Exception {
	print( TITLE, "checkLazyLoading: verify lazy loading works");
	ObjectId oid = xs[0].getOId();
	X6DO x = X6DO.createExisting( oid.toString() );
	// LazyLoading is turned on for X6DO,
	// so createExisting should not call loadData,
	// and isLoaded should return false.
	// If isLoaded returns true, we have a problem.
	if ( x.isLoaded() ) {
	    print( ERROR, test + " lazy loading not working" );
	    return false;
	}
	return true;
    }
}
