/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LBSConnection.java,v 1.9.14.1 2000/10/19 17:59:06 jasona Exp $
 */




package com.lutris.applet;

import java.applet.*;
import java.net.*;
import java.io.*;
//import java.util.*;

/**
 *   This file is the counterpart to 
 *   com.lutris.appserver.server.httpPresentation.AppletUtils. It is meant
 *   to be added to the applet's jar file or classes directory. The methods
 *   are split into two files so that the applet does not need to include
 *   any com.lutris.appserver.server.* classes. <P>
 *
 *   A dynamicly generated page contained the <APPLET...> tag that lead to
 *   the applet calling this function. That <APPLET...> tag was created by 
 *   com.lutris.appserver.server.httpPresentation.AppletUtils.createAppletTag.
 *   When the applet wants to establish a connection back to the 
 *   Multiserver, simply call the contactServer() and the extra 
 *   initialization parameters added by createAppletTag() will be used to
 *   contact the server. Specifically, the request will match the user's
 *   current session.
 *
 * @see com.lutris.appserver.server.httpPresentation.AppletUtils
 */
public class LBSConnection {

    public static final String nameParamName = "cookieName";
    public static final String valueParamName = "cookieValue";
    public static final String targetParamName = "targetURL";

    /**
     *  Do not create instances; use the static method.
     */
    private LBSConnection() {
    }

    /**
     *  Use the extra initialization parameters added to the APPLET tag
     *  by AppletUtils.createAppletTag() to open a connection back to
     *  the Multiserver, using the same session as the one used
     *  to get the HTML page that contained the APPLET tag. <P>
     *
     *  The resulting URLConnection is ready to have 
     *  <CODE>connect()</CODE> called on it. You may initialize more
     *  options if you wish, for example <CODE>setDoOutput()</CODE> or
     *  <CODE>setRequestMethod()</CODE> before calling <CODE>connect()</CODE>.
     *
     *  The resulting URLConnection may also be a HttpURLConnection, but
     *  this is not guarenteed. In the JDK on Windows and Solaris it is,
     *  but when running in Netscape Communicator as an applet, it is not.
     *  So to be safe, we return the more generic class. The main 
     *  functionality missing is the ability to set the request method.
     *  However, if you call <CODE>setDoOutput(true)</CODE> in Communicator,
     *  it will issue a POST request.
     *
     *  @param applet
     *   The applet that is calling this method.
     *  @return 
     *   A connection back to the server, or null if an error occured.
     */
    public static URLConnection contactServer(Applet applet) {
        /*
         *  Get the extra parameters. Cookie is optional.
         */
        String targetURL = applet.getParameter(targetParamName);
System.out.println("target = " + targetURL);
        if (targetURL == null)
            return null;
        String cookieName = applet.getParameter(nameParamName);
System.out.println("name = " + cookieName);
        String cookieValue = applet.getParameter(valueParamName);
System.out.println("value = " + cookieValue);
        /*
         *  Get ready to contact the server.
         */
        URL url = null;
        try {
            url = new URL(targetURL);
        } catch (MalformedURLException e) {
System.out.println("bad url!");
            return null;
        }
        URLConnection conn = null;
        try {
            conn = url.openConnection();
System.out.println("conn = " + conn);
        } catch (IOException e) {
System.out.println("io err");
            return null;
        } catch (ClassCastException e) {
System.out.println("class cast err");
            return null;
        } catch (Throwable e) {
System.out.println("threw " + e);
            return null;
        }


        /*
         * Optionally set a cookie.
         * This is used to simulate the session cookie.,
         */
        if ((cookieName != null) && (cookieValue != null)) 
            conn.setRequestProperty("Cookie", cookieName + "=" + cookieValue);
        /*
         * That's it. Don't initiate the connection, so the caller may set
         * other parameters, for example method or more headers.
         */
        return conn;
    }

}

