/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: FilterableServlet.java,v 1.8.14.1 2000/10/19 17:59:06 jasona Exp $
 */





package com.lutris.appserver.server.httpPresentation.servlet;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.enhydra.servlet.filter.Filter;


/**
 *
 *  See the class AbstractFilterableServlet for a reference implementation 
 *  that demnonstrates the responsabilities of a <CODE>FilterableServlet</CODE>.
 *
 *  This capability is used to implement the debugging servlet.
 *
 * @see com.lutris.appserver.server.httpPresentation.servlet.ServletListener
 * @see com.lutris.appserver.server.httpPresentation.servlet.AbstractServletListener
 */
public interface FilterableServlet {

    /**
     *  Add a filter to the end of the list of filters.
     *
     *  @param filter
     *    The filter to add.
     */
    public void addFilter(Filter filter);

    /**
     *  Remove a filter from the list of current filters. 
     *  Silently fails if the filter is not in the list.
     *
     *  @param filter
     *    The filter to remove.
     */
    public void removeFilter(Filter filter);

    /**
     *  Perform the action <CODE>service()</CODE> would normally perform,
     *  without using the filters. This is needed so that at the end of
     *  the filter chain the glue object can call this method to actually
     *  do the work. If it tried to call the normal service() method that
     *  would result in an infinite loop of filter processing.
     */
    public void serviceDirect(HttpServletRequest request,
                              HttpServletResponse response)
                throws ServletException, IOException;


}

