/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HttpPresentationServletGlue.java,v 1.7.14.1 2000/10/19 17:59:06 jasona Exp $
 */


package com.lutris.appserver.server.httpPresentation.servlet;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;

/**
 *  This is a small helper class, used be HttpPresentationServlet when there
 *  are filters applied to the servlet. At the end of the filter chain lies
 *  one of these objects. When my service method is called, I pass the call
 *  straight through to the serviceDirect() method of my
 *  HttpPresentationServlet. Therefore this class is the glue that connects
 *  the filter chain to the HttpPresentationServlet. Consider this an
 *  internal use only class.
 */
public class HttpPresentationServletGlue extends HttpServlet {

    /**
     *  The real servlet to pass service() calls on to.
     */
    private FilterableServlet filterableServlet;

    /**
     *  Create a new glue servlet. Calling service() on me results in
     *  serviceDirect() being called on filterableServlet. Since this is
     *  an internal-use-only class, you should not normally need to instantiate 
     *  one of these.
     *
     *  @param filterableServlet
     *    The real servlet to use.
     */
    public HttpPresentationServletGlue(FilterableServlet filterableServlet) {
        this.filterableServlet = filterableServlet;
    }



    /**
     *  Calls to this method are passed straight on to the serviceDirect()
     *  method of the FilterableServlet passed into my constructor.
     */
    public void service(HttpServletRequest request,
                        HttpServletResponse response)
                throws ServletException, IOException {
        filterableServlet.serviceDirect(request, response);
    }


    /**
     *  Get the servlet I am gluing to.
     *
     *  @return
     *    The FilterableServlet passed into my constructor.
     */
    public FilterableServlet getServlet() {
        return filterableServlet;
    }

}

