/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ServletHttpPresentationInputStream.java,v 1.12.14.1 2000/10/19 17:59:06 jasona Exp $
 */





package com.lutris.appserver.server.httpPresentation.servlet;

import java.io.IOException;
import javax.servlet.ServletInputStream;
import com.lutris.appserver.server.httpPresentation.*;

/**
 * HTTP request input stream reader for servlets.
 * This implements all methods that are defined in InputStream as calles
 * to the servlet object, since we don't know which ones are implemented by
 * ServletInputStream.
 *
 * @see javax.servlet.ServletInputStream
 */
public class ServletHttpPresentationInputStream
	extends HttpPresentationInputStream {
    private ServletInputStream inputStream;

    /**
     * Construct an object for accessing a servlet input stream.
     */
    protected ServletHttpPresentationInputStream(ServletInputStream inputStream) {
        this.inputStream = inputStream;
    }

    /**
     * Starting at the specified offset, reads into the given array of
     * bytes until all requested bytes have been read or a '\n' is
     * encountered, in which case the '\n' is read into the array as well.
     * @param b the buffer into which the data is read
     * @param off the start offset of the data
     * @param len the maximum number of bytes to read
     * @return the actual number of bytes read, or -1 if the end of the
     *         stream is reached
     * @exception IOException if an I/O error has occurred
     */
    public int readLine(byte[] b, int off, int len)
        throws IOException {
        try {
            return inputStream.readLine(b, off, len);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /*
     * InputStream methods vectored off to the ServletInputStream.
     */
    public int read() throws IOException {
        try {
            return inputStream.read();
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    public int read(byte b[]) throws IOException {
        try {
            return inputStream.read(b);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    public int read(byte b[], int off, int len) throws IOException {
        try {
            return inputStream.read(b, off, len);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    public long skip(long n) throws IOException {
        try {
            return inputStream.skip(n);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    public int available() throws IOException {
        try {
            return inputStream.available();
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    public synchronized void mark(int readlimit) {
        inputStream.mark(readlimit);
    }

    public synchronized void reset() throws IOException {
        try {
            inputStream.reset();
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    public boolean markSupported() {
        return inputStream.markSupported();
    }
}
