/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ServletHttpPresentationOutputStream.java,v 1.13.14.1 2000/10/19 17:59:06 jasona Exp $
 */





package com.lutris.appserver.server.httpPresentation.servlet;

import java.io.IOException;
import javax.servlet.ServletOutputStream;
import com.lutris.appserver.server.httpPresentation.*;

/**
 * HTTP request output stream writter for servlets.
 * This implements all methods that are defined in OutputStream as calles
 * to the servlet object, since we don't know which ones are implemented by
 * ServletOutputStream.
 *
 * @see javax.servlet.ServletOutputStream
 */
public class ServletHttpPresentationOutputStream
	extends HttpPresentationOutputStream {

    private ServletHttpPresentationResponse response;
    private ServletOutputStream outputStream;

    /**
     * Construct an object for accessing a servlet output stream.
     */
    protected ServletHttpPresentationOutputStream(ServletHttpPresentationResponse response,
                                                  ServletOutputStream outputStream) {
        this.response = response;
        this.outputStream = outputStream;
    }

    /**
     * Prints the string provided.
     * @exception IOException if an I/O error has occurred
     */
    public void print(String s) throws IOException {
        try {
            outputStream.print(s);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints the boolean provided.
     * @exception IOException if an I/O error has occurred.
     */
    public void print(boolean b) throws IOException {
        try {
            outputStream.print(b);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /*
     * Prints the character provided.
     * @exception IOException if an I/O error has occurred
     */
    public void print(char c) throws IOException {
        try {
            outputStream.print(c);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints the integer provided.
     * @exception IOException if an I/O error has occurred
     */  
    public void print(int i) throws IOException {
        try {
            outputStream.print(i);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }
 
    /**
     * Prints the long provided.
     * @exception IOException if an I/O error has occurred
     */
    public void print(long l) throws IOException {
        try {
            outputStream.print(l);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints the float provided.
     * @exception IOException if an I/O error has occurred
     */
    public void print(float f) throws IOException {
        try {
            outputStream.print(f);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints the double provided.
     * @exception IOException if an I/O error has occurred
     */
    public void print(double d) throws IOException {
        try {
            outputStream.print(d);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println() throws IOException {
        try {
            outputStream.println();
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints the string provided, followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(String s) throws IOException {
        try {
            outputStream.println(s);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints the boolean provided, followed by a CRLF.
     * @exception IOException if an I/O error has occurred.
     */
    public void println(boolean b) throws IOException {
        try {
            outputStream.println(b);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /*
     * Prints the character provided, followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(char c) throws IOException {
        try {
            outputStream.println(c);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints the integer provided, followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(int i) throws IOException {
        try {
            outputStream.println(i);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**  
     * Prints the long provided, followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */  
    public void println(long l) throws IOException {
        try {
            outputStream.println(l);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints the float provided, followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(float f) throws IOException {
        try {
            outputStream.println(f);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /**
     * Prints the double provided, followed by a CRLF.
     * @exception IOException if an I/O error has occurred
     */
    public void println(double d) throws IOException {
        try {
            outputStream.println(d);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    /*
     * OutputStream methods vectored off to the ServletOutputStream.
     */
    public void write(int b) throws IOException {
        try {
            outputStream.write(b);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    public void write(byte b[]) throws IOException {
        try {
            outputStream.write(b);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    public void write(byte b[], int off, int len) throws IOException {
        try {
            outputStream.write(b, off, len);
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    public void flush() throws IOException {
        try {
            outputStream.flush();
        } catch (IOException except) {
            throw new HttpPresentationIOException(except);
        }
    }

    //FIX:  Need MIME-write functions.
}
