/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: StreamPO.java,v 1.10.10.1 2000/10/19 17:59:06 jasona Exp $
 */




/*
 * Test presentation object to send a stream of JavaScript tags to the
 * browers.  This is used as a test of streaming data.  It write to a single
 * frame.
 *
 * Expected arguments are:
 *    frame=name
 * N.B. jolt was not used since we don't want a dependency in these tests.
 */
package com.acme.gui.presObjs;

import com.lutris.appserver.server.httpPresentation.*;


public class StreamPO implements HttpPresentation {

    private static boolean stop = false;
    
    private String getParam (HttpPresentationComms comms,
                             String name) 
            throws HttpPresentationException {
        String param = comms.request.getParameter(name);
        if (param == null) {
            throw new HttpPresentationException("Required argument \"" +
                                                name + "\" not specified");
        }
        return param;
    }

    private void wasteTime(int factor) {
        String str = "";
        for (int cnt = 0; cnt < factor; cnt++) {
            str = str + "A";
        }
    }
    

    private void updateFrameText(HttpPresentationOutputStream out,
                                 String frame,
                                 String text) 
            throws java.io.IOException {
        out.println("<SCRIPT LANGUAGE=\"Javascript\">");
        if (false) {
        out.println("parent." + frame + ".document.open()");
        out.println("parent." + frame + ".document.write(\"" + text + "\")");
        out.println("parent." + frame + ".document.close()");
        }
        out.println("document.msgBox.msg.value=\"" + text + "\"");
        out.println("</SCRIPT>");
        out.flush();
    }
    
    public void run(HttpPresentationComms comms) throws Exception {
        comms.response.setStatus(HttpPresentationResponse.SC_OK, "Good job");

        HttpPresentationOutputStream out = comms.response.getOutputStream();

        // Handle stop button; set global stop flag.
        String stopRequest = comms.request.getParameter("stopButton");
        if (stopRequest != null) {
            out.println("<CENTER>Stopped</CENTER>\n");
            stop = true;
            return;
        }


        String frame = getParam(comms, "frame");
        int maxUpdate = 1000;

        stop = false;

        out.println("<Center>\n");
        out.println("<H2>Streaming Presentation Object " + frame  + "</H2>\n");
        out.println("<FORM NAME=\"msgBox\">");
        out.println("<INPUT TYPE=\"text\" NAME=\"msg\" VALUE=\"Nothing yet\">");
        out.println("</FORM>");
        out.println("</Center>\n");
        out.flush();

        for (int cnt = 0; (cnt < maxUpdate) && (!stop); cnt++) {
            updateFrameText(out, frame, "Frame " + frame + ", update " + cnt);
            wasteTime(1000);
        }

        System.err.println("Exit PO");
    }
}
