/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Session.java,v 1.16.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.user.*;
import java.net.*;
import javax.servlet.http.*;

/**
 * Defines the interface for the Session object expected by the SessionManager..
 * 
 * @version	$Revision: 1.16.14.1 $
 * @author	Shawn McMurdo
 */
public interface Session {
    /**
     * Obtain the user associated with this session.
     * 
     * @return The user object or <CODE>null</CODE> if the
     * session is not associated with a <CODE>User</CODE> objects.
     */
    public User getUser();
 
    /**
     * Set the user associated with this session.  This will register the
     * user with the <CODE>SessionManager</CODE>. <P>
     * 
     * If it is neccessary to prevent a user from logging on multiple times,
     * this can be accomplished by synchronizing on the
     * <CODE>Sessionmanager</CODE> object and enquiring about the number of
     * users associated with a session.  It is then possible to delete other
     * sessions before adding a new session.
     *
     * @param user
     *   The user object to associate with the session.
     * @exception SessionException
     *   If the user cannot be set.
     * @see
     *   SessionManager#getSessionKeys(User)
     */
    public void setUser(User user) throws SessionException;

    /**
     * Remove the user association with this session.  This will unregister the
     * user with the <CODE>SessionManager</CODE>.
     * @exception SessionException
     *   If the user cannot be cleared.
     */
    public void clearUser() throws SessionException;

    /**
     * Obtain the unique key associated with this session.
     *
     * @return A String containing the key for this session.
     */
    public String getSessionKey();

    /**
     * Obtain the session manager associated with this session.
     *
     * @return The session manager object.
     */
    public SessionManager getSessionManager();

    /**
     * Obtain the application specific data for this session.
     *
     * @return A SessionData object containing application specific
     *   data for this session.
     */
    public SessionData getSessionData();

    /**
     * Returns true if the session is new.  A session is new if it has
     * been created but a client cookie was never used to reference it.
     * In other words, the cookie associated with the session was
     * never submitted by a client.
     *
     * @return true if the session is new.
     */
    public boolean isNew();


    /**
     *
     *
     */

     public HttpSession getHttpSession();

}



