/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PersistentSessionData.java,v 1.3.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session.persistent;

import com.lutris.appserver.server.session.*;
import com.lutris.util.KeywordValueException;
import java.io.Serializable;

/**
 * Session data used to store dynamic application
 * objects.  All objects stored in this object must be
 * serializable.
 *
 * @version	$Revision: 1.3.14.1 $
 * @author	Kyle Clark
 */
public class PersistentSessionData extends PagedSessionData {

    /**
     * This method enforces that all PersistentSessionData be
     * serializable.
     *
     * @param keyword  The keyword of the field.  This can be a simple keyword
     *  or a recursive, dot-seperated keyword path.
     * @param value The value to associate with the keyword.  The value may
     *  not be null and it must be serializable.
     * @exception KeywordValueException If the keyword is not syntactically
     *  legal or if the value is not serializable.
     */
    public synchronized void set(String keyword, Object value)
        throws KeywordValueException {
        if (!(value instanceof Serializable)) {
            throw new KeywordValueException("Value must be serializable.");
        }
        super.set(keyword, value);
    }

}

