/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PersistentSessionUserTable.java,v 1.4.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session.persistent;

import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.session.*;
import com.lutris.appserver.server.user.*;
import java.util.Hashtable;
import java.util.Enumeration;
import com.lutris.logging.*;
import com.lutris.util.*;

/**
 * Table used by StandardSessionManager to cross reference <CODE>User</CODE>
 * objects and sessions.<p>
 *
 * @version	$Revision: 1.4.14.1 $
 * @author      Kyle Clark
 */
public class PersistentSessionUserTable implements StandardSessionUserTable {

    String dbName = null;

    /**
     * @param config
     *   configuration options for this table - currently ignored.
     */
    public PersistentSessionUserTable(Config config) throws ConfigException {
        // This is a bit messy....
        dbName = PersistentSessionHome.getDatabaseName(config);
    }

    /**
     * Add a session key to the user to session xref table.
     *
     * @param session
     *   The session key.  
     * @param user
     *   the user to associated with the session key.
     */
    public void add(String sessionKey, User user) {
        // noop - maintained in DB
    }

    /**
     * Remove a session from the user to session mapping table.
     * If the session is not it the table, it is ignored.
     *
     * @param session The session object, with the user already
     * filled in. 
     */
    public void remove(String sessionKey, User user) {
        // noop - maintained in DB
    }

    /**
     * Remove a session from the user to session mapping table.
     * If the session is not it the table, it is ignored.
     *
     * @param session The session object, with the user already
     * filled in. 
     */
    public void remove(String sessionKey) {
        // noop - maintained in DB
    }

    /**
     * Get the number of sessions for a user.
     *
     * @param user The user object to check for.
     * @return The count of the number of sessions associated with this
     *  user.
     * @exception SessionException if an error occurs.
     */
    public int numSessions(User user) throws SessionException {
        try {
            UserNumSessionsQuery q = new UserNumSessionsQuery(user);
            Integer num = (Integer)DBUtil.dbQuery(q, dbName);
            return num.intValue();
        } catch (Exception e) {
            throw new SessionException(e);
        }
    }

    /**
     * Returns the session keys associated with a particular user.
     *
     * @param user The user object to check for.
     * @return An enumeration of the session keys associated
     *   with the user.
     * @exception SessionException if an error occurs.
     */
    public Enumeration getSessionKeys(User user) throws SessionException {
        try {
            UserSessionsQuery q = new UserSessionsQuery(user);
            return (Enumeration)DBUtil.dbQuery(q, dbName);
        } catch (Exception e) {
            throw new SessionException(e);
        }
    }
    
    /**
     * Shutdown this session user table as required.  The session
     * user table should not be used after this method has been called
     */
    public void shutdown() {
        // noop
    }

}
