/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HttpUtils.java,v 1.9.14.1 2000/10/19 17:59:07 jasona Exp $
 */




package com.lutris.http;
import java.io.*;
import java.util.*;
//FIX:!!!! This should not reference HTTPPresentation class; this must
//be independent of the appserver.
import com.lutris.appserver.server.httpPresentation.*;

/**
 * Utility methods useful to for HTTP.
 */
public class HttpUtils {
    /**
     * Adds the pair <b>key</b> and <b>value</b> to <b>table</b>.
     * If <b>key</b> already exists, then <b>value</b> is added
     * as the end of an array of existing values, and
     * the new array is assigned as the value for <b>key</b>.
     * If <b>key</b> does not yet exist, the <b>value</b> is assigned
     * as the value for <b>key</b>.
     * 
     * @param	table	The table to add to.
     * @param	key	The key to assign the value to.
     * @param	value	The value to assign
     */
    private static
    void AddKey(Hashtable table, String key, String value)
    {
	Object obj = table.get(key);
	if (obj == null) {
	    // First time "key" appears.
	    table.put(key, value);
	} else {
	    if (obj instanceof String) {
		// Second time.  Entry becomes an array of String.
		String[] newarray = new String[2];
		newarray[0] = obj.toString();
		newarray[1] = value;
		table.put(key, newarray);
		newarray = null;
	    } else if (obj instanceof String[]) { 
		// Third and subsequent times.
		String[] oldarray = (String[]) obj;
		String[] newarray = new String[oldarray.length +1];
		for (int i=0; i<oldarray.length; i++) {
		    newarray[i] = oldarray[i];
		}
		newarray[oldarray.length] = value;
		oldarray = null;
		table.put(key, newarray);
		newarray = null;
	    } else {
		// We should never see anything other than String
		// or String[] in this hash table.
		throw new Error("Internal Error - "
			+ "Unexpected type in QueryString Hashtable.");
	    }
	}
    }

    /**
     * Trims the left-hand side of a URL until the character '?'
     * is reached.  This is useful for raw URLs that need to be
     * parsed as query strings.
     * 
     * @param s		The string to trim.
     * @return		The trimmed string.
     */
    public static
    String trimQueryString(String s)
    {
	int pos = s.indexOf("?");
	if (pos >= 0) return s.substring(pos+1);
	return s;
    }
    
    /**
     * Decodes a query string in the format specified by the MIME
     * type <code>"application/x-www-form-urlencoded"</code>.  Such a
     * string consists of encoded key/value pairs in the form
     * <code>key=value</code>.  A hash table is returned with each
     * value indexed by its corresponding key.  If a key appears once
     * in the query string, then the value will be of type
     * <code>String</code>.  If the key appears more than once, then
     * the value will be an array of type <code>String</code>.
     * 
     * @param	s	The string to parse.
     * @return		A Hashtable containing the values indexed by
     * 			their corresponding keys.
     */
    public static
    Hashtable parseQueryString(String s)
    {
	String qs = s.replace('?', '&');
	StringTokenizer tok = new StringTokenizer(qs, "&", false);
	Hashtable table = new Hashtable();
	String pair, key, value;
	while (tok.hasMoreTokens()) {
	    pair = tok.nextToken();
	    int i = pair.indexOf("=");
	    if (i >= 0) {
		key = URLDecoder.decode(pair.substring(0, i));
		value = URLDecoder.decode(pair.substring(i+1));
		AddKey(table, key, value);
	    } else {
		int j = pair.indexOf(",");
		if (j >= 0) {
		    // Might be an image map coord. pair.
		    String xstr = pair.substring(0, j).trim();
		    xstr = URLDecoder.decode(xstr);
		    String ystr = pair.substring(j+1).trim();
		    ystr = URLDecoder.decode(ystr);
		    try {
			int xcoord = Integer.parseInt(xstr, 10);
			int ycoord = Integer.parseInt(ystr, 10);
			// Yes, we have image map coordinates.
			AddKey(table, "_imap_x", "" + xcoord);
			AddKey(table, "_imap_y", "" + ycoord);
		    } catch (Throwable t) {
			AddKey(table, "_garbage", URLDecoder.decode(pair));
		    }
		} else {
		    AddKey(table, "_garbage", URLDecoder.decode(pair));
		}
	    }
	    
	}
	return table;
    }
    
    /**
     * Parses FORM data posted by a client using the HTTP POST method
     * and the <code>application/x-www-form-urlencoded</code> MIME type.
     * 
     * @param	len	The maximum number of bytes to read from the input
     * 			stream.
     * @param	in	The input stream from which to read the form data.
     * @return		A Hashtable containing the values indexed by
     * 			their corresponding keys.
     * @exception IllegalArgumentException If the query string is 
     * 			not formatted correctly.
     * @exception IOException If an I/O error occurs while reading the
     * 			input stream.
     */
    public static
    Hashtable parsePostData(int len, InputStream in)
    throws IllegalArgumentException, IOException
    {
	byte[]	bytes = new byte[len];
	int	n, count=0;
	boolean eof=false;
	while ((count<len)&&((n = in.read(bytes, count, len - count)) >0))
	    count += n;
	char[] chars = new char[count];
	// Convert signed bytes directly to ASCII compatibility Unicode
	// chars using two's complement byte signed-unsigned conversion.
	for (n=0; n<count; n++) chars[n] = (char)((((int)bytes[n])+0x100)&0xff);
	bytes = null;
	return parseQueryString(new String(chars));
    }
    
    /**
     * Obtains the URL used by the client to make the current request,
     * without the query parameters.  Returns a StringBuffer object that
     * can be appended to, if the query string must be appended.  
     * 
     * @param req	presentation request object from which
     * 			to extract the URL.
     * @return		A string buffer object containing the extracted
     * 			URL.
     * @exception HttpPresentationException If an exception is generated
     * 			by the <code>req</code> object.
     */
    public static
    StringBuffer getRequestURL(HttpPresentationRequest req)
    throws HttpPresentationException
    {
	StringBuffer buf = new StringBuffer();
	buf.append("http://");
	buf.append(req.getServerName());
	int port = req.getServerPort();
	if (port != 80) buf.append(":" + port);
	buf.append(req.getRequestURI());
	return buf;
    }
}

