/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DebugResponse.java,v 1.11.14.1 2000/10/19 17:59:07 jasona Exp $
 */





package com.lutris.http.debug;

import java.util.Enumeration;
import javax.servlet.http.Cookie;

/**
 * Object used to access HTTP response data for debugging purposes.
 * 
 * @author	Paul Morgan
 * @since	LBS1.7
 * @version	$Revision: 1.11.14.1 $
 */
public interface DebugResponse {
    /**
     * Returns the size of the response entity data, or -1 if not known.
     */
    public int getContentLength();

    /**
     * Returns the Internet Media Type of the response entity data, or null
     * if not known.
     */
    public String getContentType();

    /**
     * Returns the status code for the response.
     */
    public int getStatusCode();

    /**
     * Returns the status message for the response.
     */
    public String getStatusMessage();

    /**
     * Returns an array of cookies to be set by this response.
     *
     * @return The array of cookies found in this response.
     */
    public Cookie[] getCookies();

    /**
     * Returns the values of the specified header for the response as
     * an array of strings, or a 0 length array if the named header does
     * not exist.
     *
     * @param name the case-insensitive header field name
     */
    public String getHeader(String name);

    /**
     * Returns an array of strings representing the header names
     * for this response. This does not have return cookies - use
     * getCookies().
     */
    public Enumeration getHeaderNames();

    /**
     * Returns the number of bytes associated with the response.
     */
    public int getTotalBytes();

    /**
     * Returns the data written to the output stream associated with this
     * request, or null if that data was not kept. Due to the potential
     * size of the data, this is an optional feature.
     */
    public String getResponseData();
}
