/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *-----------------------------------------------------------------------------
 * $Id: HttpsTestX509TrustManager.java,v 1.1.2.1 2000/07/24 23:06:02 mikew Exp $
 *-----------------------------------------------------------------------------
 */

package HTTPClient.https;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

import com.sun.net.ssl.X509TrustManager;
import com.sun.net.ssl.TrustManagerFactory;
import com.sun.net.ssl.TrustManager;

import java.security.cert.X509Certificate;
import java.security.NoSuchAlgorithmException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.cert.CertificateException;

public class HttpsTestX509TrustManager 
    implements X509TrustManager {

    //member variables
    TrustManager[] m_trustManagers = null;

    protected HttpsTestX509TrustManager(String algo, KeyStore ks)
	throws NoSuchAlgorithmException,
	       KeyStoreException,
	       FileNotFoundException,
	       IOException,
	       CertificateException {

	TrustManagerFactory tmfactory = TrustManagerFactory.getInstance(algo);

	tmfactory.init(ks);

	m_trustManagers = tmfactory.getTrustManagers();
    }

    public boolean isClientTrusted(X509Certificate[] chain) {

	//delegate to default trust managers
	for (int i=0; i<m_trustManagers.length; i++) {
	    if (((X509TrustManager)m_trustManagers[i]).isClientTrusted(chain) == true) {
		return true;
	    }
	}

	//if it used on server side, we can filter client
	//certificate decisions also
	//but for now we can ignore this

	return false;
    }

    public boolean isServerTrusted(X509Certificate[] chain) {

	// deleage actual work
	for (int i=0; i<m_trustManagers.length; i++) {
	    if (((X509TrustManager)m_trustManagers[i]).isServerTrusted(chain) == true) {
		return true;
	    }
	}

	// For testing purposes always return true, do not prompt.
	return true; 

	/*
	// filter decisions
	System.out.println ("\n\nTrustManager Alert: Untrusted Certificate recevied from Server\n");

	System.out.println ("Certificate chain received from server:");

	for (int i=0; i<chain.length; i++) {
	    System.out.println ("\n\nChain (" + i + ") -");
	    System.out.println ("Issuer DN: " + chain[i].getIssuerDN());
	    System.out.println ("\nSubject DN: " + chain[i].getSubjectDN());
	    System.out.println ("\nValidity period:\n after " + chain[i].getNotBefore() +
				"\nand before " + chain[i].getNotAfter());
	}
	System.out.println ();


	try {
	    //return getYesNoFromUser ("Do you want to continue (Y/N): ");
	}
	catch (IOException ioe) {
	    return false;
	}
	*/
    }


    public X509Certificate[] getAcceptedIssuers() {

	for (int i=0; i<m_trustManagers.length; i++) {
	    if (m_trustManagers[i] instanceof X509TrustManager) {
		return ((X509TrustManager)m_trustManagers[i]).getAcceptedIssuers();
	    }
	}

	return null;
    }


    public static X509TrustManager[] getTrustManagers(String algo, KeyStore ks)
	throws NoSuchAlgorithmException,
	       KeyStoreException,
	       FileNotFoundException,
	       IOException,
	       CertificateException {

	HttpsTestX509TrustManager[] tm = new HttpsTestX509TrustManager[1];
	tm[0] = new HttpsTestX509TrustManager(algo, ks);
	return tm;
    }

    protected static boolean getYesNoFromUser (String question)
	throws java.io.IOException {

	System.out.print (question);

	byte[] ch = new byte[100];

	System.in.read(ch);
	String chStr = new String (ch, 0, 1);

	if (chStr.equalsIgnoreCase("y") == true) {
	    return true;
	}

	return false;
    }
}




