/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ThreadResultsCollector.java,v 1.2.2.1.2.1 2000/10/19 17:59:08 jasona Exp $
 */

package org.enhydra.httpServerTest.client;

import java.util.Enumeration;

/**
 * This class provides the interface for thread results collection classes.
 * Classes implementing this interface should provide the a mechanism to 
 * collect all failed assertions and return them as an enumeration.  This
 * class is used to communicate ass failed assertions between client threads
 * and the test case collection.
 *
 * @see org.enhydra.httpServerTest.client.HttpTestClient
 * @see org.enhydra.httpServerTest.client.SingleUserGetRequestThread
 * @see org.enhydra.httpServerTest.client.SingleUserPostRequestThread
 *
 * @author Mike Ward
 **/
public interface ThreadResultsCollector {
    
    /**
     * Assertion method should only collect failed assertions.
     *
     * @param success
     *        The assertion state, only 'false' assertions are typically
     *        logged.
     * @param message
     *        The error message associated with this assesrtion.
     * @param thread
     *        The client thread making this assertion call, used to gather
     *        thread identification for error reporting.
     **/
    public void assert(boolean success, 
		       String message, 
		       BaseTestThread thread);

    /**
     * Returns an enumeration of all failed assertions.
     *
     * @return Enumeration
     *         All failed assertions.
     **/
    public Enumeration getFailedAsserts();

    /**
     * The state of failed assertions, should return 'true' if assertions
     * have failed, 'false' otherwise.
     *
     * @return boolean
     *         The state of failed assertions.
     **/
    public boolean failedAssertsState();
}
