/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: RequestDispatchForwardTest.java,v 1.1.2.1.2.1 2000/10/19 17:59:08 jasona Exp $
 */

package org.enhydra.httpServerTest.server.application.presentation;

import com.lutris.appserver.server.httpPresentation.HttpPresentationException;

import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationRequest;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.RequestDispatcher;

/**
 * This class is the generic server side test presentation object that client
 * side test threads talk to.  The purpose of this presentation object is to 
 * test the RequestDispatcher.  All this class does is to forward all requests
 * to the BasicTest.po.  In doing so, the client will verify that no request
 * information was lost.
 *
 * @see org.enhydra.httpServerTest.server.application.presentation.BasicTest
 *
 * @author Mike Ward
 **/
public class RequestDispatchForwardTest
    implements HttpPresentation {

    public static final String REDIRECT_TO_FILE = "BasicTest.po";
    /**
     * The run method for this presentation object.  This method grabs a 
     * reference to the RequestDispatcher and forwards the current request onto
     * BasicTest.po.
     *
     * @param comms
     *        The comms objects for this request.  All request, response,
     *        and session information are available from this object.
     **/
    public void run(HttpPresentationComms comms) 
        throws HttpPresentationException {

	HttpServletRequest request = 
	    comms.request.getHttpServletRequest();
	HttpServletResponse response = 
	    comms.response.getHttpServletResponse();
	
	//Grab any cgi parameters held in path info
	String pathInfo = request.getPathInfo();
	String redirectURL = REDIRECT_TO_FILE;
	if (pathInfo != null) {
	    int cgiParamIndex = pathInfo.indexOf(';');
	    if (cgiParamIndex >= 0) {
		redirectURL += pathInfo.substring(cgiParamIndex);
	    }
	}	   

	RequestDispatcher dispatcher = 
	    request.getRequestDispatcher(redirectURL);
	
	try {
	    dispatcher.forward(request, response);
	} catch (Exception e) {
	    throw new HttpPresentationException("Failed to dispatch request.",
						e);
	}
    }
}
