/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: JoltCompileException.java,v 1.6.12.1 2000/10/19 17:59:08 jasona Exp $
 */





package com.lutris.appserver.server.jolt.joltc;
import com.lutris.appserver.server.jolt.parser.*;

/**
 * Exception used to convert throw errors with a message in the same format
 * as javac errors.  The main reason this class exists is that the file name
 * that was bring processed needed to be preserved.
 */
class JoltCompileException extends Exception {
    /**
     * Construct given a file name and line number
     * @param srcFile The file where the error occured.
     * @param lineNum The line where the error occured.
     * @param msg An message describing the error.
     */
    public JoltCompileException(String msg, String srcFile, int lineNum) {
        super(srcFile + ":" + lineNum + ": " + msg);
    }

    /**
     * Construct from a ParseException.
     * @param srcFile The file where the error occured.
     * @param parseException The exception to generate the message from.
     */
    public JoltCompileException(String srcFile, ParseException parseException) {
        super(srcFile + ":" + parseException.currentToken.beginLine
            + ": " + parseException.getMessage());
    }
}
