/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: JoltCallArgs.java,v 1.7.12.1 2000/10/19 17:59:08 jasona Exp $
 */





package com.lutris.appserver.server.jolt.joltpo;

import java.util.Hashtable;
import java.util.Enumeration;

/**
 * Class use to hold arguments passed in a &lt;JOLT CALL&gt; tag.
 *
 * @version	$Revision: 1.7.12.1 $
 * @author	Mark Diekhans
 * @since	Jolt1.0
 * @see         java.util.Hashtable
 */
public class JoltCallArgs extends Hashtable {
    /**
     * Get an argument by name.
     * @param name The name of the argument, as specified in the JOLT
     *  tag.  The name is case sensitive.
     * @return The value of the argument, or null if it doesn't exist.
     */
    public String get(String name) {
        return (String)super.get(name);
    }

    /**
     * Get the list of all argument names.
     * @return An array containing the argument names
     * @see java.util.Hashtable#keys
     */
    public String[] names() {
        String[] nameList = new String[size()];
        int i = 0;
        Enumeration nameEnum = keys();
        while (nameEnum.hasMoreElements()) {
            nameList[i++] = (String)nameEnum.nextElement();
        }
        return nameList;
    }
}
