/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: JoltDumpVisitor.java,v 1.9.12.1 2000/10/19 17:59:08 jasona Exp $
 */




package com.lutris.appserver.server.jolt.parser;

/**
 * The Visitor for the Jolt parser that dumps out the tree.
 *
 * @author Shawn McMurdo
 * @version $Revision: 1.9.12.1 $
 */
public class JoltDumpVisitor implements JoltParserVisitor
{
    private int indent = 0;

    private String indentString()
    {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < indent; ++i) {
            sb.append(" ");
        }
        return sb.toString();
    }

    public Object visit(SimpleNode node, Object data)
    {
        System.out.println(indentString() + indent + " " + node +
		       ": acceptor not implemented in subclass?");
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTStart node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTHtmlSection node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTJoltContents node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTJoltParsedContents node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTJavadef node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTHtmldef node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTAnyCall node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTArgList node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTArg node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTBooleanConditional node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTDefinedConditional node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTMethodConditional node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTJoltHtml node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTJavaImport node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTInclude node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTSource node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTJavaCatch node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTJavaFinally node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }

    public Object visit(ASTJoltField node, Object data)
    {
        System.out.println(indentString() + indent + " " + node);
        ++indent;
        data = node.childrenAccept(this, data);
        --indent;
        return data;
    }
}

