/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;

// Standard imports
import java.awt.Container;
import javax.swing.JOptionPane;
import java.util.ResourceBundle;

/**
 * Class containing the version information
 */
public class KelpInfo {

    // not to be resourced
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");  // nores
    public static final String  HELP_DOC =
        "file:../kelp2/doc/kelp.html";              // nores
    private static final String VERSION = "2.0.1";    // nores
    private static final String TOOLBOX_CLASS =
        "org.enhydra.tool.ToolBoxInfo";             // nores

    //
    public static String getKelpVersion() {
        return KelpInfo.VERSION;
    }

    public static boolean verifyIDEClassPath() {
        return verifyIDEClassPath(null);
    }

    public static boolean verifyIDEClassPath(Container owner) {
        boolean valid = false;

        valid = KelpInfo.isClassPathComplete();
        StringBuffer message = new StringBuffer();

        message.append(res.getString("MissingJars1"));
        message.append(res.getString("MissingJars2"));
        if (!valid) {
            if (KelpInfo.isToolBoxInClassPath()) {
                if (!KelpInfo.isToolBoxVersionInSynch()) {
                    message.setLength(0);
                    message.append(res.getString("OutOfSync"));
                    message.append(ResUtil.format(res.getString("Kelp_version_0"),
                                                  KelpInfo.getKelpVersion()));
                    message.append('\n');
                    message.append(ResUtil.format(res.getString("ToolBox_version_0"),
                                                  ToolBoxInfo.getToolBoxVersion()));
                } else if (!ToolBoxInfo.isEnhydraInClassPath()) {
                    message.append(Constants.FILE_ENHYDRA_JAR);
                    message.append('\n');
                } else if (!ToolBoxInfo.isXMLCInClassPath()) {
                    message.append(Constants.FILE_XMLC_JAR);
                    message.append('\n');
                }
            } else {
                message.append(Constants.FILE_TOOLBOX_JAR);
                message.append('\n');
            }
            if (owner == null) {
                System.out.println(message.toString());
            } else {
                StringBuffer buf = new StringBuffer();

                buf.append(Constants.KELP);
                buf.append(Constants.TAB1);
                buf.append(KelpInfo.VERSION);
                JOptionPane.showMessageDialog(owner, message.toString(),
                                              buf.toString(),
                                              JOptionPane.ERROR_MESSAGE);
            }
        }
        return valid;
    }

    public static boolean isToolBoxInClassPath() {
        boolean inPath = true;

        try {
            Class testClass = Class.forName(TOOLBOX_CLASS);
        } catch (java.lang.ClassNotFoundException e) {
            inPath = false;
        }
        return inPath;
    }

    public static boolean isToolBoxVersionInSynch() {
        boolean inSynch = false;

        if (ToolBoxInfo.getToolBoxVersion().equalsIgnoreCase(KelpInfo.getKelpVersion())) {
            inSynch = true;
        }
        return inSynch;
    }

    public static boolean isClassPathComplete() {
        boolean complete = false;

        if (KelpInfo.isToolBoxInClassPath()) {
            if (ToolBoxInfo.isEnhydraInClassPath()
                    && ToolBoxInfo.isXMLCInClassPath()
                    && KelpInfo.isToolBoxVersionInSynch()) {
                complete = true;
            }
        }
        return complete;
    }

    /**
     * Prevent instantiation.
     */
    private KelpInfo() {}

}
