/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the license.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common;

// ToolBox imports
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.FileUtil;

// Kelp imports
import org.enhydra.kelp.common.map.Mapper;
import org.enhydra.kelp.common.importer.ResourceFilter;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterTemplateNode;

// Standard imports
import java.io.File;
import java.io.FileFilter;
import java.util.Vector;

//
public class PathUtil {

    // string not to be translated
    private final static String UNTITLED = "untitled"; // nores
    private final static String _PATH = "_PATH@"; // nores

    //
    public PathUtil() {}

    public static boolean isInputInSource(OtterProject project) {
        PathHandle inPath = null;
        PathHandle srcPath = null;
        String[]   srcs = new String[0];
        boolean    inSource = false;

        inPath = PathHandle.createPathHandle(project.getDeployInputPath());
        srcs = project.getSourcePathArray();
        for (int i = 0; i < srcs.length; i++) {
            srcPath = PathHandle.createPathHandle(srcs[i]);
            if (srcPath.parentOf(inPath) || srcPath.equals(inPath)) {
                inSource = true;
                break;
            }
        }
        return inSource;
    }

    public static boolean isTemplate(OtterNode node) {
        boolean    temp = false;
        PathHandle handleNode = null;
        PathHandle handleDeploy = null;

        handleNode = PathHandle.createPathHandle(node.getFilePath());
        if (handleNode.hasExtension(Constants.TYPE_IN)) {
            OtterProject project = node.getProject();

            handleDeploy =
                PathHandle.createPathHandle(project.getDeployInputPath());
            if (handleDeploy.parentOf(handleNode)) {
                temp = true;
            }
        }
        return temp;
    }

    public static String getDeployArchivePath(OtterProject project) {
        return getDeployArchivePath(project.getDeployRootPath(), project);
    }

    public static String getDeployArchivePath(String root,
                                              OtterProject project) {
        File         f = new File(root);
        StringBuffer path = new StringBuffer();

        path.append(f.getAbsolutePath());
        path.append(File.separator);
        path.append(Constants.DIR_ARCHIVE);
        path.append(File.separator);
        if (project == null) {
            path.append(PathUtil.UNTITLED);
        } else {
            path.append(project.getProjectName());
        }
        if (project == null || project.isWebApplication()) {
            path.append('.');
            path.append(Constants.TYPE_WAR);
        } else {
            path.append('.');
            path.append(Constants.TYPE_JAR);
        }
        return PathHandle.createPathString(path.toString());
    }

    public static String[][] expandReplacementTable(OtterProject project,
            String[][] table) {
        for (int i = 0; i < table.length; i++) {
            if (table[i][0].endsWith(PathUtil._PATH)) {
                table[i][1] = PathUtil.expandPathRelativeToProject(project,
                        table[i][1]);
            }
        }
        return table;
    }

    public static String getDeployConfPath(OtterProject project) {
        return PathUtil.getDeployConfPath(project.getDeployRootPath());
    }

    public static String getDeployConfPath(String root) {
        StringBuffer path = new StringBuffer();

        path.append(root);
        path.append(File.separator);
        path.append(Constants.DIR_CONF);
        return PathHandle.createPathString(path.toString());
    }

    public static String getDeployContentPath(OtterProject project) {
        return PathUtil.getDeployContentPath(project,
                                             project.getDeployRootPath());
    }

    public static String getDeployContentPath(OtterProject project,
                                              String root) {
        StringBuffer buf = new StringBuffer();
        String       path = new String();
        Mapper       mapper = null;

        if (project.isWebApplication()) {
            buf.append(root);
            buf.append(File.separator);
            buf.append(Constants.DIR_CONTENT);
            path = PathHandle.createPathString(buf.toString());
        } else {
            String resourcePath = new String();
            String sourcePath = new String();

            resourcePath = project.getDeployResourcePath();
            sourcePath = PathUtil.getSourcePathOf(project, resourcePath);
            mapper = new Mapper(project);
            path = mapper.getMappedOutputPath(sourcePath, resourcePath);
        }
        return path;
    }

    public static String getDefaultDeployResourcePath(OtterProject project,
            String appPackPath, String sourcePath) {
        String         path = sourcePath;
        StringBuffer   searchRootPath = new StringBuffer();
        File           firstFound = null;
        ResourceFilter resourceFilter = null;

        resourceFilter = new ResourceFilter();
        resourceFilter.setResourcePath(true);
        resourceFilter.setProject(project);
        searchRootPath.append(sourcePath);
        searchRootPath.append(File.separator);
        searchRootPath.append(appPackPath);
        firstFound = FileUtil.findFirst((FileFilter) resourceFilter,
                                        searchRootPath.toString());
        if (firstFound == null) {
            resourceFilter.setResourcePath(false);
            firstFound = FileUtil.findFirst((FileFilter) resourceFilter,
                                            searchRootPath.toString());
        }
        if (firstFound != null) {
            path = firstFound.getParent();
        }
        return PathHandle.createPathString(path);
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param propertyName
     * @param inFilename
     *
     * @see
     */
    public static void putFileRelativeToProject(OtterNode node,
                                                String propertyName,
                                                String inFilename) {
        String outFilename = compressPathRelativeToProject(node, inFilename);

        node.setProperty(propertyName, outFilename);
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param propertyName
     *
     * @return
     *
     * @see
     */
    public static File getFileRelativeToProject(OtterNode node,
                                                String propertyName) {
        File   file = null;
        String filename = node.getProperty(propertyName);

        if ((filename != null) && (filename.trim().length() > 0)) {
            filename = PathUtil.expandPathRelativeToProject(node, filename);
            file = new File(filename);
        }
        return file;
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param inPath
     *
     * @return
     *
     * @see
     */
    public static String compressPathRelativeToProject(OtterNode node,
            String inPath) {
        StringBuffer outPath = new StringBuffer();
        String       absolutePath = null;

        if ((inPath.length() > 0) && (inPath.charAt(0) == '.')) {
            outPath.append(inPath);
        } else {
            File       file = new File(node.getProject().getFilePath());
            PathHandle projectParent = null;

            projectParent = PathHandle.createPathHandle(file.getParent());
            file = new File(inPath);
            absolutePath = file.getAbsolutePath().trim();
            if (absolutePath.length() > inPath.trim().length()) {
                outPath.append(inPath);

                // Not a valid path.
            } else {
                if (projectParent.parentOf(absolutePath)) {
                    if (absolutePath.length()
                            == projectParent.getPath().length()) {
                        outPath.append('.');
                        outPath.append(File.separator);
                    } else {
                        outPath.append('.');
                        outPath.append(File.separator);
                        outPath.append(absolutePath.substring(projectParent.getPath().length()
                                                              + 1));
                    }
                } else {
                    outPath.append(absolutePath);
                }
            }
        }

        // Return with separators set to / for any OS but don't do
        // path handle to avoid decompress.
        return outPath.toString().replace('\\', '/');
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param inPath
     *
     * @return
     *
     * @see
     */
    public static String expandPathRelativeToProject(OtterNode node,
            String inPath) {
        StringBuffer outPath = new StringBuffer();
        File         projectFile = null;

        // Set separators to current OS
        inPath = inPath.replace('\\', File.separatorChar);
        inPath = inPath.replace('/', File.separatorChar);
        if ((inPath.length() == 1) && (inPath.charAt(0) == '.')) {
            projectFile = new File(node.getProject().getFilePath());
            outPath.append(projectFile.getParent());
        } else if ((inPath.length() > 0) && (inPath.charAt(0) == '.')) {

            // expand path relative to the project.
            projectFile = new File(node.getProject().getFilePath());
            outPath.append(projectFile.getParent());
            if (!projectFile.getParent().endsWith(File.separator)) {
                outPath.append(File.separator);
            }
            if ((inPath.length() > 1) && (inPath.charAt(1) == '.')) {
                outPath.append(inPath);
            } else {
                outPath.append(inPath.substring(2));
            }

        } else {
            outPath.append(inPath);
        }
        return PathHandle.createPathString(outPath.toString());
    }

    public static OtterTemplateNode[] getInputTemplates(OtterProject project) {
        OtterNode[]         input = null;
        OtterTemplateNode[] templates = null;
        OtterNodeFactory    factory = null;
        Vector              tempVector = new Vector();

        input = project.getAllInput();
        factory = project.getNodeFactory();
        for (int i = 0; i < input.length; i++) {
            PathHandle cursor = null;

            cursor = PathHandle.createPathHandle(input[i].getFilePath());
            if (cursor.hasExtension(Constants.TYPE_IN)) {
                OtterTemplateNode newNode = null;

                newNode = factory.getTemplateNode(input[i]);
                tempVector.addElement(newNode);
            }
        }
        templates = new OtterTemplateNode[tempVector.size()];
        templates = (OtterTemplateNode[]) tempVector.toArray(templates);
        tempVector.clear();
        return templates;
    }

    public static OtterNode[] getInputPassthrough(OtterProject project) {
        OtterNode[] input = null;
        OtterNode[] pass = null;
        Vector      passVector = new Vector();

        input = project.getAllInput();
        for (int i = 0; i < input.length; i++) {
            PathHandle cursor = null;

            cursor = PathHandle.createPathHandle(input[i].getFilePath());
            if (cursor.hasExtension(Constants.TYPE_IN)) {

                // skip templates
            } else {
                passVector.addElement(input[i]);
            }
        }
        pass = new OtterNode[passVector.size()];
        pass = (OtterNode[]) passVector.toArray(pass);
        passVector.clear();
        return pass;
    }

    public static String getSourcePathOf(OtterProject project,
                                         String sourceFile) {
        String[]   path = project.getSourcePathArray();
        PathHandle sourcePath = null;
        PathHandle filePath = null;
        PathHandle parentPath = null;

        filePath = PathHandle.createPathHandle(sourceFile);
        sourcePath = filePath.getParent();
        for (int i = 0; i < path.length; i++) {
            parentPath = PathHandle.createPathHandle(path[i]);
            if (parentPath.parentOf(filePath)) {
                sourcePath.setPath(parentPath.getPath());
                break;
            }
        }
        return sourcePath.getPath();
    }

}
