/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.bridge;

// Kelp imports
import org.enhydra.kelp.common.wizard.xmlc.Reporter;

// XMLC imports
import org.enhydra.xml.xmlc.XMLCError;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.enhydra.xml.xmlc.compiler.Parse;
import org.enhydra.xml.xmlc.metadata.MetaData;

// Standard imports
import java.io.PrintWriter;
import java.io.IOException;

//
public class ParserV2 implements Parser {
    private PrintWriter     traceWriter = null;
    private MetaDataHandler handler = null;
    private Parse           parser = null;

    public ParserV2(PrintWriter trace, MetaDataHandler metaData) {
        traceWriter = trace;
        handler = metaData;
        Reporter reporter = null;

        reporter = new Reporter(trace);
        parser = new Parse(reporter, trace);
    }

    public XMLCDocument parse() throws XMLCException {
        XMLCDocument doc = null;
        MetaData     metaData = null;

        metaData = (MetaData) handler.getMetaData();
        try {
            doc = parser.parse(metaData);
        } catch (XMLCException xe) {
            throw xe;
        } catch (Exception e) {
            e.printStackTrace();
            throw new XMLCException(e);
        } catch (XMLCError e) {
            e.printStackTrace();
            throw new XMLCException(e);
        } catch (Error e) {
            e.printStackTrace();
            throw new XMLCException(e);
        }
        return doc;
    }

    public MetaDataHandler getMetaDataHandler() {
        return handler;
    }

    public void setMetaDataHandler(MetaDataHandler h) {
        handler = h;
    }

    public PrintWriter getTraceWriter() {
        return traceWriter;
    }

}
