/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.map;

// ToolBox imports
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.FileUtil;

// Kelp imports
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterXMLCNode;

// Standard imports
import java.io.File;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class Mapper {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private OtterProject  project = null;
    private String[][]    map = new String[0][0];
    private String[]      path = new String[0];
    private int           scope = OtterProject.MAP_SCOPE_ALL;

    /**
     * Constructor declaration
     *
     *
     * @param p
     */
    public Mapper(OtterProject p) {
        MapEntry[] entries;

        project = p;
        scope = project.getMapScope();
        path = project.getSourcePathArray();
        if (scope != OtterProject.MAP_SCOPE_NONE) {
            map = project.getPackageMap();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param node
     *
     * @return
     */
    public String getMappedOutputPath(OtterNode node) {
        String sourcePath = new String();
        String sourceFile = new String();

        sourceFile = node.getFilePath();
        sourcePath = project.getSourcePathOf(node);
        return getMappedOutputPath(sourcePath, sourceFile);
    }

    public String getMappedOutputPath(String sourcePath, String sourceFile) {
        StringBuffer buf = new StringBuffer();
        String       out = new String();

        out = project.getClassOutputPath();
        buf.append(out);
        buf.append(File.separatorChar);
        out = getMappedPath(sourcePath, sourceFile);
        buf.append(out);
        out = PathHandle.createPathString(buf.toString());
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param node
     *
     * @return
     */
    public String getMappedSourcePath(OtterNode node) {
        StringBuffer buf = new StringBuffer();
        String       sourcePath = new String();
        String       sourceFile = new String();

        sourceFile = node.getFilePath();
        sourcePath = project.getSourcePathOf(node);
        buf.append(sourcePath);
        buf.append(File.separatorChar);
        buf.append(getMappedPath(sourcePath, sourceFile));
        return PathHandle.createPathString(buf.toString());
    }

    /**
     * Method declaration
     *
     *
     * @param node
     *
     * @return
     */
    private String getMappedPath(String sourcePath, String sourceFile) {
        String mappedPath = null;
        int    index = -1;

        index = sourceFile.lastIndexOf('.');
        if (index > sourcePath.length()) {
            mappedPath = sourceFile.substring(sourcePath.length() + 1, index);
        } else {
            mappedPath = sourceFile.substring(sourcePath.length() + 1);
        }
        mappedPath = getMappedName(mappedPath, sourceFile);
        mappedPath = mappedPath.replace('.', File.separatorChar);
        if (index > sourcePath.length()) {
            mappedPath = mappedPath + sourceFile.substring(index);
        }
        mappedPath = PathHandle.createPathString(mappedPath);
        return mappedPath;
    }

    /**
     * Method declaration
     *
     *
     * @param node
     *
     * @return
     */
    public String getClassName(OtterXMLCNode node) {
        boolean custom = false;
        int     type = node.getClassNameType();
        String  sourcePath = project.getSourcePathOf(node);
        String  rawFilePath =
            (new File(node.getFilePath())).getAbsolutePath();
        String  mappedName = new String();

        if (sourcePath.length() > 0) {
            mappedName = rawFilePath.substring(sourcePath.length(),
                                               rawFilePath.lastIndexOf('.'));
        } else {
            System.out.println(res.getString("Warning_file_not_in")
                               + rawFilePath);
            rawFilePath.substring(rawFilePath.lastIndexOf(File.separator)
                                  + 1, rawFilePath.lastIndexOf('.'));
        }
        switch (type) {
        case OtterXMLCNode.CLASS_NAME_CUSTOM:
            if (node.getCustomClassName().trim().length() > 0) {
                mappedName = node.getCustomClassName();
                custom = true;
            }
            break;
        case OtterXMLCNode.CLASS_NAME_DEFAULT:

            // already at default.
            break;
        default:
            mappedName = getMappedName(mappedName, rawFilePath);
        }
        mappedName = validateClassName(mappedName, rawFilePath, type, custom);
        return mappedName;
    }

    /**
     * Method declaration
     *
     *
     * @param mappedName
     * @param rawFilePath
     * @param type
     * @param custom
     *
     * @return
     */
    public String validateClassName(String mappedName, String inPath,
                                    int type, boolean custom) {
        String     validName = new String();
        PathHandle handle = null;

        handle = PathHandle.createPathHandle(inPath);
        validName = mappedName.trim();
        validName = validName.replace('/', '.');
        validName = validName.replace('\\', '.');
        if (validName.charAt(0) == '.') {
            validName = validName.substring(1);
        }
        if (!custom) {
            String baseName = new String();
            int    baseStart = -1;
            int    baseEnd = -1;

            baseStart = handle.getPath().lastIndexOf('/') + 1;
            if (validName.length() == 0) {

                // not in path, create packageless default name.
                validName = handle.getPath().substring(baseStart);
                baseEnd = validName.lastIndexOf('.');
                validName = validName.substring(0, baseEnd);
            }
            baseEnd = handle.getPath().lastIndexOf('.');
            baseName = handle.getPath().substring(baseStart, baseEnd);
            baseEnd = validName.lastIndexOf('.');
            validName = validName.substring(0, baseEnd + 1) + baseName
                        + handle.getExtension().toUpperCase();
        }
        return validName;
    }

    /**
     * Method declaration
     *
     *
     * @param mappedName
     * @param rawFilePath
     *
     * @return
     */
    private String getMappedName(String mappedName, String rawFilePath) {
        String   mappedPath = rawFilePath;
        String   matchPack = new String();
        String[] parentPath = null;
        String   toPackage = new String();
        boolean  useMap = false;
        boolean  mapped = false;
        boolean  isDir = false;
        int      max = -1;
        int      matchMax = -1;

        isDir = FileUtil.isDirectory(rawFilePath);
        if (map != null) {
            for (int i = 0; i < map.length; i++) {
                parentPath = new String[1];
                parentPath[0] = map[i][0];
                for (int j = 0; j < parentPath.length; j++) {
                    parentPath[j] = parentPath[j].trim();
                    if ((parentPath[j].length() > max) || (!mapped)) {
                        PathHandle parentHandle = null;

                        parentHandle =
                            PathHandle.createPathHandle(parentPath[j]);
                        max = parentPath[j].length();
                        toPackage = map[i][1].trim();
                        if (isDir) {
                            if (parentHandle.equals(rawFilePath)) {
                                matchPack = toPackage;
                                matchMax = max;
                                mapped = true;
                                break;
                            }
                        } else if (parentHandle.parentOf(rawFilePath)) {
                            matchPack = toPackage;
                            matchMax = max;
                            mapped = true;
                            break;
                        }
                    }
                }
            }
        }
        int index = 0;

        if (mapped) {
            index = rawFilePath.lastIndexOf('.');
            if (index > matchMax) {
                mappedName = matchPack
                             + rawFilePath.substring(matchMax, index);
            } else {
                mappedName = matchPack + rawFilePath.substring(matchMax);
            }
        }
        return mappedName;
    }

}
