/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.properties;

// Kelp imports
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.wizard.xmlc.MapDialog;
import org.enhydra.kelp.common.wizard.xmlc.XMLCOptionTab;

// Standard imports
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class XMLCNodePropertyPanel extends JPanel {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private OtterXMLCNode       node;
    private GridBagLayout       layoutMain;
    private JCheckBox           checkCompile;
    private JTextField          textClassName;
    private JPanel              panelClassName;
    private GridBagLayout       layoutClassName;
    private JRadioButton        radioDefault;
    private JRadioButton        radioCustom;
    private JRadioButton        radioMap;
    private JButton             buttonMap;
    private XMLCOptionTab       optionTab;
    private TitledBorder        borderClassName;
    private ButtonGroup         radioGroup;
    private JCheckBox           checkStatic;
    private LocalButtonListener buttonListener;
    private LocalCheckListener  checkListener;
    private LocalRadioListener  radioListener;

    /**
     * Constructor declaration
     *
     *
     */
    public XMLCNodePropertyPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterXMLCNode getNode() {
        return node;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     */
    public void setNode(OtterXMLCNode n) {
        node = n;
    }

    /**
     * Method declaration
     *
     */
    public void readProperties() {
        boolean b = false;
        String  s = new String();

        optionTab.setNode(node);
        optionTab.init();
        optionTab.readProperties();
        b = node.isSelected();
        checkCompile.setSelected(b);
        if (b) {
            checkStatic.setSelected(false);
        } else {
            b = node.isStatic();
            checkStatic.setSelected(b);
        }
        if (node.getClassNameType() == OtterXMLCNode.CLASS_NAME_CUSTOM) {
            radioCustom.setSelected(true);
        } else if (node.getClassNameType()
                   == OtterXMLCNode.CLASS_NAME_MAPPED) {
            radioMap.setSelected(true);
        } else {
            radioDefault.setSelected(true);
        }
        s = node.getMetaDataHandler().getClassName();
        textClassName.setText(s);
        enableCompileOptions(checkCompile.isSelected()
                             && (!checkStatic.isSelected()));
        refreshClassName();
    }

    /**
     * Method declaration
     *
     */
    public void writeProperties() {
        optionTab.writeProperties();
        boolean b = checkCompile.isSelected();

        node.setSelected(b);
        if (b) {
            node.setStatic(false);
        } else {
            b = checkStatic.isSelected();
            node.setStatic(b);
        }
        String customClassName = textClassName.getText();

        node.setCustomClassName(customClassName);
        if (radioCustom.isSelected()) {
            node.setClassNameType(OtterXMLCNode.CLASS_NAME_CUSTOM);
        } else if (radioDefault.isSelected()) {
            node.setClassNameType(OtterXMLCNode.CLASS_NAME_DEFAULT);
        } else {
            node.setClassNameType(OtterXMLCNode.CLASS_NAME_MAPPED);
        }
    }

    //
    // PRIVATE
    //

    /**
     * Method declaration
     *
     *
     * @exception Exception
     */
    private void pmInit() throws Exception {
        radioGroup =
            (ButtonGroup) Beans.instantiate(getClass().getClassLoader(),
                                            ButtonGroup.class.getName());
        radioGroup.add(radioDefault);
        radioGroup.add(radioCustom);
        radioGroup.add(radioMap);
        refreshClassName();
        radioListener = new LocalRadioListener();
        radioDefault.addActionListener(radioListener);
        radioCustom.addActionListener(radioListener);
        radioMap.addActionListener(radioListener);

        //
        buttonListener = new LocalButtonListener();
        buttonMap.addActionListener(buttonListener);

        //
        checkListener = new LocalCheckListener();
        checkCompile.addActionListener(checkListener);
        checkStatic.addActionListener(checkListener);
    }

    /**
     * Method declaration
     *
     *
     * @exception Exception
     */
    private void jbInit() throws Exception {
        checkCompile =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        textClassName =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelClassName =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutClassName =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        radioDefault =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioCustom =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioMap =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        buttonMap = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        optionTab =
            (XMLCOptionTab) Beans.instantiate(getClass().getClassLoader(),
                                              XMLCOptionTab.class.getName());
        checkStatic =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkCompile.setText(res.getString("checkCompile_Text"));
        optionTab.setLayout(optionTab.getLayout());
        panelClassName.setLayout(layoutClassName);
        borderClassName =
            BorderFactory.createTitledBorder(res.getString("Generated_class_name"));
        panelClassName.setBorder(borderClassName);
        radioDefault.setSelected(true);
        radioDefault.setText(res.getString("Default"));
        radioCustom.setText(res.getString("Custom"));
        radioMap.setText(res.getString("radioMap_Text"));
        buttonMap.setText(res.getString("buttonMap_Text"));
        checkStatic.setText(res.getString("checkStatic_Text"));
        panelClassName.add(textClassName,
                           new GridBagConstraints(0, 1, 4,
                                                  GridBagConstraints.REMAINDER,
                                                  1.0, 1.0,
                                                  GridBagConstraints.WEST,
                                                  GridBagConstraints.HORIZONTAL,
                                                  new Insets(5, 5, 5, 5), 0,
                                                  0));
        panelClassName.add(radioCustom,
                           new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(0, 0, 0, 0), 0,
                                                  0));
        panelClassName.add(radioMap,
                           new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(0, 0, 0, 0), 0,
                                                  0));
        panelClassName.add(radioDefault,
                           new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(0, 0, 0, 0), 0,
                                                  0));
        panelClassName.add(buttonMap,
                           new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(0, 0, 0, 0), 0,
                                                  0));
        this.setLayout(layoutMain);
        this.add(checkCompile,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 5, 5), 0, 0));
        this.add(panelClassName,
                 new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 15, 3, 5), 0, 0));
        this.add(optionTab,
                 new GridBagConstraints(0, 4, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 15, 5, 5), 0, 0));
        this.add(checkStatic,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 2, 5), 0, 0));
    }

    /**
     * Method declaration
     *
     *
     * @param enable
     */
    private void enableCompileOptions(boolean enable) {
        radioDefault.setEnabled(enable);
        radioCustom.setEnabled(enable);
        radioMap.setEnabled(enable);
        textClassName.setEnabled(enable && radioCustom.isSelected());
        if (textClassName.isEnabled()) {
            textClassName.setBackground(SystemColor.info);
        } else {
            textClassName.setBackground(SystemColor.control);
        }
        buttonMap.setEnabled(enable);
        optionTab.setEnabled(enable);
    }

    /**
     * Method declaration
     *
     */
    private void refreshClassName() {
        textClassName.setEnabled(radioCustom.isSelected());
        if (textClassName.isEnabled()) {
            textClassName.setBackground(SystemColor.info);
        } else {
            textClassName.setBackground(SystemColor.control);
        }
        if (node != null) {
            String className = new String();

            writeProperties();
            className = node.getMetaDataHandler().getClassName();
            textClassName.setText(className);
        }
        buttonMap.setEnabled(radioMap.isSelected());
    }

    /**
     * Method declaration
     *
     */
    private void editMap() {
        MapDialog dialog = null;
        String    t = res.getString("Edit_Project_Map");

        dialog = new MapDialog(((JDialog) this.getTopLevelAncestor()), t,
                               true);
        dialog.setProject(node.getProject());
        dialog.show();
    }

    /**
     * Class declaration
     *
     */
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonMap) {
                editMap();
                refreshClassName();
            }
        }

    }
    private class LocalRadioListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JRadioButton source = (JRadioButton) e.getSource();

            if (source.isSelected()) {
                refreshClassName();
            }
        }

    }

    /**
     * Class declaration
     *
     */
    private class LocalCheckListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = null;

            source = e.getSource();
            if (source == checkCompile && checkCompile.isSelected()) {
                checkStatic.setSelected(false);
            } else if (source == checkStatic && checkStatic.isSelected()) {
                checkCompile.setSelected(false);
            }
            enableCompileOptions(checkCompile.isSelected()
                                 && (!checkStatic.isSelected()));
        }

    }
}
