/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard;

// Kelp imports
import org.enhydra.kelp.common.Constants;

// Standard imports
import java.awt.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Arrays;

//
public class ExtensionPanel extends JPanel {
    private GridBagLayout             layoutRoot;
    private JPanel                    panelList;
    private JPanel                    panelButtons;
    private GridBagLayout             layoutList;
    private GridBagLayout             layoutButtons;
    private JLabel                    labelList;
    private JList                     listExt;
    private JButton                   buttonAdd;
    private JButton                   buttonRemove;
    private JButton                   buttonReset;
    private String[]                  resetList = null;
    private LocalButtonListener       buttonListener = null;
    private LocalListListener         listListener = null;
    private JScrollPane               scrollPane;
    private String[]                  readOnly = new String[0];
    private ExtensionChangeListener[] listeners =
        new ExtensionChangeListener[0];

    public ExtensionPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // Just for testing
    public static void main(String[] args) {
        String[]       exts = {
            Constants.TYPE_HTM, Constants.TYPE_HTML, Constants.TYPE_WML
        };
        String[]       ro = {
            Constants.TYPE_HTML
        };
        JFrame         frame = new JFrame();
        ExtensionPanel panel = new ExtensionPanel();

        panel.setExtensions(exts);
        panel.setReadOnly(ro);
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(panel, BorderLayout.CENTER);
        frame.pack();
        frame.show();
    }

    public void addListener(ExtensionChangeListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(listeners));
        if (list.contains(l)) {}
        else {
            list.add(l);
            list.trimToSize();
            listeners = new ExtensionChangeListener[list.size()];
            listeners = (ExtensionChangeListener[]) list.toArray(listeners);
        }
        list.clear();
    }

    public void removeListener(ExtensionChangeListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(listeners));
        if (list.contains(l)) {
            list.remove(l);
            list.trimToSize();
            listeners = new ExtensionChangeListener[list.size()];
            listeners = (ExtensionChangeListener[]) list.toArray(listeners);
        }
        list.clear();
    }

    public ExtensionChangeListener[] getListeners() {
        return listeners;
    }

    public String[] getReadOnly() {
        return readOnly;
    }

    public void setReadOnly(String[] r) {
        readOnly = r;
    }

    public String[] getExtensions() {
        String[]  exts = new String[0];
        ListModel model = listExt.getModel();

        exts = new String[model.getSize()];
        for (int i = 0; i < model.getSize(); i++) {
            exts[i] = model.getElementAt(i).toString();
        }
        return exts;
    }

    public void setExtensions(String[] exts) {
        if (exts == null) {
            exts = new String[0];
        }
        listExt.setListData(exts);
        listExt.updateUI();
        if (resetList == null) {
            resetList = new String[exts.length];
            for (int i = 0; i < exts.length; i++) {
                resetList[i] = new String(exts[i]);
            }
        }
    }

    //
    // PRIVATE
    //
    private void notifyListeners() {
        ExtensionChangeEvent event = null;

        event = new ExtensionChangeEvent(this, getExtensions());
        for (int i = 0; i < listeners.length; i++) {
            listeners[i].onChange(event);
        }
    }

    private void pmInit() {
        buttonListener = new LocalButtonListener();
        listListener = new LocalListListener();
        buttonRemove.addActionListener(buttonListener);
        buttonReset.addActionListener(buttonListener);
        buttonAdd.addActionListener(buttonListener);
        listExt.addListSelectionListener(listListener);
        buttonRemove.setEnabled(false);
    }

    private void jbInit() throws Exception {
        panelList = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        panelButtons = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        layoutList =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutRoot =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutButtons =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        labelList = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        listExt = (JList) Beans.instantiate(getClass().getClassLoader(),
                                            JList.class.getName());
        buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        buttonRemove =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonReset = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        labelList.setDisplayedMnemonic('X');
        labelList.setLabelFor(listExt);
        labelList.setText("Associated extensions:");
        buttonAdd.setPreferredSize(new Dimension(85, 27));
        buttonAdd.setMnemonic('A');
        buttonAdd.setText("Add...");
        buttonRemove.setPreferredSize(new Dimension(85, 27));
        buttonRemove.setMnemonic('V');
        buttonRemove.setText("Remove");
        buttonReset.setPreferredSize(new Dimension(85, 27));
        buttonReset.setText("Reset");
        scrollPane = new JScrollPane(listExt);
        scrollPane.setMinimumSize(new Dimension(100, 100));
        scrollPane.setPreferredSize(new Dimension(100, 100));
        panelList.setLayout(layoutList);
        panelList.add(labelList,
                      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(10, 10, 2, 10), 0,
                                             0));
        panelList.add(scrollPane,
                      new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.BOTH,
                                             new Insets(2, 10, 10, 10), 0,
                                             0));
        panelButtons.setLayout(layoutButtons);
        panelButtons.add(buttonAdd,
                         new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        panelButtons.add(buttonRemove,
                         new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        panelButtons.add(buttonReset,
                         new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        this.setLayout(layoutRoot);
        this.add(panelList,
                 new GridBagConstraints(0, 0, 1, 1, 0.2, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(panelButtons,
                 new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.EAST,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
    }

    private void reset() {
        setExtensions(resetList);
    }

    private String[] getSelections() {
        Object[] values = new Object[0];
        String[] selections = new String[0];

        values = listExt.getSelectedValues();
        selections = new String[values.length];
        for (int i = 0; i < selections.length; i++) {
            selections[i] = values[i].toString();
        }
        values = new Object[0];
        return selections;
    }

    private void removeSelection() {
        ArrayList list = null;
        String[]  exts = getSelections();

        if (exts.length >= 1) {
            list = new ArrayList(Arrays.asList(getExtensions()));
            for (int i = 0; i < exts.length; i++) {
                list.remove(exts[i]);
            }
            list.trimToSize();
            exts = new String[list.size()];
            exts = (String[]) list.toArray(exts);
            setExtensions(exts);
            list.clear();
        }
    }

    private boolean promptForAdd() {
        boolean change = false;
        Object  input = null;

        input = JOptionPane.showInputDialog(this, "Enter new extension",
                                            "Add Custom Extension",
                                            JOptionPane.QUESTION_MESSAGE);
        if (input == null) {}
        else {
            String[]  exts = new String[0];
            String    newExt = new String();
            ArrayList list = null;

            newExt = input.toString();
            list = new ArrayList(Arrays.asList(getExtensions()));
            if (list.contains(newExt)) {}
            else if (extensionValid(newExt)) {
                list.add(newExt);
            }
            list.trimToSize();
            exts = new String[list.size()];
            exts = (String[]) list.toArray(exts);
            setExtensions(exts);
            list.clear();
            change = true;
        }
        return change;
    }

    private boolean extensionValid(String testExt) {
        boolean valid = true;

        testExt = testExt.trim();
        if (testExt.length() == 0) {
            valid = false;
        } else if (testExt.indexOf(' ') > -1) {
            valid = false;
        } else {
            for (int i = 0; i < testExt.length(); i++) {
                char c = testExt.charAt(i);

                if (!Character.isLetterOrDigit(c)) {
                    valid = false;
                    break;
                }
            }
        }
        return valid;
    }

    class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonRemove) {
                removeSelection();
                notifyListeners();
            } else if (source == buttonReset) {
                reset();
                notifyListeners();
            } else if (source == buttonAdd) {
                if (promptForAdd()) {
                    notifyListeners();
                }
            }
        }

    }

    //
    class LocalListListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            if (e.getValueIsAdjusting()) {

                // do nothing
            } else {
                boolean  enable = true;
                String[] selections = getSelections();

                if (selections.length >= 1) {
                    ArrayList list = null;

                    list = new ArrayList(Arrays.asList(getReadOnly()));
                    for (int i = 0; i < selections.length; i++) {
                        if (list.contains(selections[i])) {
                            enable = false;
                            break;
                        }
                    }
                }
                buttonRemove.setEnabled(enable);
            }
        }

    }
}
