/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.deploy;

// Enhydra imports
import org.enhydra.xml.xmlc.XMLCVersion;

// Kelp imports
import org.enhydra.kelp.common.capture.Writer;
import org.enhydra.kelp.common.capture.WriteEvent;
import org.enhydra.kelp.common.capture.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.wizard.AbstractBuildDialog;
import org.enhydra.kelp.common.wizard.SelectionPanelEvent;

// Standard imports
import java.awt.*;
import java.awt.event.*;
import java.beans.Beans;
import javax.swing.*;
import java.util.ResourceBundle;

/**
 * This class defines the main dialog for the XMLC Wizard.
 *
 * @author Paul Mahar
 */
public class DeployBuildDialog extends AbstractBuildDialog {

    //
    private DeployPropertyPanel optionPanel;
    private DeployBuildTool     deployTool = null;

    public static void main(String[] args) {
        DeployBuildDialog dlg = new DeployBuildDialog(new Frame(),
                new String(), true);

        dlg.show();
        System.exit(0);
    }

    protected TemplateNodePropertyPanel getInputPanel() {
        return optionPanel.getInputPanel();
    }

    /**
     * Create the dialog as a child of a frame.
     *
     * @param owner
     * The parent Frame used when the dialog is modal.
     * @param title
     * The title for the dialog.
     * @param modal
     * Normally this is set to true for wizards.
     */
    public DeployBuildDialog(Frame owner, String title, boolean modal) {
        super(owner, title, modal);
        construct();
    }

    /**
     * Create the dialog as a child of another dialog.
     *
     * @param owner
     * The parent Dialog used when the dialog is modal.
     * @param title
     * The title for the dialog.
     * @param modal
     * Normally this is set to true for wizards.
     */
    public DeployBuildDialog(Dialog owner, String title, boolean modal) {
        super(owner, title, modal);
        construct();
    }

    public void onSelection(SelectionPanelEvent e) {

        // override and do nothing.
    }

    public void show() {
        selectOptionTab();
        super.show();
    }

    //
    protected String getProgressTitle() {
        return res.getString("Deployment_Progress");
    }

    private void construct() {
        try {
            optionPanel =
                (DeployPropertyPanel) Beans.instantiate(getClass().getClassLoader(),
                                                        DeployPropertyPanel.class.getName());
            init(optionPanel);
        } catch (Exception e) {
            e.printStackTrace();
        }
        buttonMake.setText(res.getString("Deploy"));
        buttonMake.setEnabled(true);
        getSelectionPanel().labelAvailable.setText(res.getString("Available_Inputs"));
        getSelectionPanel().labelSelected.setText(res.getString("Selected_Inputs"));
    }

    /**
     * Method declaration
     *
     */
    public void make() {
        if (deployTool != null) {
            deployTool.setFresh(false);
        }
        WriteListener[] wl = new WriteListener[getWriteListeners().size()];

        for (int i = 0; i < getWriteListeners().size(); i++) {
            wl[i] = (WriteListener) getWriteListeners().elementAt(i);
        }
        deployTool = new DeployBuildTool(wl);
        deployTool.setProject(getProject());
        deployTool.setNodes(getNodes());
        deployTool.setEcho(true);    // always echo from dialog
        deployTool.addProgressListener(getProgressMeter());
        getProgressMeter().addCancelListener(deployTool);
        deployTool.start();
    }

}
