/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.deploy;

// Toolbox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ArchiveTool;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.WarPlan;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.wizard.AbstractBuildTool;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.capture.Writer;
import org.enhydra.kelp.common.capture.WriteListener;

// Standard imports
import java.util.StringTokenizer;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ResourceBundle;

/**
 * CompileThread calls XMLC once for each selected HTML file.
 *
 *
 * @author
 * Paul Mahar
 */
public class DeployBuildTool extends AbstractBuildTool {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private File          startServerDirectory = null;
    private boolean       nodeOnly = false;

    public boolean isNodeOnly() {
        return nodeOnly;
    }

    public void setNodeOnly(boolean b) {
        nodeOnly = b;
    }

    /**
     * Method declaration
     *
     *
     * @param in
     *
     * @return
     */
    public static File createPolicyFile(String path) {

        // strings not to be resourced
        final String POLICY_LINE1 = "grant {";            // nores
        final String POLICY_LINE2 =
            "permission java.security.AllPermission;";    // nores
        final String POLICY_LINE3 = "};";                 // nores
        File         in = new File(path);
        File         out = null;

        if (in.exists()) {
            in.delete();
            in = null;
        }
        try {
            out = new File(path);
            out.getParentFile().mkdirs();
            out.createNewFile();
            PrintWriter writer = new PrintWriter(new FileWriter(out));

            writer.println(POLICY_LINE1);
            writer.println(POLICY_LINE2);
            writer.println(POLICY_LINE3);
            writer.flush();
            writer.close();
            out = null;
            out = new File(path);
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
        return out;
    }

    /**
     * Constructor declaration
     *
     */
    public DeployBuildTool() {
        super();
    }

    /**
     * Method declaration
     *
     *
     * @param d
     */
    public void setStartServerDirectory(File d) {
        startServerDirectory = d;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public File getStartServerDirectory() {
        return startServerDirectory;
    }

    /**
     *
     * @param listeners
     */
    public DeployBuildTool(WriteListener[] listeners) {
        super(listeners);
    }

    /**
     * Constructor declaration
     *
     *
     * @param listeners
     * @param buffer
     */
    public DeployBuildTool(WriteListener[] listeners, boolean buffer) {
        super(listeners, buffer);
    }

    /**
     * Make content, policy file, process templates, configure project for
     * running enhydra and create archive.
     */
    public void make() {
        if (!isNodeOnly()) {
            echo(new String());
            refreshProgress(5, res.getString("Deployment_starting"));
            echo(res.getString("_Deployment_starting"));
            if (isFresh()) {
                refreshProgress(5, res.getString("Deploying_content"));
                echo(res.getString("Deploying_content"));
                makeContent();
            }
            if (isFresh()) {
                if (!ToolBoxInfo.isEnhydra3()) {
                    refreshProgress(25,
                                    res.getString("Deploying_policy_file"));
                    echo(res.getString("Deploying_policy_file"));
                    makePolicy();
                }
            }
        }
        if (isFresh()) {
            if (getNodes().length >= 1) {
                refreshProgress(45, res.getString("Processing_input"));
                echo(res.getString("Processing_input"));
                makeSelectedInput();
            }
        }
        if (!isNodeOnly()) {
            if (isFresh()) {
                if (getProject().isDeployStartupJava()
                        || getProject().isDeployStartupRun()) {
                    refreshProgress(65, res.getString("Configuring_project"));
                    echo(res.getString("Configuring_project"));
                    makeStartup();
                }
            }
            if (isFresh()) {
                refreshProgress(80, res.getString("Building_archive"));
                echo(res.getString("Building_archive"));
                makeArchive();
                refreshProgress(95, res.getString("Deployment_complete"));
                echo(res.getString("_deployment_complete"));
            } else {
                echo(res.getString("_deployment_interrupted"));
            }
        }
        refreshProgress(100, new String());
    }

    /**
     */
    public void addBuiltTemplates(File[] built) {
        int              eCount = 0;
        int              selectCount = 0;
        OtterNodeFactory factory = null;

        factory = getProject().getNodeFactory();
        for (int i = 0; i < built.length; i++) {
            if (!isFresh()) {
                break;
            }
            factory.importFile(getProject(), built[i]);
        }
    }

    public void printResults() {

        // do nothing.
    }

    private void makeContent() {
        ContentBuilder builder = null;

        builder = new ContentBuilder();
        builder.setProject(getProject());
        builder.setEcho(isEcho());
        builder.setEchoWriter(getEchoWriter());
        builder.make();
    }

    /**
     * Create policy file for security manager
     */
    private File createPolicy() {
        StringBuffer policyPath = new StringBuffer();
        File         policy = null;

        policyPath.append(PathUtil.getDeployConfPath(getProject()));
        policyPath.append(File.separator);
        policyPath.append(Constants.FILE_JAVA_POLICY);
        policy =
            DeployBuildTool.createPolicyFile(policyPath.toString().replace('\\',
                '/'));
        return policy;
    }

    /**
     * Copy or create policy file as needed.
     */
    private void makePolicy() {

        // Find all images in project and copy using
        // the mapping table.
        OtterTextFileNode node = getProject().getFirstPolicy();

        if (node == null) {
            File file = null;

            file = createPolicy();
            echo(res.getString("Creating_policy_file")
                 + PathHandle.createPathString(file));
        } else {
            File source = null;
            File dest = null;

            source = new File(node.getFilePath());
            dest = new File(PathUtil.getDeployConfPath(getProject()),
                            source.getName());
            try {
                FileUtil.copyFile(source, dest);
            } catch (ToolException e) {
                echo(e);
                e.printStackTrace();
            }
        }
    }

    /**
     * Configure project for starting server
     */
    private void makeStartup() {
        if (getProject().isDeployStartupJava()) {
            OtterNodeFactory factory = null;
            StartServer      start = null;
            PathHandle       startPath = null;

            factory = getProject().getNodeFactory();
            start = new StartServer(getProject());
            start.setDirectory(getStartServerDirectory());
            try {
                start.create();
                startPath = PathHandle.createPathHandle(start.getFile());
                factory.createJavaFileNode(getProject(), startPath.getPath());
                echo(res.getString("Creating_startup") + startPath.getPath());
            } catch (java.io.IOException e) {
                echo(e);
                e.printStackTrace();
            }
        }
        if (getProject().isDeployStartupRun()) {
            getProject().configureRunClass();
        }
    }

    /**
     * Process templates
     */
    private void makeSelectedInput() {
        InputBuilder  builder = null;
        ToolException except = null;

        builder = new InputBuilder();
        builder.setEchoWriter(getEchoWriter());
        builder.setEcho(isEcho());
        builder.setProject(getProject());
        try {
            builder.make();
        } catch (ToolException e) {
            except = e;
        }
        printExceptions();
        if (except != null) {
            StringTokenizer tokenizer =
                new StringTokenizer(except.getMessage(), Constants.NEWLINE);

            while (tokenizer.hasMoreTokens()) {
                getEchoWriter().println(tokenizer.nextToken());
            }
        }
        addBuiltTemplates(builder.getBuiltTemplates());
    }

    private void makeArchive() {
        ArchiveTool tool = new ArchiveTool();
        JarPlan     plan = null;
        String      archivePath = new String();

        try {
            archivePath = PathUtil.getDeployArchivePath(getProject());
            if (getProject().isWebApplication()) {
                plan = createWarPlan();
            } else {
                plan = new JarPlan();
            }
            plan.setClasses(getProject().getClassOutputPath());
            plan.setArchivePath(archivePath);
            echo(res.getString("Creating_archive") + archivePath);
            tool.buildArchive(plan);
        } catch (ArchiveException e) {
            echo(e);
        }
    }

    private WarPlan createWarPlan() throws ArchiveException {
        WarPlan plan = null;

        plan = new WarPlan();
        plan.setContent(PathUtil.getDeployContentPath(getProject()));
        return plan;
    }

    private void echo(String message) {
        if (isEcho()) {
            getEchoWriter().println(message);
        }
    }

    private void echo(Exception e) {
        getEchoWriter().println(e.getMessage());
    }

}

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
class StartServer {
    private OtterProject project = null;
    private File         directory;
    private File         file = null;

    /**
     * Constructor declaration
     *
     *
     * @param p
     */
    protected StartServer(OtterProject p) {
        project = p;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected File getDirectory() {
        if (directory == null) {
            String[] paths = project.getSourcePathArray();

            directory = new File(paths[0]);
        }
        return directory;
    }

    /**
     * Method declaration
     *
     *
     * @param d
     */
    protected void setDirectory(File d) {
        if (d != null) {
            directory = d;
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected File getFile() {
        String filename = Constants.FILE_START_SERVER;

        file = new File(getDirectory(), filename);
        return file;
    }

    /**
     * Write a StartServer.java file for launching and stopping the server
     * with a simple graphical interface. This is useful in environments such
     * as the JDeveloper where you cannot run an arbitrary class from a project.
     *
     * @throws IOException
     */
    public void create() throws java.io.IOException {
        if ((!getFile().exists()) || project.isDeployOverwrite()) {
            PrintWriter  out = new PrintWriter(new FileWriter(getFile()));
            StringBuffer startBuf = new StringBuffer();

            startBuf.append(PathUtil.getDeployConfPath(project));
            if (!ToolBoxInfo.isEnhydra3()) {
                startBuf.append(File.separator);
                startBuf.append(Constants.FILE_BOOTSTRAP4_CONF);
            }

            // strings not to be translated
            out.println("");                                        // nores
            out.println("import java.awt.*;");                      // nores
            out.println("import java.awt.event.*;");                // nores
            out.println("");                                        // nores
            out.println("public class StartServer extends Frame {");    // nores
            out.println("  Button button = new Button();");         // nores
            out.println("");                                        // nores
            out.println("  public static void main(String[] args) {");    // nores
            out.println("    if (args == null || args.length < 1) {");    // nores
            out.println("      args = new String[1];");             // nores
            out.println("      args[0] = \""
                        + doubleEscape(startBuf.toString().replace('\\', '/'))
                        + "\";");                                   // nores
            out.println("     }");                                  // nores
            out.println("    " + ToolBoxInfo.getEnhydraMainClass()
                        + ".main(args);");                          // nores
            out.println("    System.setSecurityManager(null);");    // nores
            out.println("    StartServer frame = new StartServer();");    // nores
            out.println("    frame.setSize(200,100);");             // nores
            out.println("    frame.setLocation(new Point(200, 200));");    // nores
            out.println("    frame.show();");                       // nores
            out.println("    frame.setLocation(new Point(200, 200)); // reset for Linux AWT");    // nores
            out.println("  }");                                     // nores
            out.println("");                                        // nores
            out.println("    public StartServer() {");              // nores
            out.println("      button.setLabel(\"Shutdown Server\");");    // nores
            out.println("      button.addActionListener(new ActionListener() {");    // nores
            out.println("        public void actionPerformed(ActionEvent e) {");    // nores
            out.println("         System.exit(0);");                // nores
            out.println("        }");                               // nores
            out.println("      });");                               // nores
            out.println("      add(button);");                      // nores
            out.println("      enableEvents(java.awt.AWTEvent.WINDOW_EVENT_MASK);");    // nores
            out.println("      setTitle(\"Enhydra\");");            // nores
            out.println("    }");                                   // nores
            out.println("");                                        // nores
            out.println("  protected void processWindowEvent(java.awt.event.WindowEvent e) {");    // nores
            out.println("    super.processWindowEvent(e);");        // nores
            out.println("    if (e.getID() == java.awt.event.WindowEvent.WINDOW_CLOSING) {");    // nores
            out.println("      System.exit(0);");                   // nores
            out.println("    }");                                   // nores
            out.println("  }");                                     // nores
            out.println("");                                        // nores
            out.println("}");                                       // nores
            out.println("");                                        // nores
            out.flush();
            out.close();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param raw
     *
     * @return
     */
    private String doubleEscape(String raw) {
        StringBuffer doubled = new StringBuffer();
        String       in = new String(raw);
        int          index = in.indexOf('\\');

        while (index > -1) {
            doubled.append(in.substring(0, index));
            doubled.append('\\');
            doubled.append('\\');
            if (index == in.lastIndexOf('\\')) {
                if ((in.length() == 0)
                        || (in.charAt(in.length() - 1) != '\\')) {
                    doubled.append(in.substring(index + 1));
                }
                in = new String();
            } else {
                in = in.substring(index + 1);
            }
            index = in.indexOf('\\');
        }
        if (doubled.toString().trim().length() == 0) {

            // Should not be any \ on Linux so
            // no need to double escape.
            doubled.append(raw);
        }
        return doubled.toString();
    }

}
