/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.deploy;

// Kelp imports
import org.enhydra.kelp.common.wizard.AbstractPropertyPanel;
import org.enhydra.kelp.common.wizard.ExtensionPanel;
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.beans.*;
import java.awt.event.*;
import java.io.File;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class DeployPropertyPanel extends AbstractPropertyPanel {
    static ResourceBundle             res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private boolean                   active = false;
    private JTabbedPane               tabPane;
    private TemplateNodePropertyPanel templatePropertyPanel;
    private JCheckBox                 checkBuild;
    private JCheckBox                 checkEcho;
    private JPanel                    filler;
    private GeneralPanel              generalPanel;
    private ExtensionPanel            extensionPanel;
    private PathPanel                 pathPanel;
    private GridBagLayout             layoutMain;
    private LocalItemListener         itemListener;

    /**
     * Constructor declaration
     *
     */
    public DeployPropertyPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected TemplateNodePropertyPanel getInputPanel() {
        return templatePropertyPanel;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterProject getProject() {
        return project;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     */
    public void setProject(OtterProject p) {
        project = p;
        pathPanel.setProject(project);
        generalPanel.setProject(project);
        templatePropertyPanel.setNode(project);
    }

    public void readProperties() {
        if (project.isOpenBuild()) {

            // primetime
            checkBuild.setEnabled(true);
            checkBuild.setSelected(project.isDeployBuild());
            checkEcho.setEnabled(checkBuild.isSelected());
            checkEcho.setSelected(project.isDeployEcho());
        } else {

            // Win32 and JDeveloper
            checkBuild.setEnabled(false);
            checkBuild.setSelected(false);
            checkEcho.setEnabled(false);
            checkEcho.setSelected(false);
        }
        extensionPanel.setExtensions(project.getContentTypes());
        extensionPanel.setReadOnly(project.getDocTypes());
        pathPanel.readProperties();
        generalPanel.readProperties();
        templatePropertyPanel.readProperties();
    }

    public void writeProperties() {
        if (project != null) {
            if (project.isOpenBuild()) {

                // primetime
                project.setDeployBuild(checkBuild.isSelected());
                project.setDeployEcho(checkEcho.isSelected());
            } else {
                project.setDeployBuild(false);
                project.setDeployEcho(false);
            }
            project.setContentTypes(extensionPanel.getExtensions());
            pathPanel.writeProperties();
            generalPanel.writeProperties();
            templatePropertyPanel.writeProperties();
        }
    }

    //
    protected PathPanel getPathPanel() {
        return pathPanel;
    }

    //
    private void pmInit() {
        itemListener = new LocalItemListener();
        checkBuild.addItemListener(itemListener);
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        checkBuild =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkEcho = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JCheckBox.class.getName());
        filler = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                            JPanel.class.getName());
        generalPanel =
            (GeneralPanel) Beans.instantiate(getClass().getClassLoader(),
                                             GeneralPanel.class.getName());
        extensionPanel =
            (ExtensionPanel) Beans.instantiate(getClass().getClassLoader(),
                                               ExtensionPanel.class.getName());
        pathPanel = (PathPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  PathPanel.class.getName());
        templatePropertyPanel =
            (TemplateNodePropertyPanel) Beans.instantiate(getClass().getClassLoader(),
                TemplateNodePropertyPanel.class.getName());
        tabPane =
            (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                            JTabbedPane.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        tabPane.add(pathPanel, res.getString("Paths"));
        tabPane.add(extensionPanel, res.getString("Content_Types"));
        tabPane.add(generalPanel, res.getString("General"));
        tabPane.add(templatePropertyPanel, res.getString("Input"));
        checkBuild.setToolTipText(res.getString("deployCheckBuildlTip"));
        checkBuild.setEnabled(false);
        checkBuild.setText(res.getString("deployCheckBuildText"));
        checkEcho.setToolTipText(res.getString("deployCheckEchoTip"));
        checkEcho.setEnabled(false);
        checkEcho.setText(res.getString("deployCheckEchoText"));
        this.setLayout(layoutMain);
        this.add(checkBuild,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.NONE,
                                        new Insets(5, 5, 3, 5), 20, 0));
        this.add(checkEcho,
                 new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.NONE,
                                        new Insets(5, 5, 3, 5), 20, 0));
        this.add(filler,
                 new GridBagConstraints(2, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 3, 5), 20, 0));
        this.add(tabPane,
                 new GridBagConstraints(0, 1, 3, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(2, 5, 2, 5), 0, 0));
    }

    class LocalItemListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            checkEcho.setEnabled(e.getStateChange() == ItemEvent.SELECTED);
        }

    }
}
