/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.deploy;

// ToolBox imports
import org.enhydra.tool.common.SwingUtil;

// Kelp imports
import org.enhydra.kelp.common.wizard.AbstractPropertyPanel;
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;
import javax.swing.border.*;
import java.beans.*;
import java.util.ResourceBundle;

//
public class GeneralPanel extends AbstractPropertyPanel {
    static ResourceBundle      res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private GridBagLayout      layoutMain;
    private TitledBorder       borderResources;
    private JRadioButton       radioEnhydraApp;
    private JRadioButton       radioWebApp;
    private ButtonGroup        radioGroup;
    private LocalRadioListener radioListener = null;
    private JPanel             panelAPI;
    private GridBagLayout      layoutAPI;
    private JPanel             panelFiller;
    private BorderLayout       layoutFiller;
    private StartupPanel       startupPanel;

    public GeneralPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void setProject(OtterProject p) {
        super.setProject(p);
        startupPanel.setProject(p);
    }

    public void readProperties() {
        if (project.isWebApplication()) {
            radioWebApp.setSelected(true);
            radioEnhydraApp.setSelected(false);
        } else {
            radioWebApp.setSelected(false);
            radioEnhydraApp.setSelected(true);
        }
        startupPanel.readProperties();
    }

    public void writeProperties() {
        project.setWebApplication(radioWebApp.isSelected());
        startupPanel.writeProperties();
    }

    private void browseForPath(JTextField field, String caption) {
        File choice = null;

        choice = SwingUtil.getDirectoryChoice(this, field.getText(), caption);
        if (choice != null && choice.isDirectory()) {
            field.setText(choice.getAbsolutePath());
        }
    }

    private void pmInit() {
        radioGroup = new ButtonGroup();
        radioGroup.add(radioEnhydraApp);
        radioGroup.add(radioWebApp);
        radioListener = new LocalRadioListener();
        radioEnhydraApp.addActionListener(radioListener);
        radioWebApp.addActionListener(radioListener);
    }

    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        radioEnhydraApp =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioWebApp =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        panelAPI = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        layoutAPI =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        startupPanel =
            (StartupPanel) Beans.instantiate(getClass().getClassLoader(),
                                             StartupPanel.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        layoutFiller =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        radioEnhydraApp.setText(res.getString("radioEnhydraAppText"));
        radioEnhydraApp.setToolTipText(res.getString("radioEnhydraAppTip"));
        radioWebApp.setText(res.getString("radioWebAppText"));
        radioWebApp.setToolTipText(res.getString("radioWebAppTip"));
        startupPanel.setBorder(BorderFactory.createTitledBorder(res.getString("startupPanelBorder")));

        // API
        panelAPI.setBorder(BorderFactory.createTitledBorder(res.getString("panelAPIBorder")));
        panelAPI.setLayout(layoutAPI);
        panelFiller.setLayout(layoutFiller);
        panelAPI.add(radioWebApp,
                     new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 5, 5), 0, 0));
        panelAPI.add(radioEnhydraApp,
                     new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 5, 5), 0, 0));

        // Main
        this.setLayout(layoutMain);
        this.add(panelAPI,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 5, 5), 5, 5));
        this.add(startupPanel,
                 new GridBagConstraints(0, 2, 1, 1, 0.2, 0.2,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 10, 10));
        this.add(panelFiller,
                 new GridBagConstraints(0, 2, 1, 1, 0.2, 0.2,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 10, 10));
    }

    private void refreshPathPanel() {
        DeployPropertyPanel parentPanel = null;

        if (getParent().getParent() instanceof DeployPropertyPanel) {
            project.setWebApplication(radioWebApp.isSelected());
            parentPanel = (DeployPropertyPanel) getParent().getParent();
            parentPanel.getPathPanel().readProperties();
        }
    }

    //
    private class LocalRadioListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            refreshPathPanel();
        }

    }
}
