/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.deploy;

// ToolBox imports
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ExtensionFilter;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.wizard.AbstractPropertyPanel;

// Standard imports
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import javax.swing.*;
import javax.swing.border.*;
import java.util.ResourceBundle;

//
public class PathPanel extends AbstractPropertyPanel {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private LocalButtonListener buttonListener = null;
    private GridBagLayout       layoutMain;
    private JLabel              labelRoot;
    private JTextField          textRoot;
    private JButton             buttonRoot;
    private JLabel              labelBootstrap;
    private JTextField          textBootstrap;
    private JButton             buttonBootstrap;
    private JPanel              panelContent;
    private GridBagLayout       layoutInput;
    private GridBagLayout       layoutContent;
    private JLabel              labelInput;
    private JTextField          textInput;
    private JButton             buttonInput;
    private JLabel              labelConf;
    private JTextField          textConf;
    private JLabel              labelResource;
    private JTextField          textResource;
    private JButton             buttonResource;
    private JLabel              labelContent;
    private JTextField          textContent;
    private JLabel              labelArchive;
    private JTextField          textArchive;

    public PathPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void readProperties() {
        if (project == null) {
            System.out.println("PathPanel.readProperties: project is null");    // nores
        } else {
            textRoot.setText(project.getDeployRootPath());
            textRoot.setToolTipText(textRoot.getText());
            textBootstrap.setText(project.getDeployBootstrapPath());
            textBootstrap.setToolTipText(textBootstrap.getText());
            textInput.setText(project.getDeployInputPath());
            textInput.setToolTipText(textInput.getText());
            textResource.setText(project.getDeployResourcePath());
            textResource.setToolTipText(textResource.getText());
            refreshPaths();
        }
    }

    public void writeProperties() {
        if (project == null) {
            System.out.println("PathPanel.writeProperties: project is null");    // nores
        } else {
            project.setDeployRootPath(textRoot.getText());
            project.setDeployBootstrapPath(textBootstrap.getText());
            project.setDeployInputPath(textInput.getText());
            project.setDeployResourcePath(textResource.getText());
        }
    }

    private void pmInit() {
        Dimension dim = new Dimension();

        dim.height = (int) textRoot.getPreferredSize().getHeight();
        dim.width = 120;
        textRoot.setPreferredSize(dim);
        textBootstrap.setPreferredSize(dim);
        textArchive.setPreferredSize(dim);
        textInput.setPreferredSize(dim);
        textConf.setPreferredSize(dim);
        textResource.setPreferredSize(dim);
        textContent.setPreferredSize(dim);
        buttonListener = new LocalButtonListener();
        buttonRoot.addActionListener(buttonListener);
        buttonBootstrap.addActionListener(buttonListener);
        buttonInput.addActionListener(buttonListener);
        buttonResource.addActionListener(buttonListener);
    }

    private boolean browseForDirectory(JTextField field, String caption) {
        File    choice = null;
        boolean selection = false;

        choice = SwingUtil.getDirectoryChoice(this, field.getText(), caption);
        if (choice != null && choice.isDirectory()) {
            field.setText(choice.getAbsolutePath().replace('\\', '/'));
            field.setToolTipText(field.getText());
            selection = true;
        }
        return selection;
    }

    private void browseForFile(JTextField field, String caption, String ext) {
        File            choice = null;
        ExtensionFilter filter = new ExtensionFilter();

        filter.addExtension(ext);
        choice = SwingUtil.getFileChoice(this, new File(field.getText()),
                                         filter, caption);
        if (choice != null && choice.isFile()) {
            field.setText(choice.getAbsolutePath().replace('\\', '/'));
            field.setToolTipText(field.getText());
        }
    }

    private void refreshPaths() {
        textArchive.setText(PathUtil.getDeployArchivePath(textRoot.getText(),
                project));
        textArchive.setToolTipText(textArchive.getText());
        textConf.setText(PathUtil.getDeployConfPath(textRoot.getText()));
        textConf.setToolTipText(textConf.getText());
        textContent.setText(PathUtil.getDeployContentPath(project,
                textRoot.getText()));
        textContent.setToolTipText(textContent.getText());
    }

    private void browseForInput() {
        DeployBuildDialog dialog = null;

        if (getTopLevelAncestor() instanceof DeployBuildDialog) {
            dialog = (DeployBuildDialog) getTopLevelAncestor();
        }
        String before = textInput.getText();

        browseForDirectory(textInput, res.getString("Select_input"));
        if ((dialog == null) || before.equals(textInput.getText())) {

            // nothing to refresh;
        } else {
            project.setDeployInputPath(textInput.getText());
            dialog.getSelectionPanel().setNodes(project.getAllInput());
            dialog.getInputPanel().readFilter();
        }
    }

    private void jbInit() throws Exception {
        textRoot = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        buttonRoot = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        labelBootstrap =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        textBootstrap =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonBootstrap =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        panelContent = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        layoutInput =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutContent =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        labelInput = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                JLabel.class.getName());
        textInput =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonInput = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        labelConf = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        textConf = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        labelResource =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        textResource =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonResource =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        labelContent = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        textContent =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        labelRoot = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        labelArchive = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        textArchive =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        labelRoot.setText(res.getString("labelRoot_Text"));
        buttonRoot.setText("...");         // nores
        labelBootstrap.setText(res.getString("labelBootstrap_Text"));
        buttonBootstrap.setText("...");    // nores
        labelInput.setText(res.getString("labelInput_Text"));
        buttonInput.setText("...");        // nores
        labelConf.setMaximumSize(new Dimension(110, 17));
        labelConf.setMinimumSize(new Dimension(110, 17));
        labelConf.setPreferredSize(new Dimension(110, 17));
        labelConf.setText(res.getString("labelConf_Text"));
        labelResource.setMaximumSize(new Dimension(110, 17));
        labelResource.setMinimumSize(new Dimension(110, 17));
        labelResource.setPreferredSize(new Dimension(110, 17));
        labelResource.setText(res.getString("labelResource_Text"));
        buttonResource.setText("...");     // nores
        labelContent.setText(res.getString("labelContent_Text"));
        labelArchive.setText(res.getString("labelArchive_Text"));
        textRoot.setEditable(false);
        textBootstrap.setEditable(false);
        textResource.setEditable(false);
        textInput.setEditable(false);
        textContent.setEditable(false);
        textConf.setEditable(false);
        textArchive.setEditable(false);

        //
        panelContent.setBorder(BorderFactory.createTitledBorder(res.getString("panelContent_Border")));
        panelContent.setLayout(layoutContent);
        panelContent.add(labelResource,
                         new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.WEST,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(2, 5, 2, 5), 0,
                                                0));
        panelContent.add(textResource,
                         new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(2, 5, 2, 5), 100,
                                                0));
        panelContent.add(buttonResource,
                         new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(2, 5, 2, 5), 0,
                                                0));
        panelContent.add(labelContent,
                         new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.WEST,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(2, 5, 5, 5), 0,
                                                0));
        panelContent.add(textContent,
                         new GridBagConstraints(1, 1, 2, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(2, 5, 5, 5), 0,
                                                0));

        //
        this.setLayout(layoutMain);
        this.add(labelRoot,
                 new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(10, 10, 2, 5), 0, 0));
        this.add(textRoot,
                 new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(10, 5, 2, 5), 120, 0));
        this.add(buttonRoot,
                 new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(10, 5, 2, 5), 0, 0));
        this.add(labelBootstrap,
                 new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 10, 0, 5), 0, 0));
        this.add(textBootstrap,
                 new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 120, 0));
        this.add(buttonBootstrap,
                 new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(labelInput,
                 new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 10, 2, 5), 0, 0));
        this.add(textInput,
                 new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 120, 0));
        this.add(buttonInput,
                 new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(labelConf,
                 new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 10, 5, 5), 0, 0));
        this.add(textConf,
                 new GridBagConstraints(1, 3, 2, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 2, 5), 0, 0));
        this.add(labelArchive,
                 new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 10, 5, 5), 0, 0));
        this.add(textArchive,
                 new GridBagConstraints(1, 4, 2, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 5, 5, 5), 0, 0));
        this.add(panelContent,
                 new GridBagConstraints(0, 5, 3, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(3, 5, 3, 5), 10, 10));
    }

    private class LocalButtonListener implements ActionListener {

        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonRoot) {
                if (browseForDirectory(textRoot,
                                       res.getString("Select_deploy_root"))) {
                    refreshPaths();
                }
            } else if (source == buttonBootstrap) {
                browseForFile(textBootstrap,
                              res.getString("Select_bootstrap"),
                              Constants.TYPE_CONF);
            } else if (source == buttonInput) {
                browseForInput();
            } else if (source == buttonResource) {
                browseForDirectory(textResource,
                                   res.getString("Select_resource"));
            }
        }

    }
}
