/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.xmlc;

// ToolBox imports
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;

// Kelp imports
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.*;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class MapEditorDialog extends JDialog {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    public static final int     ACTION_OK = 0;
    public static final int     ACTION_CANCEL = 1;
    private static File         startFolder = null;
    private int                 option = ACTION_CANCEL;
    private LocalButtonListener buttonListener = null;
    private JPanel              panelMain = new JPanel();
    private GridBagLayout       layoutMain;
    private JPanel              panelEntry;
    private GridBagLayout       layoutEntry;
    private JPanel              panelButtons;
    private GridBagLayout       layoutButton;
    private JLabel              labelFolder;
    private JTextField          textFolder;
    private JButton             buttonBrowse;
    private JLabel              labelPackage;
    private JTextField          textPackage;
    private JButton             buttonCancel;
    private JButton             buttonOK;
    private OtterProject        project;

    /**
     * Constructor declaration
     *
     *
     * @param owner
     * @param title
     * @param modal
     */
    public MapEditorDialog(Frame owner, String title, boolean modal) {
        super(owner, title, modal);
        construct();
    }

    public MapEditorDialog(JDialog owner, String title, boolean modal) {
        super(owner, title, modal);
        construct();
    }

    private void construct() {
        try {
            jbInit();
            pmInit();
            pack();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Constructor declaration
     *
     */
    public MapEditorDialog() {
        this(new Frame(), new String(), false);
    }

    /**
     * Method declaration
     *
     */
    public void show() {
        setResizable(false);
        Point cPoint;

        invalidate();
        doLayout();
        pack();
        cPoint = SwingUtil.getCenteringPoint(getSize());
        setLocation(cPoint);
        super.show();
    }

    /**
     * Method declaration
     *
     */
    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonBrowse.addActionListener(buttonListener);
        buttonOK.addActionListener(buttonListener);
        buttonCancel.addActionListener(buttonListener);
        textFolder.setText(new String());
        textPackage.setText(new String());
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelEntry = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                JPanel.class.getName());
        layoutEntry =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelButtons = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        layoutButton =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        labelFolder = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                 JLabel.class.getName());
        textFolder =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonBrowse =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        labelPackage = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        textPackage =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonCancel =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonOK = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                               JButton.class.getName());
        panelMain.setLayout(layoutMain);
        panelEntry.setLayout(layoutEntry);
        panelButtons.setLayout(layoutButton);
        labelFolder.setText(res.getString("labelFolder_Text"));
        textFolder.setPreferredSize(new Dimension(250, 21));
        buttonBrowse.setText(res.getString("Browse"));
        labelPackage.setText(res.getString("labelPackage_Text"));
        textPackage.setPreferredSize(new Dimension(250, 21));
        buttonCancel.setText(res.getString("Cancel"));
        buttonOK.setText(res.getString("OK"));
        getContentPane().add(panelMain);
        panelMain.add(panelEntry,
                      new GridBagConstraints(0, 0, 1, 1, 0.2, 0.2,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.BOTH,
                                             new Insets(0, 0, 0, 0), 0, 0));
        panelEntry.add(labelFolder,
                       new GridBagConstraints(0, 0, 1, 1, 0.2, 0.2,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(5, 5, 5, 5), 0, 0));
        panelEntry.add(textFolder,
                       new GridBagConstraints(0, 1, 1, 1, 0.4, 0.4,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(5, 5, 5, 5), 200,
                                              0));
        panelEntry.add(buttonBrowse,
                       new GridBagConstraints(1, 1, 1, 1, 0.2, 0.2,
                                              GridBagConstraints.CENTER,
                                              GridBagConstraints.NONE,
                                              new Insets(5, 5, 5, 5), 0, 0));
        panelEntry.add(labelPackage,
                       new GridBagConstraints(0, 2, 1, 1, 0.2, 0.2,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(5, 5, 5, 5), 0, 0));
        panelEntry.add(textPackage,
                       new GridBagConstraints(0, 3, 1, 1, 0.4, 0.4,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(5, 5, 5, 5), 200,
                                              0));
        panelMain.add(panelButtons,
                      new GridBagConstraints(0, 1, 1, 1, 0.2, 0.2,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.BOTH,
                                             new Insets(5, 5, 5, 0), 0, 0));
        panelButtons.add(buttonOK,
                         new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        panelButtons.add(buttonCancel,
                         new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
    }

    /**
     * Method declaration
     *
     */
    protected void closeDialog() {
        setVisible(false);
        removeAll();
        dispose();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected int getOption() {
        return option;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected String getFolder() {
        return textFolder.getText();
    }

    protected void setFolder(String folder) {
        textFolder.setText(folder);
    }

    protected void setProject(OtterProject p) {
        project = p;
    }

    protected OtterProject getProject() {
        return project;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected String getPackageName() {
        return textPackage.getText();
    }

    protected void setPackageName(String pack) {
        textPackage.setText(pack);
    }

    /**
     * Method declaration
     *
     *
     * @param o
     */
    private void setOption(int o) {
        option = o;
    }

    /**
     * Method declaration
     *
     */
    private void browseAction() {
        File choice;

        if (startFolder == null ||!startFolder.exists()) {
            if (textFolder.getText().trim().length() == 0) {
                File parent = new File(getProject().getFilePath());

                parent = new File(parent.getParent());
                startFolder = parent;

                //
            } else {
                startFolder = new File(textFolder.getText());
            }
        }
        choice =
            SwingUtil.getDirectoryChoice(this, startFolder,
                                         res.getString("Select_directory_to"));
        buttonBrowse.requestFocus();
        if (choice != null) {
            textFolder.setText(PathHandle.createPathString(choice));
            startFolder = choice;    // persist this choice for next call
        }
    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class LocalButtonListener implements ActionListener {

        /**
         * Method declaration
         *
         *
         * @param event
         */
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonBrowse) {
                browseAction();
            } else if (source == buttonOK) {
                setOption(ACTION_OK);
                closeDialog();
            } else if (source == buttonCancel) {
                setOption(ACTION_CANCEL);
                closeDialog();
            }
        }

    }
}
