/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.build;

// Toolbox imports
import org.enhydra.tool.common.PathHandle;

// JBuilder imports
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.build.Builder;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.build.BuilderManager;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.jbuilder.node.PrimeDocumentNode;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// Standard imports
import java.util.List;

/**
 * This class controls building and maintaining properties for HTML/XML files
 * using the Enhydra XMLC Compiler.
 *
 */
public class XMLCBuilder implements Builder {
    public XMLCBuilder() {}

    /**
     * Register open tool
     *
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        if (KelpInfo.verifyIDEClassPath()) {
            BuilderManager.registerBuilder(new XMLCBuilder());
        }
    }

    /**
     * Called by the Build System when make or rebuild occurs
     */
    public void updateBuildProcess(BuildProcess buildProcess,
                                   Node nativeNode) {
        OtterDocumentNode docNode = null;
        OtterProject      otterProject = null;
        TextFileNode      textNode = null;
        OtterXMLCNode     xmlcNode = null;
        PathHandle        path = null;
        XMLCBuildTask     task = null;

        task = (XMLCBuildTask) buildProcess.getTask(XMLCBuildTask.SHARED_KEY);
        if (task == null) {
            task = new XMLCBuildTask();

            // There is both a map and a list
            buildProcess.putTask(XMLCBuildTask.SHARED_KEY, task);

            // Add xmlc task to top of list so genrated DOM
            // classes are available when dependent classes
            // are compiled.
            List list = buildProcess.getBuildTaskList();

            list.add(0, task);
        }
        task.increment();
        //
        if (nativeNode instanceof TextFileNode) {
            textNode = (TextFileNode) nativeNode;
            otterProject =
                new PrimeProject((JBProject) textNode.getProject());
            path =
                PathHandle.createPathHandle(textNode.getUrl().getFileObject());
            if (path.hasExtension(otterProject.getDocTypes())) {
                docNode = new PrimeDocumentNode(textNode);

                // Check if we want to build this specific HTML file.
                if (docNode.isSelected()) {
                    xmlcNode = new OtterXMLCNode(docNode);
                    task.addNode(xmlcNode);
                }
            }
        }
    }

}
