/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.node;

// ToolBox imports
import org.enhydra.tool.common.PathHandle;

// JBuilder imports
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Node;
import com.borland.primetime.vfs.Url;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import java.io.File;
import javax.swing.SwingUtilities;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class PrimeNode implements OtterNode {
    private Node      nativeNode = null;
    private Exception exception = null;

    /**
     * Constructor declaration
     *
     */
    public PrimeNode() {}

    /**
     * Constructor declaration
     *
     * @param node
     */
    public PrimeNode(Node n) {
        nativeNode = n;
    }

    public PrimeNode(OtterNode otterNode, String path) {
        JBProject nativeProject = null;
        Node containerNode = null;

        if (otterNode.getProject().getNativeProject() instanceof JBProject) {
            nativeProject =
                (JBProject) otterNode.getProject().getNativeProject();
        }
        if (otterNode.getNativeNode() instanceof Node) {
            containerNode = (Node) otterNode.getNativeNode();
        }
        if (nativeProject != null && containerNode != null) {
            Url  url = null;
            Node found = null;

            url = new Url(new File(path));
            found = nativeProject.findNode(url);
            if (found == null) {
                Node newNode = null;

                newNode = nativeProject.getNode(url);
                setNativeNode(newNode);
                SwingUtilities.invokeLater(new ParentSetter(containerNode));
            } else {
                setNativeNode(found);
            }
        }
    }


    public OtterProject getProject() {
        return (new PrimeProject((JBProject) nativeNode.getProject()));
    }

    /**
     * Method declaration
     *
     * @return
     */
    public String getFilePath() {
        File out = null;

        if (nativeNode != null) {
            if (nativeNode instanceof FileNode) {
                FileNode fileNode;

                fileNode = (FileNode) nativeNode;
                out = fileNode.getUrl().getFileObject();
            }
        }
        return PathHandle.createPathString(out);
    }

    public String getXMLCOptionFilePath() {
        String     path = new String();
        File       file = null;
        PathHandle handle = null;

        file =
            PathUtil.getFileRelativeToProject(this,
                                              PropertyKeys.NAME_XMLC_OPTION_FILEPATH);
        handle = PathHandle.createPathHandle(file);
        if (handle.isFile() && handle.hasExtension(Constants.TYPE_XMLC)) {
            path = handle.getPath();
        }
        return path;
    }

    public void setXMLCOptionFilePath(String n) {
        PathHandle handle = null;
        String     relFilename = new String();

        handle = PathHandle.createPathHandle(n);
        if (handle.isFile() && handle.hasExtension(Constants.TYPE_XMLC)) {
            relFilename = PathUtil.compressPathRelativeToProject(this,
                    handle.getPath());
        }
        setProperty(PropertyKeys.NAME_XMLC_OPTION_FILEPATH, relFilename);
    }

    public String getXMLCParameters() {
        String out = new String();

        out = getProperty(PropertyKeys.NAME_XMLC_PARAMETERS);
        out = PropUtil.cleanXMLCParameters(out);
        return out;
    }

    public void setXMLCParameters(String in) {
        String out = new String();

        out = PropUtil.cleanXMLCParameters(in);
        setProperty(PropertyKeys.NAME_XMLC_PARAMETERS, out);
    }

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     */
    public String getProperty(String property) {
        String out = null;

        if (nativeNode != null) {
            out = nativeNode.getProperty(property);
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     */
    public void setProperty(String property, String value) {
        if (nativeNode != null) {
            nativeNode.setProperty(property, value);
        }
    }

    public void setProperty(String property, int i) {
        String value = Integer.toString(i);

        setProperty(property, value);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Object getNativeNode() {
        return nativeNode;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     */
    public void setNativeNode(Object n) {
        nativeNode = (Node) n;
    }

    public OtterNode getParent() {
        OtterNode parent = null;
        Node      nativeParent = null;

        nativeParent = nativeNode.getParent();
        if (nativeParent instanceof JBProject) {
            parent = getProject();
        } else {
            parent = new PrimeNode(nativeParent);
        }
        return parent;
    }

    public boolean isSelected() {
        String in = getProperty(PropertyKeys.NAME_SELECTED);

        return PropUtil.stringToBoolean(in, false);
    }

    public void setSelected(boolean b) {
        setProperty(PropertyKeys.NAME_SELECTED, PropUtil.booleanToString(b));
    }

    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
        exception = e;
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
        return exception;
    }

    public void save() {
        FileNode fileNode = null;

        if (getNativeNode() instanceof FileNode) {
            fileNode = (FileNode) getNativeNode();
            try {
                fileNode.save();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    protected class ParentSetter implements Runnable {
        private Node conNode = null;

        protected ParentSetter(Node con) {
            conNode = con;
        }

        synchronized public void run() {
            Node      nativeNode = null;
            Node      nativeParent = null;
            JBProject nativeProject = null;

            nativeNode = (Node) getNativeNode();
            nativeNode.setParent(conNode);
            nativeProject = (JBProject) getProject().getNativeProject();
            nativeNode.check();
            nativeProject.fireChildrenChanged(nativeNode);
        }

    }
}
