/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.node;

// ToolBox imports
import org.enhydra.tool.common.ToolException;

// Kelp imports
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterImageFileNode;
import org.enhydra.kelp.common.node.OtterJavaFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;

// Primetime imports
import com.borland.primetime.node.Node;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.FolderNode;
import com.borland.jbuilder.node.JBProject;

// Standard imports
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class PrimeNodeFactory extends OtterNodeFactory {

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param name
     *
     * @return
     */
    public OtterFolderNode createFolderNode(OtterNode otterNode,
                                            String name) {
        OtterFolderNode newNode = null;

        newNode = new PrimeFolderNode(otterNode, name);
        return newNode;
    }

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param file
     *
     * @return
     */
    public OtterJavaFileNode createJavaFileNode(OtterNode otterNode,
                                                String file) {
        OtterJavaFileNode newNode = null;

        newNode = new PrimeJavaFileNode(otterNode, file);
        return newNode;
    }

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param file
     *
     * @return
     */
    public OtterDocumentNode createDocumentNode(OtterNode otterNode,
                                                String file) {
        OtterDocumentNode newNode = null;

        newNode = new PrimeDocumentNode(otterNode, file);
        return newNode;
    }

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param file
     *
     * @return
     */
    public OtterImageFileNode createImageFileNode(OtterNode otterNode,
            String file) {
        OtterImageFileNode newNode = null;

        newNode = new PrimeImageFileNode(otterNode, file);
        return newNode;
    }

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param file
     *
     * @return
     */
    public OtterTextFileNode createTextFileNode(OtterNode otterNode,
                                                String file) {
        OtterTextFileNode newNode = null;

        newNode = new PrimeTextFileNode(otterNode, file);
        return newNode;
    }

    /**
     * Method declaration
     *
     *
     * @param otterNode
     * @param file
     *
     * @return
     */
    public OtterTemplateNode createTemplateNode(OtterNode otterNode,
                                                String file) {
        OtterTemplateNode newNode = null;

        newNode = new PrimeTemplateNode(otterNode, file);
        return newNode;
    }

    /**
     * Method declaration
     *
     * @param otterNode
     *
     * @return
     */
    public OtterTemplateNode getTemplateNode(OtterNode otterNode) {
        OtterTemplateNode newNode = null;

        if (otterNode.getNativeNode() instanceof FileNode) {
            FileNode nativeNode = null;

            nativeNode = (FileNode) otterNode.getNativeNode();
            try {
              newNode = new PrimeTemplateNode(nativeNode);
            } catch (ToolException e) {
              e.printStackTrace();
              newNode = null;
            }
        }
        return newNode;
    }

    /**
     * Method declaration
     *
     *
     * @param project
     * @param node
     * @param files
     *
     * @return
     */
    public synchronized OtterJavaFileNode[] replaceGeneratedSource(OtterProject project,
            OtterNode node, String[] files) {

        // Remove previously generated sources and then add
        OtterJavaFileNode[] javaNodes = new OtterJavaFileNode[files.length];
        JBProject           nativeProject =
            (JBProject) project.getNativeProject();
        FileNode            nativeNode = (FileNode) node.getNativeNode();
        Node[]              child = nativeNode.getChildren();
        File                current = null;
        boolean             keep = false;

        for (int i = 0; i < child.length; i++) {
            if (child[i] instanceof FileNode) {
                FileNode cursor = null;

                cursor = (FileNode) child[i];
                for (int j = 0; j < files.length; j++) {
                    current = new File(files[j]);
                    if (cursor.getUrl().getFileObject().getAbsolutePath().equals(current.getAbsolutePath())) {
                        keep = true;
                        break;
                    }
                }
                if (!keep) {
                    cursor.setParent(null);
                    cursor.getUrl().getFileObject().delete();
                }
            }
        }
        for (int i = 0; i < files.length; i++) {
            try {
                Thread.sleep(100);
            } catch (Exception e) {
                e.printStackTrace();
            }
            javaNodes[i] = createJavaFileNode(node, files[i]);
        }
        return javaNodes;
    }

}
