/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.node;

// ToolBox imports
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.PathHandle;

// Kelp imports
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;

// Primetime imports
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.Node;
import com.borland.primetime.vfs.Url;
import com.borland.primetime.node.DuplicateNodeException;

// Standard imports
import java.io.File;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class PrimeTemplateNode extends PrimeTextFileNode
    implements OtterTemplateNode {

    //
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.jbuilder.Res");    // nores

    public PrimeTemplateNode(FileNode n) throws ToolException {
        super(n);
        if (!PathUtil.isTemplate(this)) {
            StringBuffer buf = new StringBuffer();

            buf.append(res.getString("Invalid_template_node"));
            buf.append('\n');
            buf.append(ResUtil.format(res.getString("Node_file_path_0_"),
                                             getFilePath()));
            buf.append('\n');
            buf.append(ResUtil.format(res.getString("Deploy_input_path_0_"),
                                             getProject().getDeployInputPath()));
            throw new ToolException(buf.toString());
        }
    }

    /**
     * Constructor declaration
     *
     *
     * @param otterNode
     * @param file
     */
    public PrimeTemplateNode(OtterNode otterNode, String file) {
        super(otterNode, file);
    }

    public boolean isBootstrap() {
        boolean boot = false;
        String  outPath = new String();
        String  bootPath = new String();

        if (isSelected()) {
            outPath = getOutputFile().getAbsolutePath().replace('\\', '/');
            bootPath = getProject().getDeployBootstrapPath().replace('\\',
                    '/');
            if (outPath.trim().equalsIgnoreCase(bootPath.trim())) {
                boot = true;
            }
        }
        return boot;
    }

    public File getOutputFile() {
        PathHandle   templatePath = null;
        File         out = null;
        StringBuffer path = new StringBuffer();
        String       thisPath = null;

        templatePath =
            PathHandle.createPathHandle(getProject().getDeployInputPath());
        thisPath = getFilePath();
        path.append(getProject().getDeployRootPath());
        if (templatePath.parentOf(thisPath)) {
            int start = getProject().getDeployInputPath().length();
            int end = thisPath.length() - 3;

            path.append(thisPath.substring(start, end));
        } else {
            File f = new File(thisPath);

            path.append(File.separator);
            path.append(f.getName().substring(0, f.getName().length() - 3));
        }
        out = new File(path.toString());
        return out;
    }

}
