/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.properties;

// ToolBox imports
import org.enhydra.tool.common.PathHandle;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// JBuilder Foundation imports
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.LightweightNode;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.properties.PropertyPageFactory;
import com.borland.primetime.properties.PropertyPage;
import com.borland.primetime.properties.PropertyManager;
import com.borland.primetime.properties.PropertyGroup;
import java.util.ResourceBundle;

//
public class XMLCProperties implements PropertyGroup {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.jbuilder.Res");

    public XMLCProperties() {}

    /**
     * Initialize this OpenTool
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        if (KelpInfo.isClassPathComplete()) {
            PropertyManager.registerPropertyGroup(new XMLCProperties());
        }
    }

    /**
     * Returns a PropertyPageFactory for the project or for an
     * HTMLFileNode. Null if topic is something else.
     *
     * TODO: ?? Add XMLFileNode ??
     */
    public PropertyPageFactory getPageFactory(final Object topic) {
        PropertyPageFactory factory = null;
        JBProject           nativeProject = null;
        OtterProject        otterProject = null;
        PathHandle          path = null;

        if (topic instanceof TextFileNode) {
            TextFileNode textNode = (TextFileNode) topic;

            nativeProject = (JBProject) textNode.getProject();
            if (nativeProject == null) {

                // Skip
            } else {
                otterProject = new PrimeProject(nativeProject);
                path =
                    PathHandle.createPathHandle(textNode.getUrl().getFileObject());
                if (path.hasExtension(otterProject.getDocTypes())) {

                    // Node property page to set your XMLC properties
                    factory =
                        (new PropertyPageFactory(res.getString("Enhydra_XMLC")) {
                        public PropertyPage createPropertyPage() {
                            return new XMLCNodePropertyPage(topic);
                        }

                    });
                }
            }
        } else if (topic instanceof LightweightNode) {

            // Package or Folder node
            LightweightNode lightNode = null;

            lightNode = (LightweightNode) topic;
            nativeProject = (JBProject) lightNode.getProject();
            if (nativeProject == null) {

                // Skip
            } else {
                factory =
                    (new PropertyPageFactory(res.getString("Enhydra_XMLC")) {
                    public PropertyPage createPropertyPage() {
                        return new XMLCPackagePropertyPage(topic);
                    }

                });
            }
        } else if (topic instanceof JBProject) {

            // Project property page to set your XMLC properties
            factory =
                (new PropertyPageFactory(res.getString("Enhydra_XMLC")) {
                public PropertyPage createPropertyPage() {
                    return new XMLCProjectPropertyPage(topic);
                }

            });
        }
        return factory;
    }

    public void initializeProperties() {}

}
