/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.wizard;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;

// JDeveloper imports
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.VersionedAddin;
import borland.jbuilder.addin.Wizard;
import borland.jbuilder.addin.WizardManager;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.swing.IconSet;

// Standard imports
import java.awt.Container;
import java.awt.Image;
import java.io.Serializable;
import java.util.ResourceBundle;
import javax.swing.JOptionPane;

//
abstract public class AbstractBasicWizard implements Serializable,
        VersionedAddin, Wizard {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.jdeveloper.Res"); // nores

    // string not to be resourced
    private final String VERSION = "2.0"; // nores

    //
    protected Project    nativeProject = null;

    public AbstractBasicWizard() {}

    abstract public void invoke(IdeEnvironment ide, Context context);

    /**
     * Wizard interface method, returns the text for the menu label
     * @return Menu label text
     */
    public String getMenuLabel() {
        return ResUtil.format(res.getString("_0_dots"), getName());
    }

    /**
     * Wizard interface method, returns information about the type of Wizard this is.
     * @return An integer representing combined flags defined by the
     * Wizard interface.
     */
    public int getInfo() {
        return Wizard.PRODUCES_FILE + Wizard.REQUIRES_PROJECT;
    }

    /**
     * Wizard interface method
     * @return little Otter
     */

    /**
     * Wizard interface method
     * @return Currently returns big Otter
     */
    public Image getBigImage() {
        return IconSet.getLargeIcon().getImage();
    }

    public Image getSmallImage() {
        return IconSet.getSmallIcon().getImage();
    }

    /**
     * Entry point exposed by the Wizard interface for early initialization.
     * Registers this Wizard object with the IDE's WizardManager object
     * @$param ide Access to all AddIn interfaces
     */
    public void startup(IdeEnvironment ide) {
        WizardManager e = ide.getWizardManager();

        e.registerWizard(this);
    }

    /**
     * For the VersionAddin Interface.
     */
    public String version() {
        return VERSION;
    }

    /**
     * Method declaration
     *
     *
     * @param projectClassPath
     *
     * @return
     *
     * @see
     */
    public boolean verifyAllClassPaths(Container owner) {
        boolean valid = false;
        String  projectClassPath = new String();

        if (nativeProject != null) {
            projectClassPath = nativeProject.getClassPath();
        }
        valid = KelpInfo.verifyIDEClassPath(owner);
        if (valid) {
            valid = projectClassPath.toLowerCase().indexOf(Constants.FILE_ENHYDRA_JAR) > 0;
            if (!valid) {
                StringBuffer buf = new StringBuffer();

                buf.append(res.getString("EnhydraLibError1"));
                buf.append('\n');
                buf.append(res.getString("EnhydraLibError2"));
                JOptionPane.showMessageDialog(owner, buf.toString(), Constants.KELP,
                                              JOptionPane.WARNING_MESSAGE);
            }
        }
        return (valid);
    }

}
