/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.wizard;

// ToolBox imports
import org.enhydra.tool.common.FileUtil;

// JDeveloper imports
import borland.jbuilder.addin.Addin;
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Wizard;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.VersionedAddin;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.common.wizard.codegen.AppGenUtil;
import org.enhydra.kelp.common.wizard.codegen.CodeGenException;
import org.enhydra.kelp.jdeveloper.node.OracleProject;

// Standard imports
import java.awt.Image;
import java.io.File;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.swing.JOptionPane;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
abstract public class AbstractCodeGenWizard implements Wizard,
        VersionedAddin {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.jdeveloper.Res"); // nores

    // string not to be resourced
    private final String VERSION = "2.0";              // nores
    private final String LIBS = "Enhydra;SunTools";    // nores

    //
    private String       imageFileName = new String();
    private Image        bigImage;

    abstract public String getDisplayName();
    abstract public String[] getSteps();

    /**
     * Method declaration
     *
     *
     * @param ide
     */
    public void startup(IdeEnvironment ide) {
        if (imageFileName.length() > 0) {

            // bigImage = imageImageLoader.loadFromResource(imageFileName,
            // ide.getIdeFrame(), getClass());
        }
    }

    /**
     * For the VersionAddin Interface.
     */
    public String version() {
        return VERSION;
    }

    public boolean isJava2(IdeEnvironment ide) {
        boolean java2 = true;
        String  ver = null;

        ver = System.getProperty(Constants.JAVA_VERSION);
        if (ver.startsWith(Constants.JDK11)) {
            java2 = false;
            String mess = new String();

            mess = ResUtil.format(res.getString("Kelp_requires_Java_2"),
                                  ver);
            JOptionPane.showMessageDialog(ide.getIdeFrame(), mess);
        }
        return java2;
    }

    /**
     * Method declaration
     *
     *
     * @param ide
     * @param context
     */
    public void invokeWizard(IdeEnvironment ide, Context context,
                             AppGenUtil util) throws CodeGenException {
        Project nativeProject = null;
        int     choice = AppGenUtil.CHOICE_CANCEL;
ide.getWorkspace().BeginUpdate();
        ide.getWorkspace().save(true);
ide.getWorkspace().reload();
ide.getWorkspace().EndUpdate();
        if (context != null) {
            nativeProject = context.getProject();
            if (nativeProject == null
                    || nativeProject.getType().equals(Container.ROOT_TYPE)) {
                nativeProject = null;
                String mess = new String();

                mess =
                    ResUtil.format(res.getString("Project_not_found"),
                                   getDisplayName());
                JOptionPane.showMessageDialog(ide.getIdeFrame(), mess);
            }
            if (nativeProject != null) {
                OtterProject otterProject = null;
                StringBuffer path = new StringBuffer();

                otterProject = new OracleProject(nativeProject);
                util.init(otterProject);
                util.getGenerator().setKelpSteps(getSteps());
                util.invokeWizard(ide.getIdeFrame());
                if (util.getChoice() == AppGenUtil.CHOICE_CANCEL) {

                    // done
                } else {

                    // Note steps should be added to replacement list!!!!
                    nativeProject.BeginUpdate();
                    nativeProject.setHTMLRootDirectory(FileUtil.toWindowsPath(otterProject.getWorkingPath()));
                    nativeProject.setHTMLSrcDirectory(Constants.DIR_SRC);
                    otterProject.setProperty(PropertyKeys.NAME_LIBRARIES,
                                             LIBS);
nativeProject.save(true);
                    nativeProject.EndUpdate();
ide.getWorkspace().BeginUpdate();
ide.getWorkspace().save(true);
ide.getWorkspace().reload();
ide.getWorkspace().EndUpdate();
                    StringBuffer buf = new StringBuffer();

                    buf.append(ResUtil.format(res.getString("The_0_Wizard"),
                                              getDisplayName()));
                    buf.append('\n');
                    buf.append(Constants.TAB4);
                    buf.append(nativeProject.getFilename());
                    buf.append('\n');
                    buf.append(ResUtil.format(res.getString("Refer_to_the_readme"),
                                              getDisplayName()));
                    JOptionPane.showMessageDialog(ide.getIdeFrame(),
                                                  buf.toString());
                }
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String toString() {
        return getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getName() {
        return getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getMenuLabel() {
        return null;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Image getBigImage() {
        return IconSet.getLargeIcon().getImage();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Image getSmallImage() {
        return IconSet.getSmallIcon().getImage();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public int getInfo() {
        return Wizard.REQUIRES_PROJECT | Wizard.PRODUCES_FILE;
    }

}
