/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.wizard;

// Oracle imports
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.HtmlNode;
import borland.jbuilder.addin.Wizard;
import borland.jbuilder.addin.WizardManager;
import borland.jbuilder.addin.VersionedAddin;
import oracle.jdeveloper.wizard.common.StandardWizardDialog;
import oracle.jdeveloper.wizard.common.OracleWizardPanel;
import oracle.jdeveloper.wizard.common.ButtonPanel;

// ToolBox imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.ProgressMeter;
import org.enhydra.tool.common.TBWizardPage;
import org.enhydra.tool.common.TBWizardPanel;
import org.enhydra.tool.common.SwingUtil;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.importer.ImportPaths;
import org.enhydra.kelp.common.importer.ImportTool;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.common.wizard.importer.ImportMapPanel;
import org.enhydra.kelp.common.wizard.importer.RootPanel;
import org.enhydra.kelp.common.wizard.importer.PathPanel;
import org.enhydra.kelp.common.wizard.importer.ImporterPanel;
import org.enhydra.kelp.jdeveloper.node.OracleProject;

// Standard imports
import java.awt.BorderLayout;
import java.awt.Image;
import java.io.File;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class ImportWizard extends AbstractBasicWizard {

    // not to be resourced
    private final String    HELP_LINK = KelpInfo.HELP_DOC + "#import";    // nores
    private final String    LIBS = "Enhydra;SunTools";    // nores

    //
    protected static String WIZARD_NAME = res.getString("Enhydra_Import");

    // AbstractBasicWizard implementation
    public String getName() {
        return ImportWizard.WIZARD_NAME;
    }

    public void invoke(IdeEnvironment ide, Context context) {
        boolean            invokeIt = true;
        OtterProject       otterProject = null;
        ImportWizardDialog dialog = null;

        if (KelpInfo.verifyIDEClassPath(ide.getIdeFrame())) {
            nativeProject = context.getProject();
            otterProject = new OracleProject(nativeProject);
            if (!isProjectEmpty()) {
                StringBuffer buf = new StringBuffer();

                buf.append(res.getString("NotEmpty1"));
                buf.append(res.getString("NotEmpty2"));
                buf.append(res.getString("NotEmpty3"));
                buf.append(res.getString("NotEmpty4"));
                int choice =
                    JOptionPane.showConfirmDialog(ide.getIdeFrame(),
                                                  buf.toString(),
                                                  ResUtil.format(res.getString("_0_Wizard_Error"), getMenuLabel()),
                                                  JOptionPane.WARNING_MESSAGE);

                if (choice == JOptionPane.CANCEL_OPTION) {
                    invokeIt = false;
                }
            }
            if (invokeIt) {
                dialog = new ImportWizardDialog(ide, getName(),
                                                nativeProject);
                dialog.show();
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     */
    private boolean isProjectEmpty() {
        Node[]  nodes = null;
        boolean empty = true;

        if (nativeProject == null) {

            // done
        } else {
            nodes = nativeProject.getChildren();
        }
        if (nodes == null) {

            // done
        } else {
            if (nodes.length > 1) {
                empty = false;
            } else if (nodes.length == 1) {
                if (!(nodes[0] instanceof HtmlNode)) {
                    empty = false;
                }
            }
        }
        return empty;
    }

    class ImportWizardDialog extends StandardWizardDialog {
        private ImportPaths  paths = null;
        private OtterProject otterProject = null;

        public ImportWizardDialog(IdeEnvironment ide, String title,
                                  Project nativeProject) {
            super(ide, title, nativeProject);
            otterProject = new OracleProject(nativeProject);
            pmInit();
        }

        private void pmInit() {
            paths = new ImportPaths();
            paths.setProject(otterProject);
            String projectPath = otterProject.getFilePath();

            if (projectPath != null && projectPath.trim().length() > 0) {
                File   file = null;
                String rootPath = new String();

                file = new File(projectPath);
                if (file.isFile()) {
                    try {
                        rootPath = file.getAbsolutePath();
                        rootPath = rootPath.substring(0,
                                                      rootPath.length() - 4);
                        file = new File(rootPath);
                        if (!file.exists()) {
                            file.mkdirs();
                        }
                        paths.setRootPath(rootPath);
                    } catch (ValidationException e) {
                        e.printStackTrace();
                    }
                }
            }
            setImage(IconSet.getOtterWizard().getImage());
            LocalWizardPage rootPage = new LocalWizardPage();
            LocalWizardPage pathPage = new LocalWizardPage();
            LocalWizardPage mapPage = new LocalWizardPage();

            rootPage.setPanel(new RootPanel());
            pathPage.setPanel(new PathPanel());
            mapPage.setPanel(new ImportMapPanel());
            rootPage.setPaths(paths);
            pathPage.setPaths(paths);
            mapPage.setPaths(paths);
            addPanel(rootPage);
            addPanel(pathPage);
            addPanel(mapPage);
        }

        public void show() {
            first();
            pack();
            centerOnScreen();
            super.show();
        }

        public void cancel() {
            closeDialog();
            super.cancel();
        }

        private void closeDialog() {
            setVisible(false);
            removeAll();
            dispose();
        }

        /**
         * Method declaration
         *
         *
         */
        public boolean finish() {
            boolean       valid = false;
            ImportTool    tool = null;
            ProgressMeter progress = null;

            tool = new ImportTool();
            if (getPanel() instanceof LocalWizardPage) {
                LocalWizardPage localPage = (LocalWizardPage) getPanel();

                valid = localPage.validatePanel();
            }
            if (valid) {
                String path = new String();

                progress = new ProgressMeter();
                progress.setOwner(this);
                progress.setModal(true);
                progress.setTitle(ImportWizard.WIZARD_NAME);
                tool.setPaths(paths);
                tool.addProgressListener(progress);
                tool.importFilesFromRoot();
                progress.startDialogThread();
                path = otterProject.getWorkingPath();
                path = FileUtil.toWindowsPath(path);
                otterProject.setProperty(PropertyKeys.NAME_LIBRARIES, LIBS);
                nativeProject.BeginUpdate();
                nativeProject.setHTMLRootDirectory(path);
                nativeProject.setHTMLSrcDirectory(Constants.DIR_SRC);
                nativeProject.EndUpdate();
                closeDialog();
            }
            return valid;
        }

    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     */
    class LocalWizardPage extends OracleWizardPanel {
        private ImporterPanel importPanel = null;
        private TBWizardPanel tbPanel = null;
        private TBWizardPage  tbPage = null;

        /**
         * Constructor declaration
         *
         */
        public LocalWizardPage() {
            super();
            pmInit();
        }

        private void pmInit() {
            BorderLayout layoutMain = null;
            BorderLayout layoutTBPanel = null;

            layoutMain = new BorderLayout();
            layoutTBPanel = new BorderLayout();
            layoutMain.setHgap(5);
            layoutMain.setVgap(5);
            setLayout(layoutMain);
            setVisible(false);
            tbPage = new TBWizardPage();
            tbPanel = new TBWizardPanel();
            tbPanel.setLayout(layoutTBPanel);
            tbPage.add(tbPanel);
        }

        /**
         * Method declaration
         *
         */
        public void enter() {
            importPanel.activated();
            enable();
            setEnabled(true);
            setVisible(true);
            add(tbPage, BorderLayout.CENTER);
            if (importPanel instanceof ImportMapPanel) {
                enableFinish(true);
            }
        }

        private void enableFinish(boolean enable) {
            if (getTopLevelAncestor() instanceof StandardWizardDialog) {
                StandardWizardDialog dialog = null;
                ButtonPanel          buttons = null;

                dialog = (StandardWizardDialog) getTopLevelAncestor();
                buttons = dialog.getButtonBar();
                buttons.enableButton(StandardWizardDialog.FINISH_BUTTON,
                                     enable);
            }
        }

        public boolean leave() {
            boolean valid = validatePanel();

            if (valid) {
                disable();
                setEnabled(false);
                setVisible(false);
                remove(tbPage);
                if (importPanel instanceof ImportMapPanel) {
                    enableFinish(false);
                }
            }
            return valid;
        }

        /**
         * Method declaration
         *
         */
        public boolean validatePanel() {
            boolean valid = true;

            try {
                importPanel.checkPage();
            } catch (ValidationException e) {
                JOptionPane.showMessageDialog(this, e.getMessage(),
                                              ImportWizard.WIZARD_NAME,
                                              JOptionPane.ERROR_MESSAGE);
                valid = false;
            }
            return valid;
        }

        /**
         * Method declaration
         *
         * @param importer
         */
        public void setPaths(ImportPaths paths) {
            importPanel.setPaths(paths);
        }

        /**
         * Method declaration
         *
         * @return
         */
        public ImportPaths getPaths() {
            return importPanel.getPaths();
        }

        public String getTitle() {
            return tbPage.getPageTitle();
        }

        protected void setPanel(ImporterPanel inPanel) {
            importPanel = inPanel;
            tbPanel.add(importPanel, BorderLayout.CENTER);
            tbPage.setInstructions(inPanel.getInstructions());
            tbPage.setPageTitle(inPanel.getPageTitle());
        }

        protected JPanel getPanel() {
            return importPanel;
        }

    }
}
