/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EnhydraAdmin.java,v 1.1.2.1.2.1 2000/10/19 17:59:08 jasona Exp $
 */





package com.lutris.appserver.admin.business;

import java.util.Vector;
import java.util.Enumeration;
import com.lutris.multiServer.MultiServer;
import org.enhydra.servlet.ServletContainer;
import org.enhydra.servlet.connectionMethods.*;
import org.enhydra.servlet.servletManager.ServletManager;
import org.enhydra.servlet.servletManager.ServletStatus;


/**
 * Utility methods used by the Enhydra administration application.
 * 
 * @version $Revision: 1.1.2.1.2.1 $
 * @author  Soheil Zahedi
 */
public class EnhydraAdmin {
    /**
     * Private constructor to prevent instantiation.
     */
    private void EnhydraAdmin() {}

    /**
     * Returns a list of connection method ids and channel IDs that map to
     * the given servlet. If there are <CODE>N</CODE> channels, 
     * the resulting array will
     * be of the form: <CODE>String result[N][2]</CODE>.
     *
     * @param servletID A valid and existing servletID.
     * @exception ??? if servletID is bogus.
     * @return all channel IDs for servlet.
     */
    public static String[][] getChannelIDs(String servletID) {
        ConnectionMethod cm;
        ChannelStatus cs;
        String[] channels, filteredChannels;
	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
	ConnectionMethodManager cmm = sc.getConnectionMethodManager();
        String[] connectionMethods = cmm.getIDs();
        Vector fc = new Vector();
        Vector fc2 = new Vector();
        int num = 0;
        for (int i = 0; i < connectionMethods.length; i++) {
            cm = cmm.get(connectionMethods[i]);
            channels = cm.getChannelIDs();
            for (int j = 0; j < channels.length; j++) {
                cs = cm.getChannelStatus(channels[j]);
                if (cs.servletID.equals(servletID)) {
                    // We have one...
                    fc.addElement(channels[j]);
                    fc2.addElement(connectionMethods[i]);
                    num++;
                }
            }
        }
        String[][] results = new String[num][];
        Enumeration e1 = fc.elements();
        Enumeration e2 = fc2.elements();
        for (int i=0; i<num; i++) {
            String id1 = (String) e1.nextElement();
            String id2 = (String) e2.nextElement();
            String[] temp = {id2, id1};
            results[i] = temp;
        }
        return results;
    }



    /**
     * Returns status for the given channel
     *
     * @param channel A valid and existing channelID
     * @return channel status, or null if not found.
     */
    public static ChannelStatus getChannelStatus(String connMethID,
						 String channelID) {
	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
	ConnectionMethodManager cmm = sc.getConnectionMethodManager();
	ConnectionMethod cm = cmm.get(connMethID);

	if (cm != null) {
	    String[] channels = cm.getChannelIDs();
	    for (int j = 0; j < channels.length; j++) {
		if (channels[j].equals(channelID))
		    // We have one...
		    return cm.getChannelStatus(channels[j]);
	    }
	}
	return null;
    }



    /**
     * Deletes the given channel. Also deletes the ConnectionMethod it
     * belongs to, if it has no other channels, so as to not leave behind
     * an empty ConnectionMethod with no channels in it.
     *
     * @param channel A valid and existing channelID
     * @exception ConnectionMethodException if there is an error getting
     *  or deleting the channels.
     */
    public static void deleteChannel(String connMethID,
				     String channelID)
	throws ConnectionMethodException {

	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
	ConnectionMethodManager cmm = sc.getConnectionMethodManager();
        ConnectionMethod cm = cmm.get(connMethID);

	if (cm != null) {
            String[] channels = cm.getChannelIDs();
            boolean dirty = false;
            for (int j = 0; j < channels.length; j++) {
                if (channels[j].equals(channelID)) {
                    cm.deleteChannel(channels[j]);
                    dirty = true;
                }
            }
            if (dirty)
                deleteCMIfEmpty(connMethID);
        }
    }



    /**
     * Enables the given channel
     *
     * @param channel A valid and existing channelID
     * @return An error message if there is an error getting
     *  or disabling the channels, or null if successfull.
     */
    public static String enableChannel(String connMethID,
				       String channelID) {

	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
	ConnectionMethodManager cmm = sc.getConnectionMethodManager();
        ConnectionMethod cm = cmm.get(connMethID);

        boolean found = false;
	if (cm != null) {
	    String[] channels = cm.getChannelIDs();
	    for (int j = 0; j < channels.length; j++) {
		if (channels[j].equals(channelID)) {
		    try {
			cm.enableChannel(channels[j]);
			found = true;
		    } catch (ConnectionMethodException e) {
			return "Unable to enable connection: " + 
			    e.getMessage();
		    }
		}
	    }
	}
        if (found) 
            return null;
        else
            return "Unable to enable connection: not found.";
    }



    /**
     * Disable the given channel
     *
     * @param channel A valid and existing channelID
     * @return An error message if there is an error getting
     *  or disabling the channels, or null if successfull.
     */
    public static String disableChannel(String connMethID,
					String channelID) {
	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
	ConnectionMethodManager cmm = sc.getConnectionMethodManager();
        ConnectionMethod cm = cmm.get(connMethID);
        boolean found = false;

	if (cm != null) {
	    String[] channels = cm.getChannelIDs();
	    for (int j = 0; j < channels.length; j++) {
		if (channels[j].equals(channelID)) {
		    try {
			cm.disableChannel(channels[j]);
			found = true;
		    } catch (ConnectionMethodException e) {
			return "Unable to disable connection: " + 
			    e.getMessage();
		    }
		}
	    }
	}

        if (found) 
            return null;
        else
            return "Unable to disable connection: not found.";
    }

    /**
     * See if the given application is running, and has an enabled channel
     * so that we can properly access it (e.g. for debugging).
     *
     * @param servletID A valid and existing servletID.
     * @return true if application is accessible, false otherwise
     */
    public static boolean isAccessible(String servletID) {
	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
        ServletManager sm = sc.getServletManager();
        ServletStatus status = sm.getStatus(servletID);
        if (!status.running) {
            return false;
        } else {
            String channels[][] = getChannelIDs(servletID);
            for (int i = 0; i < channels.length; i++) {
		String cmID = channels[i][0];
                String chID = channels[i][1];
                if (getChannelStatus(cmID, chID).enabled == true)
                    return true;
            }
        }
        return false;
    }

    /**
     * Disable or enable all channels that map to the given servlet.
     * Silently ignores errors.
     *
     * @param servletID the ID of the servlet in question.
     * @param enable If true, all the channels are enabled. Otherwise they
     * are all disabled.
     */
    public static void setServletChannelStatus(String servletID, 
                    boolean enable) {
	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
        ConnectionMethodManager cmm = sc.getConnectionMethodManager();
        if (cmm == null)
            return;
        String[] cmIDs = cmm.getIDs();
        for (int i=0; i<cmIDs.length; i++) {
            ConnectionMethod cm = cmm.get(cmIDs[i]);
            if (cm == null)
                continue;
            String[] chIDs = cm.getChannelIDs();
            for (int j=0; j<chIDs.length; j++) {
                ChannelStatus cs = cm.getChannelStatus(chIDs[j]);
                if (cs == null)
                    continue;
                if (cs.servletID.equals(servletID)) {
                    try {
                        if (enable)
                            cm.enableChannel(chIDs[j]);
                        else
                            cm.disableChannel(chIDs[j]);
                    } catch (ConnectionMethodException e) {
                    }
                }
            }
        }
    }



    /**
     *  Delete all channels that map to the given servlet.
     *  Any connection methods left with no channels are also deleted.
     *
     * @param servletId Delete channels that point to this servlet.
     */
    public static void deleteAllChannels(String servletId) {
	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
	ConnectionMethodManager cmm = sc.getConnectionMethodManager();
        if (cmm == null) {
            return;
	}
        Vector cmIds = new Vector();
        String[][] channels = EnhydraAdmin.getChannelIDs(servletId);
        for (int i=0; i<channels.length; i++) {
            String cmId = channels[i][0];
            String chId = channels[i][1];
            ConnectionMethod cm = cmm.get(cmId);
            if (cm == null)
                continue;
            // Keep track of all connection methods we touch.
            if (!cmIds.contains(cmId))
                cmIds.addElement(cmId);
            try {
                cm.deleteChannel(chId);
            } catch (ConnectionMethodException e) {
                // Just keep going.
            }
        }
        /*
         * Go back and delete any connection methods that now have no channels.
         */
        Enumeration e = cmIds.elements();
        while (e.hasMoreElements()) {
            String cmId = (String) e.nextElement();
            try {
                deleteCMIfEmpty(cmId);
            } catch (ConnectionMethodException cme) {
                // Just keep going.
            }
        }
    }

    /**
     *  Delete the specified connection method, if it has no channels.
     *
     * @param cmId The ConnectionMethod to delete (if empty).
     * @exception ConnectionMethodException If there is an error while
     * deleting the ConnectionMethod.
     */
    public static void deleteCMIfEmpty(String cmId) 
                       throws ConnectionMethodException {
	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
	ConnectionMethodManager cmm = sc.getConnectionMethodManager();
        if (cmm == null) {
            return;
	}
        ConnectionMethod cm = cmm.get(cmId);
        if (cm == null)
            return;
        String[] chIds = cm.getChannelIDs();
        if (chIds.length == 0) {
            cmm.delete(cmId);
            cm.destroy();
        }
    }

}

