/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: RemoteControl.java,v 1.10.6.1 2000/10/19 17:59:08 jasona Exp $
 */




package com.lutris.appserver.admin.presentation;

import java.net.*;
import java.io.*;
import java.util.Vector;
import com.lutris.appserver.admin.business.*;
import com.lutris.multiServer.MultiServer;
import org.enhydra.servlet.ServletContainer;
import org.enhydra.servlet.servletManager.ServletManager;
import org.enhydra.servlet.servletManager.ServletStatus;
import javax.servlet.ServletException;
import com.lutris.http.*;
import com.lutris.appserver.server.httpPresentation.*;

/**
 * This is an http presentation object, designed for communicating with
 * another program. This object starts and stops servlets.<P>
 * 
 * There is one required argument, <CODE>action</CODE>. It must have
 * one of three values: list, start or stop. For start and stop, an
 * additional argument <CODE>id</CODE> is required. <P>
 * 
 * The input is assumed to be HTTP, with URL encoded arguments.
 * The output will be HTTP, with the mime type "text/plain". There will
 * be an HTTP response code, and HTTP headers (perhaps a session
 * cookie). Inside the body of the response is the machine readable text.<P>
 *
 * Expected query string inputs: <BR>
 *     <CODE>action</CODE>	One of: <CODE>start</CODE>, 
 *              <CODE>stop</CODE> or <CODE>list</CODE>.<BR>
 *     <CODE>id</CODE>  The name of the servlet to stop or start.
 *        Not requred if <CODE>action</CODE> = <CODE>list</CODE>.
 * 
 * @author Andrew John
 * @version $Revision: 1.10.6.1 $
 */
public class RemoteControl implements HttpPresentation {

    public void run(HttpPresentationComms comms)
            throws IOException, Exception {
        //
        // Get the action from the query string.
        //
        String action = comms.request.getParameter("action");
        //
        // Get ready 
        //
        comms.response.setContentType("text/plain");
        HttpPresentationOutputStream out = comms.response.getOutputStream();
	ServletContainer sc = (ServletContainer)
		MultiServer.getService("ServletContainer");
        ServletManager sm = sc.getServletManager();
        if ((sm == null) || (action == null)) {
            out.println("ERROR");
            out.flush();
            return;
        }
        //
        // Switch off the action requested.
        //
        if (action.equalsIgnoreCase("list")) {
            String[] srvIDs = sm.getServletIDs();
            for (int i=0; i<srvIDs.length; i++) {
                ServletStatus ss = sm.getStatus(srvIDs[i]);
                if (ss == null)
                    continue; 
                if (ss.running)
                    out.println(srvIDs[i] + " RUNNING");
                else
                    out.println(srvIDs[i] + " STOPPED");
            }
        } else if (action.equalsIgnoreCase("start")) {
            String id = comms.request.getParameter("id");
            if (id == null) {
                out.println("ERROR");
                out.flush();
                return;
            }
            try {
                sm.start(id);
            } catch (ServletException e) {
                out.println("ERROR");
                out.flush();
                return;
            }
            EnhydraAdmin.setServletChannelStatus(id, true);
            out.println("OK");
        } else if (action.equalsIgnoreCase("stop")) {
            String id = comms.request.getParameter("id");
            if (id == null) {
                out.println("ERROR");
                out.flush();
                return;
            }
            EnhydraAdmin.setServletChannelStatus(id, false);
            try {
                sm.stop(id);
            } catch (ServletException e) {
                out.println("ERROR");
                out.flush();
                return;
            }
            out.println("OK");
        }
    out.flush();
    }
}

