/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MonitorChart.java,v 1.7.6.1 2000/10/19 17:59:08 jasona Exp $
 */






/* This file was created and edited with tabstops set to 8.
 * If you are using "vi" to edit this file, enter the command:
 * 
 *        :set ts=8
 *
 * to set the tabstops to 8.
 */
package com.lutris.appserver.admin.presentation.monitor.gui;


import java.awt.*;
import java.util.*;
import jclass.chart.*;


/**
 * This class is a subclass of the KLG JClass JCChart object.
 * It sets some values that all the charts on the Monitor
 * panel will want.  It also has utility functions they use.
 */
class MonitorChart
extends JCChart
{
    boolean dataLineColorSet = false;
    boolean peakLineColorSet = false;


    MonitorChart() {
	super();

        this.getChartArea().getXAxis(0).setAnnotationMethod(
            JCAxis.TIME_LABELS);
        this.getChartArea().getXAxis(0).setAnnotationRotation(
            JCAxis.ROTATE_90);
        //this.getChartArea().getXAxis(0).setTimeFormat11("yyyy.MM.dd G 'at' hh:mm:ss a z");
        this.getChartArea().getXAxis(0).setTimeFormat11("h:mm a");
        //this.getChartArea().getXAxis(0).setTimeAnnotationLocale();

	this.getLegend().setIsShowing(true);
	this.getLegend().setOrientation(JCLegend.HORIZONTAL);
	this.getLegend().setAnchor(JCLegend.SOUTH);
	this.getLegend().setBorderType(jclass.base.Border.ETCHED_IN);
	this.getLegend().setBorderWidth(MonitorPanel.LEGEND_BORDER_WIDTH);
    }


    void setChartAxisBounds() {

	MonitorDataSource dataSource =
		(MonitorDataSource)(getDataView(0).getDataSource());

	double xMin = 0;
	double xMax = 3600;
	double yMin = 0;
	double yMax = 100;

	if (dataSource != null) {

	    Vector row = dataSource.getRow(0);

	    if ((row != null) && (row.size() > 0)) {
		xMin = ((Double)(row.firstElement())).doubleValue();
	    }
	    if ((row != null) && (row.size() > 0)) {
		xMax = ((Double)(row.lastElement())).doubleValue();
	    }

	    yMin = 0;

	    /* Find the maximum Y value.  Set yMax to the
	     * first element in the list to start with, and
	     * then compare it to all the other elements.
	     */

	    row = dataSource.getRow(1);
	    if ((row != null) && (row.size() > 0)) {
		yMax = ((Double)(row.firstElement())).doubleValue();
	    }

	    for (Enumeration elements = row.elements();
		 elements.hasMoreElements();) {
		double value = ((Double)(elements.nextElement())).doubleValue();

		if (value > yMax) {
		    yMax = value;
		}
	    }

	    if (dataSource.getMax() > yMax) {
		yMax = dataSource.getMax();
	    }
	}

        this.getChartArea().getXAxis(0).setMin(xMin);
        this.getChartArea().getXAxis(0).setMax(xMax);
        this.getChartArea().getYAxis(0).setMin(yMin);
        this.getChartArea().getYAxis(0).setMax(yMax);
    }

 
    /**
     * Set the line colors.
     */
    void setLineColors() {

	if ((this.getDataView(0).getNumSeries() > 0) &&
	    (dataLineColorSet == false)) {

	    /* The data we are displaying has been initialized,
	     * so set its line color.
	     */
	    this.getDataView(0).getSeries(0).getStyle().
		setLineColor(Color.blue);
	    //this.getDataView(0).getSeries(0).getStyle().
	    //	setSymbolShape(JCSymbolStyle.NONE);
	    this.getDataView(0).getSeries(0).getStyle().
		setSymbolColor(Color.blue);

	    dataLineColorSet = true;
	}

	if ((this.getDataView(0).getNumSeries() > 1) &&
	    (peakLineColorSet == false)) {
	    /* A peak line has been added to the data, so set its
	     * color.
	     */
	    this.getDataView(0).getSeries(1).getStyle().
		setLineColor(Color.red);
	    this.getDataView(0).getSeries(1).getStyle().
		setSymbolShape(JCSymbolStyle.NONE);

	    peakLineColorSet = true;
	}
    }
}
