/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MonitorDataSource.java,v 1.9.6.1 2000/10/19 17:59:08 jasona Exp $
 */






/* This file was created and edited with tabstops set to 8.
 * If you are using "vi" to edit this file, enter the command:
 * 
 *        :set ts=8
 *
 * to set the tabstops to 8.
 */
package com.lutris.appserver.admin.presentation.monitor.gui;


import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;
import javax.swing.event.*;
import javax.swing.*;
import jclass.chart.*;
import com.lutris.appserver.admin.presentation.monitor.io.*;


/**
 */
abstract class MonitorDataSource
extends Observable
implements Chartable {

    /**
     * This is the applet of which we are a part.
     * We need a reference to the applet so we can get
     * parameter values.
     */
    private JApplet applet;


    /**
     * This object will get the latest data from the Multiserver at
     * this interval.  The chart GUI which is observing
     * this object will only be notified of new data
     * if the data actually changed.  i.e. we will only
     * call the Observable.update() method if something
     * changed.
     */
    private int updateIntervalInSeconds = 120;


    /**
     * This is the total number of requests that have been
     * made by an application since the Multiserver started keeping
     * track of them.
     */
    public long total;


    /**
     * This is the number of requests that have been
     * made over the last time interval by an application.
     * <P></P>
     * The length of the time interval is stored in
     * this object's timeIntervalInSeconds member data.
     * For example, if timeIntervalInSeconds is 60, and
     * current is 25, that means that there were 25 requests
     * over the last 60 second interval that was recorded.
     */
    public int current;


    /**
     * This is the maximum number of requests that have been
     * made over any time interval by an application since the
     * Multiserver started keeping track of them.
     * <P></P>
     * The length of the time interval is stored in
     * this object's timeIntervalInSeconds member data.
     */
    public int max;


    /**
     * This is the when the max value occurred.
     * This value is milliseconds since January 1, 1970 GMT.
     */
    public Calendar maxTime;


    /**
     * This object contains the actual data values to be plotted.
     * This is a Vector of Vectors.  It is a Vector of "rows" of
     * data.  Each row of data is itself a Vector.
     * Each element in a row Vector is a Double.
     * The first Vector contains the X values.
     * The second Vector contains the Y values.
     */
    private Vector data;


    /**
     * This is the starting time of the data.  All time values
     * are in seconds offset from this time.
     */
    private Calendar baseTime;


    /**
     * TODO - We need to figure out whether this is the client's
     * time or the server's time.  I think the server's.
     */
    private Calendar lastUpdateTime = null;



    /**
     * This is the timer we use to go get data every 
     * timeIntervalInSeconds.
     */
    private UpdateTimer updateTimer = null;


    /**
     * The chart that is displaying us.
     * This is for debugging right now.
     */
    //private MonitorChart chart;



    abstract BaseRequest createRequest();

    /**
     * This is the data source that the RequestsGraphPanel will
     * use to get its data.  The RequestsGraphPanel will be
     * an Observer of this data and so will the JCChart object
     * that is on the RequestsGraphPanel.
     * Please note, you need to call this object's startUpdate()
     * method to tell this object to start getting data from
     * the server.
     */
    MonitorDataSource(JApplet newApplet) {

	applet = newApplet;

	/* Create the data Vector and put two Vectors into
	 * it.  One is for the time values and the other is
	 * for the data values.
	 */
	data = new Vector();
	Vector row = new Vector();
	data.addElement(row);
	row = new Vector();
	data.addElement(row);
	row = new Vector();
	data.addElement(row);
	row = new Vector();
	data.addElement(row);

	/* Hack below to put some initial data into Vectors. */
	row = (Vector)(data.elementAt(0));
	row.addElement(new Double(0.0));
	row.addElement(new Double(1800.0));

	row = (Vector)(data.elementAt(1));
	row.addElement(new Double(10.0));
	row.addElement(new Double(20.0));

	row = (Vector)(data.elementAt(2));
	row.addElement(new Double(0.0));
	row.addElement(new Double(1800.0));

	row = (Vector)(data.elementAt(3));
	row.addElement(new Double(20.0));
	row.addElement(new Double(20.0));

	baseTime = Calendar.getInstance();
	baseTime.set(1998, 0, 1, 13, 30, 0);
	System.out.println("baseTime = "+
		RequestsGraphPanel.getTimeString(baseTime));

	/* Hack above to put some initial data into Vectors. */
    }


    /**
     * This tells the chart what kind of data this graph
     * contains.  ARRAY or GENERAL.
     *
     * This method is required by the Chartable interface.
     */
    public int getDataInterpretation() {
        //return(Chartable.ARRAY);
        return(Chartable.GENERAL);
    }


    /**
     * This returns one data value at a specific time.
     *
     * This method is required by the Chartable interface.
     */
    public synchronized Object getDataItem(int row, int column) {
	System.out.println("Enter getDataItem");
	if (data == null) {
            return(null);
	}

	try {
	    Vector rowData = (Vector)(data.elementAt(row));
	    Double dataItem = (Double)(rowData.elementAt(column));
	    return(dataItem);
	}
	catch(Exception e) {
	    e.printStackTrace();
	    /* Caller probably passed in a row or column index greater
	     * than the number of rows we have.
	     */
            return(null);
	}
    }


    /**
     * This method returns an entire row of data.
     * Remember that the X values are in row 0.
     * The first data series's values are in row 1.
     *
     * This method is required by the Chartable interface.
     */
    public synchronized Vector getRow(int row) {

	if (data == null) {
	    System.out.println("No data to chart.");
            return(null);
	}

	try {
	    Vector rowData = (Vector)(data.elementAt(row));
	    return(rowData);
	}
	catch(Exception e) {
	    e.printStackTrace();
	    /* Caller probably passed in a row index greater
	     * than the number of rows we have.
	     */
            return(null);
	}
    }


    /**
     * This method returns the number of rows in this data
     * source.  Remember that the X values, (i.e. time), count
     * as a row.  So, a graph with only ONE line will have
     * TWO rows.
     *
     * This method is required by the Chartable interface.
     */
    public synchronized int getNumRows() {
	System.out.println("getNumRows returning "+data.size());
	if (data == null) {
	    System.out.println("getNumRows returning 0.");
            return(0);
	}

	return(data.size());
    }


    /**
     * This method returns the point labels to be used on
     * the X axis.  Because this data source's X values are
     * time values and we are having the chart create the
     * labels, this method only needs to return <CODE>null</CODE>
     * to tell the caller that we are not generating labels.
     *
     * This method is required by the Chartable interface.
     */
    public String[] getPointLabels() {
        return(null);
    }


    /**
     * Get the name of a particular row of data.
     * This value is never displayed anywhere.
     * The label that is displayed for a row is returned
     * by getSeriesLabel(int row).
     *
     * This method is required by the Chartable interface.
     */
    public String getSeriesName(int row) {
	switch(row) {
	    case 0:
		return("Requests  ");
	    case 1:
		return("Peak");
	    default:
		return("Unknown");
	}
    }


    /**
     * Get the label to display for a particular row of data.
     *
     * This method is required by the Chartable interface.
     */
    public String getSeriesLabel(int row) {
	return(getSeriesName(row));
    }


    /**
     * Get the name of this data source.
     *
     * This method is required by the Chartable interface.
     */
    public String getName() {
        return("");
    }


    /**
     * Tell this data source how often it needs to go out and get
     * new data.
     */
    void setUpdateIntervalInSeconds(int newUpdateIntervalInSeconds) {
        updateIntervalInSeconds = newUpdateIntervalInSeconds;
    }


    /**
     * This returns the starting time of the data.
     * All data X values are some offset from this
     * value.  For example, if data points are spaced
     * one minute apart, the first X value would be
     * 0, and the next X value would be 60, etc.
     */
    Calendar getBaseTime() {
	Calendar returnValue = Calendar.getInstance();
	returnValue.setTime(baseTime.getTime());
	return(returnValue);
    }



    /**
     * This is the total number of requests that have been
     * made by an application since the Multiserver started keeping
     * track of them.
     */
    long getTotal() {
	return(total);
    }


    /**
     * This is the number of requests that have been
     * made over the last time interval by an application.
     * <P></P>
     * The length of the time interval is stored in
     * this object's timeIntervalInSeconds member data.
     * For example, if timeIntervalInSeconds is 60, and
     * current is 25, that means that there were 25 requests
     * over the last 60 second interval that was recorded.
     */
    int getCurrent() {
	return(current);
    }
    

    /**
     * This is the maximum number of requests that have been
     * made over any time interval by an application since the
     * Multiserver started keeping track of them.
     * <P></P>
     * The length of the time interval is stored in
     * this object's timeIntervalInSeconds member data.
     */
    int getMax() {
	return(max);
    }


    /**
     * This is the when the max value occurred.
     * This value is milliseconds since January 1, 1970 GMT.
     */
    Calendar getMaxTime() {
	Calendar returnValue = Calendar.getInstance();
	returnValue.setTime(maxTime.getTime());
	return(returnValue);
    }


    void setMaxTime(Calendar newMaxTime) {
	maxTime = Calendar.getInstance();
	maxTime.setTime(newMaxTime.getTime());
    }


    /**
     * Get the latest data values from the server.
     * If the latest values are different from the current
     * values we contain, notify anyone who is observing
     * us.  e.g. the chart displaying this data.
     */
    void getLatestDataFromServer() {
	System.out.println("Enter getLatestDataFromServer");

	/*
	 * Send a query off to the server.
	 */

	if (lastUpdateTime == null) {
	    /*
	     * If we have never queried the server,
	     * ask for data as far back as it has.
	     *
	     * TODO - Add code for timezone and locale.
	     */
	    //Calendar calendar = Calendar.getInstance();
	    //calendar.set(1980, 0, 0);
	    lastUpdateTime = Calendar.getInstance();
	}

	BaseRequest dataRequest = createRequest();
/*
	RequestsRequest dataRequest = new RequestsRequest();
	dataRequest.startTime = null;
	dataRequest.endTime = null;
	dataRequest.maxDataPoints = 1000;

	/* This tells the MonitorDataServer which hard coded
	 * data we want.  This is a hack until the MonitorDataServer
	 * is written to really look at the start and end time values.
	 *
	dataRequest.hackValue = hackCount++;
*/

	/* Handling of the response has not yet been made generic.
	 * We want to do something like we did with createRequest()
	 * above.
         */
	RequestsResponse response = requestData(dataRequest);

	if (response != null) {
	    /* Add the new returned data values into
	     * our local data that the graph is displaying.
	     */
	    setData(response);
	}
	else {
	    System.out.println("Response was null.");
	    System.out.println("Exit getLatestDataFromServer");
	    return;
	}

	/*
	 * If any data values have changed, notify whoever is
	 * observing us.  TODO - Figure out how to check if
	 * something has changed.
	 */
	if (true) {
	    setChanged();
	    notifyObservers(new ChartDataModelUpdate(
			    ChartDataModelUpdate.RESET, 0, 0));
	}

	System.out.println("Exit getLatestDataFromServer");
    }


    private RequestsResponse requestData(BaseRequest dataRequest) {

	System.out.println("Enter requestData");
	RequestsResponse response = null;

	String urlString;
	try {
	    /* The MonitorDataServer applet parameter should have
	     * been passed in with our applet tag.
	     */
	    urlString = applet.getParameter("MonitorDataServer");
	}
	catch(Exception e) {
	    System.err.println(
		"Problem getting applet parameter MonitorDataServer");
	    e.printStackTrace();
	    return(null);
	}

	try {
	    //System.out.println("urlString = "+urlString);
	    URL monitorDataServerURL = new URL(urlString);
	    //System.out.println("monitorDataServerURL = "+
	    //	monitorDataServerURL);
	    URLConnection urlConnection = monitorDataServerURL.openConnection();
	    //System.out.println("urlConnection = "+urlConnection);

	    /* Figure out what each of these do and document
	     * well for other people.
	     */

	    /* Tell the browser that even if it thinks it has already
	     * gotten the requested item in a previous request that it
	     * should NOT used the cached value.  i.e. go out and get
	     * it again.  (Please note, that the browser will probably
	     * ignore this if the requested object is an image.)
	     */
	    urlConnection.setUseCaches(false);

	    /* Tell the browser that the object we are expecting back
	     * is a Java serialized object.  I don't know if the call
	     * would fail if we didn't do this, but doing this is considered
	     * proper form.
	     */
	    urlConnection.setRequestProperty("Accept",
					 "application/java-serialized");

	    /* Tell the connection object that we will be sending
	     * data to the urlConnection.  We need to make sure
	     * this call to setDoOutput is BEFORE the call to
	     * setDoInput because calling setDoOutput(true) also
	     * sets doInput to false!
	     */
	    urlConnection.setDoOutput(true);

	    /* Tell the connection object that we are expecting to get
	     * a response on this socket.  This call seems unneccessary.
	     */
	    //urlConnection.setDoInput(true);

	    /* Make the connection, write out our request, and then
	     * read in the returned object.
	     */

	    // Calling connect() causes getOutputStream() to
	    // throw an exception because the connection is
	    // "already open".  I think getOutputStream() does
	    // a connect on its own.
	    // urlConnection.connect();

	    OutputStream outStream = urlConnection.getOutputStream();
	    ObjectOutputStream objOutputStream =
		new ObjectOutputStream(outStream);

	    //System.out.println("Calling objOutputStream.writeObject()");
	    objOutputStream.writeObject(dataRequest);
	    //System.out.println("Calling objOutputStream.flush()");
	    objOutputStream.flush();

	    try {
	    	objOutputStream.close();
	    }
	    catch(IOException e)
	    {
		/* IE throws an exception when I call close. */
		System.err.println("Exception calling objOutputStream.close()");
		//e.printStackTrace();
	    }

	    //System.out.println("Calling urlConnection.getInputStream()");
	    InputStream inStream = urlConnection.getInputStream();
	    ObjectInputStream objInputStream = new ObjectInputStream(inStream);

	    //System.out.println("Calling objInputStream.readObject()");
	    Object o = objInputStream.readObject();

	    //System.out.println("Calling objInputStream.close()");
	    objInputStream.close();

	    response = (RequestsResponse)o;
	    //System.out.println("response.total = "+response.total);
	    //applet.showStatus("response.total = "+response.total);
	}
	catch(Exception e) {
	    e.printStackTrace();
	    return(null);
	}

	return(response);
    }


    /**
     * Stop requesting new data from the server.  We would do this
     * when the Monitor GUI is not visible.
     */
    void stopUpdate() {

	System.out.println("Enter MonitorDataSource.stopUpdate()");
	if (updateTimer == null) {
	    /* The timer is not running. */
	    return;
	}

        updateTimer.setFinished();

	updateTimer = null;
    }


    /**
     * Start requesting new data from the server.  We would do this
     * when the Monitor GUI is becomes visible.
     */
    void startUpdate() {

	System.out.println("Enter MonitorDataSource.startUpdate()");
	if (updateTimer != null) {
	    /* The timer is already running. */
	    return;
	}
	
	updateTimer = new UpdateTimer(this, updateIntervalInSeconds);
	(new Thread(updateTimer)).start();
    }

    private synchronized void setData(RequestsResponse response) {

	System.out.println("Enter setData.");
	System.out.println("response = "+response);

	total = response.total;
	current = response.current;
	max = response.max;
	setMaxTime(response.maxTime);

	/* Figure out what the time of the last data point
	 * in our data Vector is.  Then take the new data
	 * in the response object and put it in our data Vector.
	 */

	/* Get the row that holds the time values. */
	Vector timeRow = (Vector)data.elementAt(0);

	/* Work backwards from the last time value to
	 * find the proper location for the new data.
	 * Normally, the new data will go after the
	 * last data value in our current data Vector.
	 * But, I am writing the code so that if the
	 * new data overlaps the current data, the new data
	 * will override the existing data.
	 */

	for (int newDataIndex = 0;
	     newDataIndex < response.values.length;
	     newDataIndex++) {

	    boolean newDataPointAdded = false;

	    /* Calculate the time for the new data point. 
	     * i.e. Add the offset to the base time.
	     */
	    //System.out.println("response.baseTime = "+
	    //	RequestsGraphPanel.getTimeString(response.baseTime));
	    System.out.println("response.times["+newDataIndex+"] = "+
				response.times[newDataIndex]);

	    Calendar newTime = Calendar.getInstance();
	    newTime.setTime(response.baseTime.getTime());
	    newTime.add(Calendar.SECOND, response.times[newDataIndex]);

	    /* Work backwards from the end of the current data
	     * Vector to find where the new data point should go.
	     * We start at the back because we expect the new data
	     * to usually go at the end of the current data.
	     */
	    for (int curDataIndex = (timeRow.size() - 1);
		 ((curDataIndex >= 0) && (newDataPointAdded == false));
		 curDataIndex--) {
	        //System.out.print("newDataIndex("+newDataIndex+")");
	        //System.out.println(" curDataIndex("+curDataIndex+")");

		/* Get the data point's time value.  Remember this value
		 * is an offset in seconds from the base time.
		 */
		Double curOffset = (Double)(timeRow.elementAt(curDataIndex));

		//System.out.println("curOffset = "+curOffset);

		/* Paranoid error checking. */
		if (curOffset.doubleValue() > Integer.MAX_VALUE) {
		    throw(new IllegalStateException(
			"Time offset is larger than Integer.MAX_VALUE."));
		}

		/* Add the offset to the base time.
		 *
		 * Please note the the new data's base time and the
		 * current data's base time will probably be different.
		 */
		Calendar curTime = Calendar.getInstance();
		curTime.setTime(baseTime.getTime());
		curTime.add(Calendar.SECOND, (int)(curOffset.doubleValue()));

		/*
		System.out.println("curTime = "+
				   RequestsGraphPanel.getTimeString(curTime));
		System.out.println("newTime = "+
				   RequestsGraphPanel.getTimeString(newTime));
		*/

		if (curTime.before(newTime)) {
		    /* Add the new data point into our data Vector
		     * AFTER the curTime data point.
		     */
		    insertDataPoint(response.values[newDataIndex],
				    newTime, curDataIndex+1);
		    newDataPointAdded = true;
		}
	    }

	    if (newDataPointAdded == false) {
		/* We did not add the new data point because its
		 * time was before all the current data points.
		 * So, add it at the start of the Vector.
		 */
		insertDataPoint(response.values[newDataIndex], newTime, 0);
	    }
	}

	updatePeakLine();
    }


    private void insertDataPoint(int newValue, Calendar newTime, int index) {

	/*
	System.out.println("Enter insertDataPoint");
	System.out.println("newValue("+newValue+") index("+index+")");
	System.out.println("newTime = "+
			   RequestsGraphPanel.getTimeString(newTime));
	*/

	if (baseTime == null) {
	    baseTime = Calendar.getInstance();
	    baseTime.setTime(newTime.getTime());
	}

	//System.out.println("baseTime = "+
	//	RequestsGraphPanel.getTimeString(baseTime));

	long timeOffsetInMillis = newTime.getTime().getTime() -
				  baseTime.getTime().getTime();
	double timeOffset = timeOffsetInMillis / 1000.0;

	/* Paranoid error checking. */
	if (timeOffset > Integer.MAX_VALUE) {
	    throw(new IllegalStateException(
		"Time offset is larger than Integer.MAX_VALUE."));
	}

	/* Turn the number into an integer to get rid of any
	 * part that is to the right of the decimal place.
	 */
	int timeOffsetAsInt = (int)timeOffset;
	//System.out.println("time offset from baseTime("+timeOffsetAsInt+")");

	Vector row = (Vector)data.elementAt(0);
	row.insertElementAt(new Double(timeOffsetAsInt), index);

	row = (Vector)data.elementAt(1);
	row.insertElementAt(new Double(newValue), index);
    }


    /**
     * This method updates the values in the two data arrays that
     * are used to hold the data values for the "peak" line.
     */
    private void updatePeakLine() {
	if (data == null) {
	    return;
	}

	Vector timeRow = (Vector)data.elementAt(0);
	if ((timeRow == null) || (timeRow.size() < 2)) {
	    return;
	}
	double startTime = ((Double)(timeRow.firstElement())).doubleValue();
	double endTime = ((Double)(timeRow.lastElement())).doubleValue();

	Vector peakTimeRow;
	Vector peakValueRow;
	if (data.size() < 4) {

	    System.out.println("Creating peak data values");

	    /* We have not yet created the peak data value Vectors.
	     * So create them now.
	     */
	    peakTimeRow = new Vector();
	    peakTimeRow.addElement(new Double(0.0));
	    peakTimeRow.addElement(new Double(0.0));
	    data.addElement(peakTimeRow);

	    peakValueRow = new Vector();
	    peakValueRow.addElement(new Double(0.0));
	    peakValueRow.addElement(new Double(0.0));
	    data.addElement(peakValueRow);
	}
	else {
	    peakTimeRow = (Vector)data.elementAt(2);
	    peakValueRow = (Vector)data.elementAt(3);
	}

	//System.out.println("Setting peak times to:  "+startTime+" "+endTime);
	peakTimeRow.setElementAt(new Double(startTime), 0);
	peakTimeRow.setElementAt(new Double(endTime), 1);

	//System.out.println("Setting peak value to:  "+max);
	peakValueRow.setElementAt(new Double(max), 0);
	peakValueRow.setElementAt(new Double(max), 1);
    }


    /** Quick test hack. */
    public static void main(String[] args) {

        RequestsResponse[] hackData;
        int hackCountMax = 2;



	hackData = new RequestsResponse[hackCountMax+1];

	/****************/

	RequestsResponse response = new RequestsResponse();
	response.total = 4321;
	response.current = 1234;
	response.max = 23;
	response.maxTime = Calendar.getInstance();

	response.values = new int[3];
	response.values[0] = 3;
	response.values[1] = 1;
	response.values[2] = 5;
	response.times = new int[3];
	response.times[0] = 0;
	response.times[1] = 1800;
	response.times[2] = 3600;

	response.baseTime = Calendar.getInstance();
	response.baseTime.set(1998, 0, 1, 14, 30, 0);

	hackData[0] = response;

	/****************/

	response = new RequestsResponse();
	response.total = 4322;
	response.current = 1235;
	response.max = 25;
	response.maxTime = Calendar.getInstance();

	response.values = new int[3];
	response.values[0] = 30;
	response.values[1] = 10;
	response.values[2] = 15;
	response.times = new int[3];
	response.times[0] = 0;
	response.times[1] = 1800;
	response.times[2] = 3600;

	response.baseTime = Calendar.getInstance();
	response.baseTime.set(1998, 0, 1, 12, 0, 0);

	hackData[1] = response;

	/****************/

	response = new RequestsResponse();
	response.total = 4323;
	response.current = 1236;
	response.max = 29;
	response.maxTime = Calendar.getInstance();

	response.values = new int[3];
	response.values[0] = 60;
	response.values[1] = 70;
	response.values[2] = 85;
	response.times = new int[3];
	response.times[0] = 3500;
	response.times[1] = 7100;
	response.times[2] = 14300;

	response.baseTime = Calendar.getInstance();
	response.baseTime.set(1998, 0, 1, 11, 0, 0);

	hackData[2] = response;

	/****************/

        MonitorDataSource dataSource = new RequestsDataSource(null);
	dataSource.setData(hackData[0]);
	dataSource.setData(hackData[1]);
	dataSource.setData(hackData[2]);
    }


    /*
    void setChart(MonitorChart newChart) {
	chart = newChart;
    }
    */
}


class UpdateTimer
implements Runnable {

    private MonitorDataSource dataSource;
    private int updateIntervalInSeconds;


    /**
     * Set this to true when you want this thread to finish.
     */
    private boolean finished;


    UpdateTimer(MonitorDataSource newDataSource,
		 int newUpdateIntervalInSeconds) {
	dataSource = newDataSource;
	updateIntervalInSeconds = newUpdateIntervalInSeconds;
    }


    public void run() {

	while(finished == false) {
	    try {
		dataSource.getLatestDataFromServer();
	    	Thread.sleep(updateIntervalInSeconds * 1000);
	    }
	    catch(Exception e)
	    {
		e.printStackTrace();
	    }
	}
    }


    void setFinished()
    {
	finished = true;
    }

}
