/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MonitorPanel.java,v 1.12.6.1 2000/10/19 17:59:08 jasona Exp $
 */






/* This file was created and edited with tabstops set to 8.
 * If you are using "vi" to edit this file, enter the command:
 * 
 *        :set ts=8
 *
 * to set the tabstops to 8.
 */
package com.lutris.appserver.admin.presentation.monitor.gui;


import java.awt.*;
import javax.swing.event.*;
import javax.swing.*;

/**
 */
class MonitorPanel
extends JPanel
implements ChangeListener
{
    static final int UPDATE_INTERVAL_MIN = 5;
    static final int UPDATE_INTERVAL_MAX = 300;

    static final int LEGEND_BORDER_WIDTH = 4;

    static final int GRAPH_PANEL_INSET_TOP = 5;
    static final int GRAPH_PANEL_INSET_LEFT = 10;
    static final int GRAPH_PANEL_INSET_BOTTOM = 0;
    static final int GRAPH_PANEL_INSET_RIGHT = 10;

    private static final int REQUESTS_GRAPH_PANEL_INDEX = 0;
    private static final int SESSIONS_GRAPH_PANEL_INDEX = 1;
    private static final int DB_CONNECTIONS_GRAPH_PANEL_INDEX = 2;
    private static final int CACHES_GRAPH_PANEL_INDEX = 3;
    private static final int TIME_STATS_PANEL_INDEX = 4;
    private static final int PREFERENCES_PANEL_INDEX = 5;

    private JApplet applet;

    private DynamicPanel prevPanel;

    private int updateIntervalInSeconds = 60;

    private JTabbedPane tabbedPane;


    MonitorPanel(JApplet newApplet)
    {
        super(new GridBagLayout());

	applet = newApplet;

        tabbedPane = new JTabbedPane();
        GridBagConstraints gc = new GridBagConstraints();
        gc.weightx = 1;
        gc.weighty = 1;
        gc.fill = GridBagConstraints.BOTH;
        add(tabbedPane, gc);

        tabbedPane.addChangeListener(this);

        tabbedPane.addTab("Requests", null, new JLabel("Requests Panel"));
        tabbedPane.addTab("Sessions", null, new JLabel("Sessions Panel"));
        tabbedPane.addTab("DB Connections", null,
                          new JLabel("DB Connections Panel"));
        tabbedPane.addTab("Caches", null, new JLabel("Caches Panel"));
        tabbedPane.addTab("Time Stats", null, new JLabel("Time Stats Panel"));
        tabbedPane.addTab("Preferences", null, new JLabel("Preferences Panel"));
    }


    /**
     * Get the update interval in seconds.
     */
    int getUpdateIntervalInSeconds() {
	return(updateIntervalInSeconds);
    }


    /**
     * Set the update interval in seconds.
     */
    void setUpdateIntervalInSeconds(int newUpdateIntervalInSeconds) {
	updateIntervalInSeconds = newUpdateIntervalInSeconds;

	for (int index = 0; index < tabbedPane.getTabCount(); index++) {

	    if (tabbedPane.getComponentAt(index) instanceof DynamicPanel) {
	    	DynamicPanel panel =
			(DynamicPanel)tabbedPane.getComponentAt(index);
            	if (panel != null) {
		    panel.setUpdateIntervalInSeconds(updateIntervalInSeconds);
	    	}
	    }
	}
    }


    /**
     * JTabbedPane ChangeListner
     */
    public void stateChanged(ChangeEvent e) {
        System.out.println("Enter stateChanged");

        JTabbedPane tabbedPane = (JTabbedPane)(e.getSource());
        int index = tabbedPane.getSelectedIndex();

	/* Get the newly selected panel.
	 * This is a hack for now because there is only
	 * the RequestsGraphPanel implemented.
	 */
	Component comp = tabbedPane.getSelectedComponent();
	DynamicPanel panel;
	if ((comp != null) && (comp instanceof DynamicPanel)) {
	    panel = (DynamicPanel)tabbedPane.getSelectedComponent();

	}
	else if ((comp != null) && (comp instanceof JLabel)) {
	    System.out.println("Panel currently has a label");
	    panel = null;

	}
	else {
	    panel = null;
	}

        if (panel == null) {
            panel = createTabPanel(tabbedPane, index);
        }

	/* Stop the updating of the panel being hidden. */
	if (prevPanel != null)
	{
	    prevPanel.stopUpdate();
	}

	/* The panel is being displayed, start it updating. */
	if (panel != null) {
	    panel.startUpdate();
	}

	prevPanel = panel;

        System.out.println("Exit stateChanged");
    }


    private DynamicPanel createTabPanel(JTabbedPane tabbedPane, int index) {

        //if (tabbedPane.getComponentAt(index) != null) {
        if ((tabbedPane.getComponentAt(index) instanceof JLabel) == false) {
            throw(new IllegalStateException(
                "Panel already exists at tab at index "+index));
        }

        Component newPanel = null;

        switch(index) {
            case REQUESTS_GRAPH_PANEL_INDEX:
                System.out.println("Creating requests graph panel.");
		RequestsDataSource dataSource = new RequestsDataSource(applet);
                newPanel = new RequestsGraphPanel(dataSource);
            break;

            case PREFERENCES_PANEL_INDEX:
                System.out.println("Creating preferences panel.");
		newPanel = new PreferencesPanel(this);
	    break;

            default:
                //throw(new IllegalArgumentException("No such tab index."));
		newPanel = null;
            //break;
        }

	if (newPanel == null) {
	    return(null);
	}

	((DynamicPanel)newPanel).setUpdateIntervalInSeconds(
		updateIntervalInSeconds);
        tabbedPane.setComponentAt(index, newPanel);

	return((DynamicPanel)newPanel);
    }

}
