/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: RequestsGraphPanel.java,v 1.12.6.1 2000/10/19 17:59:08 jasona Exp $
 */






/* This file was created and edited with tabstops set to 8.
 * If you are using "vi" to edit this file, enter the command:
 * 
 *        :set ts=8
 *
 * to set the tabstops to 8.
 */
package com.lutris.appserver.admin.presentation.monitor.gui;


import java.awt.*;
import java.util.*;
import java.text.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.*;
import jclass.chart.*;


/**
 */
class RequestsGraphPanel
extends JPanel
implements Observer, DynamicPanel
{
    MonitorChart chart;
    RequestsDataSource dataSource;
    JLabel totalValueLabel;
    JLabel currentValueLabel;
    JLabel peakValueLabel;
    JLabel peakTimeLabel;
    //JCAxis peakLine;
    boolean peakColorSet = false;


    RequestsGraphPanel(RequestsDataSource newDataSource)
    {
        super(new GridBagLayout());

	dataSource = newDataSource;

	JPanel graphPanel = new JPanel(new GridBagLayout());
	GridBagConstraints gc = new GridBagConstraints();
	gc.gridx = 0;
	gc.gridy = 0;
	gc.weightx = 1;
	gc.weighty = 1;
	gc.fill = GridBagConstraints.BOTH;
	gc.insets = new Insets(MonitorPanel.GRAPH_PANEL_INSET_TOP,
			       MonitorPanel.GRAPH_PANEL_INSET_LEFT,
			       MonitorPanel.GRAPH_PANEL_INSET_BOTTOM,
			       MonitorPanel.GRAPH_PANEL_INSET_RIGHT);
	add(graphPanel, gc);

	TitledBorder border = new TitledBorder("Requests per Minute");
	graphPanel.setBorder(border);

	chart = new MonitorChart();
        chart.getDataView(0).setDataSource(dataSource);
        chart.getChartArea().getYAxis(0).setTitle(
	    new JCAxisTitle("Requests"));

        gc = new GridBagConstraints();
        gc.weightx = 1;
        gc.weighty = 1;
        gc.fill = GridBagConstraints.BOTH;
        graphPanel.add(chart, gc);

	JPanel panel = new JPanel(new GridBagLayout());
        gc = new GridBagConstraints();
        gc.gridx = 0;
        gc.gridy = 1;
        gc.weightx = 1;
        gc.weighty = 0;
        gc.fill = GridBagConstraints.NONE;
	gc.anchor = GridBagConstraints.CENTER;
	gc.insets = new Insets(5, 10, 10, 10);
        add(panel, gc);

	JLabel label = new JLabel("Total:");
        gc = new GridBagConstraints();
	gc.gridx = 0;
	gc.gridy = 0;
	gc.anchor = GridBagConstraints.EAST;
        panel.add(label, gc);

	totalValueLabel = new JLabel("N/A");
        gc = new GridBagConstraints();
	gc.gridx = 1;
	gc.gridy = 0;
        gc.weightx = 1;
        gc.weighty = 0;
        gc.fill = GridBagConstraints.HORIZONTAL;
	gc.anchor = GridBagConstraints.WEST;
	gc.insets = new Insets(0, 5, 0, 0);
        panel.add(totalValueLabel, gc);

	label = new JLabel("Current:");
        gc = new GridBagConstraints();
	gc.gridx = 2;
	gc.gridy = 0;
	gc.insets = new Insets(0, 20, 0, 0);
	gc.anchor = GridBagConstraints.EAST;
        panel.add(label, gc);

	currentValueLabel = new JLabel("N/A");
        gc = new GridBagConstraints();
	gc.gridx = 3;
	gc.gridy = 0;
        gc.weightx = 1;
        gc.weighty = 0;
        gc.fill = GridBagConstraints.HORIZONTAL;
	gc.anchor = GridBagConstraints.WEST;
	gc.insets = new Insets(0, 5, 0, 0);
        panel.add(currentValueLabel, gc);

	label = new JLabel("Peak:");
        gc = new GridBagConstraints();
	gc.gridx = 0;
	gc.gridy = 1;
	gc.anchor = GridBagConstraints.EAST;
        panel.add(label, gc);

	peakValueLabel = new JLabel("N/A");
        gc = new GridBagConstraints();
	gc.gridx = 1;
	gc.gridy = 1;
        gc.weightx = 1;
        gc.weighty = 0;
        gc.fill = GridBagConstraints.HORIZONTAL;
	gc.anchor = GridBagConstraints.WEST;
	gc.insets = new Insets(0, 5, 0, 0);
        panel.add(peakValueLabel, gc);

	label = new JLabel("When:");
        gc = new GridBagConstraints();
	gc.gridx = 2;
	gc.gridy = 1;
	gc.insets = new Insets(0, 20, 0, 0);
	gc.anchor = GridBagConstraints.EAST;
        panel.add(label, gc);

	peakTimeLabel = new JLabel("N/A");
        gc = new GridBagConstraints();
	gc.gridx = 3;
	gc.gridy = 1;
        gc.weightx = 1;
        gc.weighty = 0;
        gc.fill = GridBagConstraints.HORIZONTAL;
	gc.anchor = GridBagConstraints.WEST;
	gc.insets = new Insets(0, 5, 0, 0);
        panel.add(peakTimeLabel, gc);

	/*
	 * Start listening to the data.
	 */
	dataSource.addObserver(this);
    }


    public void update(Observable obj, Object arg) {

	System.out.println("Enter update");

	if (obj != dataSource)
	{
	    throw(new IllegalArgumentException(
			"Update of object other than dataSource."));
	}

	totalValueLabel.setText(
		(new Long(dataSource.getTotal())).toString());
	/*
	System.out.println("totalValueLabel.setText("+
		(new Long(dataSource.getTotal())).toString()+")");
	*/
		
	currentValueLabel.setText(
		(new Integer(dataSource.getCurrent())).toString());
	peakValueLabel.setText(
		(new Integer(dataSource.getMax())).toString());

	if (dataSource.getMaxTime() != null) {

	    peakTimeLabel.setText(getTimeString(dataSource.getMaxTime()));
	}
	else {
	    peakTimeLabel.setText("N/A");
	}

	totalValueLabel.updateUI();
	currentValueLabel.updateUI();

	Calendar startTime = dataSource.getBaseTime();
        chart.getChartArea().getXAxis(0).setTimeBase(startTime.getTime());

	chart.setChartAxisBounds();
	chart.setLineColors();

	System.out.println("Exit update");
    }


/*
    private void setChartAxisBounds() {
	
	double xMin = 0;
	double xMax = 3600;
	double yMin = 0;
	double yMax = 100;

	if (dataSource != null) {

	    Vector row = dataSource.getRow(0);

	    if ((row != null) && (row.size() > 0)) {
		xMin = ((Double)(row.firstElement())).doubleValue();
	    }
	    if ((row != null) && (row.size() > 0)) {
		xMax = ((Double)(row.lastElement())).doubleValue();
	    }

	    yMin = 0;


	    row = dataSource.getRow(1);
	    if ((row != null) && (row.size() > 0)) {
		yMax = ((Double)(row.firstElement())).doubleValue();
	    }

	    for (Enumeration elements = row.elements();
		 elements.hasMoreElements();) {
		double value = ((Double)(elements.nextElement())).doubleValue();

		if (value > yMax) {
		    yMax = value;
		}
	    }

	    if (dataSource.getMax() > yMax) {
		yMax = dataSource.getMax();
	    }
	}

        chart.getChartArea().getXAxis(0).setMin(xMin);
        chart.getChartArea().getXAxis(0).setMax(xMax);
        chart.getChartArea().getYAxis(0).setMin(yMin);
        chart.getChartArea().getYAxis(0).setMax(yMax);
    }
*/


    /**
     * Move this to a utilities package later.
     */
    static String getTimeString(Calendar time)
    {

	/*
	 * TODO - Take locale and timezone later?
	 */
	DateFormat dateFormat = DateFormat.getTimeInstance(DateFormat.SHORT);

	// Is this needed, or is it taken care of because we
	// are using the Calendar class?
	//dateFormat.setTimeZone(...);

	Date date = time.getTime();
	return(dateFormat.format(date));
    }


    public void startUpdate() {
	if (dataSource == null)
	{
	    return;
	}

	dataSource.startUpdate();
    }


    public void stopUpdate() {
	if (dataSource == null)
	{
	    return;
	}

	dataSource.stopUpdate();
    }


    public void setUpdateIntervalInSeconds(int updateIntervalInSeconds) {
	if (dataSource == null)
	{
	    return;
	}

	dataSource.setUpdateIntervalInSeconds(updateIntervalInSeconds);
    }
}
