/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: RequestsResponse.java,v 1.11.6.1 2000/10/19 17:59:08 jasona Exp $
 */






package com.lutris.appserver.admin.presentation.monitor.io;

import java.util.*;
import java.io.*;

/**
 * This object is sent in response to a RequestsRequest object that
 * the Monitor applet sent to the MonitorDataServer.
 * <P></P>
 * We probably should have a base class that all response objects
 * are subclassed from.  (Similar to the way we have a BaseRequest object.)
 */
public class RequestsResponse
implements Serializable {

    /**
     * This is the total number of requests that have been
     * made by an application since the Multiserver started keeping
     * track of them.
     */
    public long total;


    /**
     * This is the number of requests that have been
     * made over the last time interval by an application.
     * <P></P>
     * The length of the time interval is stored in
     * this object's timeIntervalInSeconds member data.
     * For example, if timeIntervalInSeconds is 60, and
     * current is 25, that means that there were 25 requests
     * over the last 60 second interval that was recorded.
     */
    public int current;


    /**
     * This is the maximum number of requests that have been
     * made over any time interval by an application since the
     * Multiserver started keeping track of them.
     * <P></P>
     * The length of the time interval is stored in
     * this object's timeIntervalInSeconds member data.
     */
    public int max;


    /**
     * This is the when the max value occurred.
     * This value is milliseconds since January 1, 1970 GMT.
     */
    public Calendar maxTime;


    /**
     * This is the length of the time interval in seconds.
     * i.e. The data being collected is at this granularity.
     */
    public int timeIntervalInSeconds;


    /**
     * This array contains the actual data values.
     * These are the graph data point Y values.
     * Each value in this values array corresponds to
     * a time value in the times array.
     *
     * i.e. the number of requests that the application
     * made over a time interval.
     */
    public int[] values;


    /**
     * This is the time of the first data point in the
     * timeValues array.  The values in the timeValues
     * array are offsets in seconds from this base time.
     */
    public Calendar baseTime;


    /**
     * This array contains the times that the data values
     * were collected.  These are the graph data point X values.
     * The values in this array are actually a number of
     * seconds offset from baseTime.
     * Each value in this times array corresponds to
     * a data value in the values array.
     *
     * For example, if data is collected every 5 minutes,
     * the first value in this array would be 0, and the
     * second value would be 300, the third 600, etc.
     */
    public int[] times;

    public String toString() {

	String returnValue =
		"total("+total+") current("+current+") max("+max+")";

	if (values == null) {
	    returnValue += " values(null)";
	}
	else {
	    returnValue += " values.length("+values.length+")";
	}

	if (baseTime == null) {
	    returnValue += " baseTime(null)";
	}
	else {
	    long millis = baseTime.getTime().getTime();
	    returnValue += " baseTime("+(millis / 1000)+")";
	}

	return(returnValue);
    }
}
