/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ChannelStatus.java,v 1.4.12.1 2000/10/19 17:59:09 jasona Exp $
 */

package org.enhydra.servlet.connectionMethods;
 
import java.util.Vector;
import java.util.Date;


/**
    ConnectionMethods return this object, which describes the current
    status of a channel in a ConnectionMethod. Do not modify the contents
    of this structure. Only ConnectionMethods should create instances of
    this object.

    @see org.enhydra.servlet.connectionMethods.ConnectionMethod
    @see org.enhydra.servlet.filter.Filter
    @author Andy John
*/
public class ChannelStatus {

    /**
     * Constructor
     */
    public ChannelStatus()
    {
    }

    /**
     * Constructor
     * @param channel A channel to get the current status from.
     */
    public ChannelStatus(Channel channel)
    {
        this.URLPrefix = channel.getURLPrefix();
        this.servletID = channel.getServletID();
        this.enabled = channel.getEnabled();
        this.requestCount = channel.getRequestCount();
        this.filterIDs = channel.getFilterIDs();
        this.createTime = channel.getCreateTime();
        this.enabledDisabledTime = channel.getEnabledDisabledTime();
    }

    /**
        If the requested URL starts with this string, it is sent to
        the Servlet identified in this channel.
    */
    public String           URLPrefix;  

    /**
        This is the Servlet this channel should send requests to.
        This name is only valid in the ServletManager passed into the
        ConnectionManager's initialize() function.
    */
    public String           servletID;

    /**
        Is this channel enabled?
        If not, requests will be refused, and not passed on the Servlet.
    */
    public boolean          enabled;

    /**
        The number of requests sent through this channel to the Servlet.
    */
    public long         requestCount;   
    
    /**
        All the Filters currently applied to this channel.
        @see org.enhydra.servlet.filter.Filter
    */
    public String[]     filterIDs;

    /**
        The time when the channel was added to the connection method.
    */
    public Date             createTime;

    /**
        The time when this channel was most recently enabled or disabled. 
        When the channel is first created, it is disabled, and this is
        set to the same value as createTime.
    */
    public Date             enabledDisabledTime;
}

