/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ServletRecorderFilter.java,v 1.4.2.2.2.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.debug;

import java.util.Enumeration;
import javax.servlet.Servlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.enhydra.servlet.filter.*;

/**
 * Servlet transaction filter to record transaction for access via the
 * DebugManager class.
 *
 * @see		org.enhydra.servlet.filter
 * @version	$Revision: 1.4.2.2.2.1 $
 * @author	Mark Diekhans
 */
public class ServletRecorderFilter extends Filter {

    /**
     * Next id number for all filters of this class.
     */
    private static int nextId = 1;

    /**
     * Should we save a copy of all the data written to the response?
     */
    private boolean saveResponseData;

    /**
     * Debug manager that this object is associated with.
     */
    private DebugManager debugManager;

    /**
     *  The name of the servlet we are watching.
     */
    private String servletName;

    /**
     * Symbolic id of filter instance.
     */
    private String symId;

    /**
     * Construct a new recording tranaction filter.
     *
     * @param debugManager debug manager that this object is associated with.
     * @see org.enhydra.servlet.filter.Filter
     */
    public ServletRecorderFilter (DebugManager debugManager,
                                  boolean saveResponseData,
                                  String servletName) {
        super("servlet transaction recorder");
        this.debugManager = debugManager;
        this.saveResponseData = saveResponseData;
        this.servletName = servletName;
        symId = this.getClass().getName() + nextId;
        nextId++;
	setServiceInterceptor(new RecorderServiceInterceptor(debugManager, saveResponseData, servletName, null));
    }

    /**
     * Get the symbolic id.
     *
     * @return symbolic id.
     */
    public String getId() {
        return symId;
    }

    public RequestFilter wrapRequest(HttpServletRequest request,
	    HttpServletResponse response) {
	RequestRecord rr = new RequestRecord(request);
	request.setAttribute(
		"Enhydra.RecorderServiceInterceptor.requestRecord", rr);
	return rr;
    }

    public ResponseFilter wrapResponse(HttpServletRequest request,
	    HttpServletResponse response) {
	ResponseRecord rr = new ResponseRecord(response, saveResponseData);
	request.setAttribute(
		"Enhydra.RecorderServiceInterceptor.responseRecord", rr);
	return rr;
    }

}

