/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PrintTransactionResponse.java,v 1.4.12.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.filter;

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletOutputStream;
import org.enhydra.servlet.filter.*;


/*
    Print everything out.
*/
class PrintTransactionResponse extends ResponseFilter {

    private String          prefix; 
    private PrintWriter     out;
    private boolean         logStream;

    public PrintTransactionResponse (HttpServletResponse res,
                String prefix, PrintWriter out, boolean logStream) {
        super(res);
        this.prefix = prefix;
        this.out = out;
        this.logStream = logStream;
    }

    public void setContentLength (int len) {
        out.println(prefix + "setContentLength(" + len + ")");
        out.flush();
        myResponse.setContentLength(len);
    }

    public void setContentType (String type) {
        out.println(prefix + "setContentType(" + type + ")");
        out.flush();
        myResponse.setContentType(type);
    }

    public ServletOutputStream getOutputStream() throws IOException {
        ServletOutputStream os;
        try {
            os = myResponse.getOutputStream();
        } catch (IOException e) {
            out.println(prefix + "getOutputStream() threw " + e);
            out.flush();
            throw(e);
        }
        out.println(prefix + "getOutputStream() = " + os);
        out.flush();
        if (logStream) {
            String pre = prefix + "Output: ";
            return new PrintTransactionOutputStream(pre, out, os);
        } else 
            return os;
    }

    //----------------------------------------------------------------

    public boolean containsHeader(String name) {
        boolean ch = myResponse.containsHeader(name);
        out.println(prefix + "containsHeader(" + name + ") = " + ch);
        out.flush();
        return ch;
    }

    public void setStatus(int sc, String sm) {
        out.println(prefix + "setStatus(" + sc + ", " + sm + ")");
        out.flush();
        myResponse.setStatus(sc, sm);
    }

    public void setStatus(int sc) {
        out.println(prefix + "setStatus(" + sc + ")");
        out.flush();
        myResponse.setStatus(sc);
    }

    public void setHeader(String name, String value) {
        out.println(prefix + "setHeader(" + name + ", " + value + ")");
        out.flush();
        myResponse.setHeader(name, value);
    }

    public void setIntHeader(String name, int value) {
        out.println(prefix + "setIntHeader(" + name + ", " + value + ")");
        out.flush();
        myResponse.setIntHeader(name, value);
    }

    public void setDateHeader(String name, long date) {
        out.println(prefix + "setDateHeader(" + name + ", " + date + ")");
        out.flush();
        myResponse.setDateHeader(name, date);
    }

    public void sendError(int sc, String msg) throws IOException {
        out.println(prefix + "sendError(" + sc + ", " + msg + ")");
        out.flush();
        myResponse.sendError(sc, msg);
    }

    public void sendError(int sc) throws IOException {
        try {
            myResponse.sendError(sc);
        } catch (IOException e) {
            out.println(prefix + "sendError(" + sc + ") threw " + e);
            out.flush();
            throw(e);
        }
        out.println(prefix + "sendError(" + sc + ")");
        out.flush();
    }

    public void sendRedirect(String location) throws IOException {
        try {
            myResponse.sendRedirect(location);
        } catch (IOException e) {    
            out.println(prefix + "sendRedirect(" + location + ") threw " + e);
            out.flush();
            throw(e);
        }
        out.println(prefix + "sendRedirect(" + location + ")");
        out.flush();
    }
}

