/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: AddHeaders.java,v 1.4.12.1 2000/10/19 17:59:10 jasona Exp $
 */

public class AddHeaders extends MultiServerFilter {

    private int min = 0;
    private int max = 500;
    private Hashtable myHeaders = new Hashtable();

    /**
     *  There will be one instance per filter entry in the config file
     *  (usually only one). Any keys are assumed to be header names, and
     *  the values of the keys are the values for the headers.
     *
     *  @param description The description from the config file.
     *  @param initArgs The init args come from the lbs config file,
     *  with the "Filter.<ID>.InitArgs." stripped off. Will be null if
     *  no init args are specified in the config file.
     *  @exception KeywordValueException If there was an error reading
     *  the initial arguments.
     */
    public AddHeaders(String description, KeywordValueTable initArgs) 
           throws KeywordValueException {
        super(description, initArgs);
        String[] keys = initArgs.keys();
        for (int i=0; i<keys.length; i++) {
            String value = initArgs.getString(keys[i]);
            if (value == null) 
                value = "";
            myHeaders.put(keys[i], value);
        }
    }


    /**
     *  This gets called on each incoming request.
     *
     *  @param originalObjects The real Servlet, request and response
     *  objects, stored in a WrapRequest object.
     *  All calls will be passed through unchanged, except for
     *  <CODE>setHeader()</CODE> calls that match headers already sent
     *  by this filter.
     *  @return A new response object, which wraps around the one passed in.
     *  It's <CODE>setHeader()</CODE> method ignores calls to set headers
     *  that are already sent. The servlet and response objects are
     *  returned unchanged.
     *  @see org.enhydra.servlet.filter.WrapResult
     */
    public WrapResult wrap(WrapResult originalObjects) {
        return new WrapResult(originalObjects.servlet,
                              originalObjects.request,
                              new MyResponse(originalObjects.response));
    }


    /*
     *  A helper class, this is what wraps the response, sets the initial
     *  headers, then ignores requests to re-set them.  
     *  There will be one instance of this per request.
     */ 
    class MyResponse extends BasicTransactionResponse {
  
        /*
         *  Just use the BasicTransactionServlet constructor. 
         */ 
        public MyResponse(HttpServletResponse originalResponse) {
            super(originalResponse);
            Enumeration e = myHeaders.keys();
            while (e.hasMoreElements()) {
                String header = (String) e.nextElement();
                String value = (String) myHeaders.get(header);
                if ((value != null) && (value.length() != 0))
                    originalResponse.setHeader(header, value);
            }
        }

        /**
         *  If the header is one we already set, ignore this call.
         *  Otherwise, pass the call through like normal.
         *
         *  @see javax.servlet.http.HttpServletResponse
         */
        public void setHeader(String name, String value) {
            if (!myHeaders.containsKey(name))
                super.setHeader(name, value);
        }

    }


}

        
