/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SlowDown.java,v 1.4.12.1 2000/10/19 17:59:10 jasona Exp $
 */

public class SlowDown extends MultiServerFilter {

    private int min = 0;
    private int max = 500;
    private Random myRand = new Random();

    /**
     *  There will be one instance per filter entry in the config file
     *  (usually only one). If the keys "minDelayMsec" and "maxDelayMsec"
     *  are specified in the config file, they are used. Otherwise defaults
     *  of 0 and 500 are used.
     *
     *  @param description The description from the config file.
     *  @param initArgs The init args come from the lbs config file,
     *  with the "Filter.<ID>.InitArgs." stripped off. Will be null if
     *  no init args are specified in the config file.
     */
    public SlowDown(String description, KeywordValueTable initArgs) {
        super(description, initArgs);
        try {
            min = Integer.parseInt(initArgs.getString("minDelayMsec"));
        } catch (KeywordValueException e) {
        } catch (NumberFormatException e) {
        } catch (NullPointerException e) {
        }
        try {
            max = Integer.parseInt(initArgs.getString("maxDelayMsec"));
        } catch (KeywordValueException e) {
        } catch (NumberFormatException e) {
        } catch (NullPointerException e) {
        }
    }


    /**
     *  This gets called on each incoming request.
     *
     *  @param originalObjects The real Servlet, request and response
     *  objects, stored in a WrapRequest object.
     *  All calls will be passed through unchanged.
     *  @return A new Servlet object, which wraps around the one passed in.
     *  It's <CODE>service()</CODE> method delays a random amount, then 
     *  calls the original servlet's <CODE>service()</CODE> method.
     *  The request and response objects are returned unchanged.
     *  @see org.enhydra.servlet.filter.WrapResult
     */
    public WrapResult wrap(WrapResult originalObjects) {
        return new WrapResult(new MyServlet(originalObjects.servlet),
                              originalObjects.request,
                              originalObjects.response);
    }


    /*
     *  A helper class, this is what wraps the servlet and actually
     *  causes the delay. There will be one instance of this per request.
     */ 
    class MyServlet extends BasicTransactionServlet {
  
        /*
         *  Just use the BasicTransactionServlet constructor. 
         */ 
        public MyServlet(Servlet origServlet) {
            super(origServlet);
        }

        /*
         *  This is where the action happens. Delay a random amount, then
         *  call the real servlet.
         */ 
        public void service(ServletRequest req, 
                            ServletResponse resp)
                    throws ServletException, IOException {
            double n = myRand.nextDouble();
            int snooze = min + (int)((double)(max - min) * n);
            try {
                Thread.sleep(snooze);
            } catch (InterruptedException e) {
            }
            super.service(req, resp);
        }
    }


}

        
