/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EnhydraRequest.java,v 1.9.2.5.2.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.tomcat;


import javax.servlet.jsp.tagext.Tag;

import java.util.*;
import javax.servlet.http.*;
import org.apache.tomcat.core.*;
import com.lutris.logging.Logger;

import com.lutris.appserver.server.session.*;

/*
 * Extends the Tomcat Request to override the Attribute methods so that
 * they pass through to the RequestAdapter.
 * X22 This class should go away when we upgrade Tomcat to 3.1
 * X22 since the Request class becomes an interface.
 *
 * @version $Revision: 1.9.2.5.2.1 $
 * @auther Shawn McMurdo
 */
public class EnhydraRequest extends Request {

    protected HttpServletRequest eReq;
    protected SessionManager sessionManager;
    //protected
    private String sessionId=null;
    private boolean sesionIdFromCookie=true;  // because tomcat defaults to true

    private static final String SESSION_COOKIE_NAME = "JSESSIONID";
    private static final String SESSION_URLPATH_NAME = "jsessionid";

    public EnhydraRequest() {
	super();
    }

    public void setRequestAdapter(RequestAdapter reqA) {
	super.setRequestAdapter(reqA);
	// We can do this since we know that anything that we
	// set as the RequestAdapter also implements HttpServletRequest.
	eReq = (HttpServletRequest) reqA;
    }

    public Object getAttribute(String name) {
        return eReq.getAttribute(name);
    }

    public void setAttribute(String name, Object value) {
        eReq.setAttribute(name, value);
    }

    public void removeAttribute(String name) {
	eReq.removeAttribute(name);
    }

    public Enumeration getAttributeNames() {
        return eReq.getAttributeNames();
    }


    public boolean isRequestedSessionIdFromCookie() {
	return sesionIdFromCookie;
    }

    public HttpSession getSession(boolean create) {

	HttpSession httpSession = null;

	if (httpSession == null && sessionManager != null) {
            try {
                Session session = null;
                // sessionId has been set by obtainSessionId() which was called
                // before the reqest was handled.
                if (sessionId != null) {
                    getContext().getLogChannel().write(Logger.DEBUG,
                        "Session id is " + sessionId);
                    getContext().getLogChannel().write(Logger.DEBUG,
                        "SessionManager is " + sessionManager.toString());
                    session = sessionManager.getSession(Thread.currentThread(),
                            sessionId);
                }

                // the session could be null here if the browser is still up
                // since before the server was restarted. In which case ignore
                // the sessionid
                if ((session == null) && create) {
 	            String saTok = getHeader(":aff:");
	            if (saTok == null) {
		        session = sessionManager.createSession();
	            } else {
		        session = sessionManager.createSession(saTok);
	            }

                    sessionId = session.getSessionKey();
                    placeSessionIdInCookie();
                    getContext().getLogChannel().write(Logger.DEBUG,
                        "Create new session id is " + sessionId);
                }
                if (session != null) {
                    httpSession = session.getHttpSession();
                }
           }
           catch (SessionException se) {
		se.printStackTrace(System.err);
           }
	}
	return httpSession;
   }


    String getSessionId() {
        return sessionId;
    }

    public ServerSession getServerSession(boolean create) {
        return null;
    }

    public void setServerSession(ServerSession serverSession) {
        // server session is no longer used
    }

    public void setSessionManager(SessionManager sessionManager) {
        this.sessionManager = sessionManager;
    }


    //
    // This is a new session. Create a cookie and send it back into the client.
    //
    private void placeSessionIdInCookie() {
        Cookie cookie = new Cookie( SESSION_COOKIE_NAME , sessionId);
        cookie.setMaxAge(-1);                   // expires end of session
        cookie.setPath(getContext().getPath());
        cookie.setVersion(0);
	response.addSystemCookie(cookie);
    }


    /**
     * decodeURL() Searche the url for a path parameter with the name
     * JSESSIONID. If the parameter is found the value is stored in sessionID
     * and used by the sessionMAnager later on.
     * This methid is only called if no cookie was found with a sesion id.
     */

    private void decodeURL() {
        String path = getPathInfo();
        if (path != null) {
            int param = path.indexOf(';');
            if (param >= 0 ) {
                int end = path.indexOf('?');
                if (end < 0) {
                    end = path.length();
                }
                String pathParams = path.substring(param,end);
                String jSessionId = SESSION_URLPATH_NAME + "=";
                int len = jSessionId.length();
                int jsessionid = pathParams.indexOf(jSessionId);

                if (jsessionid > 0 ) {
                    sessionId = pathParams.substring(jsessionid+len,end-param);
                }

                param = path.indexOf(';');
                if (param >= 0) {
                    setPathInfo(path.substring(0,param));
                }
            }
        }
    }


    /**
     * Extract the sessionId from a cookie or the url.
     * The sessionID is stored in a cookie called JSESSIONID or a url path
     * param called jsessionid. Tomcat uses the sessionid to find a
     * SessionManager that contains all sessions belonging to this user.
     * We use the sessionID to find a sessionManager that holds all sessions
     * belonging to an application.
     */
    public void obtainSessionId() {
	Cookie cookies[]=getCookies();
        // what if cookies is null ??
        String id=null;

	for( int i=0; i<cookies.length; i++ ) {
	    Cookie cookie = cookies[i];
	    if (cookie.getName().equals(SESSION_COOKIE_NAME)) {
		id = cookie.getValue();
                try {
                    if (sessionManager != null &&
                        sessionManager.getSession(Thread.currentThread(),id) != null) {
                        sessionId = id;
                    }
                }
                catch (Exception ex) {
                    com.lutris.logging.LogChannel logChannel =
                            Logger.getCentralLogger().getChannel("Multiserver");
                    logChannel.write(Logger.ERROR,"Request getSession() fail",ex);
                }
	    }
	}
        // look for it in the url path
        if (sessionId == null) {
            decodeURL();
            sesionIdFromCookie = (sessionId != null);
        }
        if (sessionId != null) {
            setRequestedSessionId(sessionId);
        }
    }

}

