// Copyright 2000 Sims Computing, Inc.
// Licensed under the GNU Lesser General Public License.

package com.simscomputing;

/**
Indicates that an error has been detected in the software's design or
implementation. Provides the same functionality as
java.lang.IllegalStateException except that it also allows an
exception to be encapsulated.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:33 $
*/
public class IllegalStateException extends java.lang.IllegalStateException {
  private Throwable exception;

  // note: there is no default constructor because these kinds of
  // exceptions are important enough to require an error message
  // as shown in the following constructors.

  /**
  Indicates that a software bug was found. The reason for the bug is listed
  in the specified error message.

  @param String the error message
  @precondition message != null
  */
  public IllegalStateException(String message) {
    super(message);

    if (message == null) {
      throw new NullPointerException("message is null");
    } // if
  } // constructor

  /**
  Use this constructor to include an exception that was the cause of
  the software fault.

  @param String the error message
  @param Throwable the exception that triggered the bug
  @precondition message != null && exception != null
  */
  public IllegalStateException(String message, Throwable exception) {
    super(message);

    if (message == null) {
      throw new NullPointerException("message is null");
    } // if

    if (exception == null) {
      throw new NullPointerException("exception is null");
    } // if

    this.exception = exception;
  } // constructor

  // fixme: probably want to override methods from Throwable: look at ImpossibleExceptionException
  public Throwable getException() {
    return exception;
  } // getException()
} // SoftwareFaultException
