// Copyright 2000 Sims Computing
//
// This software is licensed under the GNU Lesser General Public License,
// available at http://www.gnu.org/copyleft/lesser.html.

package com.simscomputing;

/**
Indicates that an error has been detected in the software's design
or implementation.

@deprecated use java.lang.IllegalStateException or com.simscomputing.IllegalStateException instead
@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:32 $
@invariant true
*/
public class SoftwareFaultException extends RuntimeException {
  private Throwable exception;

  // note: there is no default constructor because these kinds of
  // exceptions are important enough to require an error message
  // as shown in the following constructors.

  /**
  Indicates that a software bug was found. The reason for the bug is listed
  in the specified error message.

  @param String the error message
  @precondition message != null
  */
  public SoftwareFaultException(String message) {
    super(message);

    if (message == null) {
      throw new NullPointerException("message is null");
    } // if
  } // constructor

  /**
  Use this constructor to include an exception that was the cause of
  the software fault.

  @param String the error message
  @param Throwable the exception that triggered the bug
  @precondition message != null && exception != null
  */
  public SoftwareFaultException(String message, Throwable exception) {
    super(message);

    if (message == null) {
      throw new NullPointerException("message is null");
    } // if

    if (exception == null) {
      throw new NullPointerException("exception is null");
    } // if

    this.exception = exception;
  } // constructor

  // fixme: probably want to override methods from Throwable: look at ImpossibleExceptionException
  public Throwable getException() {
    return exception;
  } // getException()
} // SoftwareFaultException
