package com.simscomputing.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyVetoException;

import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

/**
These internal frames automatically update a Window menu. The Window
menu has a group of radio buttons to indicate which InternalFrames are
in a JDesktopPane and which one is selected.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:37 $ */
public class WindowInternalFrame extends JInternalFrame 
  implements InternalFrameListener, ActionListener {
  private JMenu windowMenu;
  JRadioButtonMenuItem radioButtonMenuItem;
  boolean isUnique;

  public WindowInternalFrame(JMenu windowMenu) {
    this(windowMenu, false);
  } // constructor

  public WindowInternalFrame(JMenu windowMenu, boolean isUnique) {
    this.windowMenu = windowMenu;
    this.isUnique = isUnique;
    addInternalFrameListener(this);
  } // constructor

  //////////////////////////////////////////
  // InternalFrameListener implementation //
  //////////////////////////////////////////

  public void internalFrameOpened(InternalFrameEvent e) {
    radioButtonMenuItem = new JRadioButtonMenuItem(getTitle());
    radioButtonMenuItem.addActionListener(this);
    windowMenu.add(radioButtonMenuItem);
  } // internalFrameOpened()

  public void internalFrameActivated(InternalFrameEvent e) {
    // fixme: workaround for JDKs prior to 1.3
    if (radioButtonMenuItem != null) {
      radioButtonMenuItem.setSelected(true);
    } // if
  } // internalFrameActivated()

  public void internalFrameDeactivated(InternalFrameEvent e) {
    // fixme: workaround for JDKs prior to 1.3
    if (radioButtonMenuItem != null) {
      radioButtonMenuItem.setSelected(false);
    } // if
  } // internalFrameDeactivated()

  public void internalFrameClosed(InternalFrameEvent e) {
    // fixme: workaround for JDKs prior to 1.3
    if (radioButtonMenuItem != null) {
      windowMenu.remove(radioButtonMenuItem);
    } // if
  } // internalFrameClosed()

  public void internalFrameClosing(InternalFrameEvent e) {
  } // internalFrameClosing()

  public void internalFrameIconified(InternalFrameEvent e) {
    // fixme: workaround for JDKs prior to 1.3
    if (radioButtonMenuItem != null) {
      radioButtonMenuItem.setSelected(false);
    } // if
  } // internalFrameIconified()

  public void internalFrameDeiconified(InternalFrameEvent e) {
  } // internalFrameDeiconified()

  ///////////////////////////////////
  // ActionListener implementation //
  ///////////////////////////////////

  public void actionPerformed(ActionEvent e) {
    try {
      setIcon(false);
      toFront();
      setSelected(true);
    } // try
    catch (PropertyVetoException ex) {
      // ignore it
    } // catch
  } // actionPerformed()
} // class WindowInternalFrame
