// Java Test Bed: tools for testing Java classes
// Copyright 1999 Sims Computing
// This software is licensed under the GNU General Public License,
// available at http://www.gnu.org/copyleft/gpl.html.

package com.simscomputing.testbed;

import java.util.Iterator;
import java.util.SortedMap;

/**
A reporter that writes its result to stdout.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:33 $
*/
public class DefaultReporter implements Reporter {
  /**
  Extracts test results from 'tester' and sends the results to stdout.

  @param Tester - a tester that has already exercised a class
  */
  public void report(TestResultCollection testResultCollection) {
    Iterator iterator = testResultCollection.getIterator();
    if (iterator == null) {
      return;
    } // if

    TestResult testResult;

    // we would prefer to iterate over methods in lexicographic order
    while (iterator.hasNext()) {
      testResult = (TestResult) iterator.next();
      if (testResult.didTestPass()) {
        System.out.println(testResult.getLongName() + ": passed, " +
                           testResult.getExecutionTimeInMilliseconds() +
                           " milliseconds");
      } // if
      else {
	if (testResult.didTestFailAbnormally()) {
	  System.out.println(testResult.getLongName() + ": failed abnormally:");
	  testResult.getTestFailureException().printStackTrace(System.out);
	  System.out.println();
	} // if
	else {
	  System.out.println(testResult.getLongName() + ": failed, " +
			     testResult.getTestFailureDescription());
	} // else
      } // else
    } // while

    if (testResultCollection.didAllTestsPass()) {
      System.out.println("\nAll " + testResultCollection.getNumberOfTests() + " tests passed");
    } // if
    else {
      System.out.println("\n" + testResultCollection.getNumberOfTestsThatPassed() + " tests passed, " +
                         (testResultCollection.getNumberOfTests() - testResultCollection.getNumberOfTestsThatPassed()) + " tests failed");
    } // else
    System.out.println("Cumulative testing time = " +
                       testResultCollection.getCumulativeTestExecutionTimeInMilliseconds() +
                       " milliseconds");
  } // report()
} // class DefaultReporter
