// Java Test Bed: tools for testing Java classes
// Copyright 1999 Sims Computing
// This software is licensed under the GNU General Public License,
// available at http://www.gnu.org/copyleft/gpl.html.

package com.simscomputing.testbed;

/**
Exercises all classes specified on the command line.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:33 $
*/
public class DefaultTester implements Tester {
  /**
  Exercises all classes specified on the command line.

  @param args - list of classes to test
  */
  public static void main(String[] args) {
    Reporter reporter;

    try {
      if (args.length <= 1) {
        System.out.println("usage: DefaultTester Reporter <list of classes to test>");
        System.exit(0);
      } // if

      try {
        // instantiate the Reporter class
        Class reporterClass;
        reporterClass = Class.forName(args[0]);
        // class has been loaded
        // make sure it implements Reporter
        Object potentialReporter = reporterClass.newInstance();
        if (!(potentialReporter instanceof Reporter)) {
          System.out.println(args[0] + " does not implement Reporter");
          System.exit(0);
          return; // to satisfy the compiler
        } // if
        reporter = (Reporter) potentialReporter;
      } // try
      catch (ClassNotFoundException e) {
        // unable to load class
        System.err.println(args[0] + ": cannot load");
        return;
      } // catch
      catch (InstantiationException e) {
        // unable to instantiate class
        System.err.println(args[0] + ": cannot instantiate");
        return;
      } // catch

      // use the specified reporter to communicate the test results
      String[] classNames = new String[args.length - 1];
      System.arraycopy(args, 1, classNames, 0, args.length - 1);
      // fixme
//        reporter.report(new DefaultTester().test(classNames));
    } // try
    catch (Throwable e) {
      System.out.println("Fatal error:");
      e.printStackTrace();
      System.out.println();
      return;
    } // catch
    finally {
      System.exit(0);
    } // finally
  } // main()

  public void testAndReport(String[] testCaseCollectionClassNames) {
    // fixme
//      new DefaultReporter().report(test(testCaseCollectionClassNames));
  } // testAndReport()

  /**
  Not implemented.
  */
  public TestResultCollection exercise() throws TesterException {
    throw new TesterException("not implemented");
  }

  public String getName() {
    return "DefaultTester";
  }

  // fixme
//    public TestResultCollection test(String[] testCaseCollectionClassNames) {
//      // add each class passed in to a TestCaseCollection. Exercise that collection.
//      // Return the results.

//      TestCaseCollection testCaseCollection = new TestCaseCollection();
//      String reason;
//      for (int index = 0; index < testCaseCollectionClassNames.length; ++index) {
//        reason = null;

//        // fixme
//  //        try {
//  //          testCaseCollection.addTestCaseCollection(testCaseCollectionClassNames[index]);
//  //        } // try
//  //        catch (ClassCastException e) {
//  //          reason = testCaseCollectionClassNames[index] 
//  //            + " could not be properly cast";
//  //        } // catch


//  //        catch (ClassNotFoundException e) {
//  //          reason = testCaseCollectionClassNames[index] + " could not be loaded";
//  //        } // catch
//  //        catch (IllegalAccessException e) {
//  //          reason = testCaseCollectionClassNames[index] 
//  //            + " could not be accessed";
//  //        } // catch
//  //        catch (InstantiationException e) {
//  //          reason = testCaseCollectionClassNames[index] 
//  //            + " could not be instantiated";
//  //        } // catch
//        if (reason != null) {
//          System.out.println("Skipping " + testCaseCollectionClassNames[index]
//                             + " because " + reason + ".");
//        } // if
//      } // for index

//      // fixme
//      return testCaseCollection.exercise(null);
//    } // test()
} // class DefaultTester
