// Java Test Bed: tools for testing Java classes
// Copyright 1999 Sims Computing
// This software is licensed under the GNU General Public License,
// available at http://www.gnu.org/copyleft/gpl.html.

package com.simscomputing.testbed;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
Exercises all class files in the specified directory.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:33 $
*/
public class DirectoryTester extends TestCaseCollection { //implements Tester {
  private File directory;
  private boolean recursive;

  public DirectoryTester(final File directory, final boolean recursive) {
    this.directory = directory;
    this.recursive = recursive;
  } // constructor

  public void reset() throws TesterException {
    super.reset();

    if (!directory.isDirectory()) {
      throw new TesterException(directory + " is not a directory");
    } // if

//    /**
//    Finds all class files that extend TestCaseCollection in the
//    specified directory. Then those classes are exercised and the
//    collective results are returned.

//    @return TestResultCollection the collective results of testing all classes */
//    public TestResultCollection exercise() {
		ArrayList /* File */ testees = new ArrayList();

		findExercisableClasses(directory, new TestCaseCollectionFileFilter(), testees);

    TestResultCollection masterCollection = new TestResultCollection();
    TestResultCollection testResultCollection;
    FileTester fileTester;
		for (int index = 0; index < testees.size(); ++index) {
			// this class can now be exercised
			File f = (File) testees.get(index);

      try {
        fileTester = new FileTester(f);
//          System.out.println("************ ready for testing: " + f);
        addTestCaseCollection(fileTester);
      } // try
      catch (TesterException e) {
        // ignore
      } // catch
//        try {
//          testResultCollection = new FileTester(f).exercise();
//          masterCollection.add(testResultCollection);
//        } // try
//        catch (TesterException e) {
//          // just ignore it.
//          System.out.println(f + " failed to exercise");
//        } // catch
		} // for index

//  		return masterCollection;
  } // exercise()

	private void findExercisableClasses(File directory, FileFilter fileFilter,
																			ArrayList outputList) {
		File[] candidates = directory.listFiles(fileFilter);
		final int length = candidates != null ? candidates.length : 0;

		for (int index = 0; index < length; ++index) {
			if (recursive && candidates[index].isDirectory()) {
				findExercisableClasses(candidates[index], fileFilter, outputList);
			} // if
			else {
				outputList.add(candidates[index]);
      } // else
		} // for index
	} // findExercisableClasses()

	class TestCaseCollectionFileFilter implements FileFilter {
		/**
		Accept files that end in ".class" and implement TestCaseCollection
    */
		public boolean accept(File pathname) {
			if (pathname.isDirectory() || pathname.toString().endsWith(".class")) {
				// fixme: make sure it extends TestCaseCollections
				return true;
			} // if

			return false;
		} // accept()
	} // class TestCaseCollectionFileFilter

  public String getName() {
    return directory.getAbsolutePath();
  } // getName()

//  	public static void main(String[] args) {
//  		FileSystemTestCaseCollection fstcc = new FileSystemTestCaseCollection(".");
//  		fstcc.exercise();
//  	} // main()
} // class FileSystemTestCaseCollection
