package com.simscomputing.testbed;

import java.io.File;
import java.lang.reflect.*;
import java.net.*;
import java.util.StringTokenizer;

public class FileTester extends TestCaseCollection {
  File file;

  public FileTester(final File file) throws TesterException {
    this.file = file;
    reset();
  } // constructor

  /**
  For use by subclasses.
  */
  protected FileTester() {
    // for use by subclasses
  } // constructor

  public void reset() throws TesterException {
    try {
      super.reset();
      // retrieve the TestClassPath property
      final String testClassPath = System.getProperty("TestClassPath");
//        System.out.println("java.class.path = " + System.getProperty("java.class.path"));
//        System.out.println("TestClassPath = " + testClassPath);
      FileSystemClassLoader fileSystemClassLoader;
      if (testClassPath != null) {
        final StringTokenizer tokenizer = new StringTokenizer(testClassPath, File.pathSeparator);
        String token;
        final URL[] urls = new URL[tokenizer.countTokens()];
        int index = 0;
        while (tokenizer.hasMoreTokens()) {
          token = tokenizer.nextToken();
          
          String fileUrl = "file:" + token;
          if (token.length() > 0 && token.charAt(token.length() - 1) != File.separatorChar) {
            File fileToken = new File(token);
            if (fileToken.isDirectory()) {
              fileUrl = fileUrl + File.separatorChar;
            } // if
          } // if

          urls[index] = new URL(fileUrl);
//            System.out.println(urls[index]);
          ++index;
        } // while
        fileSystemClassLoader = new FileSystemClassLoader(urls, getClass().getClassLoader());
      } // if
      else {
        fileSystemClassLoader = new FileSystemClassLoader(new URL[0], getClass().getClassLoader());
      } // else
//        fileSystemClassLoader.exemptPackagePrefix("com.simscomputing.testbed");

      Class c = loadSpecialClass(fileSystemClassLoader);//.loadClass(file);
//        System.out.println("c's CL: " + c.getClassLoader());
 
//        Object o = c.newInstance();
//        System.out.println("loaded class: " + c);
//         testCaseCollectionClass = fileSystemClassLoader.loadClass("com.simscomputing.testbed.TestCaseCollection");

      // need to load TestCaseCollection through fileSystemClassLoader.
      // Otherwise, the TestCaseCollection class will be different than the one used
      // in the uers's test code because it's loaded from *different* class loaders.
//        Class tcc = fileSystemClassLoader.loadClass("com.simscomputing.testbed.TestCaseCollection");

      if (TestCaseCollection.class.isAssignableFrom(c)) {
//          System.out.println("does extend TestCaseCollection");
        Object tcc = c.newInstance();
//          System.out.println("the new tcc's CL = " + tcc.getClass().getClassLoader().getClass().getName());
//          if (tcc instanceof TestCaseCollection) {
//            System.out.println("still does extend TestCaseCollection");
//          } // if

        TestCaseCollection testCaseCollection = (TestCaseCollection) tcc;
        addTestCaseCollection(testCaseCollection);

//          Object testCaseCollection = c.newInstance();
//          TestResultCollection trc = testCaseCollection.exercise(this);

//          int totalTests = testCaseCollection.exercise(this);
//          System.out.println("FileTester: total tests = " + totalTests);
//          try {
//            testCaseCollection.waitUntilFinished();
//            System.out.println("********* exercising is done");
//          } // try
//          catch (InterruptedException e) {
//            // fixme
//            e.printStackTrace();
//          } // catch
//          TestResultCollection trc = testCaseCollection.getTestResults();

//  //          System.out.println("exercising " + tcc.getClass().getName());
//  //          TestResultCollection trc = tcc.exercise();

//          return trc;
      } // if
      else {
        throw new TesterException(getName() +
                                  " is not a TestCaseCollection");
      } // else
    } // try
    catch (ClassNotFoundException ex) {
      // if a file cannot be opened for testing, just report the error
      throw new TesterException(getName() + " could not be found.\n\n" + ex.getMessage());
    } // catch
    catch (IllegalAccessException ex) {
      // if a file cannot be opened for testing, just report the error
      throw new TesterException(getName() + " does not appear to be a public class or have a public default constructor.\n\n" + ex.getMessage());
    } // catch
    catch (IllegalAccessError ex) {
      // if a file cannot be opened for testing, just report the error
      throw new TesterException(getName() + " had an illegal access error.\n\n" + ex.getMessage());
    } // catch
    catch (InstantiationException ex) {
      // if a file cannot be opened for testing, just report the error
      throw new TesterException(getName() + " seems to be missing a default public constructor.\n\n" + ex.getMessage());
    } // catch
    catch (MalformedURLException ex) {
      // if a file cannot be opened for testing, just report the error
      throw new TesterException(getName() + " had a a malformed URL.\n\n" + ex.getMessage());
    } // catch
    catch (NoClassDefFoundError ex) {
      // if a file cannot be opened for testing, just report the error
      throw new TesterException(getName() + " could not have its class definition found.\n\n" + ex.getMessage());
    } // catch
  } // exercise()

  public String getName() {
    return file.getAbsolutePath();
  } // getName()

  public void progressMade(TesterProgress progress) {
//      System.out.println("progress made: " + progress);
  } 

  protected Class loadSpecialClass(FileSystemClassLoader fileSystemClassLoader) throws ClassNotFoundException {
    return fileSystemClassLoader.loadClass(file);
  } // loadSpecialClass()
} // class FileTester
