package com.simscomputing.testbed.gui;

import com.simscomputing.swing.DesktopPane;
import com.simscomputing.swing.SimsComputingTheme;
import com.simscomputing.swing.SwingUtilities;
//  import com.simscomputing.testbed.DefaultReporter;
import com.simscomputing.testbed.ClassTester;
import com.simscomputing.testbed.DirectoryTester;
import com.simscomputing.testbed.FileSystemClassLoader;
import com.simscomputing.testbed.FileTester;
import com.simscomputing.testbed.TestCaseCollection;
import com.simscomputing.testbed.Tester;
import com.simscomputing.testbed.TesterException;
import com.simscomputing.testbed.TestResultCollection;
import com.simscomputing.util.ApplicationProperties;
import com.simscomputing.util.*;

import java.awt.AWTEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.*;
import java.io.File;

import javax.swing.JFrame;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JLayeredPane;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.filechooser.FileFilter;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
The main frame for the test bed GUI.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:35 $
*/
public class Main extends JFrame implements ActionListener {
  private JMenuBar menuBar = new JMenuBar();
  private JMenu applicationMenu = new JMenu(Resources.SHORT_APPLICATION_NAME);

  private JMenuItem applicationRunClassMenuItem = new JMenuItem(Resources.APPLICATION_MENU_RUN_CLASS);
  private JMenuItem applicationRunMenuItem = new JMenuItem(Resources.APPLICATION_MENU_RUN);
  private JMenuItem applicationRunDirectoryMenuItem = new JMenuItem(Resources.APPLICATION_MENU_RUN_DIRECTORY);
  private JMenuItem applicationRunRecursiveDirectoryMenuItem = new JMenuItem(Resources.APPLICATION_MENU_RUN_DIRECTORY_RECURSIVE);

  private JMenuItem applicationOpenClassMenuItem = new JMenuItem(Resources.APPLICATION_MENU_OPEN_CLASS);
  private JMenuItem applicationOpenMenuItem = new JMenuItem(Resources.APPLICATION_MENU_OPEN);
  private JMenuItem applicationOpenDirectoryMenuItem = new JMenuItem(Resources.APPLICATION_MENU_OPEN_DIRECTORY);
  private JMenuItem applicationOpenRecursiveDirectoryMenuItem = new JMenuItem(Resources.APPLICATION_MENU_OPEN_DIRECTORY_RECURSIVE);

  private JMenuItem applicationExitMenuItem = new JMenuItem(Resources.APPLICATION_MENU_EXIT);
  private JMenu editMenu = new JMenu(Resources.EDIT_MENU);
  private JMenu windowMenu = new JMenu(Resources.WINDOW_MENU);
  private JMenu helpMenu = new JMenu(Resources.HELP_MENU);
  private JMenuItem helpAboutMenuItem = new JMenuItem(Resources.HELP_MENU_ABOUT);
  private DesktopPane desktop = new DesktopPane();
  // new objects!
  // can't initialize mruManager until we know where it will go
  // in the JMenu.
  private MruMenuManager mruManager;
  private ApplicationProperties appProperties;

//    private FileSystemClassLoader fileSystemClassLoader;

  public Main() {
    super(Resources.APPLICATION_NAME);

    Resources.setWindowMenu(windowMenu);
    Resources.setMainFrame(this);
    Resources.setDesktop(desktop);
    // new code:  initialize the ApplicationProperties object.
    try {
      appProperties = new ApplicationProperties(Resources.PROPERTIES_FILE_NAME);
      Resources.setProperties(appProperties);
    } // try
    catch (Throwable ex) {
      final ErrorDialog ed = new ErrorDialog(Resources.getMainFrame(),
                                             "Could not open properties file",
                                             ex.getMessage());
      ed.show();
    } // catch

    enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    windowMenu.setMnemonic(Resources.WINDOW_MENU_MNEMONIC);
    editMenu.setMnemonic(Resources.EDIT_MENU_MNEMONIC);

    applicationRunClassMenuItem.setMnemonic(Resources.APPLICATION_MENU_RUN_CLASS_MNEMONIC);
    applicationRunClassMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runClassMenuItem(true);
      } // actionPerformed()
    });

    applicationOpenClassMenuItem.setMnemonic(Resources.APPLICATION_MENU_OPEN_CLASS_MNEMONIC);
    applicationOpenClassMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runClassMenuItem(false);
      } // actionPerformed()
    });

    applicationRunMenuItem.setMnemonic(Resources.APPLICATION_MENU_RUN_MNEMONIC);
    applicationRunMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runMenuItem(true);
      } // actionPerformed()
    });

    applicationOpenMenuItem.setMnemonic(Resources.APPLICATION_MENU_OPEN_MNEMONIC);
    applicationOpenMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runMenuItem(false);
      } // actionPerformed()
    });

    applicationRunDirectoryMenuItem.setMnemonic(Resources.APPLICATION_MENU_RUN_DIRECTORY_MNEMONIC);
    applicationRunDirectoryMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runDirectoryMenuItem(true, false);
      } // actionPerformed()
    });

    applicationOpenDirectoryMenuItem.setMnemonic(Resources.APPLICATION_MENU_OPEN_DIRECTORY_MNEMONIC);
    applicationOpenDirectoryMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runDirectoryMenuItem(false, false);
      } // actionPerformed()
    });

    applicationRunRecursiveDirectoryMenuItem.setMnemonic(Resources.APPLICATION_MENU_RUN_DIRECTORY_RECURSIVE_MNEMONIC);
    applicationRunRecursiveDirectoryMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runDirectoryMenuItem(true, true);
      } // actionPerformed()
    });

    applicationOpenRecursiveDirectoryMenuItem.setMnemonic(Resources.APPLICATION_MENU_OPEN_DIRECTORY_RECURSIVE_MNEMONIC);
    applicationOpenRecursiveDirectoryMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runDirectoryMenuItem(false, true);
      } // actionPerformed()
    });

    applicationExitMenuItem.setMnemonic(Resources.APPLICATION_MENU_EXIT_MNEMONIC);
    applicationExitMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        exitProgram();
      } // actionPerformed()
    });

    helpAboutMenuItem.setMnemonic(Resources.HELP_MENU_ABOUT_MNEMONIC);
    helpAboutMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        AboutDialog aboutDialog = new AboutDialog(Main.this);
        SwingUtilities.centerOnParent(aboutDialog, Main.this);
        aboutDialog.show();
      } // actionPerformed()
    });

    applicationMenu.setMnemonic(Resources.SHORT_APPLICATION_NAME_MNEMONIC);

    applicationMenu.add(applicationRunClassMenuItem);
    applicationMenu.add(applicationRunMenuItem);
    applicationMenu.add(applicationRunDirectoryMenuItem);
    applicationMenu.add(applicationRunRecursiveDirectoryMenuItem);
    applicationMenu.addSeparator();

    applicationMenu.add(applicationOpenClassMenuItem);
    applicationMenu.add(applicationOpenMenuItem);
    applicationMenu.add(applicationOpenDirectoryMenuItem);
    applicationMenu.add(applicationOpenRecursiveDirectoryMenuItem);
    applicationMenu.addSeparator();

    // begin new code for mru manager!!
    // Create MruMenuManager object
    // pass in the current menu size so it knows where to start adding
    // mru items.
    if (appProperties != null) {
      mruManager = new MruMenuManager(applicationMenu, applicationMenu.getItemCount(),
                                      Resources.MAX_MRU_ENTRIES, appProperties,
                                      Resources.MRU_PROPERTY_PREFIX);
    } // if
    else {
      mruManager = new MruMenuManager(applicationMenu, applicationMenu.getItemCount(),
                                      Resources.MAX_MRU_ENTRIES);
    } // else
    // Add this object as an action listener so it can catch
    // mru item events (selection)
    mruManager.addActionListener(this);
    applicationMenu.addSeparator();
    // end new code for mru manager
    applicationMenu.add(applicationExitMenuItem);


    menuBar.add(applicationMenu);
//    menuBar.add(editMenu);
    menuBar.add(windowMenu);
    JMenuItem menuItem = new JMenuItem(Resources.EDIT_MENU_CUT,
                                       Resources.EDIT_MENU_CUT_MNEMONIC);
    editMenu.add(menuItem);
//    menuItem.addActionListener(new ActionListener() {});
    editMenu.add(new JMenuItem(Resources.EDIT_MENU_COPY,
                               Resources.EDIT_MENU_COPY_MNEMONIC));
    editMenu.add(new JMenuItem(Resources.EDIT_MENU_PASTE,
                               Resources.EDIT_MENU_PASTE_MNEMONIC));
    editMenu.add(new JMenuItem(Resources.EDIT_MENU_DEL,
                               Resources.EDIT_MENU_DEL_MNEMONIC));
    helpMenu.add(helpAboutMenuItem);
    helpMenu.setMnemonic(Resources.HELP_MENU_MNEMONIC);
    menuBar.add(helpMenu);
    setJMenuBar(menuBar);
    getContentPane().add(desktop);

    setSize(Resources.APPLICATION_SIZE);
    SwingUtilities.centerOnScreen(this);

    setVisible(true);
  } // constructor

  /**
  Implementation of ActionListener.  The only action events
  this object will pick up will be those sent from MruMenuManager,
  which means that it can act consistently with these events.
  */
  public void actionPerformed(ActionEvent e) {
    // get the source of the item (JMenu) so that
    // we can find out the file name selected.
    JMenuItem item = (JMenuItem)e.getSource();
    String itemText = item.getText();
    try {
      final int runOpenPos = itemText.indexOf(Resources.MRU_OPENRUN_SEPARATOR);
      final int separatorPos = itemText.indexOf(Resources.MRU_PREFIX_SEPARATOR);
      final String runOrOpen = itemText.substring(0, runOpenPos);
      if (runOpenPos == -1 || separatorPos == -1 ||
          (!runOrOpen.equals(Resources.MRU_OPEN) &&
          !runOrOpen.equals(Resources.MRU_RUN))) {
        final ErrorDialog ed = new ErrorDialog(Resources.getMainFrame(),
                                               "Could not test",
                                               "The collection specified does " +
                                               "not have a valid syntax."
                                                );
        ed.show();
        return;
      } // if

      final String collType = itemText.substring(runOrOpen.length() +
                                                 Resources.MRU_OPENRUN_SEPARATOR.length(),
                                                 separatorPos);
      final String fileName = itemText.substring(separatorPos +
                                                 Resources.MRU_PREFIX_SEPARATOR.length());
//        File file = new File(fileName);
      runTestCaseCollection(fileName, collType, runOrOpen.equals("run"));
    } // try
    catch (Throwable ex) {
//      ex.printStackTrace();
      final ErrorDialog ed = new ErrorDialog(Resources.getMainFrame(),
                                             "Could not test",
                                             ex.getMessage());
      ed.show();
    } // catch
  } // actionPerformed()

  // David--I extracted this code from the action event
  // that was associated with applicationOpenMenuItem so
  // that it could be called either from the FileChooser dialog
  // or the mru stuff.
  private void runTestCaseCollection(final String fileToRun,
                                     final String testCaseType,
                                     final boolean runNow) {
    try {
      ReporterInternalFrame rif;
      if (testCaseType.equals(Resources.MRU_CLASS_PREFIX)) {
        rif = new ReporterInternalFrame(runNow, new ClassTester(fileToRun));
      } // if
      else if (testCaseType.equals(Resources.MRU_FILE_PREFIX)) {
        rif = new ReporterInternalFrame(runNow, new FileTester(new File(fileToRun)));
      } // if
      else if (testCaseType.equals(Resources.MRU_DIRECTORY_PREFIX)) {
        rif = new ReporterInternalFrame(runNow, new DirectoryTester(new File(fileToRun), false));
      } // else if
      else {
        rif = new ReporterInternalFrame(runNow, new DirectoryTester(new File(fileToRun), true));
      } // else
      desktop.add(rif);
      try {
        rif.setMaximum(true);
      } // try
      catch (java.beans.PropertyVetoException ex) {
        // do nothing. ignore it.
      } // catch

      // save MRU info to the properties file
      final StringBuffer menuText = new StringBuffer();
      if (runNow) {
        menuText.append(Resources.MRU_RUN);
      } // if
      else {
        menuText.append(Resources.MRU_OPEN);
      } // else
      menuText.append(Resources.MRU_OPENRUN_SEPARATOR +
                      testCaseType + 
                      Resources.MRU_PREFIX_SEPARATOR +
                      fileToRun.toString());
      mruManager.addMruItem(menuText.toString());
    } // try
    catch (Throwable ex) {
      ex.printStackTrace();
      final ErrorDialog ed = new ErrorDialog(Resources.getMainFrame(),
                                             "Could not test",
                                             ex.getMessage());
      ed.show();
    } // catch
  } // runTestCaseCollection()

  private void runDirectoryMenuItem(final boolean runNow,
                               final boolean recursive) {
        // display a file chooser
        JFileChooser chooser = new JFileChooser(".");
        chooser.setFileHidingEnabled(true);
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        FileFilter filter = new DirectoryFilter();
        chooser.setFileFilter(filter);

        int result = chooser.showOpenDialog(Main.this);
        if (result == JFileChooser.APPROVE_OPTION) {
          if (!recursive) {
            runTestCaseCollection(chooser.getSelectedFile().getAbsolutePath(),
                                  Resources.MRU_DIRECTORY_PREFIX,
                                  runNow);
          } // if
          else {
            runTestCaseCollection(chooser.getSelectedFile().getAbsolutePath(),
                                  Resources.MRU_RECURSIVE_DIRECTORY_PREFIX,
                                  runNow);
          } // else
        } // if
  } // runDirectoryMenuItem()

  private void runClassMenuItem(final boolean runNow) {
    try {
      ClassNameDialog classNameDialog = new ClassNameDialog(Main.this);
      classNameDialog.show();
      String className = classNameDialog.getClassName();
      if (className != null) {
        runTestCaseCollection(className, Resources.MRU_CLASS_PREFIX, runNow);
      } // if
    } // try
    catch (Throwable ex) {
      final ErrorDialog ed = new ErrorDialog(Resources.getMainFrame(),
                                             "Could not test",
                                             ex.getMessage()
                                             );
      ed.show();
    } // catch
  } // runClassMenuItem()

  private void runMenuItem(final boolean runNow) {
        // display a file chooser
        final JFileChooser chooser = new JFileChooser(".");
        chooser.setFileHidingEnabled(true);
        final FileFilter filter = new ClassFileFilter();

        chooser.setFileFilter(filter);

        final int result = chooser.showOpenDialog(Main.this);
        if (result == JFileChooser.APPROVE_OPTION) {
            // Catch my notes above runTestCaseCollection method!!
            runTestCaseCollection(chooser.getSelectedFile().getAbsolutePath(),
                                  Resources.MRU_FILE_PREFIX,
                                  runNow);
        } // if
  } // runMenuItem()

  private void exitProgram() {
    try {
      mruManager.storeMruItems();
      appProperties.storeProperties();
    } // try
    catch (java.io.IOException e) {}
    finally {
      System.exit(0);
    } // finally
  } // exitProgram()

  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      exitProgram();
    } // if
  } // processWindowEvent()

  /**
  Runs the application with a "backstop" to catch all exceptions and
  display a reasonable error message.
  */
  public static void main(String[] args) {
    try {
      MetalLookAndFeel.setCurrentTheme(new SimsComputingTheme());
      new Main();
    } // try
    catch (Throwable e) {
      final ErrorDialog ed = new ErrorDialog(Resources.getMainFrame(),
                                             "Failure",
                                             e.getMessage());
      ed.show();
    } // catch
  } // main()
} // class Main
