package com.simscomputing.testbed.gui;

import com.simscomputing.swing.SwingUtilities;
import com.simscomputing.swing.WindowInternalFrame;
import com.simscomputing.testbed.TestCaseCollection;
import com.simscomputing.testbed.TesterException;
import com.simscomputing.testbed.TesterListener;
import com.simscomputing.testbed.TesterProgress;
import com.simscomputing.testbed.TestResult;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.event.InternalFrameEvent;

/**
Shows program information.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:35 $
*/
public class ProgressInternalFrame extends WindowInternalFrame 
  implements TesterListener {
  private int counter;
  private JProgressBar progressBar;
  private boolean goneYellow = false;
  private boolean goneRed = false;
  private TestCaseCollection testCaseCollection;
//    private Component parent;
  private String title;
  private JLabel progressLabel = new JLabel("Test Progress", JLabel.CENTER);

  public ProgressInternalFrame(String title) {
    super(Resources.getWindowMenu());
    
    setIconifiable(true);
    setMaximizable(true);
    setClosable(true);

    this.title = title;
    
    setTitle("Test Progress: " + title);
    progressBar = new JProgressBar();
    progressBar.setString("");
    progressBar.setStringPainted(true);
    progressBar.setForeground(Resources.GREEN);

    final Container root = getContentPane();
    root.setLayout(new GridBagLayout());
    
    final JLabel contents2 = new JLabel(title, JLabel.CENTER);

    JButton cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
//          System.out.println("************** cancelling test!!!!!!!!!!!");
//          testCaseCollection.stop();
//          thread.interrupt();
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            dispose();
          } // run()
        });
      }
    });

    final int INSET = Resources.INSET;
    root.add(progressLabel,
             new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                                    GridBagConstraints.CENTER,
                                    GridBagConstraints.HORIZONTAL,
                                    new Insets(INSET, INSET, INSET, INSET),
                                    0, 0));
    root.add(contents2,
             new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                                    GridBagConstraints.CENTER,
                                    GridBagConstraints.HORIZONTAL,
                                    new Insets(INSET, INSET, INSET, INSET),
                                    0, 0));

    root.add(progressBar,
             new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                    GridBagConstraints.CENTER,
                                    GridBagConstraints.HORIZONTAL,
                                    new Insets(INSET, INSET, INSET, INSET),
                                    0, 0));

    root.add(cancelButton,
             new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                                    GridBagConstraints.CENTER,
                                    GridBagConstraints.NONE,
                                    new Insets(INSET, INSET, INSET, INSET),
                                    0, 0));

//      pack();
    setSize(500, 200);
  } // constructor

//    public void internalFrameOpened(InternalFrameEvent e) {
//      super.internalFrameOpened(e);

//      System.out.println("*************** Progress opened");

//      // position this frame over its parent
//      toFront();
//      SwingUtilities.centerOnParent(this, parent);
//    } // internalFrameOpened()

  /**
  Sets the thread to interrupt if the Cancel button is pressed.
  */
  public void setTestCaseCollection(TestCaseCollection testCaseCollection) {
    this.testCaseCollection = testCaseCollection;
  } // setTestCaseCollection()

//    public void progressMade(TesterProgress testerProgress) {
//      if (progress != null && testerProgress != null) {
//        System.out.println("Reporter: updating progress to: " + testerProgress);
//        progress.update(testerProgress);
//      } // if
//    } // progressMade()


  public void progressMade(TesterProgress progress) {
    int size = progress.getNumberOfTestsExercised();

    if (!goneYellow && size > 0) {
      TestResult tr = progress.getTestResult(size - 1);
      if (!tr.didTestPass() && !tr.didTestFailAbnormally()) {
        goneYellow = true;
        progressBar.setForeground(Resources.YELLOW);
      } // if
    } // if

    if (!goneRed && size > 0) {
      TestResult tr = progress.getTestResult(size - 1);
      if (!tr.didTestPass() && tr.didTestFailAbnormally()) {
        goneYellow = true;
        goneRed = true;
        progressBar.setForeground(Resources.RED);
      } // if
    } // if

    progressBar.setString(progress.getTestCaseBeingExercised());
    progressBar.setMinimum(0);
    progressBar.setMaximum(progress.getTotalNumberOfTestsToExercise());
    progressBar.setValue(progress.getNumberOfTestsExercised());

    progressLabel.setText("Test Progress: " +
                          progress.getNumberOfTestsExercised() +
                          " of " + 
                          progress.getTotalNumberOfTestsToExercise());

    if (progress.isDone()) {
      javax.swing.SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          dispose();
          Resources.getDesktop().remove(ProgressInternalFrame.this);
        } // run()
      });
    } // if
  } // update


} // class ProgressInternalFrame
