/*
 * $Header: /u/cvs/Projects/EnhydraOrg/Enhydra/modules/Tomcat/src/share/org/apache/tomcat/util/Attic/XMLParser.java,v 1.2.2.1 2000/03/19 02:51:24 markd Exp $
 * $Revision: 1.2.2.1 $
 * $Date: 2000/03/19 02:51:24 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 


package org.apache.tomcat.util;

import org.apache.tomcat.util.StringManager;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Comment;
import org.w3c.dom.Text;
import org.w3c.dom.CharacterData;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;
import java.io.IOException;
import java.lang.NullPointerException;

import java.io.InputStreamReader;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xerces.readers.XCatalog;


/**
 *
 * @author James Todd [gonzo@eng.sun.com]
 */

public class XMLParser {

    private StringManager sm =
        StringManager.getManager(Constants.Package);
    private Document doc = null;
    private static final boolean DefaultCheckType = false;
    private static final String DefaultContentType =
        "text/html;charset=utf-8";
    private static final String DefaultScheme = "file";
    private static final boolean DefaultXMLValidate = true;
    // FIXME: markd@lutris.com: get this from DOM
    // private static final int PCDataType = 3;

    public XMLParser() {
    }

    public void clear() {
        doc = null;
    }

    /*
     * FIXME: markd@lutris.com.  To remove dependency on com.sun.xml,
     * several of the process method dependent on Resolver.createInputSource
     * were removed or modified.
     */
    public XMLTree process(InputStream is)
    throws IOException, SAXParseException, SAXException {
        return process(is, DefaultCheckType);
    }

    public XMLTree process(InputStream is, boolean checkType)
    throws IOException, SAXParseException, SAXException {
        InputSource inSrc = new InputSource(new InputStreamReader(is));
	inSrc.setByteStream(is);

        return parse(inSrc, checkType);
    }

    /*
    public XMLTree process(InputStream is, boolean checkType,
        String contentType)
    throws IOException, SAXParseException, SAXException {
        return process(is, checkType, contentType, DefaultScheme);
    }

    public XMLTree process(InputStream is, boolean checkType,
        String contentType, String scheme)
    throws IOException, SAXParseException, SAXException {
        return parse(Resolver.createInputSource(contentType, is,
            checkType, scheme), checkType);
    }
    */

    public XMLTree process(URL uri)
    throws IOException, SAXParseException, SAXException {
        return process(uri, DefaultCheckType);
    }

    public XMLTree process(URL uri, boolean checkType)
    throws IOException, SAXParseException, SAXException {
        return parse(new InputSource(uri.toString()), checkType);
    }

    public Document getXmlDocument() {
        return doc;
    }

    public void write()
    throws IOException {
        write(null);
    }

    public void write(OutputStream os)
    throws IOException {
        if (os == null) {
            os = System.out;
        }

	doc.getDocumentElement().normalize();
        // FIXME: markd@lutris.com:  This method was not implemented when
        // converted to xerces DOM.
        //doc.write(os);
        throw new Error(this.getClass().getName()
                        + ".write(): Not implemented");
    }

    private XMLTree parse(InputSource is)
    throws IOException, SAXParseException, SAXException {
        return parse(is, DefaultXMLValidate);
    }

    private XMLTree parse(InputSource is, boolean validate)
    throws IOException, SAXParseException, SAXException {
        DOMParser parser = new DOMParser();

        XCatalog resolver = new XCatalog();
	URL serverURL = this.getClass().getResource(
            Constants.DTD.Server.Resource);
	URL webApplicationURL = this.getClass().getResource(
            Constants.DTD.WebApplication.Resource);

	resolver.addPublicMapping(Constants.DTD.Server.PublicId,
            serverURL.toString());
	resolver.addPublicMapping(
	    Constants.DTD.WebApplication.PublicId,
	    webApplicationURL.toString());

	try {
            parser.setIncludeIgnorableWhitespace(false);
	    // parser.setFastStandalone(true);
	    parser.setEntityResolver(resolver);
	    // builder.setDisableNamespaces(true);
	    parser.parse(is);
            doc = parser.getDocument();
	} catch (Exception e) {
	    String systemId = "";
	    int lineNumber = -1;
	    String message = "";
	    String publicId = "";
	    int columnNumber = -1;

	    if (e instanceof SAXParseException) {
	        systemId = ((SAXParseException)e).getSystemId();
		lineNumber = ((SAXParseException)e).getLineNumber();
		message = ((SAXParseException)e).getMessage();
		publicId = ((SAXParseException)e).getPublicId();
		columnNumber = ((SAXParseException)e).getColumnNumber();
	    }

	    String msg = sm.getString("xml.parse") + ": " + systemId +
	        " : " + lineNumber + "\n  msg : " + message;

	    throw new SAXParseException(msg, publicId, systemId,
	        lineNumber, columnNumber);
	}

	XMLTree xmlElement = new XMLTree();
        Element element = doc.getDocumentElement();

	xmlElement.setName(element.getTagName());
	addAttributes(xmlElement, element);
	addElements(xmlElement, element);

        //System.err.println(xmlElement.toString());
        return xmlElement;
    }

    private void addAttributes(XMLTree xmlElement, Node node) {
        NamedNodeMap attributes = node.getAttributes();

        if (attributes != null) {
	    for (int i = 0; i < attributes.getLength(); i++) {
	        Node attribute = attributes.item(i);

	        xmlElement.addAttribute(attribute.getNodeName(),
                    attribute.getNodeValue());
            }
	}
    }

    // FIXME: markd@lutris.com: This code makes some bad assumptions
    // about the structure of the DOM that makes it difficult to
    // substitute other parsers.  It assumed no comment nodes and
    // and that a node had only one Text child.  It also created
    // a childXMLTree, even if it was going to be discarded.

    private void addElements(XMLTree xmlElement, Node node) {
        NodeList childNodes = node.getChildNodes();

	for (int i = 0; i < childNodes.getLength(); i++) {
	    Node childNode = childNodes.item(i);
            if (childNode instanceof Comment) {
                // Drop comments
            } else if (childNode instanceof Text) {
                String value = xmlElement.getValue();
                if (value == null) {
                    xmlElement.setValue(childNode.getNodeValue());
                } else {
                    xmlElement.setValue(value + childNode.getNodeValue());
                }
            } else {
                XMLTree childXMLTree = new XMLTree();
                childXMLTree.setName(childNode.getNodeName());
                addAttributes(childXMLTree, childNode);
	        addElements(childXMLTree, childNode);
	        xmlElement.addElement(childXMLTree);
            }
	}
    }
}
