/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.archive;

//

import org.enhydra.tool.common.ResUtil;

// Standard imports

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.jar.Attributes;
import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;
import java.util.jar.Manifest;
import java.util.ResourceBundle;

//

public class JarBuilder implements Constants {

    // not to be resourced

    static ResourceBundle       res =
        ResourceBundle.getBundle ("org.enhydra.tool.archive.Res");      // nores
    private final String        MANIFEST_VERSION_N = "Manifest-Version";        // nores
    private final String        MANIFEST_VERSION_V = "1.0";     // nores
    private final String        CREATED_BY_N = "Created-By";    // nores

    //

    private JarPlan             jarPlan = null;
    private JarOutputStream     jarStream = null;
    private String              archivePath = new String ();

    public JarPlan getJarPlan () {
        return jarPlan;
    }

    public void setJarPlan (JarPlan p) {
        jarPlan = p;
    }

    public File buildArchive () throws ArchiveException {
        File    jar = null;

        jar = openJarStream (getJarPlan ().getArchivePath ());

        addJarClasses ();
        closeJarStream ();

        return jar;
    }

    protected void closeJarStream () throws ArchiveException {
        try {
            jarStream.flush ();
            jarStream.finish ();
            jarStream.close ();
        } catch (IOException e) {
            throw new ArchiveException (e, res.getString ("Unable_to_close"));
        }
    }

    protected File openJarStream (String p) throws ArchiveException {
        FileOutputStream        fileStream = null;
        File                    archiveFile = null;
        Manifest                manifest = null;

        archivePath = p;
        archiveFile = new File (archivePath);
        archivePath = archiveFile.getAbsolutePath ();

        verifyArchiveFile (archiveFile);

        try {
            fileStream = new FileOutputStream (archiveFile);
            manifest = new Manifest ();

            manifest.getMainAttributes ().put (new Attributes.Name (MANIFEST_VERSION_N),
                                               MANIFEST_VERSION_V);

            StringBuffer        by = new StringBuffer ();

            by.append (System.getProperty (JAVA_VERSION));
            by.append (' ');
            by.append ('(');
            by.append (System.getProperty (JAVA_VENDOR));
            by.append (')');
            manifest.getMainAttributes ().put (new Attributes.Name (CREATED_BY_N),
                                               by.toString ());

            jarStream = new JarOutputStream (fileStream, manifest);
        } catch (FileNotFoundException e) {
            throw new ArchiveException (e,
                                        res.getString ("Unable_to_create"));
        } catch (IOException e) {
            throw new ArchiveException (e,
                                        res.getString ("Unable_to_create"));
        }

        return archiveFile;
    }

    protected void createEntry (File source, String entryPath)
            throws FileNotFoundException, IOException {
        JarEntry        entry = null;

        entry = new JarEntry (entryPath);

        entry.setTime (System.currentTimeMillis ());
        jarStream.putNextEntry (entry);

        if (source.getAbsolutePath ().equalsIgnoreCase (archivePath)) {

            // ignore recursive entries.

        } else {
            BufferedInputStream         in = null;
            BufferedOutputStream        out = null;

            in = new BufferedInputStream (new FileInputStream (source));
            out = new BufferedOutputStream (jarStream);

            int cursor = in.read ();

            while (cursor != -1) {
                out.write (cursor);

                cursor = in.read ();
            }

            in.close ();
            out.flush ();

            out = null;
        }

        jarStream.closeEntry ();
    }

    private void verifyArchiveFile (File archiveFile)
            throws ArchiveException {
        if (archiveFile.isDirectory ()) {
            String      exMess = res.getString ("Target_is_a_directory");

            exMess = ResUtil.format (exMess, archiveFile);

            throw new ArchiveException (exMess);
        } else if (archiveFile.getParentFile () == null) {
            String      exMess = res.getString ("Target_has_invalid");

            exMess = ResUtil.format (exMess, archiveFile);

            throw new ArchiveException (exMess);
        } else {
            archiveFile.getParentFile ().mkdirs ();
        }
    }

    private void addJarClasses () throws ArchiveException {
        File[]  files = getJarPlan ().getFilesInClasses ();

        for (int i = 0; i < files.length; i++) {
            addJarClassesFile (files[i]);
        }
    }

    private void addJarClassesFile (File source) throws ArchiveException {
        StringBuffer    entryPath = new StringBuffer ();
        String          classRoot = null;
        String          relative = null;

        //

        classRoot = getJarPlan ().getClasses ();
        relative = source.getAbsolutePath ().substring (classRoot.length ()
                + 1);
        relative = relative.replace ('\\', '/');

        //

        entryPath.append (relative);

        //

        try {
            createEntry (source, entryPath.toString ());
        } catch (IOException e) {
            String      exMess = res.getString ("JarBuilder_Unable_to");

            exMess = ResUtil.format (exMess, source);

            throw new ArchiveException (e, exMess);
        }
    }

}

