/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.archive;

// Toolbox Imports

import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ResUtil;

// Standard imports

import java.io.File;
import java.io.FileFilter;
import java.util.ResourceBundle;

//

public class WarPlan extends JarPlan {

    //

    private String      content = new String ();
    private String      webxml = new String ();

    public WarPlan (){}

    public String getContent () {
        return content;
    }

    public void setContent (String in) throws ArchiveException {
        if (in == null) {
            throw new ArchiveException (res.getString ("Content_directory_is"));
        } else {
            PathHandle  path = PathHandle.createPathHandle (in);

            if (path.isDirectory ()) {
                content = path.getPath ();
            } else {
                throw new ArchiveException (ResUtil.format (res.getString ("Content_directory_not"),
                                                            path));
            }
        }
    }

    public String getWebXML () {
        return webxml;
    }

    public void setWebXML (String in) throws ArchiveException {
        if (in == null) {
            throw new ArchiveException (res.getString ("_web_xml_is_null"));
        } else {
            PathHandle  path = PathHandle.createPathHandle (in);

            if (path.isFile ()) {
                webxml = path.getPath ();
            } else {
                throw new ArchiveException (ResUtil.format (res.getString ("_web_xml_file_not"),
                                                            path));
            }
        }
    }

    public File[] getFilesInContent () {
        File[]  files = new File[0];
        File    root = new File (getContent ());

        fileVector.removeAllElements ();

        fillFilter = new ContentFilter ();

        fillFileVector (root);

        files = (File[]) fileVector.toArray (files);

        fileVector.removeAllElements ();

        return files;
    }

    //

    private class ContentFilter implements FileFilter {
        public boolean accept (File file) {
            boolean     include = false;

            if (file.isDirectory ()) {
                include = true;
            } else if (!file.getAbsolutePath ().equals (getWebXML ())) {
                include = true;
            }

            return include;
        }

    }

}

