/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen;

import org.enhydra.tool.common.ReplacementSet;
import org.enhydra.tool.common.Replacement;
import org.enhydra.tool.common.ToolException;
import java.io.File;

/**
 * This class instantiates a set of default Replacements during
 * initialization.
 */
public class AppReplacementSet extends ReplacementSet implements Constants {

    /**
     * Replacement key for the project name which may be used as
     * a directory name and as part of some file names.
     */
    public static final String  at_PROJECT_at = "at_PROJECT_at";        // nores

    /**
     * Replacement key for the package directory.
     */
    public static final String  at_PACKAGE_DIR_at = "at_PACKAGE_DIR_at";        // nores

    /**
     * Replacement key for the Java package name.
     */
    public static final String  JAVA_PACKAGE = "@JAVA_PACKAGE@";        // nores

    /**
     * Replacement key for the shell package directory.
     */
    public static final String  SHELL_PACKAGE_DIR = "@SHELL_PACKAGE_DIR@";      // nores

    /**
     * Replacement key for the shell Root directory.
     */
    public static final String  SHELL_ROOT_PATH = "@SHELL_ROOT_PATH@";          // nores

    /**
     * Replacement key for the client type
     */
    public static final String  CLIENT = "@CLIENT@";            // nores

    /**
     * Replacement key for the content type
     */
    public static final String  CONTENT_TYPE = "@CONTENT_TYPE@";        // nores

    /**
     * Replacement key for the copyright
     */
    public static final String  COPYRIGHT = "@COPYRIGHT@";      // nores

    /**
     * Create a default set of replacement operations. This is a convenience
     * class that can be used as the basis for building custom replacement
     * set within a generator.
     */
    public AppReplacementSet () {
        try {
            this.add (new Replacement (AppReplacementSet.at_PROJECT_at,
                                       AppReplacementSet.at_PROJECT_at));
            this.add (new Replacement (AppReplacementSet.at_PACKAGE_DIR_at,
                                       AppReplacementSet.at_PACKAGE_DIR_at));
            this.add (new Replacement (AppReplacementSet.JAVA_PACKAGE,
                                       AppReplacementSet.JAVA_PACKAGE));
            this.add (new Replacement (AppReplacementSet.SHELL_PACKAGE_DIR,
                                       AppReplacementSet.SHELL_PACKAGE_DIR));
            this.add (new Replacement (AppReplacementSet.SHELL_ROOT_PATH,
                                       AppReplacementSet.SHELL_ROOT_PATH));
            this.add (new Replacement (AppReplacementSet.CLIENT,
                                       TYPE_HTML.toUpperCase ()));
            this.add (new Replacement (AppReplacementSet.CONTENT_TYPE,
                                       CONTENT_HTML));
            this.add (new Replacement (AppReplacementSet.COPYRIGHT,
                                       new String ()));
        } catch (ToolException e) {
            e.printStackTrace ();
        }
    }

}

