/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.ResUtil;
import java.util.ResourceBundle;

//
public class Main {
    static ResourceBundle res = ResourceBundle.getBundle("org.enhydra.tool.codegen.Res"); // nores
    static private final String PARAM_DEBUG = "-debug"; // nores
    static private final String PARAM_CONFIG = "-config"; // nores
    static private final String PARAM_LIST = "-list"; // nores
    static private final String PARAM_HELP = "-help"; // nores
    /**
     *
     * Invoke CodeGen and process command line parameters.
     * No arguments will default to opening the Swing based wizard.
     *
     * Basic command line options:
     * <generator name> <generator specific options>
     * Invoke generator with options
     * -help <generator name>
     * Show command line options for specified genertator.
     * -help
     * Show basic command line options
     * -config [<enhydraHome>]
     * Recreate the codegen.properties file with default
     * generators and the specified enhydra home setting.
     * -debug
     * Turn the debug flag on.
     * -verbose
     * Turn the verbose flag on.
     *
     * Generator specific help will also display if required
     * options are missing or invalid values are passed.
     */
    public static void main(String[] args) {
        System.out.println(res.getString("Enhydra_Application"));
        for (int i = 0; i < args.length; i++) {
            if (args[i].trim().equalsIgnoreCase(Main.PARAM_DEBUG)) {
                CodeGen.debug = true;
                if (args.length == 1) {
                    args = new String[0];
                }
                break;
            }
        }
        try {
            if ((args.length >= 1) && (args[0].equalsIgnoreCase(Main.PARAM_CONFIG))) {
              if (args.length >= 2) {
                CodeGen.config(args[1]);
              } else {
                CodeGen.config();
              }
            } else if ((args.length != 2) && (args.length >= 1)
                       && (args[0].length() > 0 && args[0].charAt(0) == '-')
                       && (!args[0].equalsIgnoreCase(Main.PARAM_LIST))) {
                Main.showHelp();
            } else {
                CodeGen codeGen;

                if (args.length < 1) {
                    codeGen = new CodeGen(true);
                    codeGen.invokeWizard(null);
                } else if ((args.length >= 2)
                           && args[0].trim().equalsIgnoreCase(Main.PARAM_HELP)) {
                    codeGen = new CodeGen();
                    codeGen.showGeneratorOptionHelp(args[1]);
                } else if (args[0].trim().equalsIgnoreCase(Main.PARAM_LIST)) {
                    codeGen = new CodeGen();
                    codeGen.list();
                } else {
                    codeGen = new CodeGen();
                    codeGen.invokeGenerator(args);
                }
            }
        } catch (GeneratorException e) {
            if (CodeGen.debug) {
                e.printStackTrace();
            } else {
                System.out.println(e.getMessage());
            }
        }
        System.exit(0);
    }

    /**
     * Method declaration
     *
     */
    private static void showHelp() {
        System.out.println(new String());
        System.out.println(res.getString("Usage01"));
        System.out.println(res.getString("Usage02"));
        System.out.println(res.getString("Usage03"));
        System.out.println(new String());
        System.out.println(res.getString("Usage04"));
        System.out.println(new String());
        System.out.println(res.getString("Usage05"));
        System.out.println(res.getString("Usage06"));
        System.out.println(res.getString("Usage07"));
        System.out.println(res.getString("Usage08"));
        System.out.println(ResUtil.format(res.getString("Usage09"), ToolBoxInfo.getPropertyFilename()));
        System.out.println(res.getString("Usage10"));
        System.out.println(res.getString("Usage11"));
    }


}