/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */

package org.enhydra.tool.codegen.wizard;

//
// ToolBox imports
//
import org.enhydra.tool.common.TBWizardDeck;
import org.enhydra.tool.codegen.Generator;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.ValidationException;

// Standard imports
import java.awt.Component;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class CodeGenDeck extends TBWizardDeck {
    private Generator  selection = null;
    private boolean    selectionHidden = false;

    public CodeGenDeck() {
      super();
    }

    protected void setSelectionCardHidden(boolean b) {
        selectionHidden = b && (selection != null);
        if (selectionHidden && getCardCursor() == 0) {
            next();
        }
    }

    protected boolean isSelectionCardHidden() {
        return selectionHidden;
    }

    /**
     * Method declaration
     *
     *
     * @param selection
     */
    protected void setSelection(Generator selection)
            throws GeneratorException {
        this.selection = selection;
        if (selection != null) {
            for (int i = 1; i < getCardVector().size(); i++) {
                remove((Component) getCardVector().elementAt(i));
            }
            while (getCardVector().size() > 1) {
                getCardVector().removeElementAt(getCardVector().size() - 1);
            }
            for (int i = 0; i < selection.getWizardPanels().length; i++) {
                CodeGenPage  newPage = new CodeGenPage();
                CodeGenPanel panel = selection.getWizardPanels()[i];

                newPage.add(panel);
                newPage.setPageTitle(panel.getPageTitle());
                newPage.setInstructions(panel.getInstructions());
                addWizardPage(newPage);
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    protected Generator getSelection() {
        return selection;
    }

    protected boolean isSelectionCardShowing() {
        return getCardCursor() == 0;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    protected boolean isFirstCardShowing() {
        boolean atFirst = super.isFirstCardShowing();

        if (isSelectionCardHidden()) {
            atFirst = (getCardCursor() == 1);
        }
        return (atFirst);
    }


    /**
     * Method declaration
     *
     *
     * @exception GeneratorException
     * @exception ValidationException
     *
     * @see
     */
    protected void writeOptionSet()
            throws ValidationException, GeneratorException {
        CodeGenPage page = null;

        // validate all pages before writing any
        for (int i = 0; i < getCardVector().size(); i++) {
            page = (CodeGenPage) getCardVector().elementAt(i);
            page.validateOptionSet();
        }
        for (int i = 0; i < getCardVector().size(); i++) {
            page = (CodeGenPage) getCardVector().elementAt(i);
            page.writeOptionSet();
        }
    }

}
