/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.common;

// ToolBox imports

import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.event.TemplateEvent;
import org.enhydra.tool.common.event.TemplateListener;

// Standard imports

import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Vector;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JDialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

//

public class TemplateTool {

    //

    static ResourceBundle       res =
        ResourceBundle.getBundle ("org.enhydra.tool.common.Res");       // nores

    //

    private boolean             echo = true;
    private boolean             overwrite = false;
    private boolean             swing = false;
    private TemplateFile[]      templates = new TemplateFile[0];
    private Replacement[]       replacements = new Replacement[0];
    private Vector              templateVector = new Vector ();
    private String              initSourcePath = new String ();
    private String              initDestPath = new String ();
    private String              initExt = new String ();
    private FileFilter          filter = new TemplateFilter ();
    private Vector              listenerVector = new Vector ();
    private PrintWriter         echoWriter = new PrintWriter (System.out);

    //

    public TemplateTool (){}

    public boolean isOverwrite () {
        return overwrite;
    }

    public void setOverwrite (boolean o) {
        overwrite = o;
    }

    public boolean isSwing () {
        return swing;
    }

    public void setSwing (boolean s) {
        swing = s;
    }

    public TemplateFile[] getTemplates () {
        return templates;
    }

    public void setTemplates (TemplateFile[] t) {
        templates = t;
    }

    public TemplateListener[] getTemplateListeners () {
        TemplateListener[]      templateListeners = new TemplateListener[0];

        templateListeners = new TemplateListener[listenerVector.size ()];
        templateListeners =
            (TemplateListener[]) listenerVector.toArray (templateListeners);

        return templateListeners;
    }

    public void addTemplateListener (TemplateListener l) {
        if (!listenerVector.contains (l)) {
            listenerVector.addElement (l);
        }
    }

    public void removeTemplateListener (TemplateListener l) {
        if (listenerVector.contains (l)) {
            listenerVector.removeElement (l);
        }
    }

    //
    //

    public void initTemplates (File source, File dest, String ext) {
        setInitExt (ext);
        templateVector.removeAllElements ();

        initSourcePath = source.getAbsolutePath ();
        initDestPath = dest.getAbsolutePath ();

        fillTemplateVector (source);
        setTemplates ((TemplateFile[]) templateVector.toArray (templates));
        templateVector.removeAllElements ();
    }

    public void initTemplates (File[] in, File dest, String ext) {
        setInitExt (ext);
        templateVector.removeAllElements ();

        initDestPath = dest.getAbsolutePath ();

        for (int i = 0; i < in.length; i++) {
            TemplateFile        temp = null;

            initSourcePath = in[i].getParent ();
            temp = new TemplateFile (in[i].getAbsolutePath ());

            initOutFile (temp);
            templateVector.addElement (temp);
        }

        setTemplates ((TemplateFile[]) templateVector.toArray (templates));
        templateVector.removeAllElements ();
    }

    public Replacement[] getReplacements () {
        return replacements;
    }

    public void setReplacements (Replacement[] r) {
        replacements = r;
    }

    public PrintWriter getEchoWriter () {
        return echoWriter;
    }

    public void setEchoWriter (PrintWriter w) {
        if (w == null) {
            setEcho (false);
        } else {
            echoWriter = w;
        }
    }

    public boolean isEcho () {
        if (getEchoWriter () == null) {
            echo = false;
        }

        return echo;
    }

    public void setEcho (boolean e) {
        echo = e;
    }

    public File[] createOutput () throws ToolException {
        Vector  outVector = new Vector ();
        File[]  outFiles = new File[0];
        File    out = null;

        for (int i = 0; i < templates.length; i++) {
            out = templates[i].getOutput ();

            if (templates[i].isDirectory ()) {
                out.mkdirs ();
            } else {
                if (isFileAvailable (out)) {
                    fileSearchAndReplace (templates[i], out);
                    outVector.addElement (out);
                }
            }
        }

        outFiles = (File[]) outVector.toArray (outFiles);

        outVector.removeAllElements ();

        return outFiles;
    }

    public String lineSearchAndReplace (String in) {
        String  text = in;
        String  prefix = new String ();
        String  suffix = new String ();

        for (int i = 0; i < replacements.length; i++) {
            int indexOf = text.indexOf (replacements[i].getFind ());
            int lastIndex = -1;
            int count = 0;

            while (indexOf != -1 && indexOf > lastIndex && count < 1000) {

                // Catch recursive replacements by maxing out at
                // after the 1000 replacement or break after one try
                // if imediate recursion.

                count++;
                prefix = text.substring (0, indexOf);
                suffix =
                    text.substring (indexOf
                                    + replacements[i].getFind ().length ());

                StringBuffer    buf = new StringBuffer ();

                buf.append (prefix);

                for (int j = 0; j < replacements[i].getReplaceWith ().length;
                        j++) {
                    String      rep = replacements[i].getReplaceWith ()[j];

                    if (j > 0) {
                        if (File.separatorChar == '\\') {       // Windows
                            buf.append ((char) 13);             // CR
                        }

                        buf.append ((char) 10);                 // LF
                    }

                    buf.append (rep);
                }

                buf.append (suffix);

                text = buf.toString ();
                indexOf = text.indexOf (replacements[i].getFind ());

                if ((indexOf > -1)
                        && (replacements[i].getReplaceWith ().length > 1)) {
                    break;
                }
            }
        }

        return text;
    }

    //
    //

    protected void echoWrite (String out) {
        if (isEcho ()) {
            getEchoWriter ().println (out);
            getEchoWriter ().flush ();
        }
    }

    protected void echoWrite (Exception e) {
        if (isEcho ()) {
            getEchoWriter ().println (e.getMessage ());
        }

        e.printStackTrace ();
    }

    //
    //

    private void setInitExt (String ext) {
        if (ext == null) {
            initExt = new String ();
        } else if ((ext.length () > 0) && (ext.charAt (0) == '.')) {
            initExt = ext.trim ();
        } else {
            initExt = '.' + ext.trim ();
        }
    }

    private String getInitExt () {
        return initExt;
    }

    private void notify (PathHandle in, PathHandle out) {
        TemplateListener[]      templateListeners = getTemplateListeners ();

        if (templateListeners.length == 0) {

            // done

        } else {
            TemplateEvent       event = null;

            event = new TemplateEvent (this, in, out);

            for (int i = 0; i < templateListeners.length; i++) {
                templateListeners[i].onTemplate (event);
            }
        }
    }

    private void fileSearchAndReplace (File in,
                                       File out) throws ToolException {
        BufferedReader  reader = null;
        PrintWriter     writer = null;
        File            parent = out.getParentFile ();
        PathHandle      pathOut = PathHandle.createPathHandle (out);
        PathHandle      pathIn = PathHandle.createPathHandle (in);

        notify (pathIn, pathOut);

        if ((parent != null) && (!parent.exists ())) {
            parent.mkdirs ();
        }

        try {
            reader = new BufferedReader (new FileReader (in));
            writer = new PrintWriter (new FileWriter (out));

            String      line = reader.readLine ();

            while (line != null) {
                line = lineSearchAndReplace (line);

                writer.println (line);

                line = reader.readLine ();
            }

            reader.close ();
            writer.close ();
            echoWrite (ResUtil.format (res.getString ("Creating_file_0_"),
                                       pathOut));
        } catch (IOException e) {
            throw (new ToolException (e,
                                      ResUtil.format (res.getString ("Unable_to_create_0_"),
                                                      pathOut)));
        }
    }

    private void fillTemplateVector (File root) {
        if (root.isDirectory ()) {
            File[]              children = new File[0];
            TemplateFile        template = null;
            File                out = null;

            if (filter == null){}

            if (filter == null) {
                children = root.listFiles ();
            } else {
                children = root.listFiles (filter);
            }

            int count = 0;

            for (int i = 0; i < children.length; i++) {
                if (children[i].isDirectory ()) {
                    fillTemplateVector (children[i]);
                } else if (children[i].isFile ()) {
                    count++;
                    template =
                        new TemplateFile (children[i].getAbsolutePath ());

                    initOutFile (template);
                    templateVector.addElement (template);
                }
            }

            echoWrite (ResUtil.format (res.getString ("_0_contains_1"), root,
                                       count));
        }
    }

    private void initOutFile (TemplateFile template) {
        String          suffix = new String ();
        StringBuffer    path = new StringBuffer ();
        File            out = null;

        suffix = template.getAbsolutePath ();

        if (suffix.startsWith (initSourcePath)) {
            suffix = suffix.substring (initSourcePath.length ());
        } else {
            suffix = template.getName ();
        }

        if (suffix.endsWith (getInitExt ())) {
            suffix = suffix.substring (0,
                                       suffix.length ()
                                       - getInitExt ().length ());
        }

        path.append (initDestPath);
        path.append (suffix);

        out = new File (lineSearchAndReplace (path.toString ()));

        template.setOutput (out);
    }

    /**
     * Method declaration
     *
     *
     * @param file
     *
     * @return
     */
    private boolean isFileAvailable (File file) throws ToolException {
        boolean available = true;

        if (file.exists () && (!file.isDirectory ())) {
            if (isOverwrite ()) {
                file.delete ();
            } else {
                if (isSwing ()) {
                    available = askForOverwrite (file);
                } else {

                    // command line

                    throw new ToolException (ResUtil.format (res.getString ("File_already_exists_0"),
                                                             file));
                }
            }
        }

        return available;
    }

    /**
     * Method declaration
     *
     */
    private boolean askForOverwrite (File file) throws ToolException {
        final String            YES = res.getString ("Yes");
        final String            ALL = res.getString ("Yes_to_All");
        final String            NO = res.getString ("No");
        final String            CANCEL = res.getString ("Cancel");
        final JOptionPane       optionPane = new JOptionPane ();
        String                  mess = new String ();

        mess = ResUtil.format (res.getString ("File_already_exists_n"), file);

        optionPane.setMessage (mess);

        final JDialog   dialog = optionPane.createDialog (new JFrame (),
                res.getString ("Confirm_Overwrite"));
        JButton[]       buttons = new JButton[4];
        JButton         selection = null;
        boolean         overwriteThis = false;
        boolean         cancel = false;

        for (int i = 0; i < buttons.length; i++) {
            buttons[i] = new JButton ();

            buttons[i].setActionCommand (new String () + i);
            buttons[i].addActionListener (new ActionListener () {
                public void actionPerformed (ActionEvent event) {
                    optionPane.setValue (event.getSource ());
                    dialog.setVisible (false);
                }

            });
        }

        buttons[0].setText (YES);
        buttons[1].setText (ALL);
        buttons[2].setText (NO);
        buttons[3].setText (CANCEL);
        optionPane.setOptions (buttons);
        dialog.pack ();
        dialog.show ();

        if (optionPane.getValue () instanceof JButton) {
            selection = (JButton) optionPane.getValue ();

            if (selection.getText ().equals (CANCEL)) {
                cancel = true;
            } else if (selection.getText ().equals (YES)) {
                overwriteThis = true;

                file.delete ();
            } else if (selection.getText ().equals (ALL)) {
                overwriteThis = true;

                setOverwrite (true);
                file.delete ();
            }
        }

        optionPane.removeAll ();
        dialog.removeAll ();
        dialog.dispose ();

        if (cancel) {
            throw new ToolException (res.getString ("Template_processing"));
        }

        return overwriteThis;
    }

    private class TemplateFilter implements FileFilter {
        public boolean accept (File file) {
            boolean     include = false;

            if (file.isDirectory ()) {
                include = true;
            } else if (getInitExt ().length () > 0) {
                include =
                    file.getAbsolutePath ().toLowerCase ().endsWith (getInitExt ().toLowerCase ());
            } else {
                include = true;
            }

            return include;
        }

    }

}

