/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.xml.xmlc.dom.generic;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.codegen.*;
import org.enhydra.xml.xmlc.dom.*;
import org.enhydra.xml.xmlc.compiler.ClassGenerator;
import org.enhydra.xml.xmlc.compiler.ElementInfo;
import org.enhydra.xml.xmlc.compiler.ElementTable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import org.w3c.dom.*;
import org.w3c.dom.html.HTMLDocument;

/**
 * Class to create code that creates documents using XMLCDomFactory objects.
 */
public class GenericDocCreateGenerator {
    /** 
     * Default name for the variable holding the DocumentType during
     * creation.
     */
    public static final String DOCUMENT_TYPE_VAR_NAME = "docType";

    /** 
     * Default name for the variable holding the Document during
     * creation.
     */
    public static final String DOCUMENT_VAR_NAME = "doc";

    /**
     * Constructor.
     * @param domFactory The compile-time instance of the XMLCDomFactory
     *  class.  A instance of the same class is used at runtime.
     */
    public GenericDocCreateGenerator(XMLCDomFactory domFactory) {
        //FIXME: domFactory arg no longer needed.
    }

    /**
     * Generate code to create a document.
     * @param document Document that will be recreated.
     * @param docTypeVarName Name of variable to create for docType object.
     * @param createDocVar Should a variable be created for the document
     * (otherwise assume a field).ry
     * @param docVarName Name of variable or field for document object.
     */
    public void generateDocCreate(Document document,
                                  String docTypeVarName,
                                  boolean createDocVar,
                                  String docVarName,
                                  JavaCode body) {
        String docClass = document.getClass().getName();

        DocumentType docType = document.getDoctype();
        if (docType != null) {
            body.addln("DocumentType " + docTypeVarName + " = "
                       + ClassGenerator.DOM_FACTORY_FIELD_NAME
                       + ".createDocumentType("
                       + JavaLang.createStringConst(docType.getNodeName()) + ", "
                       + JavaLang.createStringConst(docType.getPublicId()) + ", " 
                       + JavaLang.createStringConst(docType.getSystemId()) + ", " 
                       + JavaLang.createStringConst(docType.getInternalSubset()) + ");");
        }
        
        Element docElem = document.getDocumentElement();
        body.addln((createDocVar ? docClass : "") + " " + docVarName
                   + " = (" + docClass + ")"
                   + ClassGenerator.DOM_FACTORY_FIELD_NAME + ".createDocument("
                   + JavaLang.createStringConst(docElem.getNamespaceURI()) + ", "
                   + JavaLang.createStringConst(docElem.getNodeName()) + ", "
                   + ((docType == null) ? "null" : "docType") + ");");
    }

    /**
     * Generate code to create a document.
     * @param document Document that will be recreated.
     * @param createDocVar Should a variable be created for the
     *  document (otherwise assume a field).
     * @param docVarName Name of variable or field for document object.
     */
    public void generateDocCreate(Document document,
                                  boolean createDocVar,
                                  String docVarName,
                                  JavaCode body) {
        generateDocCreate(document, DOCUMENT_TYPE_VAR_NAME,
                          createDocVar, docVarName, body);
    }
}

