/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DOMType.java,v 1.1 2000/09/13 08:42:21 markd Exp $
 */

package org.enhydra.xml.xmlc.metadata;

//FIXME: add WML, cHTML.

/**
 * Enumerated type for the predefined DOM factories.
 */
public class DOMType extends EnumeratedType {
    /**
     * Constant for LazyDOM.
     */
    public static final DOMType LAZYDOM
        = new DOMType("lazydom",
                      org.enhydra.xml.xmlc.dom.lazydom.LazyDomFactory.class,
                      org.enhydra.xml.xmlc.dom.lazydom.LazyHTMLDomFactory.class);

    /**
     * Constant for Xerces DOM.
     */
    public static final DOMType XERCES
        = new DOMType("xerces",
                      org.enhydra.xml.xmlc.dom.xerces.XercesDomFactory.class,
                      org.enhydra.xml.xmlc.dom.xerces.XercesHTMLDomFactory.class);

    /**
     * XML DOM factory class for this type.
     */
    private final Class fXMLDomFactoryClass;

    /**
     * HTML DOM factory class for this type.
     */
    private final Class fHTMLDomFactoryClass;

    /**
     * Internal constructor.  Only a fixed set of types are allowed.
     */
    private DOMType(String name,
                    Class xmlDomFactoryClass,
                    Class htmlDomFactoryClass) {
        super(name);
        fXMLDomFactoryClass = xmlDomFactoryClass;
        fHTMLDomFactoryClass = htmlDomFactoryClass;
    }

    /**
     * Get the XML DOM factory class for this type.
     */
    public Class getXMLDomFactoryClass() {
        return fXMLDomFactoryClass;
    }

    /**
     * Get the HTML DOM factory class for this type.
     */
    public Class getHTMLDomFactoryClass() {
        return fHTMLDomFactoryClass;
    }

    /**
     * Get the object for the specified type.  If desiredType is null,
     * return null.
     */
    public static DOMType getType(String desiredType) {
        if (desiredType == null) {
            return null;
        } else if (LAZYDOM.fName.equals(desiredType)) {
            return LAZYDOM;
        } else if (XERCES.fName.equals(desiredType)) {
            return XERCES;
        } else {
            throw new IllegalArgumentException("Invalid DOMType: \""
                                               + desiredType
                                               + "\", expected of \""
                                               + LAZYDOM.fName + "\", or \""
                                               + XERCES.fName + "\"");
        }
    }
}
