/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MultiResourceLoader.java,v 1.1.8.2 2000/11/16 09:08:42 markd Exp $
 */

package org.enhydra.xml.xmlc.reloading;

import java.io.PrintWriter;
import java.net.URL;
import java.net.URLClassLoader;
import org.enhydra.xml.xmlc.XMLCLogger;
import com.lutris.classloader.MultiClassLoader;
import com.lutris.classloader.Resource;
import com.lutris.classloader.LocalDirResource;

/**
 * ResourceLoader for the MultiClassLoader.  This class is not compiled or
 * used for Enhydra 4.0 and greater.
 * @see com.lutris.classloader.MultiClassLoader
 */
public class MultiResourceLoader implements ResourceLoader {
    /**
     * Compile-time debugging.
     */
    private static final boolean DEBUG = false;

    /**
     * Class loader we are associated with.
     */
    private MultiClassLoader fClassLoader;

    /**
     * Logging interface.
     */
    private XMLCLogger fLogger;

    /**
     * Constructor.
     * @see ResourceLoader
     */
    public MultiResourceLoader(ClassLoader classLoader,
                               XMLCLogger logger) {
        fClassLoader = (MultiClassLoader)classLoader;
        fLogger = logger;
        if (DEBUG) {
            System.err.println("MultiResourceLoader: classpath set to:");
            URL[] url = getClassPath();
            for (int idx = 0; idx < url.length; idx++) {
                System.err.println("    " + url[idx]);
            }
        }
    }

    /**
     * @see ResourceLoader#getClassLoader
     */
    public ClassLoader getClassLoader() {
        return fClassLoader;
    }

    /**
     * Add URLs from a classloader, if possible.
     * @param loader maybe null
     */
    private void addLoaderClassPath(ClassPathBuilder classPath,
                                    ClassLoader loader) {
        if (loader instanceof MultiClassLoader) {
            classPath.addURLs(((MultiClassLoader)loader).getClassPath());
        } else if (loader instanceof URLClassLoader) {
            classPath.addURLs(((URLClassLoader)loader).getURLs());
        }
    }

    /**
     * Recursively walk chain classpath tree to collect paths in the order
     * that they would be checked my the MultiClassLoader.
     */
    private void collectClassPath(ClassPathBuilder classPath,
                                  ClassLoader loader) {
        if (loader != null) {
            collectClassPath(classPath, loader.getParent());
            addLoaderClassPath(classPath, loader);
            if (loader instanceof  MultiClassLoader) {
                collectClassPath(classPath, ((MultiClassLoader)loader).getSecondary());
            }
        }
    }

    /**
     * @see ResourceLoader#getClassPath
     */
    public URL[] getClassPath() {
        ClassPathBuilder classPath = new ClassPathBuilder();
        collectClassPath(classPath, fClassLoader);
        return classPath.getClassPath();
    }

    /**
     * @see ResourceLoader#createSelectiveClassLoader
     */
    public ClassLoader createSelectiveClassLoader(String className) {
        return new MultiSelectiveClassLoader(className, fClassLoader.getClassPath(),
                                             fClassLoader, fLogger);
    }
}
