/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MultiSelectiveClassLoader.java,v 1.3.2.1.4.1 2000/11/16 09:08:42 markd Exp $
 */

package org.enhydra.xml.xmlc.reloading;

import java.io.PrintWriter;
import java.net.URL;
import org.enhydra.xml.xmlc.XMLCLogger;
import org.enhydra.xml.xmlc.EnhydraXMLCLogger;
import com.lutris.classloader.MultiClassLoader;
import com.lutris.classloader.Resource;
import com.lutris.classloader.LocalDirResource;

/**
 * A class loader, derived from the MultiClassLoader, that selectively loads
 * classes.
 */
class MultiSelectiveClassLoader extends MultiClassLoader implements MultiClassLoader.ClassFilter {
    /** Name of class that will be loaded */
    private String fRestrictClassName;

    /**
     * Logging interface.
     */
    private XMLCLogger fLogger;

    /**
     * Constructor.
     * @see MultiClassLoader
     */
    public MultiSelectiveClassLoader(String restrictClassName,
                                     URL[] path,
                                     ClassLoader secondary,
                                     XMLCLogger logger) {
        super(path, 
              ((logger instanceof EnhydraXMLCLogger)
               ? ((EnhydraXMLCLogger)logger).getLogChannel() : null));
        fLogger = logger;
        setSecondary(secondary);
        fRestrictClassName = restrictClassName;
        addClassFilter(this);
    }

    /**
     * Class filter check.
     * @see MultiClassLoader.ClassFilter
     */
    public int loadCheck(String className) {
        boolean accepted = className.startsWith(fRestrictClassName)
            && ((className.length() == fRestrictClassName.length())
                || (className.charAt(fRestrictClassName.length()) == '$'));
        ResourceLoaderFactory.logRestrictionCheck(fLogger, this,
                                                  fRestrictClassName,
                                                  className,
                                                  accepted);
        return accepted ? MUST_LOAD : DONT_LOAD;
    }
}
