/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: RecompilationHandler.java,v 1.1 2000/09/27 08:52:13 markd Exp $
 */

package org.enhydra.xml.xmlc.reloading;

/**
 * Interface for a recompilation handler.  A recompilation handler is used
 * to check if a class is out-of-date relative to its source file and to
 * recompile the class if it is out-of-date.
 */
public interface RecompilationHandler {
    /**
     * Bind to a factory.  This is done so that loggers, etc
     * can be grabbed from the factory.
     *
     * @param factory Factory that is creating the object.  Useful
     *  for getting logging object.
     */
//FIXME: require in constructor
    public void init(XMLCReloadingFactory factory);

    /**
     * Determine if the specified class file is out-of-date relative
     * to its class file.  This should check the class file, not
     * the in-memory class.
     *
     * @param classEntry Object containing originally loaded class and
     *  the current class.
     */
    public boolean isOutOfDate(ClassEntry classEntry) throws XMLCReloadingException;

    /**
     * Recompile a class file from its source file.  Compilation
     * of the file will already be synchronized by the factory.
     *
     * @param classEntry Object containing originally loaded class and
     *  the current class.
     */
    public void recompile(ClassEntry classEntry) throws XMLCReloadingException;
}
