/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ResourceLoader.java,v 1.1.8.2 2000/11/16 09:08:42 markd Exp $
 */

package org.enhydra.xml.xmlc.reloading;

import java.io.File;
import java.net.URL;

/**
 * Interface for objects that are used to abstract particular class
 * loader implementations.  XMLC reloading/recompilation needs the
 * following functionality that is not part of ClassLoader:
 * <UL>
 * <LI> Obtaining the classpath that will be searched.
 * <LI> Create a class loader instance that will load one class and
 *      it's inner class.  It must load these class explictly and
 *      not delegate to the parent.  This is know as a `selective'
 *      class loader
 * </UL>
 * 
 * An instance of a class implementing this interface is associated with each
 * instance of XMLCReloadingFactory and it's primary clas loader.  This class
 * is then used to create the selective class loaders.
 * <P>
 * A class implement this interface must have a constructor with the
 * signature:
 * <PRE>
 *   public XXXResourceLoader(ClassLoader classLoader,
 *                            XMLCLogger logger)
 * </PRE>
 */
public interface ResourceLoader {
    /**
     * Get the class loader associated with this ResourceLoader
     * instance.
     */
    public ClassLoader getClassLoader();

    /**
     * Get the classpath that will be searched to find classes.
     * This should include all parents in the order that they
     * will be searched.  It is used for constructing a CLASSPATH
     * to pass to the Java compiler.
     */
    public URL[] getClassPath();

    /**
     * Create a selective class loader for the specified class and
     * its subclasses.  All other classes are delegated to the 
     * ClassLoader assocation with this ResourceLoader. 
     */
    public ClassLoader createSelectiveClassLoader(String className);
}
