/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XMLCReloadingException.java,v 1.1 2000/09/27 08:52:13 markd Exp $
 */

package org.enhydra.xml.xmlc.reloading;

import org.enhydra.xml.xmlc.XMLCRuntimeException;

/**
 * Runtime exception associated with reloading.
 */
public class XMLCReloadingException extends XMLCRuntimeException {
    /**
     * Output of compiled process, carried along, but not used in messages.
     */
    private String errorOutput;

    /**
     * Construct an exception without a specified cause.
     *
     * @param msg The message associated with the exception.
     */
    public XMLCReloadingException(String msg) {
        super(msg);
    }

    /**
     * Construct an exception with an associated causing exception.
     *
     * @param msg The message associated with the exception.
     * @param cause The exception or exception that cause this
     *  exception.
     */
    public XMLCReloadingException(String msg,
                                  Throwable cause) {
        super(msg, cause);
    }

    /**
     * Construct an exception from a causing exception.
     *
     * @param cause The exception or exception that cause this
     *  exception. The message will be take be this object's
     *  messasge.
     */
    public XMLCReloadingException(Throwable cause) {
        super(cause);
    }

    /**
     * Set the error output.
     */
    public void setErrorOutput(String out) {
        errorOutput = out;
    }

    /**
     * Get the error output or null if it was not set.
     */
    public String getErrorOutput() {
        return errorOutput;
    }
}
