/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package tests.org.enhydra.xml.io;

import org.enhydra.xml.io.*;
import org.enhydra.xml.xmlc.html.HTMLDocumentFactory;
import com.lutris.util.FatalExceptionError;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import java.io.*;

/**
 * Basic DOMFormatter tests, mostly of special cases.  More extensive 
 * tests are needed, however its well excerised by the XMLC tests.
 */
public class BasicTests {
    // Output directory
    public static final String OUTDIR = "output";

    // Character encodings.
    private final String UTF8 = "UTF8";
    private final String ISO8859_1 = "ISO8859_1";
    

    /**
     * Create skeleton html object with the specified text
     * in the body.
     */
    private HTMLDocument createHtml1(String title,
                                     String text) {
        HTMLDocument doc = HTMLDocumentFactory.createBasicDocument(title);
        HTMLElement body = doc.getBody();
        body.appendChild(doc.createTextNode(text));
        return doc;
    }

    /**
     * Write string to file using the specified encoding.
     */
    private void output(String fname,
                        String data,
                        String encoding) throws Exception {
        System.err.println(data);
        OutputStream out = new FileOutputStream(new File(OUTDIR, fname));

        out.write(data.getBytes(encoding));
        out.close();
    }

    /**
     * HTML formatting tests with special characters.
     */
    private void test1() throws Exception {
        DOMFormatter fmt;

        // &#146; = 0122 = 0x96
        String data = "One\u0096s check";
        HTMLDocument doc = createHtml1("", data);

        // Test 1.1, no encoding, to string
        doc.setTitle("test-1.0");
        fmt = new DOMFormatter();  // Defaults
        output("test-1.0.html", fmt.toString(doc), ISO8859_1);
    }

    /**
     * Run the tests.
     */
    private void runTests(String[] args) throws Exception {
        new File(OUTDIR).mkdirs();
        test1();
    }

    /**
     * Entry.
     */
    public static void main(String[] args) {
        try {
            new BasicTests().runTests(args);
        } catch (Throwable except) {
            except.printStackTrace();
            System.exit(1);
        }
    }
}
