/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package tests.org.enhydra.xml.io;

import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.io.*;
import org.enhydra.xml.dom.*;
import tests.org.enhydra.xml.lazydom.TestSupport;
import com.lutris.util.FatalExceptionError;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import java.io.*;

/**
 * Test DOMFormatter URLrewriting.
 */
public class URLRewriteTests extends TestSupport {
    // Output directory
    public static final String OUTDIR = "output";

    /**
     * Output a document with URL rewritting
     */
    private void outputWithRewritting(Document doc,
                                      File outFile) throws IOException {
        OutputOptions options = new OutputOptions();
        options.setURLRewriter(new URLRewriter() {
                public String rewriteURL(String urlAttrValue) {
                    return urlAttrValue + "#REWRITTEN";
                }
            });
        DOMFormatter formatter = new DOMFormatter(options);
        formatter.write(doc, outFile);
    }

    /**
     * Run the tests.
     */
    private void runTests(String[] args) throws Exception {
        int argIdx = 0;
        boolean traverse = false;
        while ((argIdx < args.length) && args[argIdx].startsWith("-")) {
            if (args[argIdx].equals("-traverse")) {
                traverse = true;
            } else {
                System.err.println("Invalid option: " + args[argIdx]);
                System.exit(1);
            }
            argIdx++;
        }
        if ((args.length - argIdx) != 3) {
            System.err.println("wrong # args: URLRewriteTests [-traverse] test-class outfile domfile");
            System.exit(1);
        }
        String testClass = args[argIdx];
        File outFile = new File(args[argIdx+1]);
        File domFile = new File(args[argIdx+2]);
        outFile.getParentFile().mkdirs();

        XMLObject xmlcDoc = loadXMLCDocClass(testClass);
        if (traverse) {
            traverseTree(xmlcDoc);
        }
        outputWithRewritting(xmlcDoc, outFile);

        PrintWriter writer = new PrintWriter(new FileWriter(domFile), true);
        DOMInfo.printTree("DOM after URL rewritting", xmlcDoc,
                          DOMInfo.PRINT_ATTR_DETAILS, writer);
        DOMStats.printStats("DOM after URL rewritting",
                            xmlcDoc, 0, writer);
        writer.close();
    }

    /**
     * Entry.
     */
    public static void main(String[] args) {
        try {
            new URLRewriteTests().runTests(args);
        } catch (Throwable except) {
            except.printStackTrace();
            System.exit(1);
        }
    }
}
