/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package tests.org.enhydra.xml.lazydom;

import tests.org.enhydra.xml.XMLTestException;
import java.io.IOException;
import java.lang.reflect.Constructor;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.lazydom.*;
import org.enhydra.xml.io.DOMParser;
import org.enhydra.xml.io.ErrorReporter;
import org.enhydra.xml.dom.DOMInfo;

/**
 * Parse an XML file into a LazyDOM.  Also handles conversions from
 * parsed LazyDOM to unexpanded LazyDOM.
 */
class LazyDOMParser {
    /**
     * Parse a document into an expanded LazyDOM.
     */
    public static LazyDocument parse(Class lazyDocumentClass,
                                     String file) throws IOException, SAXException {
        return new Parser().parse(lazyDocumentClass, file);
    }
            
    /**
     * Create a new LazyDocument.
     */
    private static LazyDocument create(Class lazyDocumentClass,
                                       LazyDocumentType docType,
                                       TemplateDOM templateDOM) {
        try {
            Constructor constr = lazyDocumentClass.getConstructor(new Class[]{DocumentType.class,
                                                                              TemplateDOM.class});
            return (LazyDocument)constr.newInstance(new Object[]{docType, templateDOM});
        } catch (Exception except) {
            throw new XMLTestException(except);
        }
    }

    /**
     * Build an unexpanded document from a template.
     */
    public static LazyDocument build(Class lazyDocumentClass,
                                     Document template) {
        // Want the real DOM, not an XMLC wrapper (if from XMLC), or LazyDOM
        // will not be correctly detected.
        if (template instanceof XMLObject) {
            template = ((XMLObject)template).getDocument();
        }
        return create(lazyDocumentClass, null,
                      new TemplateDOM((LazyDocument)template));
    }

    /**
     * Parse a XML file into an unexpaned unexpanded document.
     */
    public static LazyDocument parseBuild(Class lazyDocumentClass,
                                          String file) throws IOException, SAXException {
        return build(lazyDocumentClass,
                     parse(lazyDocumentClass, file));
    }
}

/**
 * Parser class.
 */
class Parser extends DOMParser {
    
    /**
     * Build a LazyDocument from a Document.
     */
    private LazyDocument docToLazyDoc(Document doc) {
        LazyDocument lazyDoc = new LazyDocument();
        
        // Need to clear out doc element first.
        Element docElem = lazyDoc.getDocumentElement();
        if (docElem != null) {
            System.err.println("FIXME: don't need this really");
            //lazyDoc.removeChild(docElem);
        }

        for (Node child = doc.getFirstChild(); child != null;
             child = child.getNextSibling()) {
            Node newNode = lazyDoc.importNode(child, true);
            lazyDoc.appendChild(newNode);
        }

        return lazyDoc;
    }
    
    /**
     * @see DOMParser
     */
    public LazyDocument parse(Class lazyDocumentClass,
                              String file) throws IOException, SAXException {

        setErrorHandler(new ErrorReporter(System.err));
        setValidation(true);

        // FIXME: Since the Xerces 1.2 upgrade, this Doesn't work
        // with the LazyDOM.  It causes a DTD validation error.
        // In the meantime, we just build Xerces DOM and then build
        // a LazyDOM from this.
        // FIXME: This appoach actually found some bugs, so keep the
        // copying approach as an option.
        // FIXME: fsetDocumentClassName(lazyDocumentClass.getName());
        
        Document doc = super.parse(new InputSource(file));
        return docToLazyDoc(doc);
    }
}

