/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package tests.org.enhydra.xml.lazydom;

import tests.org.enhydra.xml.XMLTestException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.FileWriter;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.enhydra.xml.lazydom.*;
import org.enhydra.xml.io.DOMParser;
import org.enhydra.xml.dom.DOMInfo;
import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.dom.SimpleDOMTraversal;
import org.enhydra.xml.dom.DOMAccess;

//FIXME: this could be more generic and shared by other tests (already used by io).

/**
 * Base class for all tests.  Provides common support routines.
 */
public class TestSupport {
    /**
     * Fail.
     */
    public void fail(String msg) {
        throw new XMLTestException(msg);
    }

    /**
     * Fail.
     */
    public void fail(String msg,
                     Throwable cause) {
        throw new XMLTestException(msg, cause);
    }

    /**
     * Print DOM to a file.
     */
    public void printNode(String msg,
                          Node node,
                          String fileName) {
        try {
            PrintWriter out = new PrintWriter(new FileWriter(fileName));
            try {
                DOMInfo.printTree(msg, node, 
                                  DOMInfo.PRINT_DEFAULT|DOMInfo.PRINT_ATTR_DETAILS,
                                  out);
            } finally {
                out.close();
            }
        } catch (Exception except) {
            throw new XMLTestException(except);
        }
        
    }

    /**
     * Load an XMLC-compiled document.
     */
    public XMLObject loadXMLCDocClass(String className) {
        try {
            Class objClass = Class.forName(className);
            return (XMLObject)objClass.newInstance();
        } catch (Exception except) {
            throw new XMLTestException(except);
        }
    }

    /**
     * Traverse a tree or subtree, cause lazy nodes to be expanded
     */
    public void traverseTree(Node root) {
        SimpleDOMTraversal traverser
            = new SimpleDOMTraversal(new SimpleDOMTraversal.Handler() {
                    public void handleNode(Node node) {
                        node.getNodeValue(); // Don't do much.
                    }
                });
        traverser.traverse(root);
    }

    /**
     * Recursively find an element-by-tag-name without expanding.
     */
    private Element accessElementByTagName(DOMAccess accessor,
                                           Node root,
                                           String tagName) {
        if ((root instanceof Element) && (((Element)root).getTagName().equals(tagName))) {
            return (Element)root;
        } else {
            Element foundNode = null;
            for (Node child = accessor.accessFirstChild(root);
                 (child != null) && (foundNode == null);
                 child = accessor.accessNextSibling(child)) {
                foundNode = accessElementByTagName(accessor, child, tagName);
            }
            return foundNode;
        }
    }

    /**
     * Find the first element of the specified tag, but do no expansion except
     * for the returned node.
     */
    public Element getElementByTagName(Document doc,
                                       Node root,
                                       String tagName) {
        DOMAccess accessor = new DOMAccess (doc);
        Element element = accessElementByTagName(accessor, root, tagName);
        if (element != null) {
            element = accessor.getExpandedElement(element);
        }
        return element;
    }
}
