#!/bin/sh
#
# Run a basic test on a HTML file, run the generate object to create
# new HTML and then verify the results.
#
#  html-basic-test [options] testid PARSER DOM RECOMP STANDALONE unqualclass [*.xmlc] input
#  
#  o -use-factory - Use the simple factory to create instances
#     of the object
#  o -should-fail - Specify if the XMLC compile should fail.
#  o -encoding htmlencoding - Specify the HTML encoding to use for the
#    read and write the HTML files.
#  o -options "options" - Pass one or more options to XMLC.  Maybe repeated.
#  o -options2 "options" - Pass one or more options to XMLC and to to second
#    xmlc compile to validate the DOM (does a -docout).
#  o -run-object-options "options" - Option to pass to r
#  o -jikes - Jikes is used, need to specify -nowarn
#  o -classloader (enhydra|multi|url) - Passed to TestAutoComp.
#  o PARSER - Parser variable from makefile.
#  o DOM - The DOM being used.
#  o RECOMP - Recompilation variable from makefile.
#  o STANDALONE - Standalong variable from makefile.
#  o unqualclass - Unqualified class name (no package).
#  o *.xmlc - XMLC option files to pass to XMLC.
#  o input - HTML file to compile.
#
#  Output files are in output/$PARSER/$RECOMP/$STANDALONE.
#
#  Setting the environment variable
#     XMLC_TEST_NONSTOP=YES
#  will keep the script from exiting non-zero on a failure.
# $Id$
#

USAGE='html-basic-test [options] testid PARSER DOM RECOMP STANDALONE unqualclass [*.xmlc] input'

set -e  # Stop on errors

# Configation
if [ "X$OSTYPE" = "Xcygwin32" ] ; then
    PS=\;
else
    PS=:
fi
JAVA="@JAVA@"
ROOT="@ROOT@"
PACKAGE="@PACKAGE@"
OUTPUT="@OUTPUT@"
XMLC_OUTPUT="@XMLC_OUTPUT@"
ENHYDRA_CLASSES="@ENHYDRA_CLASSPATH@"
TEST_CLASSES="@XMLC_TEST_CLASSES@"
TMP_DIR="tmp"
XMLC_BUILD_3_0="@XMLC_BUILD_3_0@"


# Parse options
USE_FACTORY=""
XMLC_OPTIONS=""
XMLC_OPTIONS2=""
RUN_OBJECT_OPTIONS="-check-mime-type text/html"
SHOULD_FAIL="NO"
ENCODING=""
if [ "X${XMLC_BUILD_3_0}" = "XYES" ] ; then
    CLASSLOADER="multi"
else
    CLASSLOADER="enhydra"
fi
while true ; do
    case $1 in
        -use-factory)
            USE_FACTORY="-use-factory"
            shift;;
        -should-fail)
            SHOULD_FAIL="YES"
            shift;;
        -encoding)
            shift
            ENCODING="$1"
            shift;;
        -options)
            shift
            XMLC_OPTIONS="${XMLC_OPTIONS} $1"
            shift;;
        -options2)
            shift
            XMLC_OPTIONS2="${XMLC_OPTIONS2} $1"
            shift;;
        -run-object-options)
            shift
            RUN_OBJECT_OPTIONS="${RUN_OBJECT_OPTIONS} $1"
            shift;;
        -jikes)
            XMLC_OPTIONS="${XMLC_OPTIONS} -javacflag -nowarn"
            shift;;
        -classloader)
            shift
            CLASSLOADER="$1"
            shift;;
        --) shift
            break;;
        -*) echo "Error: invalid option: $1" >&2
            exit 1;;
        *) break;;
    esac
done

if [ $# -lt 7 ] ; then
    echo "Wrong # args: ${USAGE}" >&2
    exit 1
fi

# Parse positional and *.xmlc files.

TEST_ID="$1"; shift
PARSER="$1"; shift
DOM="$1"; shift
RECOMP="$1"; shift
STANDALONE="$1"; shift
UNQUAL_CLASS="$1" ; shift

XMLC_FILES=""
while true ; do
    case $1 in
        *.xmlc)
            XMLC_FILES="${XMLC_FILES} $1"
            shift;;
        *) break;;
    esac
done

if [ $# != 1 ] ; then
    echo "Wrong # args: ${USAGE}" >&2
    exit 1
fi
INPUT="$1" ; shift

# Derived variables.
TEST_NAME=${UNQUAL_CLASS}
TEST_PACKAGE=${PACKAGE}.${PARSER}.${DOM}.${RECOMP}.${STANDALONE}
TEST_PACKAGE_DIR=${TEST_CLASSES}/`echo ${TEST_PACKAGE} | tr . /`
TEST_CLASS=${TEST_PACKAGE}.${UNQUAL_CLASS}

OUTPUT_DIR=output/${PARSER}/${DOM}/${RECOMP}/${STANDALONE}
PARSER_OUTPUT_DIR=output/${PARSER}

OUTPUT_NAME=${OUTPUT_DIR}/${TEST_NAME}
PARSER_OUTPUT_NAME=${PARSER_OUTPUT_DIR}/${TEST_NAME}

EXPECT_NAME=expected/${PARSER}/${TEST_NAME}
DOM_EXPECT_NAME=expected/${PARSER}/${DOM}/${TEST_NAME}
RECOMP_EXPECT_NAME=expected/${PARSER}/${DOM}/${TEST_NAME}

XMLC_JAR="${XMLC_OUTPUT}/lib/xmlc.jar"

# Setup to either use standalone XMLC and classpath or Enhydra stuff.
if [ "X${STANDALONE}" = "Xsa" ] ; then
    # Only put test class on class path, everything else should
    # be set in the xmlc and run-object commands.
    CLASSPATH="${TEST_CLASSES}"
    XMLC_CMD="${XMLC_OUTPUT}/bin/xmlc -d ${TEST_CLASSES}"

    # Only URLClassLoader can be used for standalone reloading tests
    CLASSLOADER="url"
else
    CLASSPATH="${ENHYDRA_CLASSES}${PS}${TEST_CLASSES}"
    XMLC_CMD="${OUTPUT}/bin/xmlc -d ${TEST_CLASSES}"
fi
export CLASSPATH

ENABLE_AUTO_EXIT='set -e'
if [ "X${XMLC_TEST_NONSTOP}" = "XYES" ] ; then
    ENABLE_AUTO_EXIT='set +e'
fi

if [ "X${DOM}" = "Xxerces" ] ; then
    XMLC_CMD="${XMLC_CMD} -domfactory org.enhydra.xml.xmlc.dom.xerces.XercesHTMLDomFactory"
fi

if [ "X${ENCODING}" != "X" ] ; then
    XMLC_OPTIONS2="${XMLC_OPTIONS2} -html:encoding ${ENCODING}"
    RUN_OBJECT_OPTIONS="${RUN_OBJECT_OPTIONS} -encoding ${ENCODING} -check-encoding ${ENCODING}"
fi

# Setup for auto-recompilation tests if specified.
if [ "X${RECOMP}" = "Xrecomp" ] ; then
    XMLC_OPTIONS="${XMLC_OPTIONS} -for-recomp"
fi

mkdir -p ${TEST_CLASSES} ${OUTPUT_DIR}

# If we are doing recompilation tests, make a copy of the 
# HTML file before compiled the file so it's not initially
# out-of-date.
if [ "X${RECOMP}" = "Xrecomp" ] ; then
    SOURCE_FILE="${TEST_PACKAGE_DIR}/`basename ${INPUT}`"
    mkdir -p ${TEST_PACKAGE_DIR}
    # Need to remove `file:' if supplied.
    cp -f `echo "${INPUT}" | sed -e 's/^file://'` ${SOURCE_FILE}
fi


# Compile the page
set +e
(set -x ; ${XMLC_CMD} ${XMLC_OPTIONS} ${XMLC_OPTIONS2} -parser ${PARSER} \
    -class ${TEST_CLASS} -sourceout ${TMP_DIR} \
    ${XMLC_FILES} ${INPUT} >${OUTPUT_NAME}.err 2>&1)
stat=$?
${ENABLE_AUTO_EXIT}
if [ $stat = 0 ] ; then
    # Ok
    if [ "X${SHOULD_FAIL}" = "XYES" ] ; then
        ../test-status 1 " $TEST_ID xmlc compile should have failed"
    fi
else
    # Failed
    if [ "X${SHOULD_FAIL}" = "XNO" ] ; then
        ../test-status 1 "$TEST_ID xmlc compile failed"
    fi
fi

# If compile should fail, just check the results.
if [ "X${SHOULD_FAIL}" = "XYES" ] ; then
    set -x
    diff ${EXPECT_NAME}.err ${OUTPUT_NAME}.err
    exit 0
fi

#
# Now that we have it compiled, we do something different on
# recomp or normal tests.
#
exit_code=0
if [ "X${RECOMP}" = "Xrecomp" ] ; then
    REVERSE_FILE="${OUTPUT_NAME}.rev.html"
    set -x

    # Create a reversed HTML file for autocomp use
    ../run-object -use-factory -reversetext ${RUN_OBJECT_OPTIONS} ${TEST_CLASS} ${REVERSE_FILE}

    # All set, try the recompilation tests.
    ../test-auto-comp -dump -classloader ${CLASSLOADER} -sourcefile ${SOURCE_FILE} -copyfile ${REVERSE_FILE} ${TEST_CLASS} >${OUTPUT_NAME}.recomp.info 2>&1

    # Check the results
    diff ${EXPECT_NAME}.err ${OUTPUT_NAME}.err || exit_code=1
    diff ${RECOMP_EXPECT_NAME}.recomp.info ${OUTPUT_NAME}.recomp.info || exit_code=1
else
    set -x
    # Run generated class to generate HTML
    ../run-object -dump ${OUTPUT_NAME}.dom ${USE_FACTORY} ${RUN_OBJECT_OPTIONS} ${TEST_CLASS} ${OUTPUT_NAME}.gen.html

    # Dump DOM and other info. In order to allow parser-specific
    # expected files to be used, which may containg warnings with file
    # names, we copy the file to location use by all test variations
    # on a single parser.
    rm -f ${PARSER_OUTPUT_NAME}.gen.html
    cp ${OUTPUT_NAME}.gen.html ${PARSER_OUTPUT_NAME}.gen.html
    ${XMLC_CMD} -parser ${PARSER} ${XMLC_OPTIONS2} -dump -methods \
        -docout ${OUTPUT_NAME}.check.html ${XMLC_FILES} \
        ${PARSER_OUTPUT_NAME}.gen.html >${OUTPUT_NAME}.gen.info 2>&1

    # This will only succeed if there is a doc type.  Ignore status, the
    # results are verified when the *.gen.info file is checked.
    grep DOCTYPE ${PARSER_OUTPUT_NAME}.gen.html >>${OUTPUT_NAME}.gen.info 2>&1 || true

    # Check the results
    diff ${EXPECT_NAME}.err ${OUTPUT_NAME}.err || exit_code=1
    diff ${DOM_EXPECT_NAME}.dom ${OUTPUT_NAME}.dom || exit_code=1
    diff ${DOM_EXPECT_NAME}.gen.info ${OUTPUT_NAME}.gen.info || exit_code=1
fi

exec ../test-status $exit_code $TEST_ID
