/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package tests.org.enhydra.xml.xmlc.metadata;

import org.enhydra.xml.dom.DOMInfo;
import org.enhydra.xml.xmlc.metadata.*;
import tests.org.enhydra.xml.xmlc.TestUtils;
import org.enhydra.xml.io.ErrorReporter;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import java.io.LineNumberReader;
import java.io.IOException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.File;

/**
 * Basic set of tests for XMLC metadata parsing.
 */
public class BasicTests {
    /**
     * Test parsing an *.xmlc file and dump the DOM to a file.
     * Optionally rewrite the metadata file
     */
    private void testParseMetaData(String xmlcFile,
                                   String dumpFile,
                                   String rewriteFile) throws Exception {
        MetaDataDocument metaDataDoc
            = MetaDataDocument.parseMetaData(new InputSource(xmlcFile),
                                             new ErrorReporter(System.err),
                                             null);
        if (dumpFile != null) {
            PrintWriter out = new PrintWriter(new FileWriter(dumpFile));
            try {
                DOMInfo.printTree(xmlcFile, metaDataDoc, out);
            } finally {
                out.close();
            }
        }
        if (rewriteFile != null) {
            metaDataDoc.serialize(new File(rewriteFile));
        }
    }
    
    /**
     * Entry.
     */
    static public void main(String[] args) {
        if ((args.length < 2) || (args.length > 3)) {
            System.err.println("Wrong # args: BasicTests in out [rewrite]");
            System.exit(1);
        }
        try {
            BasicTests tests = new BasicTests();
            tests.testParseMetaData(args[0], args[1],
                                    ((args.length > 2) ? args[2] : null));
        } catch (Exception except) {
            except.printStackTrace();
            System.exit(1);
        }
    }
}
